@@ -28,20 +28,22 @@ __license__ = "MIT"
 __date__ = "14/10/2021"
 
 
+import time
+
 from orangewidget import gui
-from orangewidget.widget import Input, Output
-from orangewidget.widget import OWBaseWidget
 from orangewidget.settings import Setting
+from orangewidget.widget import Input, Output, OWBaseWidget
+from processview.core.manager import DatasetState, ProcessManager
+from processview.core.superviseprocess import SuperviseProcess
 from silx.gui import qt
+from silx.gui.utils.concurrent import submitToQtMainThread
+
+from tomwer.core.futureobject import FutureTomwerObject
 from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.futurescan import FutureTomwerScan
+from tomwer.core.volume.volumebase import TomwerVolumeBase
 from tomwer.gui.cluster.supervisor import (
     FutureTomwerScanObserverWidget as _FutureTomwerScanObserverWidget,
 )
-from processview.core.superviseprocess import SuperviseProcess
-from processview.core.manager import ProcessManager, DatasetState
-from silx.gui.utils.concurrent import submitToQtMainThread
-import time
 
 
 class FutureSupervisorOW(OWBaseWidget, openclass=True):
@@ -68,15 +70,15 @@ class FutureSupervisorOW(OWBaseWidget, openclass=True):
     ]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
-    _static_input = Setting(dict())
+    _ewoks_default_inputs = Setting(dict())
 
     class Inputs:
         future_in = Input(
-            name="future_data",
-            type=FutureTomwerScan,
+            name="future_tomo_obj",
+            type=FutureTomwerObject,
             doc="data with some remote processing",
             multiple=True,
             default=True,
@@ -84,12 +86,13 @@ class FutureSupervisorOW(OWBaseWidget, openclass=True):
 
     class Outputs:
         data = Output(name="data", type=TomwerScanBase)
+        volume = Output(name="volume", type=TomwerVolumeBase)
 
     def __init__(self, parent=None):
         super().__init__(parent)
         # gui
         layout = gui.vBox(self.mainArea, self.name).layout()
-        self._widget = FutureTomwerScanObserverWidget(
+        self._widget = FutureTomwerObjectObserverWidget(
             parent=self, name=self.windowTitle()
         )
         layout.addWidget(self._widget)
@@ -103,38 +106,42 @@ class FutureSupervisorOW(OWBaseWidget, openclass=True):
     def convertWhenFinished(self):
         return self._widget.convertWhenFinished()
 
-    def _convertBackAutomatically(self, future_scan, status):
-        if not isinstance(future_scan, FutureTomwerScan):
+    def _convertBackAutomatically(self, future_tomo_obj, status):
+        if not isinstance(future_tomo_obj, FutureTomwerObject):
             raise TypeError(
-                f"future_scan is expected to be an instance of {FutureTomwerScan} and not {type(future_scan)}"
+                f"future_tomo_obj is expected to be an instance of {FutureTomwerObject} and not {type(future_tomo_obj)}"
             )
-        if status == "finished" and self.convertWhenFinished():
-            self._convertBack(future_scan)
+        if status in ("finished", "completed") and self.convertWhenFinished():
+            self._convertBack(future_tomo_obj)
 
-    def _convertBack(self, future_scan):
-        if not isinstance(future_scan, FutureTomwerScan):
+    def _convertBack(self, future_tomo_obj):
+        if not isinstance(future_tomo_obj, FutureTomwerObject):
             raise TypeError(
-                f"future_scan is expected to be an instance of {FutureTomwerScan} and not {type(future_scan)}"
+                f"future_tomo_obj is expected to be an instance of {FutureTomwerObject} and not {type(future_tomo_obj)}"
             )
-        self._futureHasBeenConverted(future_scan, future_scan.scan)
+        self._futureHasBeenConverted(future_tomo_obj, future_tomo_obj.tomo_obj)
 
     @Inputs.future_in
-    def add(self, future_scan, signal_id=None):
-        if future_scan is not None:
-            self._widget.addFutureScan(future_scan=future_scan)
+    def add(self, future_tomo_obj, signal_id=None):
+        if future_tomo_obj is not None:
+            self._widget.addFutureTomoObj(future_tomo_obj=future_tomo_obj)
 
-    def _futureHasBeenConverted(self, future_scan, scan):
+    def _futureHasBeenConverted(self, future_tomo_obj, tomo_obj):
         # clean client to free resources
-        self._widget.removeFutureScan(future_scan=future_scan)
-        future_scan.clear_clients()
-        if scan is not None:
-            self.Outputs.data.send(scan)
+        self._widget.removeFutureTomoObj(future_tomo_obj=future_tomo_obj)
+        if tomo_obj is not None:
+            if isinstance(tomo_obj, TomwerScanBase):
+                self.Outputs.data.send(tomo_obj)
+            elif isinstance(tomo_obj, TomwerVolumeBase):
+                self.Outputs.volume.send(tomo_obj)
 
 
-class FutureTomwerScanObserverWidget(_FutureTomwerScanObserverWidget, SuperviseProcess):
+class FutureTomwerObjectObserverWidget(
+    _FutureTomwerScanObserverWidget, SuperviseProcess
+):
     """add dataset state notification (ProcessManager) to the original FutureTomwerScanObserverWidget"""
 
-    REFRESH_FREQUENCE = 2
+    REFRESH_FREQUENCE = 10
     """time between call to updateView"""
 
     def __init__(self, name, parent=None):
@@ -156,57 +163,59 @@ class FutureTomwerScanObserverWidget(_FutureTomwerScanObserverWidget, SuperviseP
         self.stopRefresh()
         super().close()
 
-    def addFutureScan(self, future_scan: FutureTomwerScan):
-        super().addFutureScan(future_scan)
-        self._updateScanSupervisor(future_scan)
+    def addFutureTomoObj(self, future_tomo_obj: FutureTomwerObject):
+        super().addFutureTomoObj(future_tomo_obj)
+        self._updateTomoObjSupervisor(future_tomo_obj)
 
-    def removeFutureScan(self, future_scan: FutureTomwerScan):
-        self._updateScanSupervisor(future_scan)
-        super().removeFutureScan(future_scan)
+    def removeFutureTomoObj(self, future_tomo_obj: FutureTomwerObject):
+        self._updateTomoObjSupervisor(future_tomo_obj)
+        super().removeFutureTomoObj(future_tomo_obj)
 
-    def _updateScanSupervisor(self, future_scan):
-        r_id = future_scan.process_requester_id
+    def _updateTomoObjSupervisor(self, future_tomo_obj):
+        r_id = future_tomo_obj.process_requester_id
         if r_id is not None:
             requester_name = ProcessManager().get_process(r_id).name
         else:
             requester_name = "unknow"
         details = f"job spawn by {requester_name}"
-        if future_scan is None:
+        if future_tomo_obj is None:
             return
-        elif future_scan.status == "error":
+        elif future_tomo_obj.status == "error":
             state = DatasetState.FAILED
-            details = "\n".join([details, future_scan.exception()])
-        elif future_scan.status == "pending":
+        elif future_tomo_obj.status == "pending":
             details = "\n".join([details, "pending"])
             state = DatasetState.PENDING
-        elif future_scan.status == "finished":
+        elif future_tomo_obj.status in ("finished", "completed"):
+            details = future_tomo_obj.logs or "no log found"
             state = DatasetState.SUCCEED
-        elif future_scan.status == "running":
+        elif future_tomo_obj.status == "running":
             details = "\n".join([details, "running"])
             state = DatasetState.ON_GOING
-        elif future_scan.status == "cancelled":
+        elif future_tomo_obj.status == "cancelled":
             details = "\n".join([details, "job cancelled"])
             state = DatasetState.SKIPPED
-        elif future_scan.status is None:
+        elif future_tomo_obj.status is None:
             return
         else:
-            raise ValueError(f"future scan status {future_scan.status} is not managed")
+            raise ValueError(
+                f"future scan status '{future_tomo_obj.status}' is not managed, {type(future_tomo_obj.status)}"
+            )
         ProcessManager().notify_dataset_state(
-            dataset=future_scan.scan,
+            dataset=future_tomo_obj.tomo_obj,
             process=self,
             state=state,
             details=details,
         )
 
-    def _updateStatus(self, future_scan):
-        self._updateScanSupervisor(future_scan)
-        super()._updateStatus(future_scan)
+    def _updateStatus(self, future_tomo_obj):
+        self._updateTomoObjSupervisor(future_tomo_obj)
+        super()._updateStatus(future_tomo_obj)
 
 
 class _RefreshThread(qt.QThread):
     """Simple thread to call a refresh callback each refresh_frequence (seconds)"""
 
-    TIME_BETWEEN_LOOP = 0.5
+    TIME_BETWEEN_LOOP = 1.0
 
     def __init__(self, callback, refresh_frequence) -> None:
         super().__init__()
