@@ -27,15 +27,18 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "13/10/2021"
 
-from orangewidget import gui
-from orangewidget.widget import Output
-from orangewidget.widget import OWBaseWidget
-from orangewidget.settings import Setting
-from tomwer.gui.cluster.slurm import SlurmSettingsWidget
-from tomwer.core.cluster import SlurmClusterConfiguration
-import tomwer.core.process.cluster.supervisor
 import logging
 
+from silx.gui import qt
+
+from orangewidget import gui
+from orangewidget.settings import Setting
+from orangewidget.widget import Output, OWBaseWidget
+
+import tomwer.core.process.cluster.supervisor
+from tomwer.core.cluster import SlurmClusterConfiguration
+from tomwer.gui.cluster.slurm import SlurmSettingsWindow
+
 _logger = logging.getLogger(__name__)
 
 
@@ -53,35 +56,46 @@ class SlurmClusterOW(OWBaseWidget, openclass=True):
     keywords = ["tomography", "tomwer", "slurm", "cluster", "configuration"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
-    _static_input = Setting(dict())
+    _ewoks_default_inputs = Setting(dict())
 
     ewokstaskclass = tomwer.core.process.cluster.supervisor._SupervisorPlaceHolder
 
     class Outputs:
-        data_out = Output(name="cluster_config", type=SlurmClusterConfiguration)
+        config_out = Output(name="cluster_config", type=SlurmClusterConfiguration)
 
     def __init__(self, parent=None) -> None:
         super().__init__(parent)
         layout = gui.vBox(self.mainArea, self.name).layout()
-        self._widget = SlurmSettingsWidget(parent=self)
+        self._widget = SlurmSettingsWindow(parent=self)
+        self._widget.setWindowFlags(qt.Qt.Widget)
         layout.addWidget(self._widget)
 
-        if self._static_input != {}:
-            self._widget.setConfiguration(self._static_input)
+        if self._ewoks_default_inputs != {}:
+            self._widget.setConfiguration(self._ewoks_default_inputs)
 
         # trigger the signal to avoid any user request
-        self.Outputs.data_out.send(self.getConfiguration())
+        self.Outputs.config_out.send(self.getConfiguration())
 
         # connect signal / slot
         self._widget.sigConfigChanged.connect(self._configurationChanged)
 
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("_static_input", None)
+        if static_input not in (None, {}):
+            _logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     def _configurationChanged(self):
         slurmClusterconfiguration = self.getConfiguration()
-        self.Outputs.data_out.send(slurmClusterconfiguration)
-        self._static_input = slurmClusterconfiguration.to_dict()
+        self.Outputs.config_out.send(slurmClusterconfiguration)
+        self._ewoks_default_inputs = slurmClusterconfiguration.to_dict()
 
     def getConfiguration(self):
         return self._widget.getSlurmClusterConfiguration()
