@@ -29,7 +29,8 @@ __date__ = "10/01/2018"
 
 
 import logging
-from orangewidget import widget, gui
+
+from orangewidget import gui, widget
 from processview.gui.processmanager import ProcessManagerWindow
 
 logger = logging.getLogger(__name__)
@@ -51,8 +52,8 @@ class AdvancementOW(widget.OWBaseWidget, openclass=True):
     keywords = ["tomography", "process", "advancement"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     def __init__(self, parent=None):
         widget.OWBaseWidget.__init__(self, parent)
