@@ -27,22 +27,23 @@ __authors__ = ["C. Nemoz", "H. Payno"]
 __license__ = "MIT"
 __date__ = "01/12/2016"
 
-from orangewidget import widget, gui
-from tomwer.core.utils import logconfig
-from tomwer.gui.control.datalist import GenericDataListDialog
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.web.client import OWClient
-from tomwer.core.scan.scanbase import TomwerScanBase
-import tomwer.core.process.control.scanlist
-from orangewidget.settings import Setting
-from orangewidget.widget import Output, Input, OWBaseWidget
 import logging
 
+from orangewidget import gui
+from orangewidget.settings import Setting
+from orangewidget.widget import Input, Output, OWBaseWidget
+from silx.gui import qt
+
+import tomwer.core.process.control.scanlist
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.control.datalist import GenericScanListWindow
+
 logger = logging.getLogger(__name__)
 
 
-class DataListOW(OWBaseWidget, OWClient, openclass=True):
-    name = "data list"
+class DataListOW(OWBaseWidget, openclass=True):
+    name = "scan list"
     id = "orange.widgets.tomwer.scanlist"
     description = "List path to reconstructions/scans. Those path will be send to the next box once validated."
     icon = "icons/scanlist.svg"
@@ -50,8 +51,8 @@ class DataListOW(OWBaseWidget, OWClient, openclass=True):
     keywords = ["tomography", "file", "tomwer", "folder"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     _scanIDs = Setting(list())
 
@@ -61,7 +62,7 @@ class DataListOW(OWBaseWidget, OWClient, openclass=True):
         data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase)
+        data = Input(name="data", type=TomwerScanBase, multiple=True)
 
     def __init__(self, parent=None):
         """A simple annuary which register all folder containing completed scan
@@ -71,44 +72,32 @@ class DataListOW(OWBaseWidget, OWClient, openclass=True):
 
         :param parent: the parent widget
         """
-        widget.OWBaseWidget.__init__(self, parent)
-        OWClient.__init__(self)
-        self.widget = GenericDataListDialog(parent=self)
+        super().__init__(parent)
+
+        self.widget = GenericScanListWindow(parent=self)
+        self.widget.setWindowFlags(qt.Qt.Widget)
         self.widget.sigUpdated.connect(self._updateSettings)
         self._loadSettings()
         layout = gui.vBox(self.mainArea, self.name).layout()
         layout.addWidget(self.widget)
-        self.widget._sendButton.clicked.connect(self._sendList)
+        self.widget._widget._sendButton.clicked.connect(self._sendList)
 
         # expose API
         self.n_scan = self.widget.n_scan
 
         # alias used for the 'simple workflow' for now
         self.start = self._sendList
-        self.clear = self.widget.clear
+        self.clear = self.widget._widget.clear
 
     def _sendList(self):
         """Send a signal for each list to the next widget"""
-
-        def send(scan_):
-            mess = "sending one scan %s" % d
-            logger.debug(
-                mess,
-                extra={
-                    logconfig.DOC_TITLE: self.widget._scheme_title,
-                    logconfig.SCAN_ID: str(scan_),
-                },
-            )
-            self.Outputs.data.send(scan_)
-
-        for d in self.widget.datalist.items:
-            if d in self.widget.datalist._scanIDs:
-                send(scan_=self.widget.datalist._scanIDs[d])
-            else:
-                logger.error("unable to retrieve scan {}".format(d))
+        for data_id, item in self.widget.datalist._myitems.items():
+            logger.debug(f"sending {data_id}")
+            data_obj = item.data(qt.Qt.UserRole)
+            self.Outputs.data.send(data_obj)
 
     @Inputs.data
-    def add(self, scan):
+    def add(self, scan, *args, **kwargs):
         if scan is None:
             return
 
@@ -117,14 +106,24 @@ class DataListOW(OWBaseWidget, OWClient, openclass=True):
     def _loadSettings(self):
         for scan in self._scanIDs:
             assert isinstance(scan, str)
+            # kept for backward compatibility since 0.11. To be removed on the future version.
             if "@" in scan:
                 entry, file_path = scan.split("@")
-                hdf5_tomo_scan = HDF5TomoScan(entry=entry, scan=file_path)
-                self.add(hdf5_tomo_scan)
+                nxtomo_scan = NXtomoScan(entry=entry, scan=file_path)
+                self.add(nxtomo_scan)
             else:
                 self.add(scan)
 
     def _updateSettings(self):
         self._scanIDs = []
-        for scan in self.widget.datalist.items:
+        for scan in self.widget.datalist._myitems:
             self._scanIDs.append(scan)
+
+    def keyPressEvent(self, event):
+        """
+        To shortcut orange and make sure the `delete` key will be interpreted we need to overwrite this function
+        """
+        if event.key() == qt.Qt.Key_Delete:
+            self.widget._removeSelected()
+        else:
+            super().keyPressEvent(event)
