@@ -28,36 +28,35 @@ __license__ = "MIT"
 __date__ = "06/03/2020"
 
 
+from orangewidget import gui, settings, widget
+from orangewidget.widget import Output
 from silx.gui import qt
 
 import tomwer.core.process.control.datalistener.datalistener
 from orangecontrib.tomwer.widgets.utils import WidgetLongProcessing
-from orangewidget import widget, gui
-from orangewidget.widget import Output
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.blissscan import BlissScan
 from tomwer.core.process.control.datalistener import DataListener
-from tomwer.utils import docstring
-from tomwer.synctools.datalistener import DataListenerQThread
-from tomwer.gui.control.datalistener import DataListenerWidget
-from tomwer.synctools.datalistener import MockDataListenerQThread
-from tomwer.synctools.stacks.control.datalistener import DataListenerProcessStack
-from tomwer.core.process.control.nxtomomill import NxTomomillProcess
 from tomwer.core.process.control.datalistener.rpcserver import (
     send_signal_to_local_rpc_servers,
 )
-from orangewidget import settings
+from tomwer.core.process.control.nxtomomill import H5ToNxProcess
+from tomwer.core.scan.blissscan import BlissScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.control.datalistener import DataListenerWidget
+from tomwer.synctools.datalistener import DataListenerQThread, MockDataListenerQThread
+from tomwer.synctools.stacks.control.datalistener import DataListenerProcessStack
+from tomwer.utils import docstring
 
 try:
     # new HDF5Config class
     from nxtomomill.io.config import TomoHDF5Config as HDF5Config
 except ImportError:
     from nxtomomill.io.config import HDF5Config
-from typing import Iterable
+
 import functools
 import logging
 import signal
 import sys
+from typing import Iterable
 
 logger = logging.getLogger(__name__)
 
@@ -71,7 +70,7 @@ class DataListenerOW(
     Then the bliss file will be converted to .nx file, NXtomo compliant.
     """
 
-    name = "data listener"
+    name = "scan listener"
     id = "orangecontrib.widgets.tomwer.control.DataListenerOW.DataListenerOW"
     description = (
         "The widget will receive information from bliss acquisition "
@@ -91,13 +90,11 @@ class DataListenerOW(
         "NXtomo",
     ]
 
-    allows_cycle = True
-    compress_signal = False
-
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
 
-    ewoksorangeclass = (
+    ewokstaskclass = (
         tomwer.core.process.control.datalistener.datalistener._DataListenerTaskPlaceHolder
     )
 
@@ -141,30 +138,49 @@ class DataListenerOW(
         # set up
         self._loadSettings()
 
+        # for conveniance start the listener when create it.
+        self.activate(True)
+
     def _loadSettings(self):
-        if "bliss_server_configuration" in self._static_input:
-            bliss_configuration = self._static_input["bliss_server_configuration"]
+        if "bliss_server_configuration" in self._static_input:  # pylint: disable=E1135
+            bliss_configuration = self._static_input[  # pylint: disable=E1136
+                "bliss_server_configuration"
+            ]
         else:
             bliss_configuration = self._blissConfiguration
         if bliss_configuration != {}:
             self._widget.setBlissServerConfiguation(bliss_configuration)
-        if "nxtomomill_cfg_file" in self._static_input:
-            nxtomo_cfg_file = self._static_input["nxtomomill_cfg_file"]
+        if "nxtomomill_cfg_file" in self._static_input:  # pylint: disable=E1135
+            nxtomo_cfg_file = self._static_input[  # pylint: disable=E1136
+                "nxtomomill_cfg_file"
+            ]
         else:
             nxtomo_cfg_file = self._nxtomo_cfg_file
         self._widget.setCFGFilePath(nxtomo_cfg_file)
-        if "output_dir" in self._static_input:
-            self._widget.setOutputFolder(self._static_input["output_dir"])
+        if "output_dir" in self._static_input:  # pylint: disable=E1135
+            self._widget.setOutputFolder(
+                self._static_input["output_dir"]  # pylint: disable=E1136
+            )
 
     def getNXTomomillConfiguration(self):
         cfg_file = self._widget.getCFGFilePath()
+
+        def create_default_config():
+            configuration = HDF5Config()
+            # insure minimal bacward compatibility
+            if hasattr(configuration, "bam_single_file"):
+                configuration.bam_single_file = True
+            return configuration
+
         if cfg_file in (None, ""):
-            config = HDF5Config()
+            config = create_default_config()
         else:
             try:
                 config = HDF5Config.from_cfg_file(cfg_file)
-            except Exception:
-                config = HDF5Config()
+            except Exception as e:
+                logger.warning(f"Fail to load configuraiton file. Error is {e}")
+                config = create_default_config()
+
         return config
 
     def _process_bliss_file_frm_tuple(self, t):
@@ -177,13 +193,14 @@ class DataListenerOW(
         )
         configuration = self.getNXTomomillConfiguration()
         # overwrite output file
-        configuration.output_file = NxTomomillProcess.deduce_output_file_path(
+        configuration.output_file = H5ToNxProcess.deduce_output_file_path(
             bliss_scan.master_file,
             entry=bliss_scan.entry,
             outputdir=self._widget.getOutputFolder(),
+            scan=bliss_scan,
         )
         if success:
-            self._processingStack.add(scan=bliss_scan, configuration=configuration)
+            self._processingStack.add(data=bliss_scan, configuration=configuration)
         else:
             # TODO: maybe remove the acquisition ?
             pass
@@ -221,7 +238,7 @@ class DataListenerOW(
         self.processing_state(activate, info="listener active")
         self._widget.blockSignals(old)
 
-    def _signal_scan_ready(self, scan, future_scan):
+    def _signal_scan_ready(self, scan, future_tomo_obj):
         if scan is None:
             return
         assert isinstance(scan, Iterable)
@@ -241,17 +258,19 @@ class DataListenerOW(
 
     def _jsonRPCConfigChanged(self):
         self._blissConfiguration = self._widget.getBlissServerConfiguration()
-        self._static_input[
-            "bliss_server_configuration"
-        ] = self._widget.getBlissServerConfiguration()
-        self._static_input["output_dir"] = self._widget.getOutputFolder()
+        self._static_input["bliss_server_configuration"] = (  # pylint: disable=E1137
+            self._widget.getBlissServerConfiguration()
+        )
+        self._static_input["output_dir"] = (  # pylint: disable=E1137
+            self._widget.getOutputFolder()
+        )
         if self.is_active():
             self.activate(False)
             self.activate(True)
 
     def _nxtomoFileChanged(self, cfg_file):
         self._nxtomo_cfg_file = cfg_file
-        self._static_input["nxtomomill_cfg_file"] = cfg_file
+        self._static_input["nxtomomill_cfg_file"] = cfg_file  # pylint: disable=E1137
 
     def setMock(self, mock, acquisitions):
         self._mock = mock
@@ -334,10 +353,10 @@ class _PortOccupyDialog(qt.QDialog):
         self.host = host
 
         mess = (
-            "port ({port}) of {host} already in use. \n Maybe an other "
+            f"port ({port}) of {host} already in use. \n Maybe an other "
             "instance of `datalistener` is running in this session or "
             "another tomwer session. \n As this widget is connecting with "
-            "bliss we enforce it to be unique.".format(port=port, host=host)
+            "bliss we enforce it to be unique."
         )
         self.layout().addWidget(qt.QLabel(mess, self))
         self.setWindowTitle("Unable to launch two listener in parallel")
