@@ -27,35 +27,35 @@ __authors__ = ["C. Nemoz", "H. Payno"]
 __license__ = "MIT"
 __date__ = "07/12/2016"
 
+import functools
 import logging
-from orangewidget import gui
-from orangewidget import settings
-from orangewidget.widget import Output, Input
+
+from orangewidget import gui, settings
+from orangewidget.widget import Input, Output
+from processview.core.manager import DatasetState, ProcessManager
 from silx.gui import qt
 
 import tomwer.core.process.control.scantransfer
-from tomwer.core.process.control.scantransfer import ScanTransfer
-from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
-from tomwer.web.client import OWClient
-from tomwer.gui.control.datatransfert import DataTransfertSelector
-from tomwer.core.scan.scanbase import TomwerScanBase
-from processview.core.manager import ProcessManager, DatasetState
 from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
+from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
+from tomwer.core.process.control.scantransfer import ScanTransferTask
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.control.datatransfert import DataTransfertSelector
 from tomwer.utils import docstring
 
 logger = logging.getLogger(__name__)
 
 
-class DataTransfertOW(SuperviseOW, OWClient):
+class DataTransfertOW(SuperviseOW):
     """
     A simple widget managing the copy of an incoming folder to an other one
 
     :param parent: the parent widget
     """
 
-    name = "data transfert"
+    name = "data transfer"
     id = "orange.widgets.tomwer.foldertransfert"
-    description = "This widget insure data transfert of the received data "
+    description = "This widget insure data transfer of the received data "
     description += "to the given directory"
     icon = "icons/folder-transfert.svg"
     priority = 30
@@ -70,13 +70,12 @@ class DataTransfertOW(SuperviseOW, OWClient):
         "folder",
     ]
 
-    ewokstaskclass = tomwer.core.process.control.scantransfer.ScanTransfer
+    ewokstaskclass = tomwer.core.process.control.scantransfer.ScanTransferTask
 
     settingsHandler = CallbackSettingsHandler()
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
 
     dest_dir_settings = settings.Setting(str())
     """Parameters directly editabled from the TOFU interface"""
@@ -90,17 +89,17 @@ class DataTransfertOW(SuperviseOW, OWClient):
         data = Output(name="data", type=TomwerScanBase)
 
     def __init__(self, parent=None):
-        SuperviseOW.__init__(self, parent)
-        OWClient.__init__(self)
+        super().__init__(parent)
 
         self._destDir = None
         self._forceSync = False
+        self._threads = []
 
         # define GUI
         self._widget = DataTransfertSelector(
             parent=self,
             rnice_option=True,
-            default_root_folder=ScanTransfer.getDefaultOutputDir(),
+            default_root_folder=ScanTransferTask.getDefaultOutputDir(),
         )
         self._layout = gui.vBox(self.mainArea, self.name).layout()
         self._layout.addWidget(self._widget)
@@ -113,7 +112,7 @@ class DataTransfertOW(SuperviseOW, OWClient):
         if self.dest_dir_settings != "":
             self._widget.setFolder(self.dest_dir_settings)
 
-    def _requestFolder(self):
+    def _requestFolder(self):  # pragma: no cover
         """Launch a QFileDialog to ask the user the output directory"""
         dialog = qt.QFileDialog(self)
         dialog.setWindowTitle("Destination folder")
@@ -126,15 +125,9 @@ class DataTransfertOW(SuperviseOW, OWClient):
 
         return dialog.selectedFiles()[0]
 
-    def signalTransfertOk(self, input_scan, output_scan):
+    def transfertDoneCallback(self, output_scan):
         if output_scan is None:
             return
-        ProcessManager().notify_dataset_state(
-            dataset=input_scan, process=self, state=DatasetState.SUCCEED
-        )
-        ProcessManager().notify_dataset_state(
-            dataset=output_scan, process=self, state=DatasetState.SUCCEED
-        )
         self.Outputs.data.send(output_scan)
         self.scanready.emit(output_scan)
 
@@ -147,31 +140,45 @@ class DataTransfertOW(SuperviseOW, OWClient):
 
     @Inputs.data
     def process(self, scan):
-        self._process(scan=scan)
+        self._process(data=scan)
 
-    def _process(self, scan, move=False, force=True, noRsync=False):
-        if scan is None:
+    def _process(self, data, move=False, force=True, noRsync=False):
+        if data is None:
             return
+        elif not isinstance(data, TomwerScanBase):
+            raise TypeError("data is expected to be an instance of TomwerScanBase")
+
+        inputs = {
+            "data": data,
+            "move": move,
+            "overwrite": force,
+            "noRsync": noRsync,
+            "dest_dir": self._destDir,
+            "block": self._forceSync,
+            "serialize_output_data": False,
+        }
+        thread = ThreadDataTransfer(
+            inputs=inputs,
+            data=data,
+            process=self,
+        )
         try:
-            process = ScanTransfer(
-                inputs={
-                    "data": scan,
-                    "move": move,
-                    "force": force,
-                    "noRsync": noRsync,
-                    "dest_dir": self._destDir,
-                    "block": self._forceSync,
-                }
-            )
-            process.run()
+            process = ScanTransferTask(inputs=inputs)
         except Exception as e:
-            logger.error("data transfer failed. Reason is {}".format(e))
-            ProcessManager().notify_dataset_state(
-                dataset=scan,
-                process=self,
-                state=DatasetState.FAILED,
-                details=str(e),
-            )
+            logger.error(e)
+        else:
+            dest_dir = process.getDestinationDir(data.path, ask_for_output=False)
+            if dest_dir is not None:
+                thread.finished.connect(
+                    functools.partial(
+                        self.transfertDoneCallback,
+                        data._deduce_transfert_scan(
+                            process.getDestinationDir(data.path)
+                        ),
+                    )
+                )
+                thread.start()
+            self._threads.append(thread)
 
     @docstring(SuperviseOW)
     def reprocess(self, dataset):
@@ -186,3 +193,30 @@ class DataTransfertOW(SuperviseOW, OWClient):
     def isCopying(self):
         # for now only move file is handled
         return False
+
+
+class ThreadDataTransfer(qt.QThread):
+    def __init__(self, data, inputs, process) -> None:
+        super().__init__()
+        self._inputs = inputs
+        self._data = data
+        self._process = process
+
+    def run(self):
+        try:
+            process = ScanTransferTask(inputs=self._inputs)
+            process.run()
+        except Exception as e:
+            logger.error(f"data transfer failed. Reason is {e}")
+            ProcessManager().notify_dataset_state(
+                dataset=self._data,
+                process=self._process,
+                state=DatasetState.FAILED,
+                details=str(e),
+            )
+        else:
+            ProcessManager().notify_dataset_state(
+                dataset=self._data,
+                process=self._process,
+                state=DatasetState.SUCCEED,
+            )
