@@ -27,19 +27,18 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "21/07/2020"
 
-from silx.gui import qt
-from orangewidget import gui
+import logging
+
+from orangewidget import gui, settings
 from orangewidget.widget import Input, Output
+from processview.core.manager import DatasetState, ProcessManager
+from silx.gui import qt
 
 import tomwer.core.process.control.scanvalidator
-from tomwer.gui.control.datavalidator import DataValidator
-from tomwer.core.scan.scanbase import TomwerScanBase
 from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
-from processview.core.manager import ProcessManager, DatasetState
-from tomwer.core.scan.scanbase import _TomwerBaseDock
-from orangewidget import settings
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+from tomwer.gui.control.datavalidator import DataValidator
 from tomwer.utils import docstring
-import logging
 
 _logger = logging.getLogger(__name__)
 
@@ -53,7 +52,7 @@ class DataValidatorOW(SuperviseOW):
     :param parent: the parent widget
     """
 
-    name = "data validator"
+    name = "scan validator"
     id = "orange.widgets.tomwer.datavalidator"
     description = """Widget displaying results of a reconstruction and asking to
         the user if he want to validate or not the reconstruction. User can also ask
@@ -64,7 +63,6 @@ class DataValidatorOW(SuperviseOW):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
 
     ewokstaskclass = tomwer.core.process.control.scanvalidator._ScanValidatorPlaceHolder
 
@@ -128,7 +126,7 @@ class DataValidatorOW(SuperviseOW):
         # validated and so not accessible
         if (
             self._warnValManualShow is False
-            and len(self._widget._scansToValidate) >= self._NB_SCAN_BF_WARN
+            and len(self._widget._scans) >= self._NB_SCAN_BF_WARN
         ):
             mess = (
                 "Please note that the scanValidator is actually storing %s "
@@ -157,7 +155,7 @@ class DataValidatorOW(SuperviseOW):
         return self._widget.isValidationManual()
 
     def _changeReconsParamsEmited(self, scan):
-        self.Outputs.recons_param_changed.send(scan)
+        self.Outputs.recons_params_changed.send(scan)
 
     def _scanReadyEmitted(self, scan):
         ProcessManager().notify_dataset_state(
@@ -167,9 +165,13 @@ class DataValidatorOW(SuperviseOW):
 
     def _updateSettings(self):
         viewer = self._widget._centralWidget
-        self._viewer_config["mode"] = viewer.getDisplayMode()
-        self._viewer_config["slice_opt"] = viewer.getSliceOption()
-        self._viewer_config["radio_opt"] = viewer.getRadioOption()
+        self._viewer_config["mode"] = viewer.getDisplayMode()  # pylint: disable=E1137
+        self._viewer_config["slice_opt"] = (  # pylint: disable=E1137
+            viewer.getSliceOption()
+        )
+        self._viewer_config["radio_opt"] = (  # pylint: disable=E1137
+            viewer.getRadioOption()
+        )
 
     def _setSettings(self, settings):
         viewer = self._widget._centralWidget
@@ -183,7 +185,7 @@ class DataValidatorOW(SuperviseOW):
         viewer.blockSignals(old_state)
 
     def getNScanToValidate(self):
-        return len(self._widget._scansToValidate)
+        return len(self._widget._scans)
 
     def _validateScan(self, scan):
         self._widget._validateScan(scan=scan)
