@@ -27,24 +27,24 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "19/07/2018"
 
-from silx.gui import qt
+import logging
+
 from orangewidget import gui
+from orangewidget.widget import Input, Output
+from processview.core.manager import DatasetState
+from silx.gui import qt
 
 import tomwer.core.process.conditions.filters
 from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
-from orangewidget.widget import Input, Output
-from tomwer.web.client import OWClient
-from tomwer.gui.conditions.filter import FileNameFilterWidget
-from tomwer.core.scan.scanbase import TomwerScanBase
-from processview.core.manager import DatasetState
-from tomwer.utils import docstring
 from tomwer.core.process.conditions import filters
-import logging
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.conditions.filter import FileNameFilterWidget
+from tomwer.utils import docstring
 
 logger = logging.getLogger(__name__)
 
 
-class NameFilterOW(SuperviseOW, OWClient):
+class NameFilterOW(SuperviseOW):
     name = "scan filter"
     id = "orange.widgets.tomwer.filterow"
     description = (
@@ -57,20 +57,18 @@ class NameFilterOW(SuperviseOW, OWClient):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
 
-    ewokstaskclass = tomwer.core.process.conditions.filters.FileNameFilter
+    ewokstaskclass = tomwer.core.process.conditions.filters.FileNameFilterTask
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase)
+        data = Input(name="data", type=TomwerScanBase, multiple=True)
 
     class Outputs:
         data = Output(name="data", type=TomwerScanBase)
 
     def __init__(self, parent=None):
         """ """
-        SuperviseOW.__init__(self, parent)
-        OWClient.__init__(self)
+        super().__init__(parent)
 
         self.widget = FileNameFilterWidget(parent=self)
         self.widget.setContentsMargins(0, 0, 0, 0)
@@ -82,29 +80,30 @@ class NameFilterOW(SuperviseOW, OWClient):
         layout.addWidget(spacer)
 
     @Inputs.data
-    def applyfilter(self, scan):
+    def applyfilter(self, scan, *args, **kwargs):
         if scan is None:
             return
         if not isinstance(scan, TomwerScanBase):
             raise TypeError(f"{scan} is expected to be an instance of {TomwerScanBase}")
 
-        process = filters.FileNameFilter(
+        process = filters.FileNameFilterTask(
             inputs={
                 "data": scan,
                 "pattern": self.getPattern(),
                 "filter_type": self.getActiveFilter(),
                 "invert_result": self.invertFilterAction(),
+                "serialize_output_data": False,  # avoid spending time on scan serialization / deserialization when use orange
             }
         )
         process.run()
         out = process.outputs.data
         if out is not None:
             self.set_dataset_state(dataset=scan, state=DatasetState.SUCCEED)
-            logger.processSucceed("{} pass through filter".format(str(scan)))
+            logger.processSucceed(f"{scan} pass through filter")
             self._signalScanReady(scan)
         else:
             self.set_dataset_state(dataset=scan, state=DatasetState.FAILED)
-            logger.processFailed("{} NOT pass through filter".format(str(scan)))
+            logger.processFailed(f"{scan} NOT pass through filter")
 
     @docstring(SuperviseOW)
     def reprocess(self, dataset):
