@@ -27,18 +27,19 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "19/07/2018"
 
-from silx.gui import qt
+import functools
+import logging
+
 from orangewidget import gui
 from orangewidget.settings import Setting
 from orangewidget.widget import Input, Output
+from processview.core.manager import DatasetState, ProcessManager
+from silx.gui import qt
 
 import tomwer.core.process.control.timer
-from tomwer.core.scan.scanbase import TomwerScanBase
-from processview.core.manager import ProcessManager, DatasetState
 from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
+from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.utils import docstring
-import logging
-import functools
 
 _logger = logging.getLogger(__name__)
 
@@ -76,13 +77,12 @@ class TimerOW(SuperviseOW):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
     _waiting_time = Setting(int(1))
 
-    ewokstaskclass = tomwer.core.process.control.timer.Timer
+    ewokstaskclass = tomwer.core.process.control.timer.TimerTask
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase)
+        data = Input(name="data", type=TomwerScanBase, multiple=True)
 
     class Outputs:
         data = Output(name="data", type=TomwerScanBase)
@@ -99,7 +99,7 @@ class TimerOW(SuperviseOW):
         self._widget.timeChanged.connect(self._updateTime)
 
     @Inputs.data
-    def process(self, scan):
+    def process(self, scan, *args, **kwargs):
         if scan is None:
             return
 
