@@ -29,22 +29,22 @@ __authors__ = [
 __license__ = "MIT"
 __date__ = "28/09/2020"
 
-from silx.gui import qt
-from orangewidget import widget, gui
-from orangewidget.widget import Output
-from tomwer.gui.debugtools.datasetgenerator import DatasetGeneratorDialog
-from tomwer.core.scan.edfscan import EDFTomoScan
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.scan.blissscan import BlissScan
-from tomwer.core.utils.scanutils import MockEDF
-from tomwer.core.utils.scanutils import MockHDF5
-from tomwer.core.utils.scanutils import MockBlissAcquisition
-from orangewidget.settings import Setting
-from silx.gui.utils import concurrent
-import threading
 import os
+import threading
 import uuid
 
+from orangewidget import gui, widget
+from orangewidget.settings import Setting
+from orangewidget.widget import Output
+from silx.gui import qt
+from silx.gui.utils import concurrent
+
+from tomwer.core.scan.blissscan import BlissScan
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.utils.scanutils import MockBlissAcquisition, MockEDF, MockNXtomo
+from tomwer.gui.debugtools.datasetgenerator import DatasetGeneratorDialog
+
 
 class DatasetGeneratorOW(widget.OWBaseWidget, openclass=True):
     """
@@ -59,8 +59,8 @@ class DatasetGeneratorOW(widget.OWBaseWidget, openclass=True):
     keywords = ["tomography", "file", "tomwer", "dataset", "debug"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     _settings = Setting(dict())
 
@@ -150,13 +150,13 @@ class _DatasetGeneratorThread(threading.Thread):
             scan = MockEDF.mockScan(
                 scanID=self.scan_path, nRadio=self.n_proj, dim=self.dims[0]
             )
-        elif self.type_to_generate == HDF5TomoScan.__name__:
-            scan = MockHDF5(
+        elif self.type_to_generate == NXtomoScan.__name__:
+            scan = MockNXtomo(
                 scan_path=self.scan_path,
                 n_proj=self.n_proj,
                 n_ini_proj=self.n_proj,
                 create_ini_dark=self.create_ini_dark,
-                create_ini_ref=self.create_ini_flat,
+                create_ini_flat=self.create_ini_flat,
                 dim=self.dims[0],
             ).scan
         elif self.type_to_generate == BlissScan.__name__:
@@ -172,7 +172,5 @@ class _DatasetGeneratorThread(threading.Thread):
             )
             scan = mock.create_bliss_scan()
         else:
-            raise ValueError(
-                "type to generate {} not recognized".format(self.type_to_generate)
-            )
+            raise ValueError(f"type to generate {self.type_to_generate} not recognized")
         concurrent.submitToQtMainThread(self.callback, scan)
