@@ -29,10 +29,11 @@ __authors__ = [
 __license__ = "MIT"
 __date__ = "30/09/2020"
 
-from orangewidget import widget, gui
-from tomwer.gui.debugtools.objectinspector import ObjectInspector
+from orangewidget import gui, widget
 from orangewidget.widget import Input
 
+from tomwer.gui.debugtools.objectinspector import ObjectInspector
+
 
 class ObjectInspectorOW(widget.OWBaseWidget, openclass=True):
     """
@@ -47,8 +48,8 @@ class ObjectInspectorOW(widget.OWBaseWidget, openclass=True):
     keywords = ["tomography", "file", "tomwer", "dataset", "debug"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     class Inputs:
         data = Input(name="object", type=object)
