@@ -29,25 +29,30 @@ __license__ = "MIT"
 __date__ = "02/11/2020"
 
 
-from orangewidget.widget import Input, Output
+import logging
+from typing import Optional
+
 from orangewidget import gui
-from tomwer.gui.edit.dkrfpatch import DarkRefPatchWidget
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.synctools.stacks.edit.darkflatpatch import DarkFlatPatchProcessStack
 from orangewidget.settings import Setting
+from orangewidget.widget import Input, Output
+from silx.io.url import DataUrl
+
+import tomwer.core.process.edit.darkflatpatch
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.edit.dkrfpatch import DarkRefPatchWidget
+from tomwer.synctools.stacks.edit.darkflatpatch import DarkFlatPatchProcessStack
+from tomwer.utils import docstring
+
 from ...orange.managedprocess import SuperviseOW
 from ..utils import WidgetLongProcessing
-from tomwer.utils import docstring
-import tomwer.core.process.edit.darkflatpatch
-import logging
 
 _logger = logging.getLogger(__name__)
 
 
 class DarkFlatPatchOW(WidgetLongProcessing, SuperviseOW):
     """
-    Widget to define on the fly the image_key of a HDF5TomoScan
+    Widget to define on the fly the image_key of a NXtomoScan
     """
 
     name = "dark-flat-patch"
@@ -71,11 +76,10 @@ class DarkFlatPatchOW(WidgetLongProcessing, SuperviseOW):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
 
     _urlsSetting = Setting(dict())
 
-    ewokstaskclass = tomwer.core.process.edit.darkflatpatch.DarkFlatPatch
+    ewokstaskclass = tomwer.core.process.edit.darkflatpatch.DarkFlatPatchTask
 
     class Inputs:
         data = Input(name="data", type=TomwerScanBase)
@@ -106,8 +110,8 @@ class DarkFlatPatchOW(WidgetLongProcessing, SuperviseOW):
     def process(self, scan):
         if scan is None:
             return
-        elif not isinstance(scan, HDF5TomoScan):
-            _logger.error("We can only patch dark and flat for HDF5TomoScan")
+        elif not isinstance(scan, NXtomoScan):
+            _logger.error("We can only patch dark and flat for NXtomoScan")
         else:
             self._processingStack.add(scan, self.getConfiguration())
 
@@ -116,11 +120,21 @@ class DarkFlatPatchOW(WidgetLongProcessing, SuperviseOW):
         self.process(dataset)
 
     def getConfiguration(self):
+        def cast_url_to_str(url: Optional[DataUrl]):
+            if url is None:
+                return None
+            elif not isinstance(url, DataUrl):
+                raise TypeError(
+                    f"url is expected to be an optional DataUrl. {type(url)} provided instead"
+                )
+            else:
+                return url.path()
+
         return {
-            "darks_start": self.widget.getStartDarkUrl(),
-            "flats_start": self.widget.getStartFlatUrl(),
-            "darks_end": self.widget.getEndDarkUrl(),
-            "flats_end": self.widget.getEndFlatUrl(),
+            "darks_start": cast_url_to_str(self.widget.getStartDarkUrl()),
+            "flats_start": cast_url_to_str(self.widget.getStartFlatUrl()),
+            "darks_end": cast_url_to_str(self.widget.getEndDarkUrl()),
+            "flats_end": cast_url_to_str(self.widget.getEndFlatUrl()),
         }
 
     @Inputs.config_in
@@ -151,7 +165,7 @@ class DarkFlatPatchOW(WidgetLongProcessing, SuperviseOW):
                     except Exception as e:
                         _logger.error(e)
 
-    def _endProcessing(self, scan, future_scan):
+    def _endProcessing(self, scan, future_tomo_obj):
         WidgetLongProcessing._endProcessing(self, scan)
         if scan is not None:
             self.Outputs.data.send(scan)
