@@ -29,24 +29,27 @@ __license__ = "MIT"
 __date__ = "28/10/2020"
 
 
-from orangewidget.widget import Input, Output
+import logging
+
 from orangewidget import gui
+from orangewidget.widget import Input, Output
+
+import tomwer.core.process.edit.imagekeyeditor
+from tomwer.core.scan.nxtomoscan import NXtomoScan
 from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
 from tomwer.gui.edit.imagekeyeditor import ImageKeyDialog
 from tomwer.synctools.stacks.edit.imagekeyeditor import ImageKeyEditorProcessStack
-import tomwer.core.process.edit.imagekeyeditor
 from tomwer.utils import docstring
-from ..utils import WidgetLongProcessing
+
 from ...orange.managedprocess import SuperviseOW
-import logging
+from ..utils import WidgetLongProcessing
 
 _logger = logging.getLogger(__name__)
 
 
 class ImageKeyEditorOW(SuperviseOW, WidgetLongProcessing):
     """
-    Widget to define on the fly the image_key of a HDF5TomoScan
+    Widget to define on the fly the image_key of a NXtomoScan
     """
 
     name = "image-key-editor"
@@ -70,9 +73,8 @@ class ImageKeyEditorOW(SuperviseOW, WidgetLongProcessing):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
 
-    ewokstaskclass = tomwer.core.process.edit.imagekeyeditor.ImageKeyEditor
+    ewokstaskclass = tomwer.core.process.edit.imagekeyeditor.ImageKeyEditorTask
 
     class Inputs:
         data = Input(name="data", type=TomwerScanBase)
@@ -101,7 +103,7 @@ class ImageKeyEditorOW(SuperviseOW, WidgetLongProcessing):
     def process(self, scan):
         if scan is None:
             return
-        elif not isinstance(scan, HDF5TomoScan):
+        elif not isinstance(scan, NXtomoScan):
             _logger.error("You can only edit image keys for the HDF5 scans")
         else:
             self._scan = scan
@@ -123,8 +125,9 @@ class ImageKeyEditorOW(SuperviseOW, WidgetLongProcessing):
         if self._scan is None:
             return
         self._processingStack.add(self._scan, configuration=self.getConfiguration())
+        self.accept()
 
-    def _endProcessing(self, scan, future_scan):
+    def _endProcessing(self, scan, future_tomo_obj):
         WidgetLongProcessing._endProcessing(self, scan)
         if scan is not None:
             self.Outputs.data.send(scan)
