@@ -27,11 +27,21 @@ __license__ = "MIT"
 __date__ = "10/12/2021"
 
 
+import gc
+import os
+import pickle
+import tempfile
+
+import h5py
+import numpy
 import pytest
+from orangecanvas.scheme.readwrite import literal_dumps
 from silx.gui import qt
-from tomwer.test.utils import skip_gui_test
 from silx.gui.utils.testutils import TestCaseQt
+from silx.io.url import DataUrl
+
 from orangecontrib.tomwer.widgets.edit.DarkFlatPatchOW import DarkFlatPatchOW
+from tomwer.tests.utils import skip_gui_test
 
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
@@ -46,8 +56,21 @@ class TestDarkFlatPatch(TestCaseQt):
         self._window.setAttribute(qt.Qt.WA_DeleteOnClose)
         self._window.close()
         self._window = None
-        super().tearDown()
+        gc.collect()
 
     def test(self):
         self._window.show()
         self.qWaitForWindowExposed(self._window)
+
+    def test_serialiazation(self):
+        with tempfile.TemporaryDirectory() as tmp_path:
+            file_path = os.path.join(tmp_path, "darks.hdf5")
+            with h5py.File(file_path, mode="w") as h5f:
+                h5f.require_group("darks")["0"] = numpy.ones((10, 10))
+            dark_url = DataUrl(file_path=file_path, data_path="darks/0")
+            self._window.widget.setStartDarkUrl(url=dark_url)
+            config = self._window.getConfiguration()
+            pickle.dumps(config)
+            literal_dumps(config)
+
+            self._window.setConfiguration(config)
