@@ -27,16 +27,19 @@ __license__ = "MIT"
 __date__ = "10/12/2021"
 
 
+import gc
+
 import pytest
 from silx.gui import qt
-from tomwer.test.utils import skip_gui_test
 from silx.gui.utils.testutils import TestCaseQt
+
 from orangecontrib.tomwer.widgets.edit.ImageKeyEditorOW import ImageKeyEditorOW
+from tomwer.tests.utils import skip_gui_test
 
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
 class TestImageKeyEditor(TestCaseQt):
-    """Test that the axis widget work correctly"""
+    """Test that the ImageKeyEditorOW widget work correctly (at least launched)"""
 
     def setUp(self):
         super().setUp()
@@ -46,7 +49,7 @@ class TestImageKeyEditor(TestCaseQt):
         self._window.setAttribute(qt.Qt.WA_DeleteOnClose)
         self._window.close()
         self._window = None
-        super().tearDown()
+        gc.collect()
 
     def test(self):
         self._window.show()
