@@ -2,23 +2,28 @@
 the icon change (to difference them) and here the input is not a Table
 but a TomwerScanBase (use silx.gui.qt instead of AnyQt too)"""
 
-import sys
-import os
 import code
-import keyword
 import itertools
+import keyword
+import os
+import sys
 import unicodedata
 from unittest.mock import patch
-from silx.gui import qt
-from orangewidget import widget, gui
-from orangewidget.utils import itemmodels
+
+from orangewidget import gui, widget
 from orangewidget.settings import Setting, SettingsHandler
-from orangewidget.utils.widgetpreview import WidgetPreview
-from orangewidget.widget import OWBaseWidget, Input, Output
+from orangewidget.utils import itemmodels
+from orangewidget.widget import Input, Output, OWBaseWidget
+from silx.gui import qt
+from tomoscan.serie import Serie
+
 import tomwer.core.process.script.python
 from tomwer.core.cluster.cluster import SlurmClusterConfiguration
-from tomwer.core.scan.futurescan import FutureTomwerScan
+from tomwer.core.futureobject import FutureTomwerObject
 from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.volume.volumebase import TomwerVolumeBase
+from tomwer.core.tomwer_object import TomwerObject
+
 
 __all__ = ["OWPythonScript"]
 
@@ -32,7 +37,6 @@ def text_format(foreground=qt.Qt.black, weight=qt.QFont.Normal):
 
 class PythonSyntaxHighlighter(qt.QSyntaxHighlighter):
     def __init__(self, parent=None):
-
         self.keywordFormat = text_format(qt.Qt.blue, qt.QFont.Bold)
         self.stringFormat = text_format(qt.Qt.darkGreen)
         self.defFormat = text_format(qt.Qt.black, qt.QFont.Bold)
@@ -374,13 +378,34 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
     keywords = ["file", "program"]
 
     class Inputs:
+        tomo_obj = Input(
+            "tomo_obj",
+            TomwerObject,
+            replaces=["in_tomo_obj"],
+            default=True,
+            multiple=True,
+        )
+        tomo_objs = Input(
+            "tomo_objs",
+            TomwerObject,
+            replaces=["in_tomo_objs"],
+            default=True,
+            multiple=True,
+        )
         data = Input(
             "data", TomwerScanBase, replaces=["in_data"], default=True, multiple=True
         )
-        future_data = Input(
-            "future_data",
-            FutureTomwerScan,
-            replaces=["in_future_data"],
+        volume = Input(
+            "volume",
+            TomwerVolumeBase,
+            replaces=["in_volume"],
+            default=True,
+            multiple=True,
+        )
+        future_tomo_obj = Input(
+            "future_tomo_obj",
+            FutureTomwerObject,
+            replaces=["in_future_tomo_obj"],
             default=True,
             multiple=True,
         )
@@ -406,19 +431,40 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
             default=True,
             multiple=True,
         )
+        serie = Input(
+            "serie",
+            Serie,
+            replaces=["in_serie"],
+            default=True,
+            multiple=True,
+        )
 
     class Outputs:
         data = Output("data", TomwerScanBase, replaces=["out_data"])
+        tomo_obj = Output("tomo_obj", TomwerObject, replaces=["out_tomo_obj"])
+        tomo_objs = Output("tomo_objs", tuple, replaces=["out_tomo_objs"])
+        volume = Output("volume", TomwerVolumeBase, replaces=["out_volume"])
         object = Output("Object", object, replaces=["out_object"], auto_summary=False)
         configuration = Output("configuration", dict, replaces=["out_configuration"])
-        future_data = Output(
-            "future_data", FutureTomwerScan, replaces=["out_future_data"]
+        future_tomo_obj = Output(
+            "future_tomo_obj", FutureTomwerObject, replaces=["out_future_tomo_obj"]
         )
         cluster_config = Output(
             "cluster_config", SlurmClusterConfiguration, replaces=["out_cluster_config"]
         )
+        serie = Output("serie", dict, replaces=["out_serie"])
 
-    signal_names = ("data", "future_data", "object", "configuration", "cluster_config")
+    signal_names = (
+        "data",
+        "volume",
+        "future_tomo_obj",
+        "object",
+        "configuration",
+        "cluster_config",
+        "serie",
+        "tomo_obj",
+        "tomo_objs",
+    )
 
     settingsHandler = PrepareSavingSettingsHandler()
 
@@ -438,7 +484,7 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
         for name in self.signal_names:
             setattr(self, name, {})
 
-        for s in self.libraryListSource:
+        for s in self.libraryListSource:  # pylint: disable=E1133
             s.flags = 0
 
         self._cachedDocuments = {}
@@ -591,22 +637,38 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
     def set_data(self, data, id):
         self.handle_input(data, id, "data")
 
+    @Inputs.volume  # noqa F811
+    def set_volume(self, volume, id):
+        self.handle_input(volume, id, "volume")
+
     @Inputs.object  # noqa F811
-    def set_object(self, data, id):
+    def set_object(self, data, id):  # noqa F811
         self.handle_input(data, id, "object")
 
     @Inputs.configuration  # noqa F811
     def set_configuration(self, data, id):
         self.handle_input(data, id, "configuration")
 
-    @Inputs.future_data  # noqa F811
-    def set_object(self, data, id):
-        self.handle_input(data, id, "future_data")
+    @Inputs.future_tomo_obj  # noqa F811
+    def set_future_tomo_object(self, data, id):  # noqa F811 pylint: disable=E0102
+        self.handle_input(data, id, "future_tomo_obj")
 
     @Inputs.cluster_config  # noqa F811
-    def set_object(self, data, id):
+    def set_cluster_config(self, data, id):  # noqa F811 pylint: disable=E0102
         self.handle_input(data, id, "cluster_config")
 
+    @Inputs.serie  # noqa F811
+    def set_serie(self, data, id):
+        self.handle_input(data, id, "serie")
+
+    @Inputs.tomo_obj  # noqa F811
+    def set_tomo_obj(self, tomo_obj, id):
+        self.handle_input(tomo_obj, id, "tomo_obj")
+
+    @Inputs.tomo_objs  # noqa F811
+    def set_tomo_objs(self, tomo_objs, id):
+        self.handle_input(tomo_objs, id, "tomo_objs")
+
     def handleNewSignals(self):
         self.commit()
 
@@ -624,7 +686,7 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
         self.libraryList.append(Script("New script", self.text.toPlainText(), 0))
         self.setSelectedScript(len(self.libraryList) - 1)
 
-    def onAddScriptFromFile(self, *args):
+    def onAddScriptFromFile(self, *args):  # pragma: no cover
         filename, _ = qt.QFileDialog.getOpenFileName(
             self,
             "Open Python Script",
@@ -662,7 +724,7 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
             self.currentScriptIndex = current
 
     def documentForScript(self, script=0):
-        if type(script) != Script:
+        if type(script) is not Script:
             script = self.libraryList[script]
         if script not in self._cachedDocuments:
             doc = qt.QTextDocument(self)
@@ -697,7 +759,7 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
             self.text.document().setPlainText(self.libraryList[index].script)
             self.text.document().setModified(False)
 
-    def saveScript(self):
+    def saveScript(self):  # pragma: no cover
         index = self.selectedScriptIndex()
         if index is not None:
             script = self.libraryList[index]
@@ -746,14 +808,8 @@ class OWPythonScript(widget.OWBaseWidget, openclass=True):
             if not isinstance(out_var, signal_type) and out_var is not None:
                 self.Error.add_message(
                     signal,
-                    "'{}' has to be an instance of '{}'.".format(
-                        signal, signal_type.__name__
-                    ),
+                    f"'{signal}' has to be an instance of '{signal_type.__name__}'",
                 )
                 getattr(self.Error, signal)()
                 out_var = None
             getattr(self.Outputs, signal).send(out_var)
-
-
-if __name__ == "__main__":  # pragma: no cover
-    WidgetPreview(OWPythonScript).run()
