@@ -27,32 +27,35 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "04/03/2019"
 
-import tomwer.core.process.reconstruction.axis
-from ..utils import WidgetLongProcessing
-from orangewidget import gui
-from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
-from orangewidget.settings import Setting
-from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
-from orangewidget.widget import Input, Output
-from processview.gui.processmanager import ProcessManager
-from processview.core.manager import DatasetState
-from processview.core import helpers as pv_helpers
-from tomwer.core.process.reconstruction.axis.mode import AxisMode
-from tomwer.synctools.stacks.reconstruction.axis import AxisProcessStack
-from tomwer.core.process.reconstruction.axis import AxisProcess
-from tomwer.synctools.axis import QAxisRP
-from tomwer.gui.reconstruction.axis import AxisWindow
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.scan.scanbase import _TomwerBaseDock
-from silx.gui import qt
-from tomwer.core import settings
-from tomwer.core import utils
-import functools
 import copy
-
+import functools
 import logging
 
+from orangewidget import gui
+from orangewidget.settings import Setting
+from orangewidget.widget import Input, Output
+
+from processview.core import helpers as pv_helpers
+from processview.core.manager import DatasetState
+from processview.gui.processmanager import ProcessManager
+
+from silx.gui import qt
+
+import tomwer.core.process.reconstruction.axis
+from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
+from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
+from tomwer.core import settings
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.core.process.reconstruction.axis import AxisTask
+from tomwer.core.process.reconstruction.axis.mode import AxisMode
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+from tomwer.gui.reconstruction.axis import AxisWindow
+from tomwer.synctools.axis import QAxisRP
+from tomwer.synctools.stacks.reconstruction.axis import AxisProcessStack
+
+from ..utils import WidgetLongProcessing
+
 logger = logging.getLogger(__name__)
 
 
@@ -66,7 +69,7 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
                       keep alive qt widgets.
     """
 
-    name = "default center of rotation"
+    name = "center of rotation finder"
     id = "orange.widgets.tomwer.axis"
     description = "use to compute the center of rotation"
     icon = "icons/axis.png"
@@ -81,26 +84,25 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         "center of position",
     ]
 
-    ewokstaskclass = tomwer.core.process.reconstruction.axis.AxisProcess
+    ewokstaskclass = tomwer.core.process.reconstruction.axis.AxisTask
 
     want_main_area = True
     resizing_enabled = True
-    allows_cycle = True
-    compress_signal = False
 
     settingsHandler = CallbackSettingsHandler()
 
     sigScanReady = qt.Signal(TomwerScanBase)
     """Signal emitted when a scan is ready"""
 
-    _rpSetting = Setting(dict())
-    "AxisRP store as dict"
-    # this one is keep for backward compatibility. Will be removed on 0.10 I guess
-
-    static_input = Setting({"data": None, "axis_params": None})
+    _ewoks_default_inputs = Setting({"data": None, "axis_params": None, "gui": None})
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase, doc="one scan to be process")
+        data = Input(
+            name="data",
+            type=TomwerScanBase,
+            doc="one scan to be process",
+            multiple=True,
+        )
         data_recompute_axis = Input(
             name="change recons params",
             type=_TomwerBaseDock,
@@ -120,23 +122,29 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         if axis_params is not None:
             if not isinstance(axis_params, QAxisRP):
                 raise TypeError(
-                    "axis_params should be an instance of "
-                    "QAxisRP. Not {}".format(type(axis_params))
+                    f"axis_params should be an instance of QAxisRP. Not {type(axis_params)}"
                 )
         self._axis_params = axis_params or QAxisRP()
 
         # handle settings
-        axis_params_settings = self.static_input.get("axis_params", None)
-        if axis_params_settings is None:
-            axis_params_settings = self._rpSetting
-        if axis_params_settings != dict():
+        #  axis params settings
+        axis_params_settings = self._ewoks_default_inputs.get("axis_params", None)
+        if axis_params_settings not in (None, dict()):
             try:
                 self._axis_params.load_from_dict(axis_params_settings)
             except Exception as e:
-                logger.error("fail to load reconstruction settings:", str(e))
-        gui_settings = self.static_input.get("gui", None)
+                logger.error(f"fail to load reconstruction settings: {e}")
+
+        #  gui settings
+        gui_settings = self._ewoks_default_inputs.get("gui", {})
         if gui_settings is None:
-            gui_settings = self._rpSetting.get("gui", {})
+            gui_settings = {}
+
+        original_mode = self._axis_params.mode
+        if original_mode is AxisMode.manual:
+            original_cor = self._axis_params.relative_cor_value
+        else:
+            original_cor = None
 
         self.__lastAxisProcessParamsCache = None
         # used to memorize the last (reconstruction parameters, scan)
@@ -163,24 +171,35 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         self.setMode = self._widget.setMode
         self.getRadioMode = self._widget.getRadioMode
         self.getEstimatedCor = self._widget.getEstimatedCor
+        self.setEstimatedCor = self._widget.setEstimatedCor
         self.getAxis = self._widget.getAxis
         self.setValueLock = self._widget.setValueLock
         self._setModeLockFrmSettings = self._widget._setModeLockFrmSettings
         self._setValueLockFrmSettings = self._widget._setValueLockFrmSettings
 
         # load settings
-        if "mode_is_lock" in gui_settings:
-            mode_lock = gui_settings["mode_is_lock"]
-            # if the mode is manual or read ignore lock
-            if not (
-                mode_lock is True
-                and self.getRadioMode() in (AxisMode.manual, AxisMode.read)
-            ):
-                self._setModeLockFrmSettings(mode_lock)
+        try:
+            if "mode_is_lock" in gui_settings:
+                mode_lock = gui_settings["mode_is_lock"]
+                # if the mode is manual or read ignore lock
+                if not (
+                    mode_lock is True
+                    and self.getRadioMode() in (AxisMode.manual, AxisMode.read)
+                ):
+                    self._setModeLockFrmSettings(mode_lock)
+
+            if "value_is_lock" in gui_settings:
+                if gui_settings["value_is_lock"] is True:
+                    self._setValueLockFrmSettings(bool(gui_settings["value_is_lock"]))
+
+            auto_update_estimated_cor = gui_settings.get(
+                "auto_update_estimated_cor", True
+            )
+            self._widget.setAutoUpdateEstimatedCor(auto_update_estimated_cor)
+
+        except Exception as e:
+            logger.warning(f"Fail to load settings. Error is {str(e)}")
 
-        if "value_is_lock" in gui_settings:
-            if gui_settings["value_is_lock"] is True:
-                self._setValueLockFrmSettings(bool(gui_settings["value_is_lock"]))
         # expose API
         self._applyBut = self._widget._axisWidget._controlWidget._applyBut
         # connect Signal / Slot
@@ -195,10 +214,44 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         self._widget.sigMethodChanged.connect(self._updateSettingsVals)
         self._widget.sigLockModeChanged.connect(self._updateSettingsVals)
 
+        self._widget._axisWidget._radioAxis._controlWidget._mainWidget._calculationWidget._nearValueCB.toggled.connect(
+            self._updateSettingsVals
+        )
+
+        # handle special case of the manual mode
+        # force axis to manual because not handled by the widget setup dircetly
+        if original_mode in (
+            AxisMode.manual,
+            AxisMode.read,
+        ):
+            self._axis_params.mode = original_mode
+            if original_cor is not None:
+                self._axis_params.set_relative_value(original_cor)
+
+            self.setMode(original_mode)
+            # force gui update
+            self._widget.sigMethodChanged.emit("manual")
+            # hard fix, but this widget has to be redone and rework anyway
+            self._widget._axisWidget._radioAxis._controlWidget._mainWidget._calculationWidget._modeChanged()
+            if original_cor is not None:
+                self._widget._axisWidget._controlWidget._positionInfo.setPosition(
+                    relative_cor=original_cor, abs_cor=None
+                )
+
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("static_input", None)
+        if static_input not in (None, {}):
+            logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     def _processingStart(self, *args, **kwargs):
         WidgetLongProcessing._startProcessing(self)
 
-    def _scanProcessed(self, scan, future_scan):
+    def _scanProcessed(self, scan, future_tomo_obj):
         assert isinstance(scan, TomwerScanBase)
         WidgetLongProcessing._endProcessing(self, scan)
         if self.isValueLock() or self.isModeLocked():
@@ -253,7 +306,7 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
                 self.__lastAxisProcessParamsCache = params_cache
                 callback = functools.partial(self._updatePosition, self.__scan)
                 self._processingStack.add(
-                    scan=self.__scan,
+                    data=self.__scan,
                     configuration=self._axis_params.to_dict(),
                     callback=callback,
                 )
@@ -273,15 +326,11 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
                 self.__scan._axis_params.set_relative_value(
                     self._axis_params.relative_cor_value
                 )
+                _cor_value = self.__scan.axis_params.relative_cor_value
                 pv_helpers.notify_succeed(
                     process=self,
                     dataset=self.__scan,
-                    details="axis calculation defined for {}: {} (using {})"
-                    "".format(
-                        str(self.__scan.path),
-                        str(self.__scan.axis_params.relative_cor_value),
-                        "manual",
-                    ),
+                    details=f"axis calculation defined for {self.__scan.path}: {_cor_value} (using manual)",
                 )
             # validate the center of rotation
             pm = ProcessManager()
@@ -320,17 +369,14 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         msg.exec_()
 
     def _updateSettingsVals(self):
-        self._rpSetting = self._axis_params.to_dict()
-        self._rpSetting["gui"] = {
-            "mode_is_lock": self.isModeLocked(),
-            "value_is_lock": self.isValueLock(),
-        }
-        self.static_input = {
+        # remove rp setting to the advantage of 'static_input'
+        self._ewoks_default_inputs = {
             "data": None,
             "axis_params": self._axis_params.to_dict(),
             "gui": {
                 "mode_is_lock": self.isModeLocked(),
                 "value_is_lock": self.isValueLock(),
+                "auto_update_estimated_cor": self._widget._axisWidget.getAutoUpdateEstimatedCor(),
             },
         }
 
@@ -343,9 +389,6 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
     def recons_params(self):
         return self._axis_params
 
-    def _instanciateReconsParams(self):
-        return QAxisRP()
-
     def _lock_axis_controls(self, lock):
         """
 
@@ -370,13 +413,12 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         return self._widget.isModeLock()
 
     @Inputs.data
-    def new_data_in(self, scan):
+    def new_data_in(self, scan, *args, **kwargs):
         if scan is None:
             return
         scan_ = copy.copy(scan)
         if not (
-            settings.isOnLbsram(scan)
-            and utils.isLowOnMemory(settings.get_lbsram_path())
+            settings.isOnLbsram(scan) and is_low_on_memory(settings.get_lbsram_path())
         ):
             set_position = False  # avoid shift reset
             self._widget.setScan(scan=scan_, set_position=set_position)
@@ -391,7 +433,7 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         self._axis_params.frame_width = scan.dim_1
         if (
             settings.isOnLbsram(scan)
-            and utils.isLowOnMemory(settings.get_lbsram_path()) is True
+            and is_low_on_memory(settings.get_lbsram_path()) is True
         ):
             self._updatePosition(scan=scan)
             self.scan_ready(scan=scan)
@@ -402,19 +444,19 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
                     self._axis_params.relative_cor_value
                 )
                 cor = scan._axis_params.relative_cor_value
-                if isinstance(scan, HDF5TomoScan):
+                if isinstance(scan, NXtomoScan):
                     entry = scan.entry
                 else:
                     entry = "entry"
                 with scan.acquire_process_file_lock():
-                    ap = AxisProcess(
+                    ap = AxisTask(
                         process_id=self.process_id,
                         inputs={
                             "data": None,
                         },
                     )
                     try:
-                        AxisProcess._register_process(
+                        AxisTask._register_process(
                             process_file=scan.process_file,
                             entry=entry,
                             process=ap,
@@ -433,10 +475,11 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
                         state=DatasetState.SUCCEED,
                     )
             else:
-                processing_class = AxisProcess(
+                processing_class = AxisTask(
                     inputs={
                         "axis_params": self._axis_params,
                         "data": self.__scan,
+                        "serialize_output_data": False,
                     }
                 )
                 for patch in self._patches:
@@ -448,28 +491,30 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
                 self.scan_ready(scan=scan)
 
         elif self.isValueLock():
-            entry = scan.entry if isinstance(scan, HDF5TomoScan) else "entry"
+            entry = scan.entry if isinstance(scan, NXtomoScan) else "entry"
             cor = self._axis_params.relative_cor_value
             scan.axis_params.set_relative_value(cor)
             scan._axis_params.mode = "manual"
-            try:
-                AxisProcess._register_process(
-                    process_file=scan.process_file,
-                    entry=entry,
-                    process=AxisProcess,
-                    results={"center_of_rotation": cor if cor is not None else "-"},
-                    configuration=self._axis_params.to_dict(),
-                    process_index=scan.pop_process_index(),
-                    overwrite=True,
-                )
-            except Exception as e:
-                logger.warning(f"Fail to register Axis process. Error is {e}")
+            with scan.acquire_process_file_lock():
+                try:
+                    AxisTask._register_process(
+                        process_file=scan.process_file,
+                        entry=entry,
+                        process=AxisTask,
+                        results={"center_of_rotation": cor if cor is not None else "-"},
+                        configuration=self._axis_params.to_dict(),
+                        process_index=scan.pop_process_index(),
+                        overwrite=True,
+                    )
+                except Exception as e:
+                    logger.warning(f"Fail to register Axis process. Error is {e}")
             ProcessManager().notify_dataset_state(
                 dataset=scan,
-                process=AxisProcess(
+                process=AxisTask(
                     process_id=self.process_id,
                     inputs={
                         "data": None,
+                        "serialize_output_data": False,
                     },
                 ),
                 state=DatasetState.SUCCEED,
@@ -479,7 +524,7 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         elif self.isModeLocked():
             callback = functools.partial(self._updatePosition, scan)
             self._processingStack.add(
-                scan=scan, configuration=self._axis_params.to_dict(), callback=callback
+                data=scan, configuration=self._axis_params.to_dict(), callback=callback
             )
         else:
             self.activateWindow()
@@ -498,9 +543,9 @@ class AxisOW(SuperviseOW, WidgetLongProcessing):
         if scan is not None:
             # for now The behavior for reprocessing is the sama as for processing
             if hasattr(scan, "instance"):
-                self.process(scan=scan.instance)
+                self.process(scan.instance)
             else:
-                self.process(scan=scan)
+                self.process(scan)
 
     def close(self):
         self._processingStack.stop()
