@@ -27,22 +27,28 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "15/12/2021"
 
+import logging
 from typing import Optional
 
-from processview.core.manager.manager import DatasetState, ProcessManager
-from tomwer.core import settings
-from orangecontrib.tomwer.widgets import utils
-from ...orange.managedprocess import SuperviseOW
-from orangecontrib.tomwer.widgets.utils import WidgetLongProcessing
-from orangewidget.settings import Setting
 from orangewidget import gui
+from orangewidget.settings import Setting
 from orangewidget.widget import Input, Output
+from processview.core.manager.manager import DatasetState, ProcessManager
 from silx.gui import qt
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.synctools.stacks.reconstruction.castvolume import CastVolumeProcessStack
+
 import tomwer.core.process.reconstruction.nabu.castvolume
+from orangecontrib.tomwer.widgets.utils import WidgetLongProcessing
+from tomwer.core import settings
+from tomwer.core.cluster.cluster import SlurmClusterConfiguration
+from tomwer.core.futureobject import FutureTomwerObject
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.core.volume.volumebase import TomwerVolumeBase
 from tomwer.gui.reconstruction.nabu.castvolume import CastVolumeWidget
-import logging
+from tomwer.synctools.stacks.reconstruction.castvolume import CastVolumeProcessStack
+from tomwer.utils import docstring
+
+from ...orange.managedprocess import SuperviseOW
 
 _logger = logging.getLogger(__name__)
 
@@ -63,7 +69,7 @@ class CastNabuVolumeOW(WidgetLongProcessing, SuperviseOW):
     # note of this widget should be the one registered on the documentation
     name = "cast volume"
     id = "orange.widgets.tomwer.reconstruction.CastNabuVolumeOW.CastNabuVolumeOW"
-    description = "This widget will allow to cast from 32 bits to 16 bits nabu volume. Only works for tiff now"
+    description = "This widget will allow to cast a nabu volume data type / format to another one."
     icon = "icons/nabu_cast.svg"
     priority = 60
     keywords = [
@@ -82,13 +88,8 @@ class CastNabuVolumeOW(WidgetLongProcessing, SuperviseOW):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
-    allows_cycle = True
 
-    _rpSetting = Setting(dict())
-    # kept for compatibility
-
-    static_input = Setting({"data": None, "cast_volume_params": {}})
+    _ewoks_default_inputs = Setting({"data": None, "cast_volume_params": {}})
 
     sigScanReady = qt.Signal(TomwerScanBase)
     "Signal emitted when a scan is ended"
@@ -104,13 +105,37 @@ class CastNabuVolumeOW(WidgetLongProcessing, SuperviseOW):
             multiple=False,
         )
 
+        volume = Input(
+            name="volume",
+            type=TomwerVolumeBase,
+            doc="volume to be process",
+            default=False,
+            multiple=False,
+        )
+        cluster_in = Input(
+            name="cluster_config",
+            type=SlurmClusterConfiguration,
+            doc="slurm cluster to be used",
+            multiple=False,
+        )
+
     class Outputs:
         data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")
+        volume = Output(name="volume", type=TomwerVolumeBase, doc="raw volume")
+        cast_volume = Output(
+            name="cast volume", type=TomwerVolumeBase, doc="cast volume"
+        )
+        future_tomo_obj = Output(
+            name="future_tomo_obj",
+            type=FutureTomwerObject,
+            doc="future object (process remotely)",
+        )
 
     def __init__(self, parent=None):
         """ """
         SuperviseOW.__init__(self, parent)
         WidgetLongProcessing.__init__(self)
+        self._slurmCluster = None
         # processing tool
         self._processingStack = CastVolumeProcessStack(process_id=self.process_id)
         self._window = CastVolumeWidget(parent=self)
@@ -120,16 +145,29 @@ class CastNabuVolumeOW(WidgetLongProcessing, SuperviseOW):
 
         self._layout.addWidget(self._window)
 
-        self.setConfiguration(self.static_input.get("cast_volume_params", {}))
+        cast_volume_params = self._ewoks_default_inputs.get("cast_volume_params", {})
+        self.setConfiguration(cast_volume_params)
 
         # connect signal / slot
         self._window.sigConfigChanged.connect(self._updateConfig)
-        # connect signal / slot
         self._processingStack.sigComputationStarted.connect(self._startProcessing)
         self._processingStack.sigComputationEnded.connect(self._endProcessing)
 
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("static_input", None)
+        if static_input not in (None, {}):
+            _logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     @Inputs.data
-    def process(self, scan: Optional[TomwerScanBase]):
+    def process_data(self, scan: Optional[TomwerScanBase]):
+        self._process_data(scan)
+
+    def _process_data(self, scan: Optional[TomwerScanBase]):
         if scan is None:
             return
         if not isinstance(scan, TomwerScanBase):
@@ -138,7 +176,7 @@ class CastNabuVolumeOW(WidgetLongProcessing, SuperviseOW):
             )
         if (
             settings.isOnLbsram(scan)
-            and utils.isLowOnMemory(settings.get_lbsram_path()) is True
+            and is_low_on_memory(settings.get_lbsram_path()) is True
         ):
             ProcessManager().notify_dataset_state(
                 dataset=scan,
@@ -152,19 +190,60 @@ class CastNabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         else:
             self._processingStack.add(scan, configuration=self.getConfiguration())
 
+    @docstring
+    def reprocess(self, dataset):
+        if isinstance(dataset, TomwerScanBase):
+            self._process_data(dataset)
+        elif isinstance(dataset, TomwerVolumeBase):
+            self._process_volume(dataset)
+        else:
+            raise TypeError("dataset is expected to be an instance of ''")
+
+    @Inputs.cluster_in
+    def setCluster(self, slurm_cluster: Optional[SlurmClusterConfiguration]):
+        assert isinstance(
+            slurm_cluster, (type(None), SlurmClusterConfiguration)
+        ), f"Expect None of SlurmClusterConfiguration. Not {type(slurm_cluster)}"
+        self._slurmCluster = slurm_cluster
+
+    @Inputs.volume
+    def process_volume(self, volume: Optional[TomwerVolumeBase]):
+        self._process_volume(volume)
+
+    def _process_volume(self, volume: Optional[TomwerVolumeBase]):
+        if volume is None:
+            return
+        if not isinstance(volume, TomwerVolumeBase):
+            raise TypeError(
+                f"volume is expected to be an instance of {TomwerVolumeBase} not {type(volume)}"
+            )
+        else:
+            self._processingStack.add(volume, configuration=self.getConfiguration())
+
     def getConfiguration(self) -> dict:
-        return self._window.getConfiguration()
+        config = self._window.getConfiguration()
+        config["cluster_config"] = self._slurmCluster
+        return config
 
     def setConfiguration(self, configuration: dict) -> None:
         self._window.setConfiguration(configuration)
 
     def _updateConfig(self):
-        self.static_input = {
+        self._ewoks_default_inputs = {
             "data": None,
             "cast_volume_params": self.getConfiguration(),
         }
 
-    def _endProcessing(self, scan, future_scan):
-        WidgetLongProcessing._endProcessing(self, scan)
-        if scan is not None:
-            self.Outputs.data.send(scan)
+    def _endProcessing(self, obj, future_tomo_obj):
+        WidgetLongProcessing._endProcessing(self, obj)
+        if future_tomo_obj is not None:
+            self.Outputs.future_tomo_obj.send(future_tomo_obj)
+        if obj is not None:
+            if isinstance(obj, TomwerScanBase):
+                self.Outputs.data.send(obj)
+            elif isinstance(obj, TomwerVolumeBase):
+                self.Outputs.volume.send(obj)
+            # for now we store a cast_volume to the object but this is not very well design.
+            # I guess this will be removed once we move to ewoks or we need to redesign the stack approach
+            if obj.cast_volume is not None:
+                self.Outputs.cast_volume.send(obj.cast_volume)
