@@ -28,29 +28,32 @@ __license__ = "MIT"
 __date__ = "14/02/2020"
 
 
+import os
+import copy
 import logging
+from contextlib import AbstractContextManager
 from typing import Optional
 
-import tomwer.core.process.reconstruction.nabu.nabuslices
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.gui.reconstruction.nabu import check
-from tomwer.core.settings import isOnLbsram
-
-from contextlib import AbstractContextManager
-from silx.gui import qt
-from tomwer.synctools.stacks.reconstruction.nabu import NabuSliceProcessStack
-from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSlices
-from tomwer.gui.reconstruction.nabu.slices import NabuWindow
-from tomwer.core.scan.scanbase import TomwerScanBase
-from ..utils import WidgetLongProcessing
-from orangewidget.settings import Setting
 from orangewidget import gui
-from tomwer.core.scan.scanbase import _TomwerBaseDock
+from orangewidget.settings import Setting
 from orangewidget.widget import Input, Output
-from ...orange.managedprocess import SuperviseOW
+from silx.gui import qt
+
+import tomwer.core.process.reconstruction.nabu.nabuslices
 from tomwer.core.cluster import SlurmClusterConfiguration
-from tomwer.core.scan.futurescan import FutureTomwerScan
-import copy
+from tomwer.core.futureobject import FutureTomwerObject
+from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSlicesTask
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+from tomwer.core.volume.volumefactory import VolumeFactory
+from tomwer.core.process.icat.gallery import (
+    IcatScreenshots,
+    deduce_dataset_gallery_location,
+)
+from tomwer.gui.reconstruction.nabu.slices import NabuWindow
+from tomwer.synctools.stacks.reconstruction.nabu import NabuSliceProcessStack
+
+from ...orange.managedprocess import SuperviseOW
+from ..utils import WidgetLongProcessing
 
 _logger = logging.getLogger(__name__)
 
@@ -72,16 +75,10 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
-    allows_cycle = True
 
-    # TODO: replace Nabu by default nabu recons by the new orange contrib
-    static_input = Setting({"data": None, "nabu_params": None})
+    _ewoks_default_inputs = Setting({"data": None, "nabu_params": None})
 
-    _rpSetting = Setting(dict())
-    # this one is keep for backward compatibility. Will be removed on 0.10 I guess
-
-    ewokstaskclass = tomwer.core.process.reconstruction.nabu.nabuslices.NabuSlices
+    ewokstaskclass = tomwer.core.process.reconstruction.nabu.nabuslices.NabuSlicesTask
 
     sigScanReady = qt.Signal(TomwerScanBase)
     "Signal emitted when a scan is ended"
@@ -97,7 +94,7 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
             type=TomwerScanBase,
             doc="one scan to be process",
             default=True,
-            multiple=False,
+            multiple=True,
         )
         cluster_in = Input(
             name="cluster_config",
@@ -110,11 +107,19 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
         data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")
 
         future_out = Output(
-            name="future_data",
-            type=FutureTomwerScan,
+            name="future_tomo_obj",
+            type=FutureTomwerObject,
             doc="data with some remote processing",
         )
 
+        slice_urls = Output(name="slice urls", type=tuple, doc="tuple of urls created")
+
+        screenshots = Output(
+            name="screenshots",
+            type=IcatScreenshots,
+            doc="screenshots that can be saved",
+        )
+
     class DialogCM(AbstractContextManager):
         """Simple context manager to hide / show button dialogs"""
 
@@ -161,10 +166,8 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
         self._buttons.hide()
 
         # load settings
-        nabu_params = self.static_input.get("nabu_params", None)
-        if nabu_params is None:
-            nabu_params = self._rpSetting
-        if nabu_params != dict():
+        nabu_params = self._ewoks_default_inputs.get("nabu_params", None)
+        if nabu_params not in (dict(), None):
             try:
                 self._nabuWidget.setConfiguration(nabu_params)
             except Exception:
@@ -180,28 +183,25 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
         self._buttons.button(qt.QDialogButtonBox.Ok).clicked.connect(self.accept)
         self._nabuWidget.sigConfigChanged.connect(self._updateSettingsVals)
 
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("static_input", None)
+        if static_input not in (None, {}):
+            _logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     @Inputs.data
-    def process(self, scan: TomwerScanBase):
+    def process(self, scan, *args, **kwargs):
         assert isinstance(scan, (TomwerScanBase, type(None)))
         if scan is None:
             return
         scan_ = copy.copy(scan)
         scan_.clear_latest_reconstructions()
 
-        if isinstance(scan, HDF5TomoScan):
-            if not check.check_flat_series(
-                scan_, logger=_logger, user_input=isOnLbsram(scan=scan_)
-            ):
-                self.Outputs.data.send(scan_)
-                return
-        if isinstance(scan, HDF5TomoScan):
-            if not check.check_dark_series(
-                scan_, logger=_logger, user_input=isOnLbsram(scan=scan_)
-            ):
-                self.Outputs.data.send(scan_)
-                return
-
-        _logger.info("add {} to the stack".format(str(scan)))
+        _logger.info(f"add {scan} to the stack")
         # update the reconstruction mode if possible
         self._nabuWidget.setScan(scan_)
         self._processingStack.add(scan_, self.getConfiguration())
@@ -215,7 +215,7 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
 
         if scan.axis_params is None or scan.axis_params.relative_cor_value is None:
             # try to retrieve last computed cor value from nabu process
-            r_cor = NabuSlices.retrieve_last_relative_cor(scan)
+            r_cor = NabuSlicesTask.retrieve_last_relative_cor(scan)
             if scan.axis_params is None:
                 from tomwer.synctools.axis import QAxisRP
 
@@ -234,9 +234,24 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
                 if self.exec_():
                     # for now The behavior for reprocessing is the sama as for processing
                     if hasattr(scan, "instance"):
-                        self.process(scan=scan.instance)
+                        self.process(scan.instance)
                     else:
-                        self.process(scan=scan)
+                        self.process(scan)
+
+    def cancel(self, scan):
+        if scan is None:
+            return
+        if scan in self._processingStack:
+            self._processingStack.remove(scan)
+        if (
+            self._processingStack._data_currently_computed.get_identifier().to_str()
+            == scan.get_identifier().to_str()
+        ):
+            # stop current processing
+            self._processingStack.cancel()
+            # if possible process next
+            if self._processingStack.can_process_next():
+                self._processingStack._process_next()
 
     @Inputs.cluster_in
     def setCluster(self, slurm_cluster: Optional[SlurmClusterConfiguration]):
@@ -245,13 +260,37 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
         ), f"Expect None of SlurmClusterConfiguration. Not {type(slurm_cluster)}"
         self._slurmCluster = slurm_cluster
 
-    def _endProcessing(self, scan, future_scan):
+    def _endProcessing(self, scan, future_tomo_obj):
         WidgetLongProcessing._endProcessing(self, scan)
         if scan is not None:
+            # send scan
             self.Outputs.data.send(scan)
+            # send slice urls
+            slice_urls = []
+            for rec_identifier in scan.latest_reconstructions:
+                slice_urls.extend(
+                    VolumeFactory.from_identifier_to_vol_urls(rec_identifier)
+                )
+            if len(slice_urls) > 0:
+                slice_urls = tuple(slice_urls)
+                # provide list of reconstructed slices
+                self.Outputs.slice_urls.send(slice_urls)
+                # provide list of screenshots
+                self.Outputs.screenshots.send(
+                    IcatScreenshots(
+                        data_dir=deduce_dataset_gallery_location(scan),
+                        screenshots={
+                            os.path.splitext(os.path.basename(url.file_path()))[0]: url
+                            for url in slice_urls
+                        },
+                        scan=scan,
+                    )
+                )
+
             self.sigScanReady.emit(scan)
-        if future_scan is not None:
-            self.Outputs.future_out.send(future_scan)
+        if future_tomo_obj is not None:
+            # send future scan
+            self.Outputs.future_out.send(future_tomo_obj)
 
     def setDryRun(self, dry_run):
         self._processingStack.setDryRun(dry_run)
@@ -266,11 +305,10 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
         self.__exec_for_ci = True
 
     def _updateSettingsVals(self):
-        self.static_input = {
+        self._ewoks_default_inputs = {
             "data": None,
             "nabu_params": self.getConfiguration(),
         }
-        self._rpSetting = self.getConfiguration()
 
     def getConfiguration(self):
         config = self._nabuWidget.getConfiguration()
@@ -281,3 +319,12 @@ class NabuOW(WidgetLongProcessing, SuperviseOW):
         # ignore slurm cluster. Defined by the upper widget
         config.pop("cluster_config", None)
         self._nabuWidget.setConfiguration(config=config)
+
+    def keyPressEvent(self, event):
+        """The event has to be filtered since we have some children
+         that can be edited using the 'enter' key as defining the cor manually
+        (see #481)). As we are in a dialog this automatically trigger
+        'accepted'. See https://forum.qt.io/topic/5080/preventing-enter-key-from-triggering-ok-in-qbuttonbox-in-particular-qlineedit-qbuttonbox/5
+        """
+        if event.key() not in (qt.Qt.Key_Enter, qt.Qt.Key_Return):
+            super().keyPressEvent(event)
