@@ -28,30 +28,40 @@ __license__ = "MIT"
 __date__ = "14/02/2020"
 
 
-import logging
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.gui.reconstruction.nabu import check
-from contextlib import AbstractContextManager
-from silx.gui import qt
-from tomwer.synctools.stacks.reconstruction.nabu import NabuVolumeProcessStack
-from tomwer.core.process.reconstruction.nabu.nabuvolume import NabuVolume
-from tomwer.gui.reconstruction.nabu.volume import NabuVolumeWindow
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.cluster import SlurmClusterConfiguration
-from tomwer.core.scan.futurescan import FutureTomwerScan
-from orangecontrib.tomwer.widgets.utils import WidgetLongProcessing
-from orangewidget.settings import Setting
-from orangewidget import gui
-from orangewidget.widget import Input, Output
-from ...orange.managedprocess import SuperviseOW
-from tomwer.core.process.reconstruction.nabu import utils as nabu_utils
-from tomwer.core.utils.char import BETA_CHAR, DELTA_CHAR
-from collections.abc import Iterable
-from tomwer.utils import docstring
-from tomwer.core import settings
-import tomwer.core.process.reconstruction.nabu.nabuvolume
-import functools
 import copy
+import functools
+import logging
+from collections.abc import Iterable
+from contextlib import AbstractContextManager
+
+from orangewidget import gui
+from orangewidget.settings import Setting
+from orangewidget.widget import Input, Output
+from silx.gui import qt
+from tomoscan.identifier import VolumeIdentifier
+
+import tomwer.core.process.reconstruction.nabu.nabuvolume
+from orangecontrib.tomwer.widgets.utils import WidgetLongProcessing
+from tomwer.core import settings
+from tomwer.core.cluster import SlurmClusterConfiguration
+from tomwer.core.futureobject import FutureTomwerObject
+from tomwer.core.process.icat.gallery import (
+    IcatScreenshots,
+    deduce_dataset_gallery_location,
+    select_screenshot_from_volume,
+)
+from tomwer.core.process.reconstruction.nabu import utils as nabu_utils
+from tomwer.core.process.reconstruction.nabu.nabuvolume import NabuVolumeTask
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.utils.char import BETA_CHAR, DELTA_CHAR
+from tomwer.core.utils.scanutils import format_output_location
+from tomwer.core.volume.volumebase import TomwerVolumeBase
+from tomwer.core.volume.volumefactory import VolumeFactory
+from tomwer.gui.reconstruction.nabu.volume import NabuVolumeWindow
+from tomwer.synctools.stacks.reconstruction.nabu import NabuVolumeProcessStack
+from tomwer.utils import docstring
+
+from ...orange.managedprocess import SuperviseOW
 
 _logger = logging.getLogger(__name__)
 
@@ -73,17 +83,12 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
     priority = 15
     keywords = ["tomography", "nabu", "reconstruction", "volume"]
 
-    ewokstaskclass = tomwer.core.process.reconstruction.nabu.nabuvolume.NabuVolume
+    ewokstaskclass = tomwer.core.process.reconstruction.nabu.nabuvolume.NabuVolumeTask
 
     want_main_area = True
     resizing_enabled = True
-    compress_signal = False
-    allows_cycle = True
 
-    _rpSetting = Setting(dict())
-    # kept for compatibility
-
-    static_input = Setting(
+    _ewoks_default_inputs = Setting(
         {"data": None, "nabu_volume_params": None, "nabu_params": None}
     )
 
@@ -111,10 +116,23 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")
 
         future_out = Output(
-            name="future_data",
-            type=FutureTomwerScan,
+            name="future_tomo_obj",
+            type=FutureTomwerObject,
             doc="data with some remote processing",
         )
+        volume = Output(
+            name="volume",
+            type=TomwerVolumeBase,
+            doc="volume(s) created",
+        )
+        volume_urls = Output(
+            name="volume urls", type=tuple, doc="url of the volume(s) reconstructed"
+        )
+        screenshots = Output(
+            name="screenshots",
+            type=IcatScreenshots,
+            doc="screenshots that can be saved",
+        )
 
     class DialogCM(AbstractContextManager):
         """Simple context manager to hida / show button dialogs"""
@@ -128,7 +146,7 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         def __exit__(self, exc_type, exc_val, exc_tb):
             self._dialogButtonsBox.hide()
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, *args, **kwargs):
         """
         Widget which read the .hdf5 generated by octave and modify it.
         Then run a subprocess to call octave and run ftseries
@@ -143,7 +161,6 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         SuperviseOW.__init__(self, parent)
         WidgetLongProcessing.__init__(self)
         self._slurmCluster = None
-        self.__exec_for_ci = False
         # processing tool
         self._processingStack = NabuVolumeProcessStack(self, process_id=self.process_id)
 
@@ -161,12 +178,11 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         self._buttons.hide()
 
         # load settings
-        nabu_volume_params = self.static_input.get("nabu_volume_params", None)
-        if nabu_volume_params is None:
-            nabu_volume_params = self._rpSetting
-        if nabu_volume_params != dict():
+        nabu_volume_params = self._ewoks_default_inputs.get("nabu_volume_params", None)
+
+        if nabu_volume_params not in (dict(), None):
             try:
-                self._nabuWidget.setConfiguration(nabu_volume_params)
+                self.setConfiguration(nabu_volume_params)
             except Exception:
                 _logger.warning("fail to load reconstruction settings")
 
@@ -176,6 +192,16 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         self._buttons.button(qt.QDialogButtonBox.Ok).clicked.connect(self.accept)
         self._nabuWidget.sigConfigChanged.connect(self._updateSettingsVals)
 
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("static_input", None)
+        if static_input not in (None, {}):
+            _logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     @Inputs.data
     def process(self, scan: TomwerScanBase):
         assert isinstance(scan, (TomwerScanBase, type(None)))
@@ -184,31 +210,34 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         scan_ = copy.copy(scan)
         scan_.clear_latest_vol_reconstructions()
 
-        if isinstance(scan, HDF5TomoScan):
-            if not check.check_flat_series(
-                scan_, logger=_logger, user_input=settings.isOnLbsram(scan=scan_)
-            ):
-                self.Outputs.data.send(scan_)
-                return
-            if not check.check_dark_series(
-                scan_, logger=_logger, user_input=settings.isOnLbsram(scan=scan_)
-            ):
-                self.Outputs.data.send(scan_)
-                return
-
         # insure we are able to reconstruct
         if scan.nabu_recons_params in ({}, None):
             _logger.error(
-                "No reconstruction parameters found from nabu "
-                "slices for {}. You should first run slice "
-                "reconstruction prior to volume reconstruction"
-                "".format(str(scan))
+                f"No reconstruction parameters found from nabu slices for {scan}. "
+                "You should first run slice reconstruction prior to volume reconstruction"
             )
             self.Outputs.data.send(scan)
             self.sigScanReady.emit(scan)
             return
 
         config = scan.nabu_recons_params
+        # update output format if requested
+        if self._nabuWidget._mainWidget.redefineNabuFileFormat():
+            config["output"][
+                "file_format"
+            ] = self._nabuWidget._mainWidget.getNabuFileFormat()
+        # update output location if requested
+        if self._nabuWidget._mainWidget.redefineOutputLocation():
+            location = self._nabuWidget._mainWidget.getNabuOutputLocation()
+            if location is not None:
+                location = format_output_location(location=location, scan=scan)
+            config["output"]["location"] = location
+            config["output"][
+                "output_dir_mode"
+            ] = (
+                self._nabuWidget._mainWidget._outputLocationWidget.getOutputDirMode().value
+            )
+
         if "phase" in config and "delta_beta" in config["phase"]:
             pag_dbs = config["phase"]["delta_beta"]
             if isinstance(pag_dbs, str):
@@ -220,9 +249,7 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
                     pass
             if len(pag_dbs) > 1:
                 _logger.warning(
-                    "Several value found for {} / {}. Volume"
-                    "reconstruction take one at most."
-                    "".format(DELTA_CHAR, BETA_CHAR)
+                    f"Several value found for {DELTA_CHAR} / {BETA_CHAR}. Volume reconstruction take one at most."
                 )
                 timeout = NabuVolumeOW.TIMEOUT if settings.isOnLbsram(scan) else None
                 self._dialogDB = _DeltaBetaSelectorDialog(
@@ -247,7 +274,7 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
             or dataset.axis_params.relative_cor_value is None
         ):
             # try to retrieve last computed cor value from nabu process
-            r_cor = NabuVolume.retrieve_last_relative_cor(dataset)
+            r_cor = NabuVolumeTask.retrieve_last_relative_cor(dataset)
             if dataset.axis_params is None:
                 from tomwer.synctools.axis import QAxisRP
 
@@ -261,6 +288,21 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
 
         self.process(dataset)
 
+    def cancel(self, scan):
+        if scan is None:
+            return
+        if scan in self._processingStack:
+            self._processingStack.remove(scan)
+        if (
+            self._processingStack._data_currently_computed.get_identifier().to_str()
+            == scan.get_identifier().to_str()
+        ):
+            # stop current processing
+            self._processingStack.cancel()
+            # if possible process next
+            if self._processingStack.can_process_next():
+                self._processingStack._process_next()
+
     @Inputs.cluster_in
     def setCluster(self, cluster):
         self._slurmCluster = cluster
@@ -269,19 +311,57 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         db = dialog.getSelectedValue()
         if db is not None:
             try:
-                scan.nabu_recons_params["phase"]["delta_beta"] = db
+                scan.nabu_recons_params["phase"]["delta_beta"] = (db,)
             except Exception as e:
                 logging.error(e)
             else:
                 self.process(scan=scan)
 
-    def _endProcessing(self, scan, future_scan):
+    def _endProcessing(self, scan, future_tomo_obj):
         WidgetLongProcessing._endProcessing(self, scan)
         if scan is not None:
+            # send scan
             self.Outputs.data.send(scan)
             self.sigScanReady.emit(scan)
-        if future_scan is not None:
-            self.Outputs.future_out.send(future_scan)
+            # send volume urls
+            volume_urls = []
+            for volume_id in scan.latest_vol_reconstructions:
+                assert isinstance(volume_id, VolumeIdentifier)
+                volume_urls.extend(VolumeFactory.from_identifier_to_vol_urls(volume_id))
+            if len(volume_urls) > 0:
+                # send list of reconcstructed volume
+                self.Outputs.volume_urls.send(tuple(volume_urls))
+                # send list of screenshots
+                screenshots = {}
+                [
+                    screenshots.update(
+                        select_screenshot_from_volume(
+                            VolumeFactory.create_tomo_object_from_identifier(volume_id)
+                        )
+                    )
+                    for volume_id in scan.latest_vol_reconstructions
+                ]
+                self.Outputs.screenshots.send(
+                    IcatScreenshots(
+                        data_dir=deduce_dataset_gallery_location(scan),
+                        screenshots=screenshots,
+                        scan=scan,
+                    ),
+                )
+
+            # send volume identifier(s)  # only one expected
+            n_rec_volumes = len(scan.latest_vol_reconstructions)
+            if n_rec_volumes > 0:
+                if n_rec_volumes > 1:
+                    _logger.warning(
+                        f"{n_rec_volumes} volume reconstructed when at most one expected"
+                    )
+                volume = VolumeFactory.create_tomo_object_from_identifier(
+                    scan.latest_vol_reconstructions[0]
+                )
+                self.Outputs.volume.send(volume)
+        if future_tomo_obj is not None:
+            self.Outputs.future_out.send(future_tomo_obj)
 
     def setDryRun(self, dry_run):
         self._processingStack.setDryRun(dry_run)
@@ -291,14 +371,8 @@ class NabuVolumeOW(WidgetLongProcessing, SuperviseOW):
         self.raise_()
         self.show()
 
-    def _replaceExec_(self):
-        """used for CI, replace the exec_ call ny"""
-        self.__exec_for_ci = True
-
     def _updateSettingsVals(self):
-        self._rpSetting = self.getConfiguration()
-        # kept for compatibility
-        self.static_input = {
+        self._ewoks_default_inputs = {
             "data": None,
             "nabu_volume_params": self.getConfiguration(),
             "nabu_params": None,
@@ -344,13 +418,10 @@ class _DeltaBetaSelectorDialog(qt.QDialog):
         self.layout().addWidget(self._buttons)
         self._timeout = timeout
         if self._timeout is None:
-            self.setWindowTitle(
-                "Select one value for {} / {}".format(DELTA_CHAR, BETA_CHAR)
-            )
+            self.setWindowTitle(f"Select one value for {DELTA_CHAR} / {BETA_CHAR}")
         else:
             self.setWindowTitle(
-                "Select one value for {} / {}. (close automatically in {} "
-                "sec.)".format(DELTA_CHAR, BETA_CHAR, self._timeout)
+                f"Select one value for {DELTA_CHAR} / {BETA_CHAR}. (close automatically in {self._timeout} sec.)"
             )
 
         # connect signal / slot
@@ -388,8 +459,7 @@ class _DeltaBetaSelectorDialog(qt.QDialog):
             self.reject()
         else:
             self.setWindowTitle(
-                "Select one value for {} / {}. (close automatically in {} "
-                "sec.)".format(DELTA_CHAR, BETA_CHAR, self._timeout)
+                f"Select one value for {DELTA_CHAR} / {BETA_CHAR}. (close automatically in {self._timeout} sec.)"
             )
             self._displayTimer.start(1000)
 
