@@ -28,39 +28,38 @@ __license__ = "MIT"
 __date__ = "01/02/2021"
 
 
-from tomwer.gui.reconstruction.nabu import check
-from ..utils import WidgetLongProcessing
-from orangewidget import gui
-from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
-from orangewidget.settings import Setting
-from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
-from orangewidget.widget import Input, Output
-from tomwer.synctools.stacks.reconstruction.saaxis import SAAxisProcessStack
-from tomwer.synctools.saaxis import QSAAxisParams
-from tomwer.synctools.axis import QAxisRP
-from tomwer.core.process.reconstruction.axis import AxisProcess
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from silx.gui import qt
-from tomwer.gui.reconstruction.saaxis.saaxis import SAAxisWindow as _SAAxisWindow
-from processview.core.manager import ProcessManager, DatasetState
-from tomwer.core import settings
-from tomwer.core import utils
-from tomwer.core.process.reconstruction.saaxis.saaxis import SAAxisProcess
-from tomwer.core.scan.scanbase import _TomwerBaseDock
-import tomwer.core.process.reconstruction.saaxis.saaxis
-from tomwer.core.cluster import SlurmClusterConfiguration
-from tomwer.core.scan.futurescan import FutureTomwerScan
-from processview.core import helpers as pv_helpers
 import functools
 import logging
 from typing import Union
 
+from orangewidget import gui
+from orangewidget.settings import Setting
+from orangewidget.widget import Input, Output
+
+from processview.core import helpers as pv_helpers
+from processview.core.manager import DatasetState, ProcessManager
+from silx.gui import qt
+
+import tomwer.core.process.reconstruction.saaxis.saaxis
+from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
+from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
+from tomwer.core import settings
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.core.cluster import SlurmClusterConfiguration
+from tomwer.core.process.reconstruction.axis import AxisTask
+from tomwer.core.process.reconstruction.saaxis.saaxis import SAAxisTask
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+from tomwer.gui.reconstruction.saaxis.saaxis import SAAxisWindow as _SAAxisWindow
+from tomwer.synctools.axis import QAxisRP
+from tomwer.synctools.saaxis import QSAAxisParams
+from tomwer.synctools.stacks.reconstruction.saaxis import SAAxisProcessStack
+
+from ..utils import WidgetLongProcessing
+
 _logger = logging.getLogger(__name__)
 
 
 class SAAxisWindow(_SAAxisWindow):
-
     sigResultsToBeShow = qt.Signal()
     """signal emit when some results are ready to be display"""
 
@@ -77,7 +76,6 @@ class SAAxisWindow(_SAAxisWindow):
         self._clusterConfig = None
 
     def setClusterConfig(self, cluster_config: dict):
-
         if not isinstance(
             cluster_config, (dict, type(None), SlurmClusterConfiguration)
         ):
@@ -104,23 +102,21 @@ class SAAxisWindow(_SAAxisWindow):
         else:
             axis_params_info = self.getQAxisRP().get_simple_str()
 
-            text = "start automatic cor for {} with {}".format(scan, axis_params_info)
+            text = f"start automatic cor for {scan} with {axis_params_info}"
             _logger.inform(text)
-            cor_estimation_process = AxisProcess(
+            cor_estimation_process = AxisTask(
                 inputs={
                     "axis_params": self.getQAxisRP(),
                     "data": scan,
                     "wait": True,
+                    "serialize_output_data": False,
                 },
                 process_id=-1,
             )
             try:
                 cor_estimation_process.run()
             except Exception as e:
-                text = (
-                    "Unable to run automatic cor calculation. Reason is "
-                    "{}.".format(e)
-                )
+                text = f"Unable to run automatic cor calculation. Reason is {e}"
                 _logger.error(text)
                 msg = qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Critical)
@@ -129,9 +125,7 @@ class SAAxisWindow(_SAAxisWindow):
                 return None
             else:
                 cor = scan.axis_params.relative_cor_value
-                text = "automatic cor computed for {}: {} ({})".format(
-                    scan, cor, axis_params_info
-                )
+                text = f"automatic cor computed for {scan}: {cor} ({axis_params_info})"
                 _logger.inform(text)
                 self.setEstimatedCorPosition(value=cor)
                 self.getAutomaticCorWindow().hide()
@@ -150,7 +144,7 @@ class SAAxisWindow(_SAAxisWindow):
             self._mightUpdateResult, scan, self.isAutoFocusLock()
         )
         self._processing_stack.add(
-            scan=scan, configuration=self.getConfiguration(), callback=callback
+            data=scan, configuration=self.getConfiguration(), callback=callback
         )
 
     def _validate(self):
@@ -172,16 +166,27 @@ class SAAxisWindow(_SAAxisWindow):
                 self.setCurrentCorValue(scan.saaxis_params.autofocus)
             pm = ProcessManager()
             details = pm.get_dataset_details(
-                dataset_id=scan.get_dataset_identifier(), process=self._processing_stack
+                dataset_id=scan.get_identifier(), process=self._processing_stack
             )
-            ProcessManager().notify_dataset_state(
-                dataset=scan,
-                process=self._processing_stack,
-                details=details,
-                state=DatasetState.WAIT_USER_VALIDATION,
+            current_state = pm.get_dataset_state(
+                dataset_id=scan.get_identifier(), process=self._processing_stack
             )
-            self.sigResultsToBeShow.emit()
+            if current_state not in (
+                DatasetState.CANCELLED,
+                DatasetState.FAILED,
+                DatasetState.SKIPPED,
+            ):
+                ProcessManager().notify_dataset_state(
+                    dataset=scan,
+                    process=self._processing_stack,
+                    details=details,
+                    state=DatasetState.WAIT_USER_VALIDATION,
+                )
+                self.sigResultsToBeShow.emit()
         if validate:
+            _logger.processSucceed(
+                f"saaxis processing succeeded with {scan.axis_params.relative_cor_value} as cor value"
+            )
             self.validateScan(scan)
 
     def wait_processing(self, wait_time):
@@ -194,14 +199,15 @@ class SAAxisWindow(_SAAxisWindow):
         if scan is None:
             return
         assert isinstance(scan, TomwerScanBase)
-        selected_cor_value = self.getCurrentCorValue()
+        selected_cor_value = self.getCurrentCorValue() or scan.saaxis_params.autofocus
+        # if validate is done manually then pick current cor value; else we are in 'auto mode' and get it from the autofocus.
         details = ProcessManager().get_dataset_details(
-            dataset_id=scan.get_dataset_identifier(), process=self._processing_stack
+            dataset_id=scan.get_identifier(), process=self._processing_stack
         )
         if details is None:
             details = ""
         if selected_cor_value is None:
-            infos = "no selected cor value. {} skip SAAXIS".format(scan)
+            infos = f"no selected cor value. {scan} skip SAAXIS"
             infos = "\n".join((infos, details))
             _logger.warning(infos)
             scan.axis_params.set_relative_value(None)
@@ -210,16 +216,20 @@ class SAAxisWindow(_SAAxisWindow):
             )
         else:
             scan.axis_params.set_relative_value(selected_cor_value)
-            infos = "cor selected for {}: relative: {}, absolute: {}".format(
-                scan,
-                scan.axis_params.relative_cor_value,
-                scan.axis_params.absolute_cor_value,
-            )
+            if scan.nabu_recons_params is not None:
+                if "reconstruction" not in scan.nabu_recons_params:
+                    scan.nabu_recons_params["reconstruction"] = {}
+                scan.nabu_recons_params["reconstruction"][
+                    "rotation_axis_position"
+                ] = scan.axis_params.absolute_cor_value
+            r_cor = scan.axis_params.relative_cor_value
+            a_cor = scan.axis_params.absolute_cor_value
+            infos = f"cor selected for {scan}: relative: {r_cor}, absolute: {a_cor}"
             infos = "\n".join((infos, details))
             pv_helpers.notify_succeed(
                 process=self._processing_stack, dataset=scan, details=infos
             )
-        SAAxisProcess.process_to_tomwer_processes(
+        SAAxisTask.process_to_tomwer_processes(
             scan=scan,
         )
         self.Outputs.data.send(scan)
@@ -260,12 +270,14 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
     Details about :ref:`saaxis score calculation`
     """
 
-    name = "semi-automated center of rotation"
+    name = "multi-cor (sa-axis)"
     id = "orange.widgets.tomwer.sa_axis"
-    description = "use to compute the center of rotation semi automatic way"
+    description = "Reconstruct a slice with different center of rotation (cor) values"
     icon = "icons/saaxis.png"
     priority = 21
     keywords = [
+        "multi",
+        "multi-cor",
         "tomography",
         "semi automatic",
         "half automatic",
@@ -278,23 +290,17 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
         "saaxis",
     ]
 
-    ewokstaskclass = tomwer.core.process.reconstruction.saaxis.saaxis.SAAxisProcess
+    ewokstaskclass = tomwer.core.process.reconstruction.saaxis.saaxis.SAAxisTask
 
     want_main_area = True
     resizing_enabled = True
-    allows_cycle = True
-    compress_signal = False
 
     settingsHandler = CallbackSettingsHandler()
 
     sigScanReady = qt.Signal(TomwerScanBase)
     """Signal emitted when a scan is ready"""
 
-    _rpSetting = Setting(dict())
-    # kept for compatibility
-    "SAAxisRP store as dict"
-
-    static_input = Setting({"data": None, "sa_axis_params": None})
+    _ewoks_default_inputs = Setting({"data": None, "sa_axis_params": None})
 
     class Inputs:
         data = Input(
@@ -318,12 +324,6 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
     class Outputs:
         data = Output(name="data", type=TomwerScanBase, doc="one scan to be process")
 
-        future_out = Output(
-            name="future_data",
-            type=FutureTomwerScan,
-            doc="data with some remote processing",
-        )
-
     def __init__(self, parent=None):
         """
 
@@ -338,14 +338,11 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
         )
         self._layout.addWidget(self._widget)
 
-        sa_axis_params = self.static_input.get("sa_axis_params", None)
-        if sa_axis_params is None:
-            sa_axis_params = self._rpSetting
-        self.setConfiguration(sa_axis_params)
+        sa_axis_params = self._ewoks_default_inputs.get("sa_axis_params", None)
+        self.setConfiguration(sa_axis_params or {})
 
         # connect signal / slot
         self._widget.sigConfigurationChanged.connect(self._updateSettings)
-        self.destroyed.connect(self._widget.stop)
         self._widget._processing_stack.sigComputationStarted.connect(
             self._startProcessing
         )
@@ -356,6 +353,27 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
         # expose API
         self.wait_processing = self._widget.wait_processing
 
+    def getWaitingOverlay(self):
+        return self._widget._tabWidget._resultsViewer._plot.getWaiterOverlay()
+
+    def _startProcessing(self, *args, **kwargs):
+        self.getWaitingOverlay().show()
+        super()._startProcessing()
+
+    def _endProcessing(self, *args, **kwargs):
+        self.getWaitingOverlay().hide()
+        super()._endProcessing()
+
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("static_input", None)
+        if static_input not in (None, {}):
+            _logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     def setConfiguration(self, configuration):
         if "workflow" in configuration:
             autofocus_lock = configuration["workflow"].get("autofocus_lock", None)
@@ -367,6 +385,9 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
     def getCurrentCorValue(self):
         return self._widget.getCurrentCorValue()
 
+    def setEstimatedCorPosition(self, value):
+        self._widget.setEstimatedCorPosition(value=value)
+
     def load_sinogram(self):
         self._widget.loadSinogram()
 
@@ -389,26 +410,10 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
             scan.saaxis_params = QSAAxisParams()
         self._skipCurrentScan(new_scan=scan)
 
-        # check scan can be processed
-        if isinstance(scan, HDF5TomoScan):
-            if not check.check_flat_series(
-                scan, logger=_logger, user_input=settings.isOnLbsram(scan=scan)
-            ):
-                self.Outputs.data.send(scan)
-                return
-            if not check.check_dark_series(
-                scan, logger=_logger, user_input=settings.isOnLbsram(scan=scan)
-            ):
-                self.Outputs.data.send(scan)
-                return
-
-        if settings.isOnLbsram(scan) and utils.isLowOnMemory(
-            settings.get_lbsram_path()
-        ):
+        if settings.isOnLbsram(scan) and is_low_on_memory(settings.get_lbsram_path()):
             self.notify_skip(
                 scan=scan,
-                details="saaxis has been skiped for {} because "
-                "of low space in lbsram".format(scan),
+                details=f"saaxis has been skiped for {scan} because of low space in lbsram",
             )
             self.Outputs.data.send(scan)
         else:
@@ -431,11 +436,12 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
         if scan is None or str(scan) == str(new_scan):
             return
         current_scan_state = ProcessManager().get_dataset_state(
-            dataset_id=scan.get_dataset_identifier(), process=self
+            dataset_id=scan.get_identifier(), process=self
         )
         if current_scan_state in (
             DatasetState.PENDING,
             DatasetState.WAIT_USER_VALIDATION,
+            DatasetState.CANCELLED,
         ):
             details = "Was pending and has been replaced by another scan."
             self.notify_skip(scan=scan, details=details)
@@ -444,7 +450,22 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
     @Inputs.data_recompute
     def reprocess(self, dataset):
         self.lockAutofocus(False)
-        self.process(scan=dataset)
+        self.process(dataset)
+
+    def cancel(self, scan):
+        if scan is None:
+            return
+        if scan in self._widget._processing_stack:
+            self._widget._processing_stack.remove(scan)
+        if (
+            self._widget._processing_stack._data_currently_computed.get_identifier().to_str()
+            == scan.get_identifier().to_str()
+        ):
+            # stop current processing
+            self._widget._processing_stack.cancel()
+            # if possible process next
+            if self._widget._processing_stack.can_process_next():
+                self._widget._processing_stack._process_next()
 
     def validateCurrentScan(self):
         self._widget.validateCurrentScan()
@@ -452,18 +473,14 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
     def _updateSettings(self):
         config = self._widget.getConfiguration()
         config.pop("cluster_config", None)
-        self._rpSetting = config
-        self._rpSetting["workflow"] = {
-            "autofocus_lock": self._widget.isAutoFocusLock(),
-        }
 
-        self.static_input = {
+        self._ewoks_default_inputs = {
             "data": None,
             "sa_axis_params": self._widget.getConfiguration(),
         }
-        self.static_input["sa_axis_params"][
-            "autofocus_lock"
-        ] = self._widget.isAutoFocusLock()
+        self._ewoks_default_inputs["sa_axis_params"]["workflow"] = {
+            "autofocus_lock": self._widget.isAutoFocusLock(),
+        }
 
     def _raiseResults(self):
         if not self.isAutoFocusLock():
@@ -471,13 +488,5 @@ class SAAxisOW(SuperviseOW, WidgetLongProcessing):
             self.show()
             self._widget.showResults()
 
-    def stop(self):
-        self._widget.stop()
-
-    def close(self):
-        self.stop()
-        self._widget = None
-        super().close()
-
     def getConfiguration(self):
         return self._widget.getConfiguration()
