@@ -28,32 +28,32 @@ __license__ = "MIT"
 __date__ = "04/05/2021"
 
 
-from tomwer.gui.reconstruction.nabu import check
-from ..utils import WidgetLongProcessing
+import functools
+import logging
+
 from orangewidget import gui
-from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
 from orangewidget.settings import Setting
-from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
 from orangewidget.widget import Input, Output
-from tomwer.core.process.reconstruction.sadeltabeta import SADeltaBetaProcess
-from tomwer.synctools.stacks.reconstruction.sadeltabeta import SADeltaBetaProcessStack
-from tomwer.synctools.sadeltabeta import QSADeltaBetaParams
-from tomwer.synctools.axis import QAxisRP
+from processview.core import helpers as pv_helpers
+from processview.core.manager import DatasetState, ProcessManager
 from silx.gui import qt
+
+import tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta
+from orangecontrib.tomwer.orange.managedprocess import SuperviseOW
+from orangecontrib.tomwer.orange.settings import CallbackSettingsHandler
+from tomwer.core import settings
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.core.cluster import SlurmClusterConfiguration
+from tomwer.core.process.reconstruction.sadeltabeta import SADeltaBetaTask
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
 from tomwer.gui.reconstruction.sadeltabeta import (
     SADeltaBetaWindow as _SADeltaBetaWindow,
 )
-from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
-from processview.core.manager import ProcessManager, DatasetState
-import tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta
-from tomwer.core import settings
-from tomwer.core import utils
-from tomwer.core.cluster import SlurmClusterConfiguration
-from tomwer.core.scan.futurescan import FutureTomwerScan
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from processview.core import helpers as pv_helpers
-import functools
-import logging
+from tomwer.synctools.axis import QAxisRP
+from tomwer.synctools.sadeltabeta import QSADeltaBetaParams
+from tomwer.synctools.stacks.reconstruction.sadeltabeta import SADeltaBetaProcessStack
+
+from ..utils import WidgetLongProcessing
 
 _logger = logging.getLogger(__name__)
 
@@ -61,6 +61,7 @@ _logger = logging.getLogger(__name__)
 class SADeltaBetaWindow(_SADeltaBetaWindow):
     def __init__(self, Outputs, parent=None, process_id=None):
         _SADeltaBetaWindow.__init__(self, parent=parent)
+
         self.Outputs = Outputs
         self._sa_delta_beta_params = QSADeltaBetaParams()
         self._processing_stack = SADeltaBetaProcessStack(
@@ -69,7 +70,6 @@ class SADeltaBetaWindow(_SADeltaBetaWindow):
         self._clusterConfig = None
 
     def setClusterConfig(self, cluster_config: dict):
-
         if not isinstance(
             cluster_config, (dict, type(None), SlurmClusterConfiguration)
         ):
@@ -91,7 +91,7 @@ class SADeltaBetaWindow(_SADeltaBetaWindow):
             self._mightUpdateResult, scan, self.isAutoFocusLock()
         )
         self._processing_stack.add(
-            scan=scan, configuration=self.getConfiguration(), callback=callback
+            data=scan, configuration=self.getConfiguration(), callback=callback
         )
 
     def _validate(self):
@@ -109,12 +109,25 @@ class SADeltaBetaWindow(_SADeltaBetaWindow):
             )
             if scan.sa_delta_beta_params.autofocus is not None:
                 self.setCurrentDeltaBetaValue(scan.sa_delta_beta_params.autofocus)
-            ProcessManager().notify_dataset_state(
-                dataset=scan,
-                process=self._processing_stack,
-                details="processing done. Wait for user validation",
-                state=DatasetState.WAIT_USER_VALIDATION,
+
+            pm = ProcessManager()
+            details = pm.get_dataset_details(
+                dataset_id=scan.get_identifier(), process=self._processing_stack
             )
+            current_state = ProcessManager().get_dataset_state(
+                dataset_id=scan.get_identifier(), process=self._processing_stack
+            )
+            if current_state not in (
+                DatasetState.CANCELLED,
+                DatasetState.FAILED,
+                DatasetState.SKIPPED,
+            ):
+                ProcessManager().notify_dataset_state(
+                    dataset=scan,
+                    process=self._processing_stack,
+                    details=details,
+                    state=DatasetState.WAIT_USER_VALIDATION,
+                )
         if validate:
             self.validateScan(scan)
 
@@ -128,11 +141,11 @@ class SADeltaBetaWindow(_SADeltaBetaWindow):
         if scan is None:
             return
         assert isinstance(scan, TomwerScanBase)
-        selected_db_value = self.getCurrentDeltaBetaValue()
+        selected_db_value = (
+            self.getCurrentDeltaBetaValue() or scan.sa_delta_beta_params.autofocus
+        )
         if selected_db_value is None:
-            infos = "no selected delta / beta value. {} skip SADeltaBetaParams".format(
-                scan
-            )
+            infos = f"no selected delta / beta value. {scan} skip SADeltaBetaParams"
             _logger.warning(infos)
             scan.sa_delta_beta_params.set_db_selected_value(None)
             pv_helpers.notify_skip(
@@ -140,13 +153,16 @@ class SADeltaBetaWindow(_SADeltaBetaWindow):
             )
         else:
             scan.sa_delta_beta_params.set_db_selected_value(selected_db_value)
-            infos = "delta / beta selected for {}: {},".format(
-                scan, scan.sa_delta_beta_params.value
-            )
+            if scan.nabu_recons_params is not None:
+                if "phase" not in scan.nabu_recons_params:
+                    scan.nabu_recons_params["phase"] = {}
+                scan.nabu_recons_params["phase"]["delta_beta"] = (selected_db_value,)
+            _db_value = scan.sa_delta_beta_params.value
+            infos = f"delta / beta selected for {scan}: {_db_value}"
             pv_helpers.notify_succeed(
                 process=self._processing_stack, dataset=scan, details=infos
             )
-        SADeltaBetaProcess.process_to_tomwer_processes(
+        SADeltaBetaTask.process_to_tomwer_processes(
             scan=scan,
         )
         self.Outputs.data.send(scan)
@@ -182,16 +198,14 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
     Details about :ref:`sadeltabeta score calculation`
     """
 
-    name = "semi automatic delta/beta calculation"
+    name = "multi-pag (sa-delta/beta calculation)"
     id = "orange.widgets.tomwer.sa_delta_beta"
-    description = (
-        "compute several delta / beta value to get the optimal "
-        "delta / beta value (before reconstructing a volume for "
-        "example."
-    )
+    description = "Reconstruct a slice with several delta / beta values."
     icon = "icons/delta_beta_range.png"
     priority = 22
     keywords = [
+        "multi",
+        "multi-pag",
         "tomography",
         "semi automatic",
         "half automatic",
@@ -211,23 +225,18 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
     ]
 
     ewokstaskclass = (
-        tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta.SADeltaBetaProcess
+        tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta.SADeltaBetaTask
     )
 
     want_main_area = True
     resizing_enabled = True
-    allows_cycle = True
-    compress_signal = False
 
     settingsHandler = CallbackSettingsHandler()
 
     sigScanReady = qt.Signal(TomwerScanBase)
     """Signal emitted when a scan is ready"""
 
-    _rpSetting = Setting(dict())
-    "settings store as dict"
-
-    static_input = Setting({"data": None, "sa_delta_beta_params": None})
+    _ewoks_default_inputs = Setting({"data": None, "sa_delta_beta_params": None})
 
     class Inputs:
         data = Input(name="data", type=TomwerScanBase, default=True, multiple=False)
@@ -246,12 +255,6 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
     class Outputs:
         data = Output(name="data", type=TomwerScanBase)
 
-        future_out = Output(
-            name="future_data",
-            type=FutureTomwerScan,
-            doc="data with some remote processing",
-        )
-
     def __init__(self, parent=None):
         """
 
@@ -267,14 +270,13 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
         )
         self._layout.addWidget(self._widget)
 
-        sa_delta_beta_params = self.static_input.get("sa_delta_beta_params", None)
-        if sa_delta_beta_params is None:
-            sa_delta_beta_params = self._rpSetting
-        self.setConfiguration(sa_delta_beta_params)
+        sa_delta_beta_params = self._ewoks_default_inputs.get(
+            "sa_delta_beta_params", None
+        )
+        self.setConfiguration(sa_delta_beta_params or {})
 
         # connect signal / slot
         self._widget.sigConfigurationChanged.connect(self._updateSettings)
-        self.destroyed.connect(self._widget.stop)
         self._widget._processing_stack.sigComputationStarted.connect(
             self._startProcessing
         )
@@ -282,6 +284,16 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
         # expose API
         self.wait_processing = self._widget.wait_processing
 
+    def __new__(cls, *args, **kwargs):
+        # ensure backward compatibility with 'static_input'
+        static_input = kwargs.get("stored_settings", {}).get("static_input", None)
+        if static_input not in (None, {}):
+            _logger.warning(
+                "static_input has been deprecated. Will be replaced by _ewoks_default_inputs in the workflow file. Please save the workflow to apply modifications"
+            )
+            kwargs["stored_settings"]["_ewoks_default_inputs"] = static_input
+        return super().__new__(cls, *args, **kwargs)
+
     def setConfiguration(self, configuration):
         if "workflow" in configuration:
             autofocus_lock = configuration["workflow"].get("autofocus_lock", None)
@@ -315,26 +327,10 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
             scan.sa_delta_beta_params = QSADeltaBetaParams()
         self._skipCurrentScan(new_scan=scan)
 
-        # check scan can be processed
-        if isinstance(scan, HDF5TomoScan):
-            if not check.check_flat_series(
-                scan, logger=_logger, user_input=settings.isOnLbsram(scan=scan)
-            ):
-                self.Outputs.data.send(scan)
-                return
-            if not check.check_dark_series(
-                scan, logger=_logger, user_input=settings.isOnLbsram(scan=scan)
-            ):
-                self.Outputs.data.send(scan)
-                return
-
-        if settings.isOnLbsram(scan) and utils.isLowOnMemory(
-            settings.get_lbsram_path()
-        ):
+        if settings.isOnLbsram(scan) and is_low_on_memory(settings.get_lbsram_path()):
             self.notify_skip(
                 scan=scan,
-                details="sa-delta-beta has been skiped for {} because "
-                "of low space in lbsram".format(scan),
+                details=f"sa-delta-beta has been skiped for {scan} because of low space in lbsram",
             )
             self.Outputs.data.send(scan)
         else:
@@ -353,7 +349,7 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
         if scan is None or str(scan) == str(new_scan):
             return
         current_scan_state = ProcessManager().get_dataset_state(
-            dataset_id=scan.get_dataset_identifier(), process=self
+            dataset_id=scan.get_identifier(), process=self
         )
         if current_scan_state in (
             DatasetState.PENDING,
@@ -366,7 +362,7 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
     @Inputs.data_recompute
     def reprocess(self, dataset):
         self.lockAutofocus(False)
-        self.process(scan=dataset)
+        self.process(dataset)
 
     @Inputs.cluster_in
     def setCluster(self, cluster):
@@ -378,27 +374,39 @@ class SADeltaBetaOW(SuperviseOW, WidgetLongProcessing):
     def _updateSettings(self):
         config = self._widget.getConfiguration()
         config.pop("cluster_config", None)
-        self._rpSetting = config
-        self._rpSetting["workflow"] = {
-            "autofocus_lock": self._widget.isAutoFocusLock(),
-        }
-        # keep for backward compatibility
-
-        self.static_input = {
+        self._ewoks_default_inputs = {
             "data": None,
             "sa_delta_beta_params": self._widget.getConfiguration(),
         }
-        self.static_input["sa_delta_beta_params"]["workflow"] = {
+        self._ewoks_default_inputs["sa_delta_beta_params"]["workflow"] = {
             "autofocus_lock": self._widget.isAutoFocusLock(),
         }
 
-    def close(self):
-        self.stop()
-        self._widget = None
-        super().close()
-
-    def stop(self):
-        self._widget.stop()
-
     def getConfiguration(self):
         return self._widget.getConfiguration()
+
+    def cancel(self, scan):
+        if scan is None:
+            return
+        if scan in self._widget._processing_stack:
+            self._widget._processing_stack.remove(scan)
+        if (
+            self._widget._processing_stack._data_currently_computed.get_identifier().to_str()
+            == scan.get_identifier().to_str()
+        ):
+            # stop current processing
+            self._widget._processing_stack.cancel()
+            # if possible process next
+            if self._widget._processing_stack.can_process_next():
+                self._widget._processing_stack._process_next()
+
+    def getWaitingOverlay(self):
+        return self._widget._tabWidget._resultsViewer._plot.getWaiterOverlay()
+
+    def _startProcessing(self, *args, **kwargs):
+        self.getWaitingOverlay().show()
+        super()._startProcessing()
+
+    def _endProcessing(self, *args, **kwargs):
+        self.getWaitingOverlay().hide()
+        super()._endProcessing()
