@@ -27,17 +27,17 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "21/07/2020"
 
-from silx.gui import qt
-from orangewidget import widget, gui
-from orangewidget.widget import Input
-from tomwer.gui.visualization.dataviewer import DataViewer
-from tomwer.core.scan.scanbase import TomwerScanBase
-from orangewidget import settings
-from tomwer.gui import utils
-from tomwer.gui import icons
-import tomwer.core.process.visualization.dataviewer
 import logging
 
+from orangewidget import gui, settings, widget
+from orangewidget.widget import Input
+from silx.gui import qt
+
+import tomwer.core.process.visualization.dataviewer
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui import icons, utils
+from tomwer.gui.visualization.dataviewer import DataViewer
+
 _logger = logging.getLogger(__name__)
 
 
@@ -58,18 +58,24 @@ class DataViewerOW(widget.OWBaseWidget, openclass=True):
     keywords = ["tomography", "file", "tomwer", "acquisition", "validation"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     ewokstaskclass = tomwer.core.process.visualization.dataviewer._DataViewerPlaceHolder
 
     _viewer_config = settings.Setting(dict())
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase)
+        data = Input(
+            name="data",
+            type=TomwerScanBase,
+            multiple=True,
+        )
 
     def __init__(self, parent=None):
-        widget.OWBaseWidget.__init__(self, parent)
+        super().__init__(parent)
+        self._imagejThreads = []
+        # threads used to open frames with image j
         self._layout = gui.vBox(self.mainArea, self.name).layout()
         self.viewer = DataViewer(parent=self)
         self.viewer.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)
@@ -96,7 +102,7 @@ class DataViewerOW(widget.OWBaseWidget, openclass=True):
             _logger.warning("No active image defined")
         else:
             try:
-                utils.open_url_with_image_j(current_url)
+                self._imagejThreads.append(utils.open_url_with_image_j(current_url))
             except OSError as e:
                 msg = qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Warning)
@@ -105,7 +111,7 @@ class DataViewerOW(widget.OWBaseWidget, openclass=True):
                 msg.exec_()
 
     @Inputs.data
-    def addScan(self, scan):
+    def addScan(self, scan, *args, **kwargs):
         if scan is None:
             return
         self.viewer.setScan(scan)
@@ -114,9 +120,15 @@ class DataViewerOW(widget.OWBaseWidget, openclass=True):
         return qt.QSize(400, 500)
 
     def _updateSettings(self):
-        self._viewer_config["mode"] = self.viewer.getDisplayMode()
-        self._viewer_config["slice_opt"] = self.viewer.getSliceOption()
-        self._viewer_config["radio_opt"] = self.viewer.getRadioOption()
+        self._viewer_config["mode"] = (  # pylint: disable=E1137
+            self.viewer.getDisplayMode().value
+        )
+        self._viewer_config["slice_opt"] = (  # pylint: disable=E1137
+            self.viewer.getSliceOption().value
+        )
+        self._viewer_config["radio_opt"] = (  # pylint: disable=E1137
+            self.viewer.getRadioOption().value
+        )
 
     def _setSettings(self, settings):
         old_state = self.viewer.blockSignals(True)
@@ -129,6 +141,7 @@ class DataViewerOW(widget.OWBaseWidget, openclass=True):
         self.viewer.blockSignals(old_state)
 
     def close(self):
+        [thread.quit() for thread in self._imagejThreads]
         self.viewer.close()
         self.viewer = None
         super().close()
