@@ -27,13 +27,16 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "07/09/2020"
 
-from silx.gui import qt
-from orangewidget import widget, gui
+import logging
+
+from orangewidget import gui, widget
 from orangewidget.widget import Input
+
+from silx.gui import qt
+
+import tomwer.core.process.visualization.diffviewer
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.gui.visualization.diffviewer import DiffFrameViewer
-import tomwer.core.process.visualization.diffviewer
-import logging
 
 _logger = logging.getLogger(__name__)
 
@@ -44,7 +47,7 @@ class DiffViewerOW(widget.OWBaseWidget, openclass=True):
     Allows to compare two random frame.
     """
 
-    name = "diff viewer"
+    name = "diff frame viewer"
     id = "orangecontrib.tomwer.widgets.visualization.diffviewerow"
     description = "Allows comparison between two random frame from a scan"
     icon = "icons/diff.png"
@@ -52,8 +55,8 @@ class DiffViewerOW(widget.OWBaseWidget, openclass=True):
     keywords = ["tomography", "diff", "tomwer", "compare", "comparison"]
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     ewokstaskclass = tomwer.core.process.visualization.diffviewer._DiffViewerPlaceHolder
 
@@ -73,4 +76,4 @@ class DiffViewerOW(widget.OWBaseWidget, openclass=True):
         self.viewer.addScan(scan)
 
     def sizeHint(self):
-        return qt.QSize(400, 500)
+        return qt.QSize(500, 700)
