@@ -27,12 +27,16 @@ __authors__ = ["C. Nemoz", "H. Payno"]
 __license__ = "MIT"
 __date__ = "01/08/2018"
 
-from orangewidget import widget, gui
+import logging
+
+from orangewidget import gui, widget
 from orangewidget.widget import Input
+
+from silx.gui import qt
+
+import tomwer.core.process.visualization.radiostack
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.gui.stacks import RadioStack
-import tomwer.core.process.visualization.radiostack
-import logging
 
 logger = logging.getLogger(__name__)
 
@@ -57,14 +61,16 @@ class RadioStackOW(widget.OWBaseWidget, openclass=True):
 
     ewokstaskclass = tomwer.core.process.visualization.radiostack._RadioStackPlaceHolder
 
-    allows_cycle = True
-    compress_signal = False
-
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase)
+        data = Input(
+            name="data",
+            type=TomwerScanBase,
+            multiple=True,
+        )
 
     def __init__(self, parent=None):
         widget.OWBaseWidget.__init__(self, parent)
@@ -73,7 +79,18 @@ class RadioStackOW(widget.OWBaseWidget, openclass=True):
         self._box.layout().addWidget(self._viewer)
 
     @Inputs.data
-    def addLeafScan(self, scanID):
+    def addLeafScan(self, scanID, *args, **kwargs):
         if scanID is None:
             return
         self._viewer.addLeafScan(scanID)
+
+    def keyPressEvent(self, e):
+        # TODO: fixme
+        # here we want to avoid loading imageJ when enter is pressed.
+        # the correct way would be to install an event filer
+        # but this is ignored because the KeyPressEvent goes other it.
+        # I don't really see why too annoyed at the point to look deeper
+        if e.key() in (qt.Qt.Key_Enter, qt.Qt.Key_Return):
+            pass
+        else:
+            super().keyPressEvent(e)
