@@ -27,20 +27,22 @@ __authors__ = ["C. Nemoz", "H. Payno"]
 __license__ = "MIT"
 __date__ = "19/03/2018"
 
-from silx.gui import qt
-from orangewidget import widget, gui
-from orangewidget.widget import Input
-from tomwer.web.client import OWClient
-from tomwer.gui.samplemoved import SampleMovedWidget
-from tomwer.core.scan.scanbase import TomwerScanBase
-import tomwer.core.process.visualization.samplemoved
-import os
 import logging
+import os
+
+from orangewidget import gui, widget
+from orangewidget.widget import Input
+
+from silx.gui import qt
+
+import tomwer.core.process.visualization.samplemoved
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.samplemoved import SampleMovedWidget
 
 logger = logging.getLogger(__name__)
 
 
-class SampleMovedOW(widget.OWBaseWidget, OWClient, openclass=True):
+class SampleMovedOW(widget.OWBaseWidget, openclass=True):
     """
     Simple widget exposing two images side by side to see if a sample moved
     during the acquisition.
@@ -64,15 +66,14 @@ class SampleMovedOW(widget.OWBaseWidget, OWClient, openclass=True):
     )
 
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
-    compress_signal = False
 
     class Inputs:
         data = Input(name="data", type=TomwerScanBase)
 
     def __init__(self, parent=None):
-        widget.OWBaseWidget.__init__(self, parent)
-        OWClient.__init__(self)
+        super().__init__(parent)
 
         layout = gui.vBox(self.mainArea, self.name).layout()
 
@@ -106,6 +107,7 @@ class SampleMovedOW(widget.OWBaseWidget, OWClient, openclass=True):
 
             self._mainWidget.clearOnLoadActions()
             self._mainWidget.setImages(rawSlices)
+            self._mainWidget.setScan(scan)
             self._mainWidget.setOnLoadAction(scan.flat_field_correction)
 
     def clear(self):
