@@ -27,13 +27,16 @@ __authors__ = ["C. Nemoz", "H. Payno"]
 __license__ = "MIT"
 __date__ = "01/08/2018"
 
-from orangewidget import widget, gui
+import logging
+
+from orangewidget import gui, widget
 from orangewidget.widget import Input
+
+import tomwer.core.process.visualization.sinogramviewer
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.gui.visualization.sinogramviewer import SinogramViewer
-import tomwer.core.process.visualization.sinogramviewer
+
 from ..utils import WidgetLongProcessing
-import logging
 
 logger = logging.getLogger(__name__)
 
@@ -59,10 +62,8 @@ class SinogramViewerOW(WidgetLongProcessing, widget.OWBaseWidget, openclass=True
         tomwer.core.process.visualization.sinogramviewer._SinogramViewerPlaceHolder
     )
 
-    allows_cycle = True
-    compress_signal = False
-
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
 
     class Inputs:
@@ -78,7 +79,6 @@ class SinogramViewerOW(WidgetLongProcessing, widget.OWBaseWidget, openclass=True
         # connect signal / slot
         self._viewer.sigSinoLoadStarted.connect(self._startProcessing)
         self._viewer.sigSinoLoadEnded.connect(self._endProcessing)
-        self.destroyed.connect(self._viewer._stopAnimationThread)
 
     @Inputs.data
     def addLeafScan(self, scanID):
