@@ -28,13 +28,17 @@ __license__ = "MIT"
 __date__ = "04/05/2018"
 
 
-from orangewidget import widget, gui
-from orangewidget.widget import Input
-from tomwer.gui.stacks import SliceStack
-from tomwer.core.scan.scanbase import TomwerScanBase
-import tomwer.core.process.visualization.slicestack
 import logging
 
+from orangewidget import gui, widget
+from orangewidget.widget import Input
+
+from silx.gui import qt
+
+import tomwer.core.process.visualization.slicestack
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.stacks import SliceStack
+
 logger = logging.getLogger(__name__)
 
 
@@ -72,14 +76,16 @@ class SlicesStackOW(widget.OWBaseWidget, openclass=True):
 
     ewokstaskclass = tomwer.core.process.visualization.slicestack._SliceStackPlaceHolder
 
-    allows_cycle = True
-    compress_signal = False
-
     want_main_area = True
+    want_control_area = False
     resizing_enabled = True
 
     class Inputs:
-        data = Input(name="data", type=TomwerScanBase)
+        data = Input(
+            name="data",
+            type=TomwerScanBase,
+            multiple=True,
+        )
 
     def __init__(self, parent=None):
         widget.OWBaseWidget.__init__(self, parent)
@@ -88,7 +94,18 @@ class SlicesStackOW(widget.OWBaseWidget, openclass=True):
         self._box.layout().addWidget(self._viewer)
 
     @Inputs.data
-    def addLeafScan(self, scanID):
+    def addLeafScan(self, scanID, *args, **kwargs):
         if scanID is None:
             return
         self._viewer.addLeafScan(scanID)
+
+    def keyPressEvent(self, e):
+        # TODO: fixme
+        # here we want to avoid loading imageJ when enter is pressed.
+        # the correct way would be to install an event filer
+        # but this is ignored because the KeyPressEvent goes other it.
+        # I don't really see why too annoyed at the point to look deeper
+        if e.key() in (qt.Qt.Key_Enter, qt.Qt.Key_Return):
+            pass
+        else:
+            super().keyPressEvent(e)
