@@ -1,34 +1,37 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import sys
 import argparse
-from silx.gui import qt
-import signal
 import functools
-from tomwer.gui import icons
-from tomwer.gui.utils.splashscreen import getMainSplashScreen
-from tomwer.gui.reconstruction.axis import AxisWindow
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.synctools.axis import QAxisRP
-from tomwer.core.process.reconstruction.axis.axis import AxisProcess, NoAxisUrl
-from tomwer.core.process.reconstruction.axis.axis import AxisMode
-from tomwer.core.process.reconstruction.darkref.darkrefs import (
-    requires_normed_dark_and_flat,
-)
-from tomwer.core.utils.resource import increase_max_number_file
-from silx.io.url import DataUrl
 import logging
+import signal
+import sys
+import silx
+from silx.gui import qt
 
+from tomwer.core.process.reconstruction.axis.axis import (
+    AxisMode,
+    AxisTask,
+    NoAxisUrl,
+)
+from tomwer.core.process.reconstruction.darkref.darkrefs import (
+    requires_reduced_dark_and_flat,
+)
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils.resource import increase_max_number_file
+from tomwer.gui import icons
+from tomwer.gui.reconstruction.axis import AxisWindow
+from tomwer.gui.utils.splashscreen import getMainSplashScreen
+from tomwer.synctools.axis import QAxisRP
 
 logging.basicConfig(level=logging.WARNING)
 _logger = logging.getLogger(__name__)
 
 
 class _AxisProcessGUI(AxisWindow):
-    def __init__(self, scan, axis_params):
+    def __init__(self, scan, axis_params, backend=None):
         axis_params.frame_width = scan.dim_1
-        AxisWindow.__init__(self, axis_params=axis_params)
+        super().__init__(axis_params=axis_params, backend=backend)
         self.hideLockButton()
         self.hideApplyButton()
         self.setScan(scan=scan)
@@ -43,11 +46,12 @@ class _AxisProcessGUI(AxisWindow):
             ("start axis calculation with", scan.axis_params.axis_url_1.url.path())
         )
         _logger.info(mess)
-        process = AxisProcess(
+        process = AxisTask(
             inputs={
                 "data": scan,
                 "axis_params": self._axis_params,
                 "wait": wait,
+                "serialize_output_data": False,
             }
         )
         try:
@@ -76,10 +80,12 @@ class _AxisProcessGUI(AxisWindow):
 def main(argv):
     parser = argparse.ArgumentParser(description=__doc__)
     parser.add_argument(
-        "--scan-path",
+        "scan_path",
         help="For EDF acquisition: provide folder path, for HDF5 / nexus "
         "provide the master file",
-        default=None,
+    )
+    parser.add_argument(
+        "--entry", help="For Nexus files: entry in the master file", default=None
     )
     parser.add_argument(
         "--use-sinogram",
@@ -88,21 +94,6 @@ def main(argv):
         action="store_true",
         default=False,
     )
-    parser.add_argument(
-        "--radio1",
-        help="If you cannot provide scan-path or master file you should "
-        "provide urls of the two radios to use for calculation",
-        default=None,
-    )
-    parser.add_argument(
-        "--radio2",
-        help="If you cannot provide scan-path or master file you should "
-        "provide urls of the two radios to use for calculation",
-        default=None,
-    )
-    parser.add_argument(
-        "--entry", help="For Nexus files: entry in the master file", default=None
-    )
     parser.add_argument(
         "--debug",
         dest="debug",
@@ -113,7 +104,7 @@ def main(argv):
     parser.add_argument(
         "--mode",
         default=None,
-        help="Use a specific mode. Available modes are {}" "".format(AxisMode.values()),
+        help=f"Use a specific mode. Available modes are {AxisMode.values()}",
     )
     parser.add_argument(
         "--full-image",
@@ -121,16 +112,26 @@ def main(argv):
         default=False,
         help="Define the mode 'full' when display shifted images from manual" " mode",
     )
-
+    parser.add_argument(
+        "--use-opengl-plot",
+        help="Use OpenGL for plots (instead of matplotlib)",
+        action="store_true",
+        default=False,
+    )
     options = parser.parse_args(argv[1:])
 
     if options.debug:
         logging.root.setLevel(logging.DEBUG)
 
+    if options.use_opengl_plot:
+        silx.config.DEFAULT_PLOT_BACKEND = "gl"
+    else:
+        silx.config.DEFAULT_PLOT_BACKEND = "matplotlib"
+
     increase_max_number_file()
 
     global app  # QApplication must be global to avoid seg fault on quit
-    app = qt.QApplication.instance() or qt.QApplication([])
+    app = qt.QApplication.instance() or qt.QApplication(["tomwer"])
     splash = getMainSplashScreen()
     qt.QApplication.setOverrideCursor(qt.Qt.WaitCursor)
     qt.QApplication.processEvents()
@@ -154,25 +155,14 @@ def main(argv):
         scan = ScanFactory.create_scan_object(
             scan_path=options.scan_path, entry=options.entry
         )
-    except ValueError as e:
-        print(e)
+    except ValueError:
         scan = ScanFactory.mock_scan()
-        if options.radio1 is None:
-            raise ValueError("At least a file or a scan path should be given")
 
-    requires_normed_dark_and_flat(scan=scan, logger_=_logger)
+    requires_reduced_dark_and_flat(scan=scan, logger_=_logger)
     # define the process_index is any tomwer_processes_existing
     scan.set_process_index_frm_tomwer_process_file()
     if scan.axis_params is None:
         scan.axis_params = QAxisRP()
-    if options.radio1 is not None:
-        radio1 = DataUrl(file_path=options.radio1, scheme="fabio")
-        scan.axis_params.axis_url_1 = radio1
-        if options.radio2 is not None:
-            radio2 = DataUrl(file_path=options.radio2, scheme="fabio")
-        else:
-            radio2 = None
-        scan.axis_params.axis_url_2 = radio2
 
     if options.debug:
         _logger.setLevel(logging.DEBUG)
