@@ -28,32 +28,26 @@ __license__ = "MIT"
 __date__ = "14/12/2021"
 
 
-from orangewidget.workflow import config
-import tomwer.version
-from .splash import splash_screen, getIcon
-from tomwer.gui import icons
-from orangewidget.workflow.widgetsscheme import WidgetsScheme as _WidgetsScheme
 import pkg_resources
-from silx.gui import qt
 from orangecanvas.application.outputview import (
     TerminalTextDocument as _TerminalTextDocument,
 )
-from . import environ
 from orangewidget.settings import set_widget_settings_dir_components
+from orangewidget.workflow import config
+from silx.gui import qt
+
+import tomwer.version
+from tomwer.gui import icons
+
+from . import environ
+from .splash import getIcon, splash_screen
+from .widgetsscheme import WidgetsScheme
 
 
 def version():
     return tomwer.version.version
 
 
-class WidgetsScheme(_WidgetsScheme):
-    """Simple redefinition of the WidgetsScheme to allow loops"""
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.set_loop_flags(self.AllowLoops)
-
-
 class TomwerConfig(config.Config):
     """
     Configuration defined for tomwer
@@ -65,6 +59,9 @@ class TomwerConfig(config.Config):
 
     def init(self):
         super().init()
+        qt.QApplication.setApplicationName(self.ApplicationName)
+        qt.QApplication.setOrganizationDomain(self.OrganizationDomain)
+        qt.QApplication.setApplicationVersion(self.ApplicationVersion)
         qt.QApplication.setApplicationDisplayName(self.ApplicationName)
         widget_settings_dir_cfg = environ.get_path("widget_settings_dir", "")
         if widget_settings_dir_cfg:
@@ -115,9 +112,9 @@ class TomwerConfig(config.Config):
 
         all_eps = sorted(
             all_eps,
-            key=lambda ep: 0
-            if ep.dist.project_name.lower() in ("orange3", "tomwer")
-            else 1,
+            key=lambda ep: (
+                0 if ep.dist.project_name.lower() in ("orange3", "tomwer") else 1
+            ),
         )
         return iter(all_eps)
 
@@ -129,12 +126,12 @@ class TomwerConfig(config.Config):
         #: Submit a bug report action in the Help menu
         "Bug Report": "https://gitlab.esrf.fr/tomotools/tomwer/-/issues",
         #: A url quick tour/getting started url
-        "Quick Start": "http://www.edna-site.org/pub/doc/tomwer/video/canvas/start_up/",
+        "Quick Start": "https://www.youtube.com/playlist?list=PLddRXwP6Z6F9KOu1V5o6H24KPH5Ikuk2f",
         #: The 'full' documentation, should be something like current /docs/
         #: but specific for 'Visual Programing' only
-        "Documentation": "http://www.edna-site.org/pub/doc/tomwer/latest/",
+        "Documentation": "https://tomotools.gitlab-pages.esrf.fr/tomwer/index.html",
         #: YouTube tutorials
-        "Screencasts": "http://www.edna-site.org/pub/doc/tomwer/video",
+        "Screencasts": "https://www.youtube.com/@tomotools",
     }
 
 
@@ -145,12 +142,12 @@ class TomwerSplashScreen(qt.QSplashScreen):
         pixmap=None,
         textRect=None,
         textFormat=qt.Qt.PlainText,
-        **kwargs
+        **kwargs,
     ):
         super(TomwerSplashScreen, self).__init__(pixmap=icons.getQPixmap("tomwer"))
 
     def showMessage(self, message, alignment=qt.Qt.AlignLeft, color=qt.Qt.black):
-        version = "tomwer version {}".format(tomwer.version.version)
+        version = f"tomwer version {tomwer.version.version}"
         super().showMessage(version, qt.Qt.AlignLeft | qt.Qt.AlignBottom, qt.Qt.white)
 
 
@@ -188,17 +185,19 @@ class TerminalTextDocument(_TerminalTextDocument):
     def writeWithFormat(self, string: str, charformat) -> None:
         assert qt.QThread.currentThread() is self.thread()
         # remove linux reset sequence
-        string = string.replace("\033[0m", "")
-        # remove linux reset sequence
-        string = string.replace("\033[1;%dm", "")
-        # remove linux reset sequence
-        string = string.replace("[1m", "")
-        string = string.replace("[1;30m", "")
-        string = string.replace("[1;31m", "")
-        string = string.replace("[1;32m", "")
-        string = string.replace("[1;33m", "")
-        string = string.replace("[1;34m", "")
-        string = string.replace("[1;35m", "")
+        from tomwer.core.log import logger as tomwer_logger
+
+        for sequence in (
+            tomwer_logger._RESET_SEQ,
+            tomwer_logger._BOLD_SEQ,
+            tomwer_logger._BLACK,
+            tomwer_logger._RED,
+            tomwer_logger._GREEN,
+            tomwer_logger._YELLOW,
+            tomwer_logger._BLUE,
+            tomwer_logger._MAGENTA,
+        ):
+            string = string.replace(sequence, "")
 
         color = self.get_log_level(string) or qt.Qt.red
         charformat.setForeground(color)
