@@ -1,18 +1,21 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import logging
-import sys
-from silx.gui import qt
 import argparse
-from tomwer.gui import icons
-from tomwer.gui.utils.splashscreen import getMainSplashScreen
-from tomwer.gui.reconstruction.nabu.slices import NabuDialog
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.process.reconstruction.nabu.nabuslices import run_slices_reconstruction
-from tomwer.gui.visualization.dataviewer import DataViewer as _DataViewer
-from tomwer.core.utils.resource import increase_max_number_file
+import logging
 import signal
+import sys
+
+from silx.gui import qt
+
+from tomwer.core.process.reconstruction.nabu.nabuslices import run_slices_reconstruction
+from tomwer.core.process.reconstruction.utils.cor import relative_pos_to_absolute
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils.resource import increase_max_number_file
+from tomwer.gui import icons
+from tomwer.gui.reconstruction.nabu.slices import NabuDialog
+from tomwer.gui.utils.splashscreen import getMainSplashScreen
+from tomwer.gui.visualization.dataviewer import DataViewer as _DataViewer
 
 logging.basicConfig(level=logging.WARNING)
 _logger = logging.getLogger(__name__)
@@ -41,16 +44,10 @@ class DataViewer(_DataViewer):
         if event.key() is qt.Qt.Key_F5:
             self._updateDisplay()
 
-    def event(self, event) -> bool:
-        if event.type() == qt.QEvent.DeferredDelete:
-            self.cleanBeforeQuit()
-            self.closed.emit()
-        return super().event(event)
-
 
 class NabuProcessingThread(qt.QThread):
-    def init(self, scan, config, dry_run=False):
-        self._scan = scan
+    def init(self, data, config, dry_run=False):
+        self._scan = data
         self._config = config
         self._dry_run = dry_run
 
@@ -142,7 +139,9 @@ class NabuWindow(NabuDialog):
         cor = self.getCOR()
         if cor != "auto":
             # move the cor value to the nabu reference
-            cor = cor + self.__scan.dim_1 // 2
+            cor = relative_pos_to_absolute(
+                relative_pos=cor, det_width=self.__scan.dim_1
+            )
         config["reconstruction"]["rotation_axis_position"] = str(cor)
         return config
 
@@ -165,7 +164,7 @@ class NabuWindow(NabuDialog):
     def _launchComputation(self, scan, config):
         # update the processing thread
         thread = self.getProcessingThread()
-        thread.init(scan=scan, config=config, dry_run=self._dry_run)
+        thread.init(data=scan, config=config, dry_run=self._dry_run)
         thread.finished.connect(self._computationEnded)
 
         # start processing
@@ -180,10 +179,6 @@ class NabuWindow(NabuDialog):
             self._dataViewer.close()
             self._dataViewer = None
 
-    def _aboutToQuit(self):
-        if self._dataViewer is not None:
-            self._dataViewer.cleanBeforeQuit()
-
 
 def main(argv):
     import os
@@ -216,7 +211,7 @@ def main(argv):
 
     increase_max_number_file()
 
-    app = qt.QApplication.instance() or qt.QApplication([])
+    app = qt.QApplication.instance() or qt.QApplication(["tomwer"])
 
     qt.QLocale.setDefault(qt.QLocale(qt.QLocale.English))
     qt.QLocale.setDefault(qt.QLocale.c())
@@ -243,7 +238,6 @@ def main(argv):
     widget.setWindowIcon(icons.getQIcon("tomwer"))
     splash.finish(widget)
     widget.show()
-    qt.QApplication.instance().aboutToQuit.connect(widget._aboutToQuit)
 
     app.exec_()
 
