@@ -1,17 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import sys
 import argparse
-from silx.gui import qt
+import logging
 import signal
+import sys
+
+from silx.gui import qt
+
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils.resource import increase_max_number_file
 from tomwer.gui import icons
 from tomwer.gui.utils.splashscreen import getMainSplashScreen
-from tomwer.core.scan.scanfactory import ScanFactory
 from tomwer.gui.visualization.dataviewer import DataViewer
-from tomwer.core.utils.resource import increase_max_number_file
-import logging
-
 
 logging.basicConfig(level=logging.WARNING)
 _logger = logging.getLogger(__name__)
@@ -31,16 +32,31 @@ def main(argv):
         default=None,
         nargs="?",
     )
+    parser.add_argument(
+        "--use-opengl-plot",
+        help="Use OpenGL for plots (instead of matplotlib)",
+        action="store_true",
+        default=False,
+    )
 
     options = parser.parse_args(argv[1:])
 
     scan = ScanFactory.create_scan_object(
         scan_path=options.scan_path, entry=options.entry
     )
+
+    import silx
+
+    if options.use_opengl_plot:
+        silx.config.DEFAULT_PLOT_BACKEND = "gl"
+    else:
+        silx.config.DEFAULT_PLOT_BACKEND = "matplotlib"
+
     increase_max_number_file()
 
     global app  # QApplication must be global to avoid seg fault on quit
-    app = qt.QApplication.instance() or qt.QApplication([])
+
+    app = qt.QApplication.instance() or qt.QApplication(["tomwer"])
     splash = getMainSplashScreen()
     qt.QApplication.setOverrideCursor(qt.Qt.WaitCursor)
     qt.QApplication.processEvents()
@@ -57,7 +73,7 @@ def main(argv):
     timer.timeout.connect(lambda: None)
 
     window = DataViewer(parent=None)
-    window.setDisplayMode("radios")
+    window.setDisplayMode("projections-radios")
     window.setWindowTitle("tomwer: scan-viewer")
     window.setWindowIcon(icons.getQIcon("tomwer"))
     window.setScan(scan)
@@ -75,6 +91,7 @@ def getinputinfo():
 
 def sigintHandler(*args):
     """Handler for the SIGINT signal."""
+    qt.QApplication.closeAllWindows()  # needed because get a waiting thread behind
     qt.QApplication.quit()
 
 
