@@ -34,25 +34,31 @@ __license__ = "MIT"
 __date__ = "22/09/2017"
 
 import logging
+import os
+from sys import platform
+
 from tomwer.core.log.processlog import (
     PROCESS_ENDED_NAME,
-    PROCESS_SKIPPED_NAME,
-    PROCESS_INFORM_NAME,
-    PROCESS_STARTED_NAME,
     PROCESS_FAILED_NAME,
+    PROCESS_INFORM_NAME,
+    PROCESS_SKIPPED_NAME,
+    PROCESS_STARTED_NAME,
     PROCESS_SUCCEED_NAME,
 )
-import os
 
 # The background is set with 40 plus the number of the color, and the
 # foreground with 30
 
 # These are the sequences need to get colored ouput
 _RESET_SEQ = "\033[0m"
-_COLOR_SEQ = "\033[1;%dm"
 _BOLD_SEQ = "\033[1m"
 
-_BLACK, _RED, _GREEN, _YELLOW, _BLUE, _MAGENTA, _CYAN, _WHITE = range(8)
+_BLACK = "\033[30m"
+_RED = "\033[31m"
+_GREEN = "\033[32m"
+_YELLOW = "\033[33m"
+_BLUE = "\033[34m"
+_MAGENTA = "\033[35m"
 
 LOG_COLORS = {
     "WARNING": _MAGENTA,
@@ -80,14 +86,15 @@ def _formatter_message(message, use_color=True):
 class _ColoredFormatter(logging.Formatter):
     def __init__(self, msg):
         logging.Formatter.__init__(self, msg)
-        self.use_color = os.environ.get("ORANGE_COLOR_STDOUT_LOG", "False") == "True"
+        if platform == "win32":
+            self.use_color = False
+        else:
+            self.use_color = os.environ.get("ORANGE_COLOR_STDOUT_LOG", "True") == "True"
 
     def format(self, record):
         levelname = record.levelname
         if self.use_color is True and levelname in LOG_COLORS:
-            levelname_color = (
-                _COLOR_SEQ % (30 + LOG_COLORS[levelname]) + levelname + _RESET_SEQ
-            )
+            levelname_color = LOG_COLORS[levelname] + levelname + _RESET_SEQ
             record.levelname = levelname_color
         record.asctime = self.formatTime(record, self.datefmt)
         return logging.Formatter.format(self, record)
