@@ -33,13 +33,15 @@ __date__ = "19/07/2018"
 
 import fnmatch
 import logging
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.scanfactory import ScanFactory
-import tomwer.version
 import os
 import re
 
+import tomwer.version
 from tomwer.core.process.task import Task
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.scan.scanfactory import ScanFactory
+
+from tomwer.core.utils.deprecation import deprecated_warning
 
 _logger = logging.getLogger(__name__)
 
@@ -130,10 +132,16 @@ class UnixFileNamePatternFilter(_PatternBaseFilter):
         return not match
 
 
-class FileNameFilter(
-    _PatternBaseFilter, Task, input_names=("data",), output_names=("data",)
+class FileNameFilterTask(
+    _PatternBaseFilter,
+    Task,
+    input_names=("data", "pattern"),
+    output_names=("data",),
+    optional_input_names=("serialize_output_data",),
 ):
-    """Filter than can call several filter type"""
+    """
+    Task to filter a scan according to his name and a 'unix file name pattern' or a 'regular expression'
+    """
 
     UNIX_PATTERN_FILTER = "unix file name pattern"
 
@@ -148,8 +156,17 @@ class FileNameFilter(
         REGULAR_EXP_FILTER: RegularExpressionFilter,
     }
 
-    def __init__(self, varinfo=None, inputs=None, label=None):
-        Task.__init__(self, varinfo=varinfo, inputs=inputs, label=label)
+    def __init__(
+        self, varinfo=None, inputs=None, node_id=None, node_attrs=None, execinfo=None
+    ):
+        Task.__init__(
+            self,
+            varinfo=varinfo,
+            inputs=inputs,
+            node_id=node_id,
+            node_attrs=node_attrs,
+            execinfo=execinfo,
+        )
         pattern = inputs.get("pattern", None)
         if pattern is None:
             raise ValueError("'pattern' should be provided")
@@ -186,7 +203,7 @@ class FileNameFilter(
         assert isinstance(scan, TomwerScanBase)
 
         if not self.isFiltered(os.path.basename(scan.path)):
-            if self._return_dict:
+            if self.get_input_value("serialize_output_data", True):
                 self.outputs.data = scan.to_dict()
             else:
                 self.outputs.data = scan
@@ -206,3 +223,17 @@ class FileNameFilter(
     def definition():
         """definition of the process"""
         return "filter a scan according to his name"
+
+
+class FileNameFilter(FileNameFilterTask):
+    def __init__(
+        self, varinfo=None, inputs=None, node_id=None, node_attrs=None, execinfo=None
+    ):
+        deprecated_warning(
+            name="tomwer.core.process.conditions.filters.FileNameFilter",
+            type_="class",
+            reason="improve readibility",
+            since_version="1.2",
+            replacement="FileNameFilterTask",
+        )
+        super().__init__(varinfo, inputs, node_id, node_attrs, execinfo)
