@@ -28,20 +28,25 @@ __license__ = "MIT"
 __date__ = "05/07/2017"
 
 
-import socket
-import os
-from tomwer.core import settings
-import tomwer.version
-from tomwer.core.process.task import BaseProcessInfo
 import logging
-from .rpcserver import DataListenerThread
-from nxtomomill import converter as nxtomomill_converter
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomoscan.io import HDF5File
-from silx.io.utils import h5py_read_dataset
-from ewokscore.task import Task as EwoksTask
+import os
+import socket
+import time
 import typing
 
+from ewokscore.task import Task as EwoksTask
+from nxtomomill import converter as nxtomomill_converter
+from silx.io.utils import h5py_read_dataset
+from silx.io.utils import open as open_hdf5
+
+import tomwer.version
+from tomwer.core import settings
+from tomwer.core.process.task import BaseProcessInfo
+from tomwer.core.scan.blissscan import BlissScan
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+
+from .rpcserver import DataListenerThread
+
 try:
     # new HDF5Config class
     from nxtomomill.io.config import TomoHDF5Config as HDF5Config
@@ -53,7 +58,7 @@ try:
 
     _SCAN_NUMBER_PATH = _Acquisition._SCAN_NUMBER_PATH
 except ImportError:
-    from nxtomomill.converter.hdf5.acquisition.baseaquisition import BaseAcquisition
+    from nxtomomill.converter.hdf5.acquisition.baseacquisition import BaseAcquisition
 
     _SCAN_NUMBER_PATH = BaseAcquisition._SCAN_NUMBER_PATH
 
@@ -64,10 +69,11 @@ def nxtomomill_input_callback(entry, desc):
     """note: for the data listener we want to avoid any user intereaction.
     In order to avoid any time lost"""
     if entry == "energy":
-        _logger.warning("Energy is missing {}. set to {}".format(entry, 19.0))
-        return 19.0
+        default_energy = 19.0
+        _logger.warning(f"Energy is missing. Set it to default to {default_energy}")
+        return default_energy
     else:
-        _logger.warning("missing {}. Won't be set".format(entry))
+        _logger.warning(f"missing {entry}. Won't be set")
         return None
 
 
@@ -96,6 +102,11 @@ class DataListener(BaseProcessInfo):
     TIMOUT_READ_FILE = 30
     "When the event 'scan_ended' is received all data might not have been write" " yet"
 
+    SWMR_MODE = None
+    """The bliss writer is not using the swmr mode. This class has independant behavior regarding the tomoscan / nxotmo get_swmr_mode which is
+    dedicated to the internal tomotools behavior
+    """
+
     def __init__(self):
         super().__init__()
         self._host = settings.JSON_RPC_HOST
@@ -108,7 +119,7 @@ class DataListener(BaseProcessInfo):
     @staticmethod
     def program_name():
         """Name of the program used for this processing"""
-        return "data listener"
+        return "scan listener"
 
     @staticmethod
     def program_version():
@@ -161,7 +172,7 @@ class DataListener(BaseProcessInfo):
         elif isinstance(properties, dict):
             self._settings = properties
         else:
-            raise TypeError("invalid type: {}".format(type(properties)))
+            raise TypeError(f"invalid type: {type(properties)}")
 
     def run(self):
         pass
@@ -180,12 +191,12 @@ class DataListener(BaseProcessInfo):
         :return: tuple scan succeeded.
         """
         if not os.path.isfile(sample_file):
-            raise ValueError("Given file {} does not exists.".format(sample_file))
+            raise ValueError(f"Given file {sample_file} does not exists.")
         scans = []
         for mf_emf in self.convert(bliss_file=sample_file, entry=entry):
             master_file, entry_master_file = mf_emf
             if master_file is not None and entry_master_file is not None:
-                scan = HDF5TomoScan(scan=master_file, entry=entry_master_file)
+                scan = NXtomoScan(scan=master_file, entry=entry_master_file)
 
                 try:
                     # register process.
@@ -218,7 +229,7 @@ class DataListener(BaseProcessInfo):
         """Return the list of scans dir for this bliss_file / entry"""
 
         def get_scan_indexes():
-            with HDF5File(bliss_file, "r", swmr=True) as h5f:
+            with open_hdf5(bliss_file) as h5f:
                 entry_node = h5f[entry]
                 if _SCAN_NUMBER_PATH in entry_node:
                     return h5py_read_dataset(entry_node[_SCAN_NUMBER_PATH])
@@ -243,7 +254,7 @@ class DataListener(BaseProcessInfo):
         """Return the proposal file of the experimentation if registred by the
         data listener"""
         if entry is None:
-            with HDF5File(process_file, "r", swmr=True) as h5f:
+            with open_hdf5(process_file) as h5f:
                 entries = BaseProcessInfo._get_process_nodes(
                     root_node=h5f, process=DataListener
                 )
@@ -258,7 +269,7 @@ class DataListener(BaseProcessInfo):
                     entry = list(entries.keys())[0]
                     _logger.info("take %s as default entry" % entry)
 
-        with HDF5File(process_file, "r", swmr=True) as h5f:
+        with open_hdf5(process_file) as h5f:
             dl_nodes = BaseProcessInfo._get_process_nodes(
                 root_node=h5f[entry], process=DataListener
             )
@@ -300,7 +311,7 @@ class DataListener(BaseProcessInfo):
         """Return the proposal file of the experimentation if registred by the
         data listener"""
         if entry is None:
-            with HDF5File(process_file, "r", swmr=True) as h5f:
+            with open_hdf5(process_file) as h5f:
                 entries = BaseProcessInfo._get_process_nodes(
                     root_node=h5f, process=DataListener
                 )
@@ -315,7 +326,7 @@ class DataListener(BaseProcessInfo):
                     entry = list(entries.keys())[0]
                     _logger.info("take %s as default entry" % entry)
 
-        with HDF5File(process_file, "r", swmr=True) as h5f:
+        with open_hdf5(process_file) as h5f:
             dl_nodes = BaseProcessInfo._get_process_nodes(
                 root_node=h5f[entry], process=DataListener
             )
@@ -349,7 +360,7 @@ class DataListener(BaseProcessInfo):
         """Return the proposal file of the experimentation if registred by the
         data listener"""
         if entry is None:
-            with HDF5File(process_file, "r", swmr=True) as h5f:
+            with open_hdf5(process_file) as h5f:
                 entries = BaseProcessInfo._get_process_nodes(
                     root_node=h5f, process=DataListener
                 )
@@ -364,7 +375,7 @@ class DataListener(BaseProcessInfo):
                     entry = list(entries.keys())[0]
                     _logger.info("take %s as default entry" % entry)
 
-        with HDF5File(process_file, "r", swmr=True) as h5f:
+        with open_hdf5(process_file) as h5f:
             dl_nodes = BaseProcessInfo._get_process_nodes(
                 root_node=h5f[entry], process=DataListener
             )
@@ -399,7 +410,7 @@ class DataListener(BaseProcessInfo):
         specific sequence data
         """
         if entry is None:
-            with HDF5File(process_file, "r", swmr=True) as h5f:
+            with open_hdf5(process_file) as h5f:
                 entries = BaseProcessInfo._get_process_nodes(
                     root_node=h5f, process=DataListener
                 )
@@ -414,7 +425,7 @@ class DataListener(BaseProcessInfo):
                     entry = list(entries.keys())[0]
                     _logger.info("take %s as default entry" % entry)
 
-        with HDF5File(process_file, "r", swmr=True) as h5f:
+        with open_hdf5(process_file) as h5f:
             dl_nodes = BaseProcessInfo._get_process_nodes(
                 root_node=h5f[entry], process=DataListener
             )
@@ -458,7 +469,9 @@ class DataListener(BaseProcessInfo):
         :return: tuple (output file, output file entry). Both are set to None
                  if conversion fails.
         """
-        from tomwer.core.process.control.nxtomomill import NxTomomillProcess
+        from tomwer.core.process.control.nxtomomill import (
+            H5ToNxProcess as NxTomomillProcess,
+        )
 
         conf = self.get_configuration()
         if conf is None:
@@ -466,7 +479,12 @@ class DataListener(BaseProcessInfo):
         configuration = HDF5Config.from_dict(conf)
         if configuration.output_file is None:
             output_file_path = NxTomomillProcess.deduce_output_file_path(
-                bliss_file, entry=entry, outputdir=None
+                bliss_file,
+                entry=entry,
+                outputdir=None,
+                scan=BlissScan(
+                    master_file=bliss_file, entry=entry, proposal_file=bliss_file
+                ),
             )
         else:
             output_file_path = configuration.output_file
@@ -485,25 +503,22 @@ class DataListener(BaseProcessInfo):
         # write
         def sequence_is_finished():
             try:
-                with HDF5File(bliss_file, "r", swmr=True) as h5f:
+                with open_hdf5(bliss_file) as h5f:
                     end_scan_path = "/".join((entry, "end_time"))
                     return end_scan_path in h5f
             except Exception:
                 return False
 
         timeout = self.TIMOUT_READ_FILE
-        import time
 
         while timeout > 0 and not sequence_is_finished():
-            timeout = timeout - 0.2
+            timeout -= 0.2
             time.sleep(0.2)
 
         if timeout <= 0:
-            _logger.error(
-                "unable to access {}@{}. (Write never ended)".format(entry, bliss_file)
-            )
+            _logger.error(f"unable to access {entry}@{bliss_file}. (Write never ended)")
             return ((None, None),)
-        # one more delay to insure we can read it
+        # one more delay to insure we can read it. Some frame might need more time to be dump from lima.
         time.sleep(4)
 
         # force some parameters
@@ -522,8 +537,7 @@ class DataListener(BaseProcessInfo):
             )
         except Exception as e:
             _logger.error(
-                "Fail to convert from tango file: %s to NXTomo."
-                "Conversion error is: %s" % (bliss_file, e)
+                f"Fail to convert from tango file: {bliss_file} to NXTomo. Error is: {e}"
             )
             return ((None, None),)
         else:
@@ -539,7 +553,7 @@ class DataListener(BaseProcessInfo):
         return res == "Y"
 
     def _signal_scan_ready(self, scan):
-        assert isinstance(scan, HDF5TomoScan)
+        assert isinstance(scan, NXtomoScan)
         pass
 
     def activate(self, activate=True):
