@@ -27,19 +27,19 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "09/06/2020"
 
-import time
-import logging
-import threading
-import typing
-from silx.utils.enum import Enum as _Enum
-from multiprocessing import Process
-from werkzeug.wrappers import Request, Response
-from werkzeug.serving import run_simple
-from jsonrpc import JSONRPCResponseManager, dispatcher
-import socket
-from psutil import process_iter
 import getpass
+import logging
+import socket
+import threading
+import time
+import typing
+from multiprocessing import Process
 
+from jsonrpc import JSONRPCResponseManager, dispatcher
+from psutil import process_iter
+from silx.utils.enum import Enum as _Enum
+from werkzeug.serving import run_simple
+from werkzeug.wrappers import Request, Response
 
 _logger = logging.getLogger(__name__)
 
@@ -68,7 +68,7 @@ class BlissAcquisition:
         self.proposal_file = proposal_file
         self.sample_file = sample_file
         self.scan_numbers = []
-        self.status = BlissAcquisition
+        self.status = TangoAcquisitionStatus.STARTED
         self._start_time = start_time
         self._end_time = None
         self._error = None
@@ -151,7 +151,6 @@ class _BaseDataListenerThread:
     def _rpc_sequence_started(
         self, saving_file, scan_title, sequence_scan_number, proposal_file, sample_file
     ):
-
         self._current_sequence = BlissAcquisition(
             file_path=saving_file,
             entry_name=sequence_scan_number,
@@ -178,25 +177,23 @@ class _BaseDataListenerThread:
             self.scan_added(acquisition=self._current_sequence, scan_number=scan_number)
 
     def scan_added(self, acquisition: BlissAcquisition, scan_number: int):
-        print("scan {} started".format(scan_number))
+        print(f"scan {scan_number} started")
 
     def _rpc_scan_ended(self, scan_number):
         if self._current_sequence is not None:
             self.scan_ended(acquisition=self._current_sequence, scan_number=scan_number)
 
     def scan_ended(self, acquisition: BlissAcquisition, scan_number):
-        print("scan {} ended".format(scan_number))
+        print(f"scan {scan_number} ended")
 
-    def _rpc_sequence_ended(self, saving_file, sequence_scan_number, success):
+    def _rpc_sequence_ended(self, saving_file, sequence_scan_number, success: bool):
         if success is False:
-            _logger.warning(
-                "sequence {}@{} failed".format(sequence_scan_number, saving_file)
-            )
+            _logger.warning(f"sequence {sequence_scan_number}@{saving_file} failed")
             if self._current_sequence is not None:
                 try:
                     self.sequence_failed(self._current_sequence)
                 except Exception as e:
-                    _logger.warning("Fail to catch cancel scan. error is", e)
+                    _logger.warning(f"Fail to catch cancel scan. error is {e}")
 
             return
 
@@ -213,23 +210,18 @@ class _BaseDataListenerThread:
                 )
 
                 _logger.warning(
-                    "End of {} detected but does not fit the "
-                    "current acquisition {}".format(name, name_current)
+                    f"End of {name} detected but does not fit the current acquisition {name_current}"
                 )
 
     def sequence_ended(self, acquisition: BlissAcquisition):
         """callback when a sequence end (with success)"""
         print(
-            "sequence {}@{} ended. Succeed: {}".format(
-                acquisition.entry, acquisition.master_file, acquisition.status
-            )
+            f"sequence {acquisition.entry}@{acquisition.master_file} ended. Succeed: {acquisition.status}"
         )
 
     def sequence_failed(self, acquisition: BlissAcquisition):
         """callback when a sequence fails (probably canceled)"""
-        print(
-            "sequence {}@{} failed".format(acquisition.entry, acquisition.master_file)
-        )
+        print(f"sequence {acquisition.entry}@{acquisition.master_file} failed")
 
     @Request.application
     def application(self, request):
@@ -243,17 +235,17 @@ class _BaseDataListenerThread:
 
     def start(self) -> None:
         self._stop = False
-        _logger.info("launch rpc server on {}:{}".format(self._host, self._port))
+        _logger.info(f"launch rpc server on {self._host}:{self._port}")
         self._server = Process(
             target=run_simple, args=(self._host, self._port, self.application)
         )
         self._server.start()
-        _logger.info("launcher server with pid {}".format(self._server.pid))
+        _logger.info(f"launcher server with pid {self._server.pid}")
 
     def stop(self):
         self._stop = True
         if self._server is not None:
-            _logger.info("stop rpc server on {}:{}".format(self._host, self._port))
+            _logger.info(f"stop rpc server on {self._host}:{self._port}")
             self._server.terminate()
             self._server.join(_BaseDataListenerThread._JOIN_TIMEOUT)
             self._server = None
@@ -291,9 +283,7 @@ def send_signal_to_local_rpc_servers(signal, port, extended_find=True):
                         "tomwer",
                         "orange-canvas",
                     ):
-                        _logger.warning(
-                            "send {} signal to pid {}" "".format(signal, proc.pid)
-                        )
+                        _logger.warning(f"send {signal} signal to pid {proc.pid}")
                         proc.send_signal(signal)
                         found = True
                         return
@@ -309,17 +299,8 @@ def send_signal_to_local_rpc_servers(signal, port, extended_find=True):
                     # make sure we will kill the correct process
                     if conns.laddr.port == port:
                         _logger.warning(
-                            "process pid: {} - {} seems to be one occupying port {}"
-                            "".format(proc.pid, proc.name(), port)
+                            f"process pid: {proc.pid} - {proc.name()} seems to be one occupying port {port}"
                         )
                         return
             except (psutil.PermissionError, psutil.AccessDenied):
                 pass
-
-
-if __name__ == "__main__":
-    worker = _BaseDataListenerThread(host="localhost", port=4000)
-    worker.run()
-    time.sleep(20)
-    worker.stop()
-    exit(0)
