@@ -30,29 +30,42 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "30/09/2019"
 
+import logging
 import os
-import time
 import threading
-from tomwer.core.signal import Signal
-from tomwer.core.process.utils import LastReceivedScansDict
-from tomwer.core.process.task import BaseProcessInfo
+import time
+from typing import Optional
+
+from ewokscore.task import Task as EwoksTask
+
 from tomwer.core.process.control.datawatcher.datawatcherobserver import (
     _DataWatcherObserver,
     _OngoingObservation,
 )
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.settings import get_lbsram_path, get_dest_path
-from tomwer.core.utils import logconfig
-from .status import *  # noqa F403
-import logging
+from tomwer.core.process.task import BaseProcessInfo
+from tomwer.core.process.utils import LastReceivedScansDict
+from tomwer.core.scan.blissscan import BlissScan
 from tomwer.core.scan.scanbase import TomwerScanBase
-from ewokscore.task import Task as EwoksTask
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.settings import get_dest_path, get_lbsram_path
+from tomwer.core.signal import Signal
+from tomwer.core.utils import logconfig
+
+from .status import *  # noqa F403
 
 logger = logging.getLogger(__name__)
 
 
-class _DataWatchertaskPlaceHolder(EwoksTask, output_names=("data",)):
-    pass
+class DataWatcherEwoksTask(EwoksTask, input_names=["data"], output_names=["data"]):
+    """
+    For now the data watcher is a 'special case'. Because it will trigger downstream workflows / prcessing each time it discover
+    a new scan.
+    To move to 'default' ewoks workflows. Like launching it from the command line the simpler for now is to
+    simply pass the `data` input that user can fill manually.
+    """
+
+    def run(self):
+        self.outputs.data = self.inputs.data
 
 
 class _DataWatcher(BaseProcessInfo):
@@ -90,6 +103,14 @@ class _DataWatcher(BaseProcessInfo):
         self._setCurrentStatus("not processing")
         self._launchObservation()
         self._checkThread = None
+        self._serialize_output_data = False
+
+    def set_serialize_output_data(self, serialize: bool):
+        self._serialize_output_data = serialize
+
+    def _launchObservation(self):
+        """Main function of the widget"""
+        raise NotImplementedError("Base class")
 
     def _initClass(self):
         self.srcPattern = get_lbsram_path()
@@ -105,6 +126,7 @@ class _DataWatcher(BaseProcessInfo):
         """Pattern to look in the acquisition file to know of the acquisition
          is ended or not"""
         self.startByOldest = False
+        self.linux_filter_file_pattern = None
 
     def _switchObservation(self):
         """Switch the status of observation"""
@@ -127,6 +149,7 @@ class _DataWatcher(BaseProcessInfo):
             assert len(obsMethod) == 1 or (
                 type(obsMethod[1]) is dict and len(obsMethod) == 2
             )
+            obsMethod = obsMethod[0]
         self.obsMethod = obsMethod
         if self.isObserving:
             self._restartObservation()
@@ -184,15 +207,6 @@ class _DataWatcher(BaseProcessInfo):
 
             return True
 
-    def isObserving(self):
-        """
-
-        :return bool: True if the widget is actually observing the root
-                      directory
-
-        """
-        return self.isObserving
-
     def _setIsObserving(self, b):
         self.isObserving = b
 
@@ -206,6 +220,19 @@ class _DataWatcher(BaseProcessInfo):
     def getFolderObserved(self):
         return self.folderObserved
 
+    def getLinuxFilePattern(self):
+        return self.linux_filter_file_pattern
+
+    def setLinuxFilePattern(self, pattern: Optional[str]):
+        if pattern is None:
+            self.linux_filter_file_pattern = pattern
+        else:
+            pattern = pattern.lstrip(" ").rstrip(" ")
+            if pattern == "":
+                self.linux_filter_file_pattern = None
+            else:
+                self.linux_filter_file_pattern = pattern
+
     def setFolderObserved(self, path):
         assert type(path) is str
         if not os.path.isdir(path):
@@ -251,7 +278,6 @@ class _DataWatcher(BaseProcessInfo):
         """Init the thread running the datawatcher functions"""
         if self.observationThread is None:
             self._createDataWatcher()
-
         headDir = self.getFolderObserved()
         if headDir is None or not os.path.isdir(headDir):
             self._messageNotDir(headDir)
@@ -261,11 +287,12 @@ class _DataWatcher(BaseProcessInfo):
         # update information on the head folder and the start by the oldest
         self.observationThread.setHeadFolder(headDir)
         self.observationThread.setObservationMethod(self.obsMethod)
+        self.observationThread.setUnixFileNamePattern(self.linux_filter_file_pattern)
 
         return True
 
-    def _messageNotDir(self, dir):
-        message = "Given path (%s) isn't a directory." % dir
+    def _messageNotDir(self, dir_):
+        message = "Given path (%s) isn't a directory." % dir_
         logger.warning(message, extra={logconfig.DOC_TITLE: self._scheme_title})
 
     def _createDataWatcher(self):
@@ -299,10 +326,7 @@ class _DataWatcher(BaseProcessInfo):
                                 _scan = ScanFactory.create_scan_object(scan_path=scanID)
                             except Exception as e:
                                 logger.error(
-                                    "Fail to create TomoBase from",
-                                    scanID,
-                                    "Error is",
-                                    e,
+                                    f"Fail to create TomoBase from {scanID}. Error is {e}"
                                 )
                             else:
                                 self._signalScanReady(scan=_scan)
@@ -352,14 +376,14 @@ class _DataWatcher(BaseProcessInfo):
             try:
                 scan = ScanFactory.create_scan_object(scan_path=info)
             except Exception as e:
-                logger.warning("Fail to create scan object from", info, "Reason is", e)
+                logger.warning(f"Fail to create scan object from {info}. Reason is {e}")
             else:
                 self._signalScanReady(scan=scan)
 
     def _signalScanReady(self, scan):
-        assert isinstance(scan, TomwerScanBase)
+        assert isinstance(scan, (TomwerScanBase, BlissScan))
         self.lastFoundScans.add(scan)
-        self.sigScanReady.emit(scan)
+        self.sigScanReady.emit(scan)  # pylint: disable=E1101
 
     def mockObservation(self, folder):
         # simple mocking emitting a signal to say that the given folder is valid
@@ -400,7 +424,7 @@ class DataWatcher(_DataWatcher):
     """
 
     sigTMStatusChanged = Signal(str)
-    sigScanReady = Signal(TomwerScanBase)
+    sigScanReady = Signal(object)
 
     import threading
 
@@ -414,7 +438,7 @@ class DataWatcher(_DataWatcher):
     def _signalScanReady(self, scan):
         assert isinstance(scan, TomwerScanBase)
         super()._signalScanReady(scan)
-        if self._return_dict:
+        if self._serialize_output_data:
             value = scan.to_dict()
         else:
             value = scan
@@ -429,7 +453,7 @@ class DataWatcher(_DataWatcher):
         # manage data watcher observation
         if self.observationThread is None or not self._observation_thread_running():
             if self._initObservation() is False:
-                self.currentStatus = self._setCurrentStatus("failure")
+                self._setCurrentStatus("failure")
                 logger.info(
                     "failed on observation",
                     extra={logconfig.DOC_TITLE: self._scheme_title},
