@@ -33,28 +33,32 @@ __license__ = "MIT"
 __date__ = "09/02/2017"
 
 
+import logging
 import os
 import threading
+import time
 from collections import OrderedDict
+from fnmatch import fnmatch
+from typing import Optional
+
+import h5py
 
 from tomwer.core.process.control.datawatcher import status as datawatcherstatus
-from tomwer.web.client import OWClient
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.utils.threads import LoopThread
+
+from .datawatcherprocess import _DataWatchEmpty
 from .edfdwprocess import (
     _DataWatcherProcessParseInfo,
     _DataWatcherProcessUserFilePattern,
     _DataWatcherProcessXML,
 )
-from .hdf5dwprocess import _DataWatcherProcessHDF5
-from .datawatcherprocess import _DataWatchEmpty
-from tomwer.core.utils.threads import LoopThread
-import logging
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.scan.edfscan import EDFTomoScan
+from .hdf5dwprocess import _BlissScanWatcherProcess, _DataWatcherProcessHDF5
 
 logger = logging.getLogger(__name__)
 
 
-class _DataWatcherFixObserverMixIn(OWClient):
+class _DataWatcherStaticObserverMixIn:
     def __init__(
         self,
         scanID,
@@ -73,8 +77,6 @@ class _DataWatcherFixObserverMixIn(OWClient):
         :param Union[str,None] patternObs:
         :param observationRegistry: what registry the observation should update
         """
-        OWClient.__init__(self)
-
         self.obsMethod = obsMethod
         self.srcPattern = srcPattern
         self.destPattern = destPattern
@@ -84,6 +86,7 @@ class _DataWatcherFixObserverMixIn(OWClient):
         self.dataWatcherProcess = self._getDataWatcherProcess()
         self.status = "not processing"
         self.observationRegistry = observationRegistry
+        self.file_name_pattern = None
 
     def setProperties(self, properties):
         assert isinstance(properties, dict)
@@ -101,7 +104,12 @@ class _DataWatcherFixObserverMixIn(OWClient):
             self.dataWatcherProcess.quitting = True
 
     def run(self):
-        if not os.path.isdir(self.path):
+        look_for_hdf5_file = self.obsMethod in (
+            datawatcherstatus.BLISS_SCAN_END,
+            datawatcherstatus.NXtomo_END,
+        )
+
+        if not look_for_hdf5_file and not os.path.isdir(self.path):
             logger.info("can't observe %s, not a directory" % self.path)
             self.status = "failure"
             if self.observationRegistry:
@@ -149,31 +157,39 @@ class _DataWatcherFixObserverMixIn(OWClient):
         return
 
     def _getDataWatcherProcess(self):
-        assert self.path is not None
-        if HDF5TomoScan.directory_contains_scan(directory=self.path):
-            raise TypeError("HDF5 is not managed by the DataWatcher")
+        if self.obsMethod == datawatcherstatus.DET_END_XML:
+            return _DataWatcherProcessXML(
+                dataDir=self.path,
+                srcPattern=self.srcPattern,
+                destPattern=self.destPattern,
+            )
+        elif self.obsMethod == datawatcherstatus.PARSE_INFO_FILE:
+            return _DataWatcherProcessParseInfo(
+                dataDir=self.path,
+                srcPattern=self.srcPattern,
+                destPattern=self.destPattern,
+            )
+        elif self.obsMethod == datawatcherstatus.DET_END_USER_ENTRY:
+            return _DataWatcherProcessUserFilePattern(
+                dataDir=self.path,
+                srcPattern=self.srcPattern,
+                destPattern=self.destPattern,
+                pattern=self.patternObs,
+            )
+        elif self.obsMethod == datawatcherstatus.BLISS_SCAN_END:
+            return _BlissScanWatcherProcess(
+                dataDir=self.path,
+                srcPattern=self.srcPattern,
+                destPattern=self.destPattern,
+            )
+        elif self.obsMethod == datawatcherstatus.NXtomo_END:
+            return _DataWatcherProcessHDF5(
+                dataDir=self.path,
+                srcPattern=self.srcPattern,
+                destPattern=self.destPattern,
+            )
         else:
-            if self.obsMethod == datawatcherstatus.DET_END_XML:
-                return _DataWatcherProcessXML(
-                    dataDir=self.path,
-                    srcPattern=self.srcPattern,
-                    destPattern=self.destPattern,
-                )
-            elif self.obsMethod == datawatcherstatus.PARSE_INFO_FILE:
-                return _DataWatcherProcessParseInfo(
-                    dataDir=self.path,
-                    srcPattern=self.srcPattern,
-                    destPattern=self.destPattern,
-                )
-            elif self.obsMethod == datawatcherstatus.DET_END_USER_ENTRY:
-                return _DataWatcherProcessUserFilePattern(
-                    dataDir=self.path,
-                    srcPattern=self.srcPattern,
-                    destPattern=self.destPattern,
-                    pattern=self.patternObs,
-                )
-            else:
-                raise ValueError("requested observation method not recognized")
+            raise ValueError("requested observation method not recognized")
 
     def setSourceToDestinationPatterns(self, srcPattern, destPattern):
         """set the patterns to replace strings sequence in directories path.
@@ -198,6 +214,9 @@ class _DataWatcherFixObserverMixIn(OWClient):
         self.srcPattern = srcPattern
         self.destPattern = destPattern
 
+    def setUnixFileNamePattern(self, pattern):
+        self.file_name_pattern = pattern
+
     def setObservationMethod(self, obsMeth, info=None):
         """
         Set if we are looking for the .xml file
@@ -228,7 +247,7 @@ class _DataWatcherFixObserverMixIn(OWClient):
                 self.patternObs = None
 
 
-class _DataWatcherFixObserver(_DataWatcherFixObserverMixIn, threading.Thread):
+class _DataWatcherStaticObserver(_DataWatcherStaticObserverMixIn, threading.Thread):
     """Observe one specific directory and signal when the state of this
     directory change"""
 
@@ -242,7 +261,7 @@ class _DataWatcherFixObserver(_DataWatcherFixObserverMixIn, threading.Thread):
         observationRegistry,
     ):
         threading.Thread.__init__(self)
-        _DataWatcherFixObserverMixIn.__init__(
+        _DataWatcherStaticObserverMixIn.__init__(
             self,
             scanID=scanID,
             obsMethod=obsMethod,
@@ -253,7 +272,7 @@ class _DataWatcherFixObserver(_DataWatcherFixObserverMixIn, threading.Thread):
         )
 
 
-class _DataWatcherObserver_MixIn(OWClient):
+class _DataWatcherObserver_MixIn:
     """Thread launching the data watcher process (observation of acquisition)
 
     :param str headDir: the root dir to make to fin dacquisition
@@ -282,8 +301,8 @@ class _DataWatcherObserver_MixIn(OWClient):
         srcPattern=None,
         destPattern=None,
         ignoredFolders=None,
+        file_name_pattern=None,
     ):
-        OWClient.__init__(self)
         self.observations = observationClass()
         self.observations.ignoredFolders = (
             [] if ignoredFolders is None else ignoredFolders
@@ -294,6 +313,7 @@ class _DataWatcherObserver_MixIn(OWClient):
         self.setObservationMethod(obsMethod)
         self.srcPattern = srcPattern
         self.destPattern = destPattern
+        self._file_name_pattern = file_name_pattern
         self.dataWatcherProcess = None
         self._patternObs = None
         """The pattern to use for the DET_END_USER_ENTRY method"""
@@ -301,24 +321,59 @@ class _DataWatcherObserver_MixIn(OWClient):
         self.restartEvent = None
         self._firstRun = True
 
+    @property
+    def file_name_pattern(self) -> Optional[str]:
+        return self._file_name_pattern
+
+    @file_name_pattern.setter
+    def file_name_pattern(self, pattern: Optional[str]):
+        if pattern is None or pattern.lstrip(" ").rstrip(" ") == "":
+            self._file_name_pattern = None
+        else:
+            self._file_name_pattern = pattern.lstrip(" ").rstrip(" ")
+
     def resetObservations(self):
         self.observations.reset()
 
     def registerRestartProcess(self, event):
         self.restartEvent = event
 
-    def process(self):
-        def process_observations(directory):
-            if (
-                self.observations.isObserving(directory) is False
-                and self.dataWatcherProcess._isScanDirectory(directory)
-                and directory not in self.observations.ignoredFolders
-            ):
-                self.observe(directory)
+    def run(self):
+        look_for_hdf5_file = self.obsMethod in (
+            datawatcherstatus.BLISS_SCAN_END,
+            datawatcherstatus.NXtomo_END,
+        )
 
-            for f in os.listdir(directory):
-                if os.path.isdir(os.path.join(directory, f)):
-                    process_observations(os.path.join(directory, f))
+        def process_observations(file_path):
+            do_observation = (
+                self.observations.isObserving(file_path) is False
+                and (
+                    (look_for_hdf5_file and h5py.is_hdf5(file_path))
+                    or (self.dataWatcherProcess._isScanDirectory(file_path))
+                )
+                and file_path not in self.observations.ignoredFolders
+            )
+            if (
+                do_observation
+                and self.file_name_pattern is not None
+                and not fnmatch(os.path.basename(file_path), self.file_name_pattern)
+            ):
+                do_observation = False
+            if do_observation:
+                # do the observation
+                self.observe(file_path)
+
+            if os.path.isdir(file_path):
+                try:
+                    for f in os.listdir(file_path):
+                        full_file_path = os.path.join(file_path, f)
+                        if os.path.isdir(full_file_path) or (
+                            look_for_hdf5_file and h5py.is_hdf5(full_file_path)
+                        ):
+                            # parse files to check if some observations should be done
+                            process_observations(full_file_path)
+                except Exception:
+                    pass
 
         if self._firstRun or (
             self.restartEvent is not None and self.restartEvent.isSet()
@@ -330,14 +385,12 @@ class _DataWatcherObserver_MixIn(OWClient):
             if not os.path.isdir(self.headDir):
                 logger.warning("can't observe %s, not a directory" % self.headDir)
                 return
-            self._check_scans_ready()
+            self._check_scans_ready()  # pylint: disable=E1101
             self.dataWatcherProcess = self._getDataWatcherProcess()
             process_observations(self.headDir)
             self._processObservation()
-            self._check_scans_ready()
+            self._check_scans_ready()  # pylint: disable=E1101
         else:
-            import time
-
             time.sleep(0.5)
 
     def _processObservation(self):
@@ -354,6 +407,9 @@ class _DataWatcherObserver_MixIn(OWClient):
             self.dataWatcherProcess.quitting = True
             self.lastObsDir = None
 
+    def setUnixFileNamePattern(self, pattern):
+        self.file_name_pattern = pattern
+
     def setObservationMethod(self, obsMeth, info=None):
         """
         Set if we are looking for the .xml file from .edf observation or any
@@ -384,7 +440,7 @@ class _DataWatcherObserver_MixIn(OWClient):
             else:
                 self._patternObs = None
 
-        for dir, thread in self.observations.dict.items():
+        for _, thread in self.observations.dict.items():
             thread.setObservationMethod(obsMeth, info=info)
 
     def setHeadFolder(self, headDir):
@@ -413,11 +469,11 @@ class _DataWatcherObserver_MixIn(OWClient):
         """
         self.srcPattern = srcPattern
         self.destPattern = destPattern
-        for dir, thread in self.observations.items():
+        for _, thread in self.observations.items():
             thread.setSourceToDestinationPatterns(self.srcPattern, self.destPattern)
 
     def observe(self, scanID):
-        self.observations.add(self._getObserver(scanID))
+        self.observations.add(self._getObserver(scanID))  # pylint: disable=E1101
 
     def cancelObservation(self, scanID):
         if self.observations.isObserving(scanID) is False:
@@ -434,12 +490,18 @@ class _DataWatcherObserver_MixIn(OWClient):
         return self.observations.isObserving(scanID)
 
     def _getDataWatcherProcess(self):
-        if HDF5TomoScan.directory_contains_scan(self.headDir):
+        if self.obsMethod == datawatcherstatus.NXtomo_END:
             return _DataWatcherProcessHDF5(
                 dataDir=self.headDir,
                 srcPattern=self.srcPattern,
                 destPattern=self.destPattern,
             )
+        elif self.obsMethod == datawatcherstatus.BLISS_SCAN_END:
+            return _BlissScanWatcherProcess(
+                dataDir=self.headDir,
+                srcPattern=self.srcPattern,
+                destPattern=self.destPattern,
+            )
         elif EDFTomoScan.directory_contains_scan(
             self.headDir, src_pattern=self.srcPattern, dest_pattern=self.destPattern
         ):
@@ -514,7 +576,7 @@ class _DataWatcherObserver(_DataWatcherObserver_MixIn, LoopThread):
             self.latestScanReady.append(scanID)
 
     def _getObserver(self, scanID):
-        return _DataWatcherFixObserver(
+        return _DataWatcherStaticObserver(
             scanID=scanID,
             obsMethod=self.obsMethod,
             srcPattern=self.srcPattern,
@@ -561,13 +623,13 @@ class _OngoingObservation(object):
     def add(self, observer):
         if self.isObserving(observer.path) is False:
             with self.lock:
-                assert isinstance(observer, _DataWatcherFixObserverMixIn)
+                assert isinstance(observer, _DataWatcherStaticObserverMixIn)
                 self.dict[observer.path] = observer
                 self.obsAddedEvent.set()
                 self._latestObsEnded.append(observer.path)
 
     def remove(self, observer):
-        assert isinstance(observer, _DataWatcherFixObserverMixIn)
+        assert isinstance(observer, _DataWatcherStaticObserverMixIn)
         if self.isObserving(observer.path) is True:
             # observer.sigStatusChanged.disconnect(self._updateStatus)
             observer.quit()
