@@ -31,16 +31,15 @@ __license__ = "MIT"
 __date__ = "14/10/2016"
 
 
-from glob import glob
-import subprocess
-from tomwer.core.signal import Signal
-from tomwer.core.process.reconstruction.ftseries.params.fastsetupdefineglobals import *  # noqa F403
-from tomwer.web.client import OWClient
 import logging
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.scan.edfscan import EDFTomoScan
-from .status import OBSERVATION_STATUS
 import os
+import subprocess
+from glob import glob
+
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.signal import Signal
+
+from .status import OBSERVATION_STATUS
 
 logger = logging.getLogger(__name__)
 
@@ -58,7 +57,7 @@ Useful tools
 """
 
 
-class _DataWatcherProcess(OWClient):
+class _DataWatcherProcess:
     """
     DataWatcherProcess is the process managing the observation of acquisition.
     Since we want to loop infinitly on a root folder we have to ignore the
@@ -108,7 +107,7 @@ class _DataWatcherProcess(OWClient):
     INITIAL_STATUS = "not processing"
 
     def __init__(self, dataDir, srcPattern=None, destPattern=None):
-        super(_DataWatcherProcess, self).__init__()
+        super().__init__()
         self.RootDir = os.path.abspath(dataDir)
         self.parsing_dir = os.path.abspath(dataDir)
         self.oldest = 0
@@ -171,7 +170,7 @@ class _DataWatcherProcess(OWClient):
             if self.RootDir.startswith(self.srcPattern):
                 source = os.path.join(self.RootDir, self.parsing_dir)
                 target = source.replace(self.srcPattern, self.destPattern, 1)
-                info = "Start synchronization between %s and %s" % (source, target)
+                info = f"Start synchronization between {source} and {target}"
                 self.sigNewInformation.emit(info)
 
                 target_dirname = os.path.dirname(target)
@@ -224,10 +223,6 @@ class _DataWatcherProcess(OWClient):
                 gd = glob(os.path.join(directory, EDFTomoScan.INFO_EXT))
                 if len(gd) > 0:
                     self.detector = "Dimax"
-        elif HDF5TomoScan.directory_contains_scan(
-            directory, src_pattern=self.srcPattern, dest_pattern=self.destPattern
-        ):
-            scan = HDF5TomoScan(scan=directory)
         else:
             return False
 
