@@ -31,16 +31,14 @@ __license__ = "MIT"
 __date__ = "30/09/2019"
 
 
-from .datawatcherprocess import _DataWatcherProcess
 import logging
 import os
 import shutil
-from tomwer.core.scan.edfscan import EDFTomoScan
-from silx.third_party.EdfFile import EdfFile
-from subprocess import getstatusoutput as myux
 from glob import glob
-import time
-import sys
+
+from tomwer.core.scan.edfscan import EDFTomoScan
+
+from .datawatcherprocess import _DataWatcherProcess
 
 _logger = logging.getLogger(__name__)
 try:
@@ -52,26 +50,6 @@ except ImportError:
     has_rsync = True
 
 
-def get_dir_size(dir):
-    if not os.path.isdir(dir):
-        err = "%s is not a directory, can't get size" % dir
-        raise ValueError(err)
-    else:
-        aux = myux("du -ms " + dir.replace(" ", "\ "))  # noqa W605
-        if len(aux) < 2:
-            return 0
-        return float((aux[1].split("\t"))[0])
-
-
-def get_info_val(lines, key):
-    r = range(len(lines))
-    key = key + "="
-    for i in r:
-        if key in lines[i]:
-            val = float(lines[i].split("=")[1])
-    return val
-
-
 class _DataWatcherEDFProcess(_DataWatcherProcess):
     """
     Base class for edf acquisition observation
@@ -100,7 +78,7 @@ class _DataWatcherEDFProcess(_DataWatcherProcess):
     def _removeAcquisition(self, scanID, reason):
         if os.path.exists(scanID) and os.path.isdir(scanID):
             if self._removed is None:
-                _logger.info("removing folder %s because %s" % (scanID, reason))
+                _logger.info(f"removing folder {scanID} because {reason}")
                 if has_rsync:
                     RSyncManager().removeDir(scanID)
                     # avoid multiple removal as removal is asynchronous and might
@@ -109,29 +87,6 @@ class _DataWatcherEDFProcess(_DataWatcherProcess):
                 else:
                     shutil.rmtree(scanID)
 
-    def info_validate(self, infoname):
-        """
-        Check that file found is a valid one.
-
-        :return: True if the file is valid
-        """
-        assert os.path.isfile(infoname)
-        try:
-            fd = open(infoname, "r")
-            lines = fd.read()
-            fd.close()
-
-            lg = len(self.FILE_INFO_KEYS)
-            vali = 0
-            for i in range(lg):
-                vali = vali + (self.FILE_INFO_KEYS[i] in lines)
-
-            return vali == lg
-        except Exception as e:
-            e = sys.exc_info()[0]
-            _logger.error("fail to get info from %s" % infoname)
-            raise e
-
     def is_abort(self):
         if os.path.exists(path=self.scan_name):
             return EDFTomoScan(scan=self.scan_name).is_abort(
@@ -150,6 +105,26 @@ class _DataWatcherProcessXML(_DataWatcherEDFProcess):
     def __init__(self, dataDir, srcPattern, destPattern):
         _DataWatcherEDFProcess.__init__(self, dataDir, srcPattern, destPattern)
 
+        # hack for acquisition on lbsram:
+        # in this case .info and .xml are in /data instead of /lbsram. To simplify we copy those from /data to /lbsram
+        # not very elegant but as this is expected to be 'legacy code'
+        if self.srcPattern is not None and self.destPattern is not None:
+            for ext in (self.INFO_EXT, self.XML_EXT):
+                aux = self.parsing_dir.split(os.path.sep)
+                file_on_lbsram = os.path.join(
+                    self.RootDir, self.parsing_dir, aux[len(aux) - 1] + ext
+                )
+
+                file_on_nice = file_on_lbsram.replace(
+                    self.srcPattern, self.destPattern, 1
+                )
+                if os.path.exists(file_on_nice) and not os.path.exists(file_on_lbsram):
+                    try:
+                        shutil.copyfile(file_on_nice, file_on_lbsram)
+                    except Exception:
+                        # in case for example dest pattern doesn't exists
+                        pass
+
     def is_data_complete(self):
         self._sync()
         aux = self.parsing_dir.split(os.path.sep)
@@ -191,134 +166,34 @@ class _DataWatcherProcessUserFilePattern(_DataWatcherEDFProcess):
 
 class _DataWatcherProcessParseInfo(_DataWatcherEDFProcess):
     """
-    This method will look for a '[scan].xml' pattern
+    This method will look for a '[scan].info' pattern
     """
 
-    TYPES = {
-        "SignedByte": 1,
-        "UnsignedByte": 1,
-        "SignedShort": 2,
-        "UnsignedShort": 2,
-        "SignedInteger": 4,
-        "UnsignedInteger": 4,
-        "SignedLong": 4,
-        "UnsignedLong": 4,
-        "Signed64": 8,
-        "Unsigned64": 8,
-        "FloatValue": 4,
-        "Float": 4,
-        "DoubleValue": 8,
-    }
-
-    TIME_LOOP_DATA_INCOMPLETE = 4
-    """Time in second between two iterations to check if the data are complete
-    or not"""
-
     def __init__(self, dataDir, srcPattern, destPattern):
         _DataWatcherEDFProcess.__init__(self, dataDir, srcPattern, destPattern)
-        self.Dim1 = None
-        self.Dim2 = None
-        self.Tomo = None
 
     @staticmethod
     def get_data_size(edfType):
-        if edfType in _DataWatcherProcessParseInfo.TYPES:
-            return _DataWatcherProcessParseInfo.TYPES[edfType]
+        if edfType in _DataWatcherProcessParseInfo.TYPES:  # pylint: disable=E1101
+            return _DataWatcherProcessParseInfo.TYPES[edfType]  # pylint: disable=E1101
         else:
             return 2
 
-    def info_validate(self):
-        """
-        Check that file found is a valid one and set acquisition information
-
-        :return: True if the file is valid
-        """
-        if _DataWatcherEDFProcess.info_validate(self):
-            assert os.path.isfile(self.infoname)
-            try:
-                fd = open(self.infoname, "r")
-                lines = fd.read()
-                fd.close()
-
-                self.curdir = os.path.join(self.RootDir, self.parsing_dir)
-                if not os.path.isdir(self.curdir):
-                    return False
-                self.dirsize = get_dir_size(self.curdir)
-
-                lines = lines.split("\n")
-
-                self.Dim1 = int(get_info_val(lines, "Dim_1"))
-                self.Dim2 = int(get_info_val(lines, "Dim_2"))
-                self.Tomo = int(get_info_val(lines, "TOMO_N"))
-            except Exception:
-                e = sys.exc_info()[0]
-                _logger.error("fail to get info from %s" % self.infoname)
-                raise e
-            else:
-                return True
-
     def is_data_complete(self):
         self._sync()
 
-        try:
-            limit = True
-            while limit:
-                try:
-                    filno = os.open(self.dataname, os.O_RDONLY)
-                    limit = False
-                except Exception:
-                    if self.quitting is True:
-                        return -1
-                    time.sleep(self.TIME_LOOP_DATA_INCOMPLETE)
+        aux = self.parsing_dir.split(os.path.sep)
+        info_file = os.path.join(
+            self.RootDir, self.parsing_dir, aux[len(aux) - 1] + self.INFO_EXT
+        )
 
-            os.close(filno)
+        if self.srcPattern is None:
+            self.scan_completed = os.path.isfile(info_file)
+        else:
+            infofilenice = info_file.replace(self.srcPattern, self.destPattern, 1)
 
-            old_size = 0
-            datafile1_ready = False
-            while datafile1_ready is False:
-                if self.quitting is True:
-                    return -1
-                time.sleep(self.TIME_LOOP_DATA_INCOMPLETE)
-                mystat = os.stat(self.dataname)
-                fstat_st_size = mystat.st_size
-                if fstat_st_size > old_size:
-                    old_size = fstat_st_size
-                else:
-                    datafile1_ready = True
-
-            edf = EdfFile(self.dataname).Images[0]
-            self.dataType = _DataWatcherProcessParseInfo.get_data_size(edf.DataType)
-            headerLength = edf.DataPosition
-            multif = 1
-            if self.detector == "Dimax":
-                multif = self.Tomo
-
-            deltas = (
-                multif * (self.Dim1 * self.Dim2 * self.dataType + headerLength)
-                - fstat_st_size
+            self.scan_completed = os.path.isfile(infofilenice) or os.path.isfile(
+                info_file
             )
-            if (deltas == 0) and (self.detector == "Dimax"):
-                self.scan_completed = True
-                return 1
 
-            """
-            1st data file has been completed. Wait all data files are present, i.e. check if > nprojections
-            """
-            limit = True
-            while limit:
-                r = glob(self.curdir + "/*")
-                if len(r) > self.Tomo:
-                    limit = False
-                else:
-                    if self.quitting is True:
-                        return -1
-
-                    _logger.info("waiting for more data")
-                    self._sync()
-                    time.sleep(1)
-
-            self.scan_completed = True
-            return 1
-        except Exception as e:
-            _logger.error("fail during is_data_validate")
-            raise e
+        return self.scan_completed
