@@ -1,49 +1,29 @@
-# coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
-
-__authors__ = ["H. Payno"]
-__license__ = "MIT"
-__date__ = "05/07/2017"
-
-
+import fnmatch
+import logging
 import os
 import shutil
-import fnmatch
+from tomwer.core.utils.deprecation import deprecated_warning
 
-from tomwer.core.process.task import Task
-
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.settings import get_lbsram_path, get_dest_path
-from tomwer.core.signal import Signal
-from tomwer.core.utils import logconfig, rebaseParFile
-import logging
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.edfscan import EDFTomoScan
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.io.utils import get_linked_files_with_entry
 import tomwer.version
+from tomwer.core.process.reconstruction.nabu.settings import NABU_CFG_FILE_FOLDER
+from tomwer.core.process.reconstruction.nabu.utils import update_cfg_file_after_transfer
+from tomwer.core.process.task import Task
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.settings import get_dest_path, get_lbsram_path
+from tomwer.core.signal import Signal
+from tomwer.core.utils import logconfig
+from tomwer.core.utils.spec import rebaseParFile
+from tomwer.core.utils.scanutils import data_identifier_to_scan
+from tomwer.io.utils import get_linked_files_with_entry
+from tomwer.core.process.reconstruction.saaxis.saaxis import (
+    DEFAULT_RECONS_FOLDER as MULTI_COR_DEFAULT_FOLDER,
+)
+from tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta import (
+    DEFAULT_RECONS_FOLDER as MULTI_DB_DEFAULT_FOLDER,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -56,7 +36,21 @@ else:
     has_rsync = True
 
 
-class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
+class ScanTransferTask(
+    Task,
+    input_names=("data",),
+    optional_input_names=(
+        "serialize_output_data",
+        "copying",
+        "block",
+        "turn_off_print",
+        "dest_dir",
+        "move",
+        "noRsync",
+        "overwrite",
+    ),
+    output_names=("data",),
+):
     """Manage the copy of scan.
 
     .. warning : the destination directory is find out from the file system
@@ -67,8 +61,16 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
 
     scanready = Signal(TomwerScanBase)
 
-    def __init__(self, varinfo=None, inputs=None, label=None):
-        super().__init__(varinfo=varinfo, inputs=inputs, label=label)
+    def __init__(
+        self, varinfo=None, inputs=None, node_id=None, node_attrs=None, execinfo=None
+    ):
+        super().__init__(
+            varinfo=varinfo,
+            inputs=inputs,
+            node_id=node_id,
+            node_attrs=node_attrs,
+            execinfo=execinfo,
+        )
         self.turn_off_print = inputs.get("turn_off_print", False)
         self._destDir = inputs.get("dest_dir", None)
         """
@@ -78,7 +80,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         self._copying = inputs.get("copying", False)
         force_sync = inputs.get("force_sync", None)
         if force_sync is not None:
-            from silx.utils.deprecation import deprecated_warning
+            from tomwer.core.utils.deprecation import deprecated_warning
 
             deprecated_warning(
                 type_="Parameter",
@@ -89,17 +91,19 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         else:
             force_sync = False
 
-        self._block = inputs.get("block", force_sync)
+        self._block = self.get_input_value("block", force_sync)
 
-        self._move = inputs.get("move", False)
+        self._move = self.get_input_value("move", False)
         if not isinstance(self._move, bool):
             raise TypeError("move is expected to be a boolean")
 
-        self._force = inputs.get("force", False)
-        if not isinstance(self._force, bool):
+        self._overwrite = self.get_input_value("overwrite", False)
+        if not isinstance(self._overwrite, bool):
             raise TypeError("move is expected to be a boolean")
 
-        self._noRsync = inputs.get("noRsync", False)  # TODO: rename noRsync to no_rsync
+        self._noRsync = self.get_input_value(
+            "noRsync", False
+        )  # TODO: rename noRsync to no_rsync
         if not isinstance(self._noRsync, bool):
             raise TypeError("move is expected to be a boolean")
 
@@ -134,7 +138,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
 
         return ""
 
-    def _process_edf_scan(self, scan, move=False, force=True, noRsync=False):
+    def _process_edf_scan(self, scan, move=False, overwrite=True, noRsync=False):
         if not isinstance(scan, EDFTomoScan):
             raise TypeError(f"{scan} is expected to be an instance of {EDFTomoScan}")
         outputdir = self.getDestinationDir(scan.path)
@@ -144,7 +148,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         self._pretransfertOperations(scan.path, outputdir)
         # as we are in the workflow we want this function to be bloking.
         # so we will not used a thread for folder synchronization
-        # for now rsync is not delaing with force option
+        # for now rsync is not delaing with overwrite option
         if not has_rsync or noRsync is True or RSyncManager().has_rsync() is False:
             logger.info("Can't use rsync, copying files")
             try:
@@ -152,18 +156,18 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
                     self._moveFiles(
                         scanPath=scan.path,
                         outputdir=os.path.dirname(outputdir),
-                        force=force,
+                        overwrite=overwrite,
                     )
                 else:
                     self._copyFiles(
-                        scanPath=scan.path, outputdir=outputdir, force=force
+                        scanPath=scan.path, outputdir=outputdir, overwrite=overwrite
                     )
             except shutil.Error as e:
                 raise e
             else:
                 output_scan = scan._deduce_transfert_scan(outputdir)
                 try:
-                    output_scan._update_latest_recons_urls(
+                    output_scan._update_latest_recons_identifiers(
                         old_path=scan.path, new_path=output_scan.path
                     )
                 except Exception as e:
@@ -184,7 +188,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
             self._signalCopying(scanID=source, outputdir=target)
             output_scan = scan._deduce_transfert_scan(outputdir)
             try:
-                output_scan._update_latest_recons_urls(
+                output_scan._update_latest_recons_identifiers(
                     old_path=scan.path, new_path=output_scan.path
                 )
             except Exception as e:
@@ -215,7 +219,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         )
 
     def _get_hdf5_sample_file_or_nx_dst(self, bliss_sample_file):
-        bliss_sample_file = os.path.realpath(bliss_sample_file)
+        bliss_sample_file = os.path.abspath(bliss_sample_file)
         if self._destDir is not None:
             rel_path = os.path.join(*bliss_sample_file.split(os.sep)[-2:])
             return os.path.join(self._destDir, rel_path)
@@ -239,7 +243,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         return self._getOutputDirSpec() or self._getOutputDirLBS(bliss_sample_file)
 
     def _process_hdf5_scan(self, scan) -> TomwerScanBase:
-        assert isinstance(scan, HDF5TomoScan)
+        assert isinstance(scan, NXtomoScan)
         from tomwer.core.process.control.datalistener import DataListener
 
         files_sources = []
@@ -312,7 +316,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
             files_sources.append(scan.master_file)
             files_dest.append(new_nx_file)
             delete_opt.append(True)
-            output_scan = HDF5TomoScan(scan=new_nx_file, entry=scan.entry)
+            output_scan = NXtomoScan(scan=new_nx_file, entry=scan.entry)
         else:
             output_scan = None
 
@@ -368,10 +372,10 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
 
         # manage folders
         patterns = [
-            "nabu_cfg_files",
+            NABU_CFG_FILE_FOLDER,
             "*slice*",
-            "saaxis_results",
-            "sadeltabeta_results",
+            MULTI_COR_DEFAULT_FOLDER,
+            MULTI_DB_DEFAULT_FOLDER,
             "steps_file_basename_nabu_sinogram_save_step",
         ]
         for folder_ in os.listdir(dir_name):
@@ -418,6 +422,10 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
             os.path.normpath(os.path.join(os.path.dirname(dst_nx_file), file_path))
             for file_path in relative_linked_files_src
         ]
+        relative_linked_files_src = [
+            os.path.normpath(os.path.join(os.path.dirname(src_nexus_file), file_path))
+            for file_path in relative_linked_files_src
+        ]
         return dst_nx_file, relative_linked_files_src, relative_linked_files_dst
 
     def run(self):
@@ -426,17 +434,16 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         :param scan: the path to the file we want to move/process
         :type scan: :class:`.TomoBase`
         :param move: if True, directly move the files. Otherwise copy the files
-        :param force: if True then force the copy even if the file/folder already
+        :param overwrite: if True then force the copy even if the file/folder already
             exists
         :param bool noRSync: True if we wan't do sue shutil instead of rsync.
         """
-        scan = self.inputs.data
+        scan = data_identifier_to_scan(self.inputs.data)
         if scan is None:
             raise ValueError("'scan' should be provided")
 
         if scan is None:
             self.outputs.data = None
-        force = self._force
 
         _scan = scan
         if type(_scan) is dict:
@@ -447,9 +454,12 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         logger.info("synchronisation with scanPath")
         if isinstance(scan, EDFTomoScan):
             output_scan = self._process_edf_scan(
-                scan=scan, move=self._move, force=force, noRsync=self._noRsync
+                scan=scan,
+                move=self._move,
+                overwrite=self.get_input_value("overwrite", True),
+                noRsync=self._noRsync,
             )
-        elif isinstance(scan, HDF5TomoScan):
+        elif isinstance(scan, NXtomoScan):
             if self._move is True:
                 raise NotImplementedError("move option not implemented")
             if self._noRsync is True:
@@ -457,7 +467,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
             output_scan = self._process_hdf5_scan(scan=scan)
         else:
             raise TypeError("Other scan than EDF or HDF5 are not managed")
-        if self._return_dict:
+        if self.get_input_value("serialize_output_data", True):
             self.outputs.data = output_scan.to_dict()
         else:
             self.outputs.data = output_scan
@@ -465,6 +475,20 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
     def _pretransfertOperations(self, scanfolder, outputdir):
         """Operation to be run before making the transfert of the scan"""
         self._updateParFiles(scanfolder, outputdir)
+        self._updateNabuConfigFiles(scanfolder, outputdir)
+
+    def _updateNabuConfigFiles(self, scanfolder, outputdir):
+        nabu_cfg_folder = os.path.join(scanfolder, NABU_CFG_FILE_FOLDER)
+        if os.path.exists(nabu_cfg_folder):
+            for _file in os.listdir(nabu_cfg_folder):
+                try:
+                    update_cfg_file_after_transfer(
+                        config_file_path=os.path.join(nabu_cfg_folder, _file),
+                        old_path=scanfolder,
+                        new_path=outputdir,
+                    )
+                except Exception:
+                    pass
 
     def _updateParFiles(self, scanfolder, outputdir):
         """Update all path contained in the .par files to fit the new outpudir"""
@@ -483,7 +507,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
 
         results = {"input_scan": str(input_scan), "output_scan": str(output_scan)}
         entry = "entry"
-        if isinstance(output_scan, HDF5TomoScan):
+        if isinstance(output_scan, NXtomoScan):
             entry = output_scan.entry
         self.register_process(
             process_file=output_scan.process_file,
@@ -493,7 +517,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
             configuration=None,
         )
         logger.processSucceed(
-            "transfer succeed of {} to {}".format(str(input_scan), str(output_scan)),
+            f"transfer succeed of {input_scan} to {output_scan}",
             extra={
                 logconfig.DOC_TITLE: self._scheme_title,
                 logconfig.FROM: str(input_scan),
@@ -509,13 +533,11 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         assert isinstance(output_scan, TomwerScanBase)
         self.scanready.emit(output_scan)
 
-    def _copyFiles(self, scanPath, outputdir, force):
+    def _copyFiles(self, scanPath, outputdir, overwrite):
         """Copying files and removing them"""
         assert type(scanPath) is str
         assert type(outputdir) is str
         assert os.path.isdir(scanPath)
-        #        if force is False:
-        #            assert(os.path.isdir(outputdir))
         # create the destination dir
         if not os.path.isdir(outputdir):
 
@@ -529,7 +551,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         for f in os.listdir(scanPath):
             file = os.path.join(scanPath, f)
             fileDest = os.path.join(outputdir, f)
-            if force is True:
+            if overwrite is True:
                 if os.path.isdir(fileDest):
                     shutil.rmtree(fileDest)
                 if os.path.isfile(fileDest):
@@ -547,11 +569,9 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         info = "sucessfuly removed file at %s !!!" % scanPath
         logger.info(info)
 
-    def _moveFiles(self, scanPath, outputdir, force):
+    def _moveFiles(self, scanPath, outputdir, overwrite):
         """Function simply moving files"""
         assert os.path.isdir(scanPath)
-        if force is False:
-            assert os.path.isdir(outputdir)
 
         logger.debug(
             "synchronisation with scanPath",
@@ -559,7 +579,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         )
 
         target = os.path.join(outputdir, os.path.basename(scanPath))
-        if force is True and os.path.isdir(target):
+        if overwrite is True and os.path.isdir(target):
             shutil.rmtree(target)
         shutil.move(scanPath, outputdir)
 
@@ -583,7 +603,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         else:
             return None
 
-    def getDestinationDir(self, scanPath):
+    def getDestinationDir(self, scanPath, ask_for_output=True):
         """Return the destination directory. The destination directory is the
         root directory"""
         if self._destDir is not None:
@@ -593,7 +613,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         scanIDPath = os.path.abspath(scanPath)
 
         outputdir = self._getOutputDirSpec() or self._getOutputDirLBS(scanIDPath)
-        if outputdir is None:
+        if outputdir is None and ask_for_output:
             outputdir = self._requestFolder()
 
         return outputdir
@@ -601,7 +621,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
     def setDestDir(self, dist):
         """Force the outpudir to dist.
 
-        :param str dist: path to the folder. If None remove force behavior
+        :param str dist: path to the folder. If None remove overwrite behavior
         """
         self._destDir = dist
         if self._destDir is not None and os.path.isdir(self._destDir):
@@ -618,7 +638,7 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
             print("### copying files ", scanID, " to ", outputdir)
             print("### ...")
 
-        info = "start moving folder from %s to %s" % (scanID, outputdir)
+        info = f"start moving folder from {scanID} to {outputdir}"
         logger.processStarted(info, extra={logconfig.DOC_TITLE: self._scheme_title})
 
     def _signalCopyFailed(self):
@@ -660,3 +680,17 @@ class ScanTransfer(Task, input_names=("data",), output_names=("data",)):
         :return:
         """
         return os.path.join(self.getDestinationDir(scan), os.path.basename(scan))
+
+
+class ScanTransfer(ScanTransferTask):
+    def __init__(
+        self, varinfo=None, inputs=None, node_id=None, node_attrs=None, execinfo=None
+    ):
+        deprecated_warning(
+            name="tomwer.core.process.control.scantransfer.ScanTransfer",
+            type_="class",
+            reason="improve readibility",
+            since_version="1.2",
+            replacement="ScanTransferTask",
+        )
+        super().__init__(varinfo, inputs, node_id, node_attrs, execinfo)
