@@ -27,23 +27,25 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "18/06/2017"
 
-from collections import OrderedDict
-from tomwer.core.signal import Signal
-from tomwer.core import settings
-from tomwer.core import utils
-from tomwer.core.process.task import BaseProcessInfo
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.utils import logconfig
 import logging
-from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+
 from ewokscore.task import Task as EwoksTask
 
+from tomwer.core import settings
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.core.process.task import BaseProcessInfo
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.signal import Signal
+from tomwer.core.utils import logconfig
+from tomwer.core.utils.scanutils import data_identifier_to_scan
+
 logger = logging.getLogger(__name__)
 
 
 class _ScanValidatorPlaceHolder(EwoksTask, input_names=["data"], output_names=["data"]):
     def run(self):
-        self.outputs.data = self.inputs.data
+        self.outputs.data = data_identifier_to_scan(self.inputs.data)
 
 
 class ScanValidator(BaseProcessInfo):
@@ -56,7 +58,6 @@ class ScanValidator(BaseProcessInfo):
 
     def __init__(self, memoryReleaser):
         super().__init__()
-        self._scansToValidate = OrderedDict()
         self._manualValidation = True
         self._hasToLimitScanBlock = settings.isOnLbsram()
         self._memoryReleaser = memoryReleaser
@@ -74,13 +75,9 @@ class ScanValidator(BaseProcessInfo):
     def _clearMemoryReleaser(self):
         if self._memoryReleaser is not None:
             self._memoryReleaser.should_be_stopped = True
-            self._memoryReleaser.wait()
+            self._memoryReleaser.wait(4000)
             self._memoryReleaser = None
 
-    @property
-    def lastReceivedRecons(self):
-        return list(self._scansToValidate.values())[0]
-
     def addScan(self, scan):
         """
         Return the index on the current orderred dict
@@ -91,22 +88,18 @@ class ScanValidator(BaseProcessInfo):
         _ftserie = scan
         if type(scan) is str:
             _ftserie = ScanFactory.create_scan_object(_ftserie)
-        info = "Scan %s has been added by the Scan validator" % _ftserie
-        logger.info(info)
+        logger.info(f"Scan {_ftserie} has been added by the Scan validator")
 
-        _ftserie = scan
-        if type(scan) is str:
-            _ftserie = ScanFactory.create_scan_object(_ftserie)
-        self._scansToValidate[str(_ftserie)] = _ftserie
+        self._scans[str(_ftserie)] = _ftserie
 
-        index = len(self._scansToValidate) - 1
+        index = len(self._scans) - 1
 
         self._freeStackIfNeeded()
         return index
 
     def _freeStackIfNeeded(self):
         # if we are low in memory in lbsram: we will automatically validate the current scan
-        isLowMemoryLbs = utils.isLowOnMemory(settings.get_lbsram_path()) is True
+        isLowMemoryLbs = is_low_on_memory(settings.get_lbsram_path()) is True
         if not self.isValidationManual():
             self._validateStack()
         elif isLowMemoryLbs:
@@ -131,8 +124,8 @@ class ScanValidator(BaseProcessInfo):
                        located on lbsram.
         Validate all the scans in the stack.
         """
-        for scanID in list(self._scansToValidate.keys()):
-            scan = self._scansToValidate[scanID]
+        for scanID in list(self._scans.keys()):
+            scan = self._scans[scanID]
             if filter_ is None or settings.isOnLbsram(scan):
                 self._validated(scan)
 
@@ -178,7 +171,7 @@ class ScanValidator(BaseProcessInfo):
         """Callback when the validate button is pushed"""
         if scan is not None:
             assert isinstance(scan, TomwerScanBase)
-            info = "%s has been validated" % str(scan)
+            info = f"{str(scan)} has been validated"
             logger.processEnded(
                 info,
                 extra={
@@ -187,10 +180,8 @@ class ScanValidator(BaseProcessInfo):
                 },
             )
             self._sendScanReady(scan)
-            # in some case the scan can not be in the _scansToValidate
+            # in some case the scan can not be in the _scans
             # (if the signal) come from an other window that 'scan to treat'
-            if str(scan) in self._scansToValidate:
-                del self._scansToValidate[str(scan)]
             if str(scan) in self._scans:
                 del self._scans[str(scan)]
 
@@ -207,11 +198,9 @@ class ScanValidator(BaseProcessInfo):
                 },
             )
             self._sendScanCanceledAt(scan)
-            # in some case the scan can not be in the _scansToValidate
+            # in some case the scan can not be in the _scans
             # (if the signal) come from an other window that 'scan to treat'
-            if str(scan) in self._scansToValidate:
-                del self._scansToValidate[str(scan)]
-            if str(scan) in self._scansToValidate:
+            if str(scan) in self._scans:
                 del self._scans[str(scan)]
             self.clear()
 
@@ -228,8 +217,8 @@ class ScanValidator(BaseProcessInfo):
         if type(ftserie) is str:
             _ftserie = ScanFactory.create_scan_object(_ftserie)
 
-        if _ftserie.path in self._scansToValidate:
-            del self._scansToValidate[str(_ftserie)]
+        if _ftserie.path in self._scans:
+            del self._scans[str(_ftserie)]
         self._sendUpdateReconsParam(_TomwerBaseDock(tomo_instance=_ftserie))
 
     def setManualValidation(self, b):
@@ -251,19 +240,20 @@ class ScanValidator(BaseProcessInfo):
         """
         return self._manualValidation
 
-    def _sendScanReady(self):
+    def _sendScanReady(self, scan):
         raise RuntimeError("ScanValidator is a pure virtual class.")
 
-    def _sendScanCanceledAt(self):
+    def _sendScanCanceledAt(self, scan):
         raise RuntimeError("ScanValidator is a pure virtual class.")
 
-    def _sendUpdateReconsParam(self):
+    def _sendUpdateReconsParam(self, scan):
         raise RuntimeError("ScanValidator is a pure virtual class.")
 
     def clear(self):
-        scans = self._scans.values()
+        scans = list(self._scans.values())
+        # need to be copy because cancel will update self._scans
         for scan in scans:
-            self._cancelScan(scan=scans[scan])
+            self._cancelScan(scan=scan)
 
 
 class ScanValidatorP(ScanValidator):
