@@ -29,38 +29,44 @@ __authors__ = ["C.Nemoz", "H.Payno"]
 __license__ = "MIT"
 __date__ = "19/03/2019"
 
-from .params import (
-    AxisRP,
-    DEFAULT_CMP_TAKE_LOG,
-    DEFAULT_CMP_OVERSAMPLING,
-    DEFAULT_CMP_SUBSAMPLING_Y,
-    DEFAULT_CMP_THETA,
-)
-from .mode import AxisMode
-from .projectiontype import ProjectionType
-from .anglemode import CorAngleMode
-from tomwer.core.process.task import Task
-from processview.core.superviseprocess import SuperviseProcess
-from tomwer.core.scan.scanbase import TomwerScanBase
-from processview.core.manager import ProcessManager
-from processview.core.manager import DatasetState
-from tomwer.core.scan.scanfactory import ScanFactory
 import logging
-from tomwer.core.utils import logconfig
+from typing import Optional, Union
+
 import numpy
-from typing import Union
-from tomwer.utils import docstring
-from tomwer.core.utils import image
-import tomwer.version
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.process.reconstruction.nabu.utils import check_sinogram_half
 from nabu.estimation.cor import (
     CenterOfRotation,
     CenterOfRotationAdaptiveSearch,
     CenterOfRotationGrowingWindow,
     CenterOfRotationSlidingWindow,
+    CenterOfRotationOctaveAccurate,
 )
-from nabu.pipeline.estimators import SinoCORFinder
+from nabu.pipeline.estimators import SinoCORFinder, CORFinder
+from nabu.resources.nxflatfield import update_dataset_info_flats_darks
+from processview.core.manager import DatasetState, ProcessManager
+from processview.core.superviseprocess import SuperviseProcess
+from tomwer.core.utils.deprecation import deprecated_warning
+
+import tomwer.version
+from tomwer.core.process.reconstruction.utils.cor import absolute_pos_to_relative
+from tomwer.core.process.task import Task
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils import image, logconfig
+from tomwer.core.utils.scanutils import data_identifier_to_scan
+from tomwer.utils import docstring
+
+from .mode import AxisMode
+from .params import (
+    DEFAULT_CMP_N_SUBSAMPLING_Y,
+    DEFAULT_CMP_NEAR_POS,
+    DEFAULT_CMP_NEAR_WIDTH,
+    DEFAULT_CMP_OVERSAMPLING,
+    DEFAULT_CMP_TAKE_LOG,
+    DEFAULT_CMP_THETA,
+    AxisRP,
+)
+from .projectiontype import ProjectionType
 
 try:
     from nabu.pipeline.estimators import CompositeCOREstimator
@@ -69,9 +75,8 @@ except ImportError:
 else:
     has_composite_cor_finder = True
 from silx.io.url import DataUrl
-from tomoscan.io import HDF5File
 from silx.io.utils import h5py_read_dataset
-import nabu
+from silx.io.utils import open as open_hdf5
 
 _logger = logging.getLogger(__name__)
 if not has_composite_cor_finder:
@@ -84,24 +89,47 @@ pc_height = 10.0 / 100.0
 pc_width = 50.0 / 100.0
 
 
+def _absolute_pos_to_relative_with_warning(
+    absolute_pos: float, det_width: Optional[int]
+):
+    """
+    nabu returns the value as absolute. tomwer needs it as relative
+    Also handle the case (unlikely) the detector width cannot be found
+    """
+    if det_width is None:
+        det_width = 2048
+        _logger.warning("unable to find image width. Set width to 2048")
+    else:
+        det_width = det_width
+    return absolute_pos_to_relative(absolute_pos=absolute_pos, det_width=det_width)
+
+
 def adapt_tomwer_scan_to_nabu(scan: TomwerScanBase):
     """simple util to convert tomwer scan to a nabu DataAnalizer and
     updating infos regarding flat and dark if needed
     """
     dataset_infos = scan.to_nabu_dataset_analyser()
-    if nabu.version > "2021.1.0" and isinstance(scan, HDF5TomoScan):
-        # since this version nabu does not the load of dark / flat mean/median
-        # directly in the function but outside. So we need an extra call to
-        # update_dataset_info_flats_darks
-        from nabu.resources.nxflatfield import update_dataset_info_flats_darks
+    if isinstance(scan, NXtomoScan):
+        try:
+            update_dataset_info_flats_darks(
+                dataset_infos,
+                flatfield_mode=None,
+            )
+        except ValueError as exception:
+            # nabu raise an error if no darks / flats set. But this can make sense at this stage if the NXtomo has no
+            # raw dark / flat and is already normalized. In this case only fire a warning
+            if (
+                scan.reduced_darks is not None
+                and len(scan.reduced_darks) > 0
+                and scan.reduced_flats is not None
+                and len(scan.reduced_flats) > 0
+            ):
+                raise exception
+            else:
+                _logger.warning(
+                    "Fail to update nabu dataset info flats and darks. Expected if the dataset contains already normalized projections"
+                )
 
-        update_dataset_info_flats_darks(
-            dataset_infos,
-            processes_file=scan.get_relative_file(
-                "nabu_processes.h5", with_dataset_prefix=True
-            ),
-            flatfield_mode=None,
-        )
     return dataset_infos
 
 
@@ -127,7 +155,7 @@ def compute_cor_nabu_growing_window(
     """
     nabu_class = CenterOfRotationGrowingWindow(horz_fft_width=horz_fft_width)
     # value return is relative
-    return nabu_class.find_shift(
+    res = nabu_class.find_shift(
         img_1=radio_1,
         img_2=numpy.fliplr(radio_2) if flip_frame_2_lr else radio_2,
         side=side,
@@ -135,6 +163,59 @@ def compute_cor_nabu_growing_window(
         padding_mode=padding_mode,
         median_filt_shape=None,
     )
+    if isinstance(res, numpy.ndarray):
+        if len(res) == 1:
+            return res[0]
+        else:
+            raise ValueError(
+                "nabu rsult is expected to be a scalar, numpy array found. Please upgrade nabu this issue is expected to be solved"
+            )
+    else:
+        return res
+
+
+def compute_cor_nabu_growing_window_radios(
+    scan: TomwerScanBase,
+):
+    """
+    Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
+
+    :param TomwerScanBase scan:
+
+    :return:
+    """
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    projection_angles = scan.get_proj_angle_url()
+    projection_angles_i = {
+        value.path(): key for key, value in projection_angles.items()
+    }
+    url_radio_1, url_radio_2 = AxisTask.get_inputs_urls(scan=scan)
+    angle_radio_1 = float(projection_angles_i[url_radio_1.url.path()])
+    angle_radio_2 = float(projection_angles_i[url_radio_2.url.path()])
+    radio_angles = tuple(numpy.radians((angle_radio_1, angle_radio_2)))
+
+    corfinder = CORFinder(
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="growing-window",
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
+        radio_angles=radio_angles,
+        logger=_logger,
+    )
+    res = corfinder.find_cor()  # Returns absolute cor
+    if isinstance(res, numpy.ndarray):
+        if len(res) == 1:
+            res = res[0]
+        else:
+            raise ValueError(
+                "nabu rsult is expected to be a scalar, numpy array found. Please upgrade nabu this issue is expected to be solved"
+            )
+
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def compute_cor_nabu_growing_window_sinogram(
@@ -144,27 +225,33 @@ def compute_cor_nabu_growing_window_sinogram(
     Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
 
     :param TomwerScanBase scan:
-    :raises: ValueError if half acquisition with an odd number of projections
 
     :return:
     """
-    check_sinogram_half(scan)
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
     corfinder = SinoCORFinder(
-        adapt_tomwer_scan_to_nabu(scan),
-        slice_idx=scan.axis_params.sinogram_line,
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="sino-growing-window",
+        slice_idx=scan.axis_params.sinogram_line or "middle",
         subsampling=scan.axis_params.sinogram_subsampling,
-        do_flatfield=True,
-        cor_options=scan.axis_params.get_nabu_cor_options(),
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
         logger=_logger,
     )
-    res = corfinder.find_cor(method=AxisMode.growing_window.value)
-    # value return is absolute
-    if scan.dim_1 is None:
-        _logger.warning("unagle to find image width. Set half width to 1024")
-        res = res - 1024
-    else:
-        res = res - scan.dim_1 // 2
-    return res
+    res = corfinder.find_cor()
+    if isinstance(res, numpy.ndarray):
+        if len(res) == 1:
+            res = res[0]
+        else:
+            raise ValueError(
+                "nabu rsult is expected to be a scalar, numpy array found. Please upgrade nabu this issue is expected to be solved"
+            )
+
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def compute_scan_sino_coarse_to_fine(scan):
@@ -172,26 +259,24 @@ def compute_scan_sino_coarse_to_fine(scan):
     Compute center of rotation from `sino-coarse-to-fine` algorithm for given
     scan
     :param scan:
-    :raises: ValueError if half acquisition with an odd number of projections
     :return:
     """
-    check_sinogram_half(scan)
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
     corfinder = SinoCORFinder(
-        adapt_tomwer_scan_to_nabu(scan),
-        slice_idx=scan.axis_params.sinogram_line,
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method=AxisMode.sino_coarse_to_fine.value,
+        slice_idx=scan.axis_params.sinogram_line or "middle",
         subsampling=scan.axis_params.sinogram_subsampling,
-        do_flatfield=True,
-        cor_options=scan.axis_params.get_nabu_cor_options(),
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
         logger=_logger,
     )
-    res = corfinder.find_cor(method=AxisMode.sino_coarse_to_fine.value)
-    # value return is absolute
-    if scan.dim_1 is None:
-        _logger.warning("unagle to find image width. Set half width to 1024")
-        res = res - 1024
-    else:
-        res = res - scan.dim_1 // 2
-    return res
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def compute_scan_composite_coarse_to_fine(scan: TomwerScanBase):
@@ -199,44 +284,59 @@ def compute_scan_composite_coarse_to_fine(scan: TomwerScanBase):
     Compute center of rotation from `sino-coarse-to-fine` algorithm for given
     scan
     :param scan:
-    :raises: ValueError if half acquisition with an odd number of projections
     :return:
     """
     if not has_composite_cor_finder:
         _logger.error("unable to find nabu CompositeCOREstimator")
         return None
 
-    check_sinogram_half(scan)
-    theta, subsampling_y, oversampling, take_log = get_composite_options(scan)
+    (
+        theta,
+        n_subsampling_y,
+        oversampling,
+        take_log,
+        near_pos,
+        near_width,
+    ) = get_composite_options(scan)
+
+    # as the new corfinder is not yet merged in the main branch
+    # allow some tolerance for the "side" argument that is there only
+    # in the new one
+
+    cor_options = scan.axis_params.get_nabu_cor_options_as_dict()
+    for key in "low_pass", "high_pass":
+        if key in cor_options:
+            cor_options[key] = int(cor_options[key])
     corfinder = CompositeCOREstimator(
-        adapt_tomwer_scan_to_nabu(scan),
-        theta=theta,
-        subsampling_y=subsampling_y,
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        theta_interval=theta,
+        n_subsampling_y=n_subsampling_y,
         oversampling=oversampling,
-        cor_options=scan.axis_params.get_nabu_cor_options(),
+        cor_options=cor_options,
         logger=_logger,
         take_log=take_log,
     )
-    res = corfinder.find_cor(method=AxisMode.composite_coarse_to_fine.value)
-    # value return is absolute
-    if scan.dim_1 is None:
-        _logger.warning("unagle to find image width. Set half width to 1024")
-        res = res - 1024
-    else:
-        res = res - scan.dim_1 // 2
-    return res
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def get_composite_options(scan):
     theta = scan.axis_params.composite_options.get("theta", DEFAULT_CMP_THETA)
-    subsampling_y = scan.axis_params.composite_options.get(
-        "subsampling_y", DEFAULT_CMP_SUBSAMPLING_Y
+    n_subsampling_y = scan.axis_params.composite_options.get(
+        "n_subsampling_y", DEFAULT_CMP_N_SUBSAMPLING_Y
     )
     oversampling = scan.axis_params.composite_options.get(
         "oversampling", DEFAULT_CMP_OVERSAMPLING
     )
     take_log = scan.axis_params.composite_options.get("take_log", DEFAULT_CMP_TAKE_LOG)
-    return theta, subsampling_y, oversampling, take_log
+
+    near_pos = scan.axis_params.composite_options.get("near_pos", DEFAULT_CMP_NEAR_POS)
+    near_width = scan.axis_params.composite_options.get(
+        "near_width", DEFAULT_CMP_NEAR_WIDTH
+    )
+    return theta, n_subsampling_y, oversampling, take_log, near_pos, near_width
 
 
 def compute_scan_cor_nabu_growing_window(scan):
@@ -245,11 +345,10 @@ def compute_scan_cor_nabu_growing_window(scan):
     value.
 
     :param `.TomoBase` scan: scan to process
-    :raises: ValueError if half acquisition with an odd number of projections
     :return: Union[float, None]
     """
     if not scan.axis_params.use_sinogram:
-        radio_1, radio_2 = AxisProcess.get_inputs(scan=scan)
+        radio_1, radio_2 = AxisTask.get_inputs(scan=scan)
         if radio_1 is None or radio_2 is None:
             raise NoAxisUrl("Unable to find projections for nabu axis calculation")
     else:
@@ -267,13 +366,7 @@ def compute_scan_cor_nabu_growing_window(scan):
     if scan.axis_params.use_sinogram:
         return compute_cor_nabu_growing_window_sinogram(scan=scan)
     else:
-        return compute_cor_nabu_growing_window(
-            radio_1=radio_1.copy(),
-            radio_2=radio_2.copy(),
-            side=scan.axis_params.side,
-            padding_mode=scan.axis_params.padding_mode,
-            flip_frame_2_lr=scan.axis_params.flip_lr,
-        )
+        return compute_cor_nabu_growing_window_radios(scan=scan)
 
 
 def compute_cor_nabu_sliding_window(
@@ -301,7 +394,7 @@ def compute_cor_nabu_sliding_window(
     :return:
     """
     nabu_class = CenterOfRotationSlidingWindow(horz_fft_width=horz_fft_width)
-    return nabu_class.find_shift(
+    res = nabu_class.find_shift(
         img_1=radio_1,
         img_2=numpy.fliplr(radio_2) if flip_frame_2_lr else radio_2,
         side=side,
@@ -309,6 +402,43 @@ def compute_cor_nabu_sliding_window(
         padding_mode=padding_mode,
         median_filt_shape=None,
     )
+    return res
+
+
+def compute_cor_nabu_sliding_window_radios(
+    scan,
+):
+    """
+    Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
+
+    :param TomwerScanBase scan:
+
+    :return:
+    """
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    projection_angles = scan.get_proj_angle_url()
+    projection_angles_i = {
+        value.path(): key for key, value in projection_angles.items()
+    }
+    url_radio_1, url_radio_2 = AxisTask.get_inputs_urls(scan=scan)
+    angle_radio_1 = float(projection_angles_i[url_radio_1.url.path()])
+    angle_radio_2 = float(projection_angles_i[url_radio_2.url.path()])
+    radio_angles = tuple(numpy.radians((angle_radio_1, angle_radio_2)))
+
+    corfinder = CORFinder(
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="sliding-window",
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
+        radio_angles=radio_angles,
+        logger=_logger,
+    )
+    res = corfinder.find_cor()  # Returns absolute cor.
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def compute_cor_nabu_sliding_window_sinogram(
@@ -318,27 +448,25 @@ def compute_cor_nabu_sliding_window_sinogram(
     Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
 
     :param TomwerScanBase scan:
-    :raises: ValueError if half acquisition with an odd number of projections
 
     :return:
     """
-    check_sinogram_half(scan)
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
     corfinder = SinoCORFinder(
-        adapt_tomwer_scan_to_nabu(scan),
-        slice_idx=scan.axis_params.sinogram_line,
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="sino-sliding-window",
+        slice_idx=scan.axis_params.sinogram_line or "middle",
         subsampling=scan.axis_params.sinogram_subsampling,
-        do_flatfield=True,
-        cor_options=scan.axis_params.get_nabu_cor_options(),
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
         logger=_logger,
     )
-    res = corfinder.find_cor(method=AxisMode.sliding_window.value)
-    # value return is absolute
-    if scan.dim_1 is None:
-        _logger.warning("unagle to find image width. Set half width to 1024")
-        res = res - 1024
-    else:
-        res = res - scan.dim_1 // 2
-    return res
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def compute_scan_cor_nabu_sliding_window(scan):
@@ -347,11 +475,10 @@ def compute_scan_cor_nabu_sliding_window(scan):
     value.
 
     :param `.TomoBase` scan: scan to process
-    :raises: ValueError if half acquisition with an odd number of projections
     :return: Union[float, None]
     """
     if not scan.axis_params.use_sinogram:
-        radio_1, radio_2 = AxisProcess.get_inputs(scan=scan)
+        radio_1, radio_2 = AxisTask.get_inputs(scan=scan)
         if radio_1 is None or radio_2 is None:
             raise NoAxisUrl("Unable to find projections for nabu axis calculation")
     else:
@@ -369,13 +496,129 @@ def compute_scan_cor_nabu_sliding_window(scan):
     if scan.axis_params.use_sinogram:
         return compute_cor_nabu_sliding_window_sinogram(scan=scan)
     else:
-        return compute_cor_nabu_sliding_window(
-            radio_1=radio_1.copy(),
-            radio_2=radio_2.copy(),
-            side=scan.axis_params.side,
-            padding_mode=scan.axis_params.padding_mode,
-            flip_frame_2_lr=scan.axis_params.flip_lr,
-        )
+        return compute_cor_nabu_sliding_window_radios(scan=scan)
+
+
+def compute_scan_fourier_angles(scan):
+    """
+    run 'scan_fourier_angles' algorithm for the requested scan
+    """
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    corfinder = SinoCORFinder(
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="fourier-angles",
+        slice_idx=scan.axis_params.sinogram_line or "middle",
+        subsampling=scan.axis_params.sinogram_subsampling,
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
+        logger=_logger,
+    )
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
+
+
+def compute_scan_octave_accurate_radios(
+    scan,
+):
+    """
+    Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
+
+    :param TomwerScanBase scan:
+
+    :return:
+    """
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    projection_angles = scan.get_proj_angle_url()
+    projection_angles_i = {
+        value.path(): key for key, value in projection_angles.items()
+    }
+    url_radio_1, url_radio_2 = AxisTask.get_inputs_urls(scan=scan)
+    angle_radio_1 = float(projection_angles_i[url_radio_1.url.path()])
+    angle_radio_2 = float(projection_angles_i[url_radio_2.url.path()])
+    radio_angles = tuple(numpy.radians((angle_radio_1, angle_radio_2)))
+
+    corfinder = CORFinder(
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="octave-accurate",
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
+        radio_angles=radio_angles,
+        logger=_logger,
+    )
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
+
+
+def compute_scan_octave_accurate(scan):
+    """
+    Compute center of rotation from `octave-accurate` algorithm
+    scan
+    :param scan:
+    :return:
+    """
+    cor_options = scan.axis_params.get_nabu_cor_options_as_dict()
+    radio_1, radio_2 = AxisTask.get_inputs(scan=scan)
+    extra_options = {}
+    for key in "low_pass", "high_pass":
+        if key in cor_options:
+            extra_options[key] = float(cor_options[key])
+
+    corfinder = CenterOfRotationOctaveAccurate(cor_options=cor_options)
+    res = corfinder.find_shift(
+        img_1=radio_1,
+        img_2=numpy.fliplr(radio_2) if scan.axis_params.flip_lr else radio_2,
+        side=scan.axis_params.side,
+        padding_mode=scan.axis_params.padding_mode,
+        **extra_options,
+    )
+    return res
+
+
+def compute_cor_nabu_centered_radios(
+    scan,
+):
+    """
+    Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
+
+    :param TomwerScanBase scan:
+
+    :return:
+    """
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    projection_angles = scan.get_proj_angle_url()
+    projection_angles_i = {
+        value.path(): key for key, value in projection_angles.items()
+    }
+    url_radio_1, url_radio_2 = AxisTask.get_inputs_urls(scan=scan)
+    angle_radio_1 = float(projection_angles_i[url_radio_1.url.path()])
+    angle_radio_2 = float(projection_angles_i[url_radio_2.url.path()])
+    radio_angles = tuple(numpy.radians((angle_radio_1, angle_radio_2)))
+
+    corfinder = CORFinder(
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="centered",
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
+        radio_angles=radio_angles,
+        logger=_logger,
+    )
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
 
 
 def compute_cor_nabu_centered(
@@ -384,6 +627,7 @@ def compute_cor_nabu_centered(
     padding_mode,
     flip_frame_2_lr=True,
     horz_fft_width=False,
+    vert_fft_width=False,
 ):
     """
     Call nabu.preproc.alignement.CenterOfRotation.find_shift
@@ -400,7 +644,9 @@ def compute_cor_nabu_centered(
 
     :return:
     """
-    nabu_class = CenterOfRotation(horz_fft_width=horz_fft_width)
+    nabu_class = CenterOfRotation(
+        horz_fft_width=horz_fft_width, vert_fft_width=vert_fft_width
+    )
     return nabu_class.find_shift(
         img_1=radio_1,
         img_2=numpy.fliplr(radio_2) if flip_frame_2_lr else radio_2,
@@ -420,7 +666,7 @@ def compute_scan_cor_nabu_centered(scan):
 
     """
     assert scan.axis_params is not None
-    radio_1, radio_2 = AxisProcess.get_inputs(scan=scan)
+    radio_1, radio_2 = AxisTask.get_inputs(scan=scan)
     if radio_1 is None or radio_2 is None:
         raise NoAxisUrl("Unable to find projections for nabu axis calculation")
 
@@ -429,12 +675,7 @@ def compute_scan_cor_nabu_centered(scan):
         "mode %s" % scan.axis_params.padding_mode
     )
 
-    return compute_cor_nabu_centered(
-        radio_1=radio_1.copy(),
-        radio_2=radio_2.copy(),
-        padding_mode=scan.axis_params.padding_mode,
-        flip_frame_2_lr=scan.axis_params.flip_lr,
-    )
+    return compute_cor_nabu_centered_radios(scan)
 
 
 def compute_cor_nabu_global(
@@ -464,6 +705,42 @@ def compute_cor_nabu_global(
     )
 
 
+def compute_cor_nabu_global_radios(
+    scan: TomwerScanBase,
+):
+    """
+    Call nabu.preproc.alignement.CenterOfRotationGrowingWindow.find_shift
+
+    :param TomwerScanBase scan:
+
+    :return:
+    """
+    has_darks = scan.reduced_darks is not None and len(scan.reduced_darks) > 0
+    has_flats = scan.reduced_flats is not None and len(scan.reduced_flats) > 0
+
+    projection_angles = scan.get_proj_angle_url()
+    projection_angles_i = {
+        value.path(): key for key, value in projection_angles.items()
+    }
+    url_radio_1, url_radio_2 = AxisTask.get_inputs_urls(scan=scan)
+    angle_radio_1 = float(projection_angles_i[url_radio_1.url.path()])
+    angle_radio_2 = float(projection_angles_i[url_radio_2.url.path()])
+    radio_angles = tuple(numpy.radians((angle_radio_1, angle_radio_2)))
+
+    corfinder = CORFinder(
+        dataset_info=adapt_tomwer_scan_to_nabu(scan),
+        method="global",
+        do_flatfield=has_darks and has_flats,
+        cor_options=scan.axis_params.get_nabu_cor_options_as_dict(),
+        radio_angles=radio_angles,
+        logger=_logger,
+    )
+    res = corfinder.find_cor()
+    return _absolute_pos_to_relative_with_warning(
+        absolute_pos=res, det_width=scan.dim_1
+    )
+
+
 def compute_scan_cor_nabu_global(scan):
     """
     Call to nabu.preproc.alignment.CenterOfRotation from the scan axis_params
@@ -474,7 +751,7 @@ def compute_scan_cor_nabu_global(scan):
 
     """
     assert scan.axis_params is not None
-    radio_1, radio_2 = AxisProcess.get_inputs(scan=scan)
+    radio_1, radio_2 = AxisTask.get_inputs(scan=scan)
     if radio_1 is None or radio_2 is None:
         raise NoAxisUrl("Unable to find projections for nabu axis calculation")
 
@@ -482,13 +759,7 @@ def compute_scan_cor_nabu_global(scan):
         "compute scan axis from nabu CenterOfRotation with padding "
         "mode %s" % scan.axis_params.padding_mode
     )
-
-    return compute_cor_nabu_global(
-        radio_1=radio_1.copy(),
-        radio_2=radio_2.copy(),
-        padding_mode=scan.axis_params.padding_mode,
-        flip_frame_2_lr=scan.axis_params.flip_lr,
-    )
+    return compute_cor_nabu_global_radios(scan)
 
 
 def get_stdmax_column(x: numpy.ndarray) -> float:
@@ -512,8 +783,12 @@ class NoAxisUrl(Exception):
     pass
 
 
-class AxisProcess(
-    Task, SuperviseProcess, input_names=("data",), output_names=("data",)
+class AxisTask(
+    Task,
+    SuperviseProcess,
+    input_names=("data",),
+    output_names=("data",),
+    optional_input_names=("serialize_output_data",),
 ):
     """
     Process used to compute the center of rotation of a scan
@@ -525,14 +800,34 @@ class AxisProcess(
     _CALCULATIONS_METHODS = {
         AxisMode.centered: compute_scan_cor_nabu_centered,
         AxisMode.global_: compute_scan_cor_nabu_global,
-        AxisMode.sliding_window: compute_scan_cor_nabu_sliding_window,
-        AxisMode.growing_window: compute_scan_cor_nabu_growing_window,
+        AxisMode.sliding_window_sinogram: compute_scan_cor_nabu_sliding_window,
+        AxisMode.sliding_window_radios: compute_scan_cor_nabu_sliding_window,
+        AxisMode.growing_window_sinogram: compute_scan_cor_nabu_growing_window,
+        AxisMode.growing_window_radios: compute_scan_cor_nabu_growing_window,
         AxisMode.sino_coarse_to_fine: compute_scan_sino_coarse_to_fine,
         AxisMode.composite_coarse_to_fine: compute_scan_composite_coarse_to_fine,
+        AxisMode.near: compute_scan_composite_coarse_to_fine,
+        AxisMode.sino_fourier_angles: compute_scan_fourier_angles,
+        AxisMode.octave_accurate_radios: compute_scan_octave_accurate,
     }
 
-    def __init__(self, process_id=None, varinfo=None, inputs=None, label=None):
-        Task.__init__(self, varinfo=varinfo, inputs=inputs, label=label)
+    def __init__(
+        self,
+        process_id=None,
+        varinfo=None,
+        inputs=None,
+        node_id=None,
+        node_attrs=None,
+        execinfo=None,
+    ):
+        Task.__init__(
+            self,
+            varinfo=varinfo,
+            inputs=inputs,
+            node_id=node_id,
+            node_attrs=node_attrs,
+            execinfo=execinfo,
+        )
         if "recons_params" in inputs:
             raise KeyError("do not use 'recons_params' but use 'axis_params' instead")
         axis_params = inputs.get("axis_params", None)
@@ -556,9 +851,11 @@ class AxisProcess(
         """Recons parameters register before locking the position"""
 
     def set_configuration(self, configuration):
-        self.set_recons_params(
-            recons_params=AxisRP.from_dict(configuration["_rpSetting"])
-        )
+        if "_rpSetting" in configuration:
+            recons_params = AxisRP.from_dict(configuration["_rpSetting"])
+        else:
+            recons_params = AxisRP.from_dict(configuration)
+        self.set_recons_params(recons_params=recons_params)
 
     def set_recons_params(self, recons_params):
         assert isinstance(recons_params, AxisRP)
@@ -590,8 +887,9 @@ class AxisProcess(
         .. note:: this simply call `compute`.
                   But this is needed for the AxisProcessThreaded class
         """
-        scan = self.inputs.data
+        scan = data_identifier_to_scan(self.inputs.data)
         if scan is None:
+            self.outputs.data = None
             return
 
         if isinstance(scan, TomwerScanBase):
@@ -599,7 +897,7 @@ class AxisProcess(
         elif isinstance(scan, dict):
             scan = ScanFactory.create_scan_object_frm_dict(scan)
         else:
-            raise ValueError("input type {} is not managed".format(scan))
+            raise ValueError(f"input type {scan} is not managed")
 
         _logger.info("start axis calculation for %s" % scan.path)
         self._axis_params.frame_width = scan.dim_1
@@ -607,7 +905,6 @@ class AxisProcess(
         try:
             scan_res = self.compute(scan=scan)
         except Exception as e:
-            raise e
             scan_res = None
             error = e
         else:
@@ -627,31 +924,26 @@ class AxisProcess(
         finally:
             if cor != "...":
                 self._process_end(scan, cor=cor, error=error)
-        self.outputs.data = scan_res
 
-    def reprocess(self, scan):
-        """
-        Reprocess the scan. For now simply call process
-
-        :param scan:
-        """
-        return self.process(scan=scan)
+        if self.get_input_value("serialize_output_data", True):
+            self.outputs.data = scan_res.to_dict()
+        else:
+            self.outputs.data = scan_res
 
     def _process_end(self, scan, cor, error=None):
         assert isinstance(scan, TomwerScanBase)
         if scan.process_file is not None:
             entry = "entry"
-            if isinstance(scan, HDF5TomoScan):
+            if isinstance(scan, NXtomoScan):
                 entry = scan.entry
-            with scan.acquire_process_file_lock():
-                self.register_process(
-                    process_file=scan.process_file,
-                    entry=entry,
-                    results={"center_of_rotation": cor if cor is not None else "-"},
-                    configuration=self._axis_params.to_dict(),
-                    process_index=scan.pop_process_index(),
-                    overwrite=True,
-                )
+            self.register_process(
+                process_file=scan.process_file,
+                entry=entry,
+                results={"center_of_rotation": cor if cor is not None else "-"},
+                configuration=self._axis_params.to_dict(),
+                process_index=scan.pop_process_index(),
+                overwrite=True,
+            )
 
         try:
             extra = {
@@ -680,7 +972,7 @@ class AxisProcess(
                     dataset=scan, process=self, state=DatasetState.FAILED, details=info
                 )
             else:
-                info = "axis calculation defined for {}: {} (using {})" "".format(
+                info = "axis calculation defined for {}: {} (using {})".format(
                     str(scan.path),
                     str(scan.axis_params.relative_cor_value),
                     scan.axis_params.mode.value,
@@ -708,15 +1000,15 @@ class AxisProcess(
         ):
             return scan.axis_params.axis_url_1, scan.axis_params.axis_url_2
         else:
-            _radio_1, _radio_2 = scan.getRadiosForAxisCalc(
-                mode=CorAngleMode.manual_selection
+            _radio_1, _radio_2 = scan.get_opposite_projections(
+                mode=scan.axis_params.angle_mode
             )
         return _radio_1, _radio_2
 
     @staticmethod
     def get_inputs(scan):
         assert isinstance(scan, TomwerScanBase)
-        radio_1, radio_2 = AxisProcess.get_inputs_urls(scan=scan)
+        radio_1, radio_2 = AxisTask.get_inputs_urls(scan=scan)
         if radio_1 and radio_2:
             mess = " ".join(
                 ("input radios are", radio_1.url.path(), "and", radio_2.url.path())
@@ -780,7 +1072,7 @@ class AxisProcess(
 
         :param TomwerScanBase scan: scan for which we want to compute the axis
                               position.
-        :return: scan as a TomoBase or a dict if _return_dict activated
+        :return: scan as a TomoBase or a dict if serialize_output_data activated
         """
         _logger.info("compute center of rotation for %s" % scan.path)
         try:
@@ -792,8 +1084,7 @@ class AxisProcess(
             scan_name = scan.path or "undef scan"
             scan.axis_params.set_relative_value(None)
             raise Exception(
-                "Fail to compute axis position for {} reason is {}"
-                "".format(scan_name, e)
+                f"Fail to compute axis position for {scan_name} reason is {e}"
             )
         else:
             scan.axis_params.set_relative_value(position)
@@ -804,18 +1095,12 @@ class AxisProcess(
                 method = "sinogram"
             else:
                 method = scan.axis_params.mode.value
+            r_cor_value = scan.axis_params.relative_cor_value
             mess = (
-                "Compute axis position ({value}) with {method} for {scan_name}".format(
-                    value=str(scan.axis_params.relative_cor_value),
-                    method=method,
-                    scan_name=scan_name,
-                )
+                f"Compute axis position ({r_cor_value}) with {method} for {scan_name}"
             )
             _logger.info(mess)
-        if self._return_dict:
-            return scan.to_dict()
-        else:
-            return scan
+        return scan
 
     def setMode(self, mode, value):
         if mode is AxisMode.manual:
@@ -879,10 +1164,8 @@ class AxisProcess(
         :return:
         """
         if entry is None:
-            with HDF5File(process_file, "r", swmr=True) as h5f:
-                entries = AxisProcess._get_process_nodes(
-                    root_node=h5f, process=AxisProcess
-                )
+            with open_hdf5(process_file) as h5f:
+                entries = AxisTask._get_process_nodes(root_node=h5f, process=AxisTask)
                 if len(entries) == 0:
                     _logger.info("unable to find a Axis process in %s" % process_file)
                     return None
@@ -892,9 +1175,9 @@ class AxisProcess(
                     entry = list(entries.keys())[0]
                     _logger.info("take %s as default entry" % entry)
 
-        with HDF5File(process_file, "r", swmr=True) as h5f:
-            axis_nodes = AxisProcess._get_process_nodes(
-                root_node=h5f[entry], process=AxisProcess
+        with open_hdf5(process_file) as h5f:
+            axis_nodes = AxisTask._get_process_nodes(
+                root_node=h5f[entry], process=AxisTask
             )
             index_to_path = {}
             for key, index in axis_nodes.items():
@@ -924,3 +1207,23 @@ class AxisProcess(
                     else:
                         res = h5py_read_dataset(results_node["center_of_rotation"])
             return res
+
+
+class AxisProcess(AxisTask):
+    def __init__(
+        self,
+        process_id=None,
+        varinfo=None,
+        inputs=None,
+        node_id=None,
+        node_attrs=None,
+        execinfo=None,
+    ):
+        deprecated_warning(
+            name="tomwer.core.process.reconstruction.axis.axis.AxisProcess",
+            type_="class",
+            reason="improve readibility",
+            since_version="1.2",
+            replacement="AxisTask",
+        )
+        super().__init__(process_id, varinfo, inputs, node_id, node_attrs, execinfo)
