@@ -28,28 +28,28 @@ __license__ = "MIT"
 __date__ = "04/03/2019"
 
 
-from silx.utils.enum import Enum as _Enum
-
-from ...utils import _assert_param_instance
-from .anglemode import CorAngleMode
-from tomwer.core.utils import getParametersFromParOrInfo
-from .projectiontype import ProjectionType
-from .mode import AxisMode
-from collections import namedtuple
-from silx.io.url import DataUrl
-import silx.io.utils
 import logging
-import numpy
 import os
-import h5py
+from collections import namedtuple
 from typing import Union
-from tomwer.core.process.task import Task
-from silx.utils.deprecation import deprecated
 
-try:
-    from nabu.preproc.phase import PaganinPhaseRetrieval
-except ImportError:
-    from tomwer.third_party.nabu.preproc.phase import PaganinPhaseRetrieval
+import h5py
+import numpy
+from silx.io.url import DataUrl
+from silx.utils.deprecation import deprecated
+from silx.utils.enum import Enum as _Enum
+from tomoscan.esrf.scan.utils import get_data
+
+from tomwer.core.process.task import Task
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.utils.spec import getParametersFromParOrInfo
+
+from .anglemode import CorAngleMode
+from .mode import AxisMode, AXIS_MODE_METADATAS
+from .projectiontype import ProjectionType
+
+from nabu.preproc.phase import PaganinPhaseRetrieval
+
 
 _logger = logging.getLogger(__name__)
 
@@ -63,7 +63,9 @@ _NO_PAG = "nopag"
 
 
 DEFAULT_CMP_THETA = 10
-DEFAULT_CMP_SUBSAMPLING_Y = 10
+DEFAULT_CMP_N_SUBSAMPLING_Y = 10
+DEFAULT_CMP_NEAR_POS = 0
+DEFAULT_CMP_NEAR_WIDTH = 20
 DEFAULT_CMP_OVERSAMPLING = 4
 DEFAULT_CMP_TAKE_LOG = True
 
@@ -75,7 +77,7 @@ class AxisCalculationInput(_Enum):
     transmission = _calculation_conf(ProjectionType.transmission, False)
     transmission_pag = _calculation_conf(ProjectionType.transmission, True)
 
-    def name(self):
+    def name(self):  # pylint: disable=E0102
         if self.value.paganin is True:
             return " ".join((self.value.projection_type.value, "paganin"))
         else:
@@ -98,7 +100,7 @@ class AxisCalculationInput(_Enum):
             for member in cls:
                 if value in (member.value, member.name()):
                     return member
-            raise ValueError("Cannot convert: %s" % value)
+            raise ValueError(f"Cannot convert: {value}")
 
 
 class AxisResource(object):
@@ -132,7 +134,7 @@ class AxisResource(object):
         if self.__url is None:
             return None
         if self.__raw_data is None:
-            self.__raw_data = silx.io.utils.get_data(self.url)
+            self.__raw_data = get_data(self.url)
         return self.__raw_data
 
     @data.setter
@@ -149,14 +151,18 @@ class AxisResource(object):
 
     def normalize_data(self, scan, log_):
         """Normalize data for axis calculation"""
+        if not isinstance(scan, TomwerScanBase):
+            raise TypeError(
+                f"scan is expected to be an instance of {TomwerScanBase}. {type(scan)} provided"
+            )
         if self.__url is None:
             return None
         else:
             self.__norme_data = scan.data_flat_field_correction(data=self.data)
             if log_ is True:
-                self.__norme_data[
-                    numpy.isnan(self.__norme_data)
-                ] = self.__norme_data.max()
+                self.__norme_data[numpy.isnan(self.__norme_data)] = (
+                    self.__norme_data.max()
+                )
                 if self.__norme_data.max() < 0:
                     _logger.error("max data value < 0 unable to compute log")
                 else:
@@ -167,9 +173,9 @@ class AxisResource(object):
                     except Exception as e:
                         _logger.error("Fail to apply log on radio" + e)
             else:
-                self.__norme_data[
-                    numpy.isnan(self.__norme_data)
-                ] = self.__norme_data.min()
+                self.__norme_data[numpy.isnan(self.__norme_data)] = (
+                    self.__norme_data.min()
+                )
 
         return self.__norme_data
 
@@ -220,6 +226,28 @@ class AxisRP:
     AXIS_POSITION_PAR_KEY = "ROTATION_AXIS_POSITION"
     """Key used for the axis position in par files"""
 
+    _MANAGED_KEYS = (
+        "MODE",
+        "POSITION_VALUE",
+        "CALC_INPUT_TYPE",
+        "USE_SINOGRAM",
+        "ANGLE_MODE",
+        "SINOGRAM_LINE",
+        "AXIS_URL_1",
+        "AXIS_URL_2",
+        "LOOK_AT_STDMAX",
+        "NEAR_WX",
+        "FINE_STEP_X",
+        "SCALE_IMG2_TO_IMG1",
+        "NEAR_POSITION",
+        "SINOGRAM_SUBSAMPLING",
+        "PADDING_MODE",
+        "FLIP_LR",
+        "COMPOSITE_OPTS",
+        "SIDE",
+        "COR_OPTIONS",
+    )
+
     def __init__(self):
         self.__mode = AxisMode.manual
         """Mode used for defining the COR (center of rotation)"""
@@ -258,17 +286,21 @@ class AxisRP:
         """do image scaling"""
         self.__padding_mode = None
         self.__frame_width = None
-        self.__side = "all"
+        self.__side = "right"
         """side of the cor. Requested by nabu cor algorithms growing-window
         and sliding-window"""
         self.__flip_lr = True
         self.__composite_options = {
             "theta": DEFAULT_CMP_THETA,
-            "subsampling_y": DEFAULT_CMP_SUBSAMPLING_Y,
+            "n_subsampling_y": DEFAULT_CMP_N_SUBSAMPLING_Y,
             "oversampling": DEFAULT_CMP_OVERSAMPLING,
             "take_log": DEFAULT_CMP_TAKE_LOG,
+            "near_pos": DEFAULT_CMP_NEAR_POS,
+            "near_width": DEFAULT_CMP_NEAR_WIDTH,
         }
         """specific options for composite cor search"""
+        self.__extra_cor_options = ""
+        """Automatic cor options as str. side, near_pos and near_width are provided independantly"""
 
     @property
     def mode(self):
@@ -283,18 +315,28 @@ class AxisRP:
                     name = AxisMode.global_.name
                 elif name == "accurate":
                     _logger.info(
-                        "convert axis mode {} to {} (renamed)"
-                        "".format(name, AxisMode.centered.name)
+                        f"convert axis mode {name} to {AxisMode.centered.name} (renamed)"
                     )
                     name = AxisMode.centered.name
+                elif name == "growing-window":
+                    _logger.warning(
+                        f"growing-window mode has been removed. Replace it by {AxisMode.growing_window_radios.value}"
+                    )
+                    name = AxisMode.growing_window_radios.name
+                elif name == "sliding-window":
+                    _logger.warning(
+                        f"sliding-window mode has been removed. Replace it by {AxisMode.sliding_window_radios.value}"
+                    )
+                    name = AxisMode.sliding_window_radios.name
                 try:
                     _mode = getattr(AxisMode, name)
                 except Exception:
                     _mode = AxisMode.from_value(name)
             except Exception:
-                raise ValueError("Fail to create axis mode from %s" % mode)
+                raise ValueError(f"Fail to create axis mode from {mode}")
         else:
-            _assert_param_instance(mode, AxisMode)
+            if not isinstance(mode, AxisMode):
+                raise TypeError(f"mode is expected to be an instance of {AxisMode}")
             _mode = mode
         self.__mode = _mode
         self.changed()
@@ -307,7 +349,7 @@ class AxisRP:
     def frame_width(self, value):
         if not isinstance(value, (type(None), float, int)):
             raise TypeError(
-                "Value is expected to be None or a float. Not {}".format(type(value))
+                f"Value is expected to be None or a float. Not {type(value)}"
             )
         if value is None:
             self.__frame_width = value
@@ -341,7 +383,10 @@ class AxisRP:
         return self.__absolute_value
 
     def set_relative_value(self, value):
-        _assert_param_instance(value, (int, float, str, type(None)))
+        if not isinstance(value, (int, float, str, type(None))):
+            raise TypeError(
+                f"value is expected to be an instance of {int} {float}, {str} or {None}. {type(value)} provided"
+            )
         if value is None:
             changed = self.__relative_value is not None
         elif isinstance(value, str) and value == "...":
@@ -385,8 +430,8 @@ class AxisRP:
 
     @side.setter
     def side(self, side):
-        if side not in ("all", "left", "right", "center"):
-            raise ValueError("side '{}' is not managed".format(side))
+        if side not in ("all", "left", "right", "center", "near"):
+            raise ValueError(f"side '{side}' is not managed")
         if self.__side != side:
             self.__side = side
             self.changed()
@@ -553,8 +598,19 @@ class AxisRP:
             self.changed()
 
     @property
-    def padding_mode(self):
-        return self.__padding_mode
+    def extra_cor_options(self) -> str:
+        return self.__extra_cor_options
+
+    @extra_cor_options.setter
+    def extra_cor_options(self, cor_options: str):
+        if not isinstance(cor_options, str):
+            raise TypeError(f"{type(cor_options)} provided when {str} expected")
+        else:
+            clean_opts_str = (
+                cor_options.replace(" ", "").strip(";").rstrip(";").replace(";", " ; ")
+            )
+            self.__extra_cor_options = clean_opts_str
+            self.changed()
 
     @property
     def composite_options(self) -> dict:
@@ -573,11 +629,25 @@ class AxisRP:
             raise TypeError(
                 f"opts is expected to be an instance of dict not {type(opts)}"
             )
+        # insure backward compatibility
+        if "subsampling_y" in opts:
+            opts["n_subsampling_y"] = opts.pop("subsampling_y")
         for key in opts.keys():
-            if key not in ("theta", "oversampling", "subsampling_y", "take_log"):
+            if key not in (
+                "theta",
+                "oversampling",
+                "n_subsampling_y",
+                "take_log",
+                "near_pos",
+                "near_width",
+            ):
                 raise KeyError(f"{key} is not recogized")
         self.__composite_options = opts
 
+    @property
+    def padding_mode(self):
+        return self.__padding_mode
+
     @padding_mode.setter
     def padding_mode(self, mode):
         if self.__padding_mode != mode:
@@ -611,10 +681,7 @@ class AxisRP:
             ):
                 _dict = getParametersFromParOrInfo(file_path)
                 if self.AXIS_POSITION_PAR_KEY.lower() not in _dict:
-                    err = "fail to find `%s` in %s" % (
-                        self.AXIS_POSITION_PAR_KEY,
-                        file_path,
-                    )
+                    err = f"fail to find `{self.AXIS_POSITION_PAR_KEY}` in {file_path}"
                     _logger.info(err)
                     raise KeyError(err)
                 else:
@@ -628,7 +695,7 @@ class AxisRP:
                             res = float(values[0])
                         except Exception:
                             _logger.info(
-                                "fail to load center of rotation " "from ", file_path
+                                f"fail to load center of rotation from {file_path}"
                             )
                             return None
                         else:
@@ -665,10 +732,10 @@ class AxisRP:
         :return: center of rotation or none
         :rtype: Union[None, float]
         """
-        from .axis import AxisProcess  # avoid cyclic import
+        from .axis import AxisTask  # avoid cyclic import
 
         processes = Task.get_processes_frm_type(
-            process_file=file_path, process_type=AxisProcess, entry=entry
+            process_file=file_path, process_type=AxisTask, entry=entry
         )
         if len(processes) == 0:
             _logger.warning("failed to find any information on center " "of rotation")
@@ -676,8 +743,7 @@ class AxisRP:
         elif len(processes) > 1:
             _logger.info("found more than one value computed for " "center of rotation")
         _logger.info(
-            "load center of rotation from %s: %s"
-            % (file_path, processes[0].results["center_of_rotation"])
+            f"load center of rotation from {file_path}: {processes[0].results['center_of_rotation']}"
         )
         cor = processes[0].results["center_of_rotation"]
         if cor in [None, ""]:
@@ -728,6 +794,8 @@ class AxisRP:
             "PADDING_MODE": self.padding_mode,
             "FLIP_LR": self.flip_lr,
             "COMPOSITE_OPTS": self.composite_options,
+            "SIDE": self.side,
+            "COR_OPTIONS": self.extra_cor_options,
         }
         return _dict
 
@@ -738,6 +806,12 @@ class AxisRP:
         return axis
 
     def load_from_dict(self, _dict):
+        # Convert managed keys to upper case
+        _dict = {
+            key.upper() if key.upper() in self._MANAGED_KEYS else key: value
+            for key, value in _dict.items()
+        }
+
         if "MODE" in _dict:
             self.mode = _dict["MODE"]
         if "POSITION_VALUE" in _dict:
@@ -774,6 +848,9 @@ class AxisRP:
             self.flip_lr = bool(_dict["FLIP_LR"])
         if "COMPOSITE_OPTS" in _dict:
             self.composite_options = _dict["COMPOSITE_OPTS"]
+        if "SIDE" in _dict:
+            self.side = _dict["SIDE"]
+        self.extra_cor_options = _dict.get("COR_OPTIONS", "")
 
     def copy(self, axis_params, copy_axis_url=True, copy_flip_lr=True):
         assert isinstance(axis_params, AxisRP)
@@ -793,6 +870,7 @@ class AxisRP:
         self.padding_mode = axis_params.padding_mode
         self.side = axis_params.side
         self.composite_options = axis_params.composite_options
+        self.extra_cor_options = axis_params.extra_cor_options
         if copy_axis_url:
             self.axis_url_1 = axis_params.axis_url_1
             self.axis_url_2 = axis_params.axis_url_2
@@ -807,18 +885,66 @@ class AxisRP:
         pass
 
     def get_simple_str(self):
-        results = "{}".format(self.mode.value)
-        if self.mode in (AxisMode.growing_window, AxisMode.sliding_window):
-            extra_info = "side: {}, use sinogram: {}".format(
-                self.side, self.use_sinogram
-            )
+        """
+        special information as a str for mode able to handle both sinogram and radios
+        """
+        results = f"{self.mode.value}"
+        if self.mode in (
+            AxisMode.growing_window_radios,
+            AxisMode.growing_window_sinogram,
+            AxisMode.sliding_window_radios,
+            AxisMode.sliding_window_sinogram,
+        ):
+            extra_info = f"side: {self.side}, use sinogram: {self.use_sinogram}"
             results = ", ".join((results, extra_info))
         return results
 
-    def get_nabu_cor_options(self) -> str:
-        """return cor option for nabu"""
-        if self.mode in (AxisMode.sliding_window, AxisMode.growing_window):
-            options = "side='{}'".format(self.side)
-        else:
-            options = None
+    def get_nabu_cor_options_as_dict(self) -> str:
+        options = {}
+        if self.mode is AxisMode.near:
+            self.side = "near"
+        request_side = len(AXIS_MODE_METADATAS[self.mode].valid_sides) > 0
+        if request_side:
+
+            if self.side == "near":
+                options["side"] = self.composite_options.get(
+                    "near_pos", self.estimated_cor
+                )
+            else:
+                options["side"] = self.side
+
+        if self.side == "near":
+            near_width = self.composite_options.get("near_width", 20.0)
+            options["near_width"] = near_width
+
+        # append "extra_cor_options" to already handled cor options
+        extra_cor_options = self.extra_cor_options.replace(" ", "")
+
+        if extra_cor_options != "":
+            for opt in self.extra_cor_options.replace(" ", "").split(";"):
+                if len(opt.split("=")) == 2:
+                    key, value = opt.split("=")
+                    options[key] = value
+                else:
+                    _logger.info(f"ignore option {opt}. Invalid synthax")
         return options
+
+    @deprecated(replacement="get_nabu_cor_options_as_str", since_version="1.1")
+    def get_nabu_cor_options(self) -> str:
+        return self.get_nabu_cor_options_as_str()
+
+    def get_nabu_cor_options_as_str(self) -> str:
+        """return cor option for nabu"""
+
+        def cast_key_value(key, value):
+            if key in ("side",):
+                return f"{key}='{value}'"
+            else:
+                return f"{key}={value}"
+
+        return " ; ".join(
+            [
+                cast_key_value(key, value)
+                for key, value in self.get_nabu_cor_options_as_dict().items()
+            ]
+        )
