@@ -29,12 +29,15 @@ __date__ = "07/03/2019"
 
 
 import enum
+
 from silx.utils.enum import Enum as _Enum
+from tomoscan.framereducer.method import ReduceMethod
+
 from tomwer.core.process.reconstruction.darkref.settings import (
     DARKHST_PREFIX,
     REFHST_PREFIX,
 )
-from tomwer.core.process.reconstruction.ftseries.params.base import _ReconsParam
+from tomwer.core.process.reconstruction.paramsbase import _ReconsParam
 from tomwer.core.utils.deprecation import deprecated
 
 
@@ -47,21 +50,19 @@ class When(_Enum):
     after = (2,)
 
 
-@enum.unique
-class Method(_Enum):
-    none = (0,)
-    average = (1,)
-    median = 2
-    first = 10
-    last = 11
-
-    @classmethod
-    def from_value(cls, value):
-        if isinstance(value, str):
-            for m in Method:
-                if value.lower() == m.name.lower():
-                    return m
-        return super().from_value(value)
+def _from_value_reduce_method_backward_compatibility(value) -> ReduceMethod:
+    if value in ("none", 0, (0,)):
+        return ReduceMethod.NONE
+    elif value in ("average", 1, (1,)):
+        return ReduceMethod.MEAN
+    elif value in ("median", 2, (2,)):
+        return ReduceMethod.MEDIAN
+    elif value in ("first", 10, (10,)):
+        return ReduceMethod.FIRST
+    elif value in ("last", 11, (11,)):
+        return ReduceMethod.FIRST
+    else:
+        return ReduceMethod.from_value(value)
 
 
 class DKRFRP(_ReconsParam):
@@ -70,14 +71,14 @@ class DKRFRP(_ReconsParam):
     def __init__(self):
         _ReconsParam.__init__(self)
         self.__do_when = When.before
-        self.__dark_calc = Method.average
+        self.__dark_calc = ReduceMethod.MEAN
         self.__overwrite_dark = False
         self.__remove_dark = False
         self.__dark_pattern = "darkend[0-9]{3,4}"
-        self.__ref_calc = Method.median
+        self.__flat_calc = ReduceMethod.MEDIAN
         self.__overwrite_flat = False
-        self.__remove_ref = False
-        self.__ref_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"
+        self.__remove_flats = False
+        self.__flat_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"
         self.__dark_prefix = DARKHST_PREFIX
         self.__ref_prefix = REFHST_PREFIX
 
@@ -87,10 +88,10 @@ class DKRFRP(_ReconsParam):
             "DARKOVE": self.__class__.overwrite_dark,
             "DARKRMV": self.__class__.remove_dark,
             "DKFILE": self.__class__.dark_pattern,
-            "REFSCAL": self.__class__.ref_calc_method,
+            "REFSCAL": self.__class__.flat_calc_method,
             "REFSOVE": self.__class__.overwrite_flat,
-            "REFSRMV": self.__class__.remove_ref,
-            "RFFILE": self.__class__.ref_pattern,
+            "REFSRMV": self.__class__.remove_raw_flats,
+            "RFFILE": self.__class__.flat_pattern,
         }
 
     @property
@@ -114,11 +115,8 @@ class DKRFRP(_ReconsParam):
 
     @dark_calc_method.setter
     def dark_calc_method(self, method):
-        assert isinstance(method, (int, Method, str))
-        if isinstance(method, str):
-            _dark_calc = getattr(Method, method.lower())
-        else:
-            _dark_calc = Method(method)
+        assert isinstance(method, (int, ReduceMethod, str))
+        _dark_calc = _from_value_reduce_method_backward_compatibility(method)
 
         if self.__dark_calc != _dark_calc:
             self.__dark_calc = _dark_calc
@@ -163,19 +161,16 @@ class DKRFRP(_ReconsParam):
             self.changed()
 
     @property
-    def ref_calc_method(self):
+    def flat_calc_method(self):
         """Dark calculation method (None, Average, Median)"""
-        return self.__ref_calc
+        return self.__flat_calc
 
-    @ref_calc_method.setter
-    def ref_calc_method(self, method):
-        assert isinstance(method, (int, Method, str))
-        if isinstance(method, str):
-            _ref_calc = getattr(Method, method.lower())
-        else:
-            _ref_calc = Method(method)
-        if self.__ref_calc != _ref_calc:
-            self.__ref_calc = _ref_calc
+    @flat_calc_method.setter
+    def flat_calc_method(self, method):
+        assert isinstance(method, (int, ReduceMethod, str))
+        _ref_calc = _from_value_reduce_method_backward_compatibility(method)
+        if self.__flat_calc != _ref_calc:
+            self.__flat_calc = _ref_calc
             self.changed()
 
     @property
@@ -202,42 +197,44 @@ class DKRFRP(_ReconsParam):
             self.changed()
 
     @property
-    def remove_ref(self):
-        """Remove original ref files when done"""
-        return self.__remove_ref
+    def remove_raw_flats(self):
+        """Remove original flat / ref files when done (for EDF only)"""
+        return self.__remove_flats
 
-    @remove_ref.setter
-    def remove_ref(self, remove):
+    @remove_raw_flats.setter
+    def remove_raw_flats(self, remove):
         # TODO: float should be removed, but this is a legacy from fastomo3
         assert isinstance(remove, (int, bool, float))
-        _remove_ref = remove
-        if self.__remove_ref != _remove_ref:
-            self.__remove_ref = _remove_ref
+        _remove_flats = remove
+        if self.__remove_flats != _remove_flats:
+            self.__remove_flats = _remove_flats
             self.changed()
 
     @property
-    def ref_pattern(self):
-        """File pattern to detect references"""
-        return self.__ref_pattern
+    def flat_pattern(self):
+        """File pattern to detect EDF flats files"""
+        return self.__flat_pattern
 
-    @ref_pattern.setter
-    def ref_pattern(self, pattern):
-        if pattern != self.__ref_pattern:
-            self.__ref_pattern = pattern
+    @flat_pattern.setter
+    def flat_pattern(self, pattern):
+        if pattern != self.__flat_pattern:
+            self.__flat_pattern = pattern
             self.changed()
 
     @property
-    def ref_prefix(self):
+    def flat_prefix(self):
+        """flat prefix for EDF"""
         return self.__ref_prefix
 
-    @ref_prefix.setter
-    def ref_prefix(self, prefix):
+    @flat_prefix.setter
+    def flat_prefix(self, prefix):
         if prefix != self.__ref_prefix:
             self.__ref_prefix = prefix
             self.changed()
 
     @property
     def dark_prefix(self):
+        """dark prefix for EDF"""
         return self.__dark_prefix
 
     @dark_prefix.setter
@@ -247,7 +244,7 @@ class DKRFRP(_ReconsParam):
             self.changed()
 
     def _set_remove_opt(self, rm):
-        self.remove_ref = rm
+        self.remove_raw_flats = rm
         self.remove_dark = rm
 
     def _set_skip_if_exist(self, skip):
@@ -257,14 +254,14 @@ class DKRFRP(_ReconsParam):
     def to_dict(self):
         _dict = {
             "DOWHEN": self.do_when.name,
-            "DARKCAL": self.dark_calc_method.name.split(".")[-1].title(),
+            "DARKCAL": self.dark_calc_method.value,
             "DARKOVE": int(self.overwrite_dark),
             "DARKRMV": int(self.remove_dark),
             "DKFILE": self.dark_pattern,
-            "REFSCAL": self.ref_calc_method.name.split(".")[-1].title(),
+            "REFSCAL": self.flat_calc_method.value,
             "REFSOVE": int(self.overwrite_flat),
-            "REFSRMV": int(self.remove_ref),
-            "RFFILE": self.ref_pattern,
+            "REFSRMV": int(self.remove_raw_flats),
+            "RFFILE": self.flat_pattern,
         }
         _dict.update(self.unmanaged_params)
         return _dict
@@ -276,13 +273,28 @@ class DKRFRP(_ReconsParam):
         return params
 
     def load_from_dict(self, _dict):
+        # Convert managed keys to upper case
+        _dict = {
+            key.upper() if key.upper() in self.managed_params else key: value
+            for key, value in _dict.items()
+        }
+
         self._load_unmanaged_params(_dict=_dict)
-        self.do_when = getattr(When, _dict["DOWHEN"])
-        self.dark_calc_method = getattr(Method, _dict["DARKCAL"].lower())
-        self.overwrite_dark = _dict["DARKOVE"]
-        self.remove_dark = _dict["DARKRMV"]
-        self.dark_pattern = _dict["DKFILE"]
-        self.ref_calc_method = getattr(Method, _dict["REFSCAL"].lower())
-        self.overwrite_flat = _dict["REFSOVE"]
-        self.remove_ref = _dict["REFSRMV"]
-        self.ref_pattern = _dict["RFFILE"]
+        if "DOWHEN" in _dict:
+            self.do_when = getattr(When, _dict["DOWHEN"])
+        if "DARKCAL" in _dict:
+            self.dark_calc_method = _dict["DARKCAL"]
+        if "DARKOVE" in _dict:
+            self.overwrite_dark = _dict["DARKOVE"]
+        if "DARKRMV" in _dict:
+            self.remove_dark = _dict["DARKRMV"]
+        if "DKFILE" in _dict:
+            self.dark_pattern = _dict["DKFILE"]
+        if "REFSCAL" in _dict:
+            self.flat_calc_method = _dict["REFSCAL"]
+        if "REFSOVE" in _dict:
+            self.overwrite_flat = _dict["REFSOVE"]
+        if "REFSRMV" in _dict:
+            self.remove_raw_flats = _dict["REFSRMV"]
+        if "RFFILE" in _dict:
+            self.flat_pattern = _dict["RFFILE"]
