@@ -28,10 +28,11 @@ __license__ = "MIT"
 __date__ = "04/05/2021"
 
 
-from tomwer.core.process.reconstruction.scores.params import SABaseParams
-from tomwer.core.process.reconstruction.nabu.utils import retrieve_lst_of_value_from_str
 import typing
 
+from tomwer.core.process.reconstruction.nabu.utils import retrieve_lst_of_value_from_str
+from tomwer.core.process.reconstruction.scores.params import SABaseParams
+
 
 class SADeltaBetaParams(SABaseParams):
     """Parameters for the semi-automatic axis calculation"""
@@ -62,25 +63,23 @@ class SADeltaBetaParams(SABaseParams):
         return self._selected_db
 
     def to_dict(self) -> dict:
-        dict = super().to_dict()
-        dict.update(
+        my_dict = super().to_dict()
+        my_dict.update(
             {
                 "delta_beta_values": self.delta_beta_values,
                 "selected_db_value": self.selected_delta_beta_value,
             }
         )
-        return dict
+        return my_dict
 
     def load_from_dict(self, dict_: dict):
         if not isinstance(dict_, dict):
-            raise TypeError(
-                "dict_ should be an instance of dict not {}".format(type(dict_))
-            )
+            raise TypeError(f"dict_ should be an instance of dict not {type(dict_)}")
         super().load_from_dict(dict_)
         if "delta_beta_values" in dict_:
             self.delta_beta_values = dict_["delta_beta_values"]
         if "selected_db_value" in dict_:
-            self.set_db_selected_value(value=dict["selected_db_value"])
+            self.set_db_selected_value(value=dict_["selected_db_value"])
 
     @staticmethod
     def from_dict(dict_):
@@ -99,12 +98,9 @@ class SADeltaBetaParams(SABaseParams):
         if self.slice_indexes is None:
             missing_information.append("slice index not provided")
         if len(missing_information) > 0:
+            missing_information_str = " ; ".join(missing_information)
             raise ValueError(
-                str(
-                    "Some informations are missing: {}".format(
-                        " ; ".join(missing_information)
-                    )
-                )
+                f"Some informations are missing: {missing_information_str}"
             )
 
     def __str__(self):
