@@ -28,12 +28,13 @@ __license__ = "MIT"
 __date__ = "18/02/2021"
 
 
-from silx.utils.enum import Enum as _Enum
-import numpy
-import sys
 import logging
+import sys
 from typing import Union
 
+import numpy
+from silx.utils.enum import Enum as _Enum
+
 _logger = logging.getLogger(__name__)
 
 
@@ -76,7 +77,20 @@ class ComputedScore:
         elif method is ScoreMethod.TOMO_CONSISTENCY:
             return self.tomo_consistency
         else:
-            raise ValueError("{} is an unrecognized method".format(method))
+            raise ValueError(f"{method} is an unrecognized method")
+
+    def __eq__(self, __value: object) -> bool:
+        if not isinstance(__value, ComputedScore):
+            return False
+        else:
+            return (
+                self.total_variation == __value.total_variation
+                and self.std == __value.std
+                and self.tomo_consistency == __value.tomo_consistency
+            )
+
+    def __str__(self) -> str:
+        return f"std: {self.std} - tv: {self.total_variation} - tomo-consistency: {self.tomo_consistency}"
 
 
 def compute_tomo_consistency(
@@ -160,11 +174,11 @@ def compute_score(
         elif data.shape[2] == 1:
             data = data.reshape(data.shape[0], data.shape[1])
         else:
-            raise ValueError("Data is expected to be 2D. Not {}".format(data.ndim))
+            raise ValueError(f"Data is expected to be 2D. Not {data.ndim}D")
     elif data.ndim == 2:
         pass
     else:
-        raise ValueError("Data is expected to be 2D. Not {}".format(data.ndim))
+        raise ValueError(f"Data is expected to be 2D. Not {data.ndim}D")
     if method is ScoreMethod.TOMO_CONSISTENCY:
         try:
             return compute_tomo_consistency(
@@ -177,9 +191,7 @@ def compute_score(
         except ImportError:
             return None
         except Exception as e:
-            _logger.warning(
-                "Fail to compute 'tomo consistency' score. " "Reason is {}".format(e)
-            )
+            _logger.warning(f"Fail to compute 'tomo consistency' score. Reason is {e}")
             return None
     else:
         fct = _METHOD_TO_FCT.get(method, None)
@@ -187,14 +199,16 @@ def compute_score(
     if fct is not None:
         return fct(data)
     else:
-        raise ValueError("{} is not handled".format(method))
+        raise ValueError(f"{method} is not handled")
 
 
 def get_disk_mask_radius(datasets_) -> int:
     """compute the radius to use for the mask"""
     radius = sys.maxsize
     # get min radius
-    for cor, (url, data) in datasets_.items():
+    for _, (_, data) in datasets_.items():
+        if data is None:
+            continue
         assert data.ndim == 2, "data is expected to be 2D"
         min_ = numpy.array(data.shape).min()
         if radius >= min_:
@@ -205,14 +219,12 @@ def get_disk_mask_radius(datasets_) -> int:
 def apply_roi(data, radius, url) -> numpy.array:
     """compute the square included in the circle of radius and centered
     in the middle of the data"""
-    half_width = int(radius / 2 ** 0.5)
+    half_width = int(radius / 2**0.5)
     center = numpy.array(data.shape[:]) // 2
     min_x, max_x = center[0] - half_width, center[0] + half_width
     min_y, max_y = center[1] - half_width, center[1] + half_width
     try:
         return data[min_y:max_y, min_x:max_x]
     except Exception:
-        _logger.error(
-            "Fail to apply roi for {}. Take the entire dataset".format(url.path())
-        )
+        _logger.error(f"Fail to apply roi for {url.path()}. Take the entire dataset")
         return data
