@@ -34,66 +34,52 @@ __license__ = "MIT"
 __date__ = "27/07/2021"
 
 
-import h5py
-import numpy
+import logging
+import typing
+from collections import namedtuple
 from datetime import datetime
 from typing import Union
-from silx.io.dictdump import dicttoh5
-from collections import namedtuple
-from silx.io.dictdump import h5todict
-from tomoscan.io import HDF5File
-import typing
-import logging
+
+import h5py
+import numpy
+from ewokscore.task import Task as _EwoksTask
+from ewokscore.taskwithprogress import TaskWithProgress as _EwoksTaskWithProgress
+from silx.io.dictdump import dicttoh5, h5todict
 from silx.io.utils import h5py_read_dataset
-import multiprocessing
-import silx
-from ewokscore.task import Task as EwoksTask
-
-
-mult_manager = multiprocessing.Manager()
+from silx.io.utils import open as open_hdf5
+from tomoscan.io import HDF5File
+from tomwer.core.utils.locker import FileLockerManager
 
 
 _logger = logging.getLogger(__name__)
 
-# TODO: should be removed
-_input_desc = namedtuple("_input_desc", ["name", "type", "handler", "doc"])
-# TODO: should be removed
-
-_output_desc = namedtuple("_output_desc", ["name", "type", "doc"])
-
 _process_desc = namedtuple(
     "_process_desc", ["process_order", "configuration", "results"]
 )
 
 
-_MULTIPROCESSING_LOCKERS = mult_manager.dict()
-
-
 class BaseProcessInfo:
     """Tomwer base process class"""
 
-    MULTIPROCESSING_LOCKERS = mult_manager.dict()
-
     _output_values = {}
     # TODO: look at this: not sure this is needed anymore
 
     def __init__(self, inputs=None):
+        """
+        :param bool: return_dict: if True serialize (to_dict / from_dict functions) between each task
+        """
         if inputs is None:
             inputs = {}
-        self._scheme_title = inputs.get(
-            "scheme_title", None
-        )  # title of the associated scheme
-        self._return_dict = inputs.get("return_dict", False)
+
+        self._scheme_title = (
+            "scheme_title"  # TODO: have a look, this must be get somewhere and reused ?
+        )
 
         """should the return type of the handler should be TomoBase instance
         objects or dict"""
         self._settings = {}
-
-    @staticmethod
-    def _get_lock(file_name):
-        if file_name not in _MULTIPROCESSING_LOCKERS:
-            _MULTIPROCESSING_LOCKERS[file_name] = mult_manager.Lock()
-        return _MULTIPROCESSING_LOCKERS[file_name]
+        self._cancelled = False
+        # a useful variable that can be set to True if the task has been cancelled
 
     @staticmethod
     def properties_help():
@@ -128,12 +114,6 @@ class BaseProcessInfo:
         """
         self._output_values[key] = value
 
-    def key_exist(self, key):
-        for _output in self.outputs:
-            if _output.name == key:
-                return True
-        return False
-
     @staticmethod
     def program_name():
         """Name of the program used for this processing"""
@@ -195,19 +175,20 @@ class BaseProcessInfo:
         :type interpretations: Union[None, dict
         """
         assert process_file is not None
-        try:
-            self._register_process(
-                process_file=process_file,
-                entry=entry,
-                process=self,
-                configuration=configuration,
-                results=results,
-                process_index=process_index,
-                interpretations=interpretations,
-                overwrite=overwrite,
-            )
-        except IOError as e:
-            _logger.error(e)
+        with FileLockerManager().get_lock(file_name=process_file):
+            try:
+                self._register_process(
+                    process_file=process_file,
+                    entry=entry,
+                    process=self,
+                    configuration=configuration,
+                    results=results,
+                    process_index=process_index,
+                    interpretations=interpretations,
+                    overwrite=overwrite,
+                )
+            except IOError as e:
+                _logger.error(e)
 
     @staticmethod
     def _register_process(
@@ -248,91 +229,71 @@ class BaseProcessInfo:
         def get_process_path():
             return "/".join((entry or "entry", process_name))
 
-        # save it to the file
-        with Task._get_lock(process_file):
-            # needs an extra lock for multiprocessing
+        # save it to the file (lock should be handled upper)
+        with HDF5File(process_file, mode="a") as h5f:
+            nx_process = h5f.require_group(get_process_path())
+            if "NX_class" not in nx_process.attrs:
+                nx_process.attrs["NX_class"] = "NXprocess"
+            if overwrite:
+                for key in (
+                    "program",
+                    "results",
+                    "version",
+                    "date",
+                    "sequence_index",
+                    "class_instance",
+                    "configuration",
+                ):
+                    if key in nx_process:
+                        del nx_process[key]
+
+            # write process information
+            nx_process["program"] = process.program_name()
+            nx_process["version"] = process.program_version()
+            nx_process["date"] = datetime.now().replace(microsecond=0).isoformat()
+            nx_process["sequence_index"] = numpy.int32(process_index)
+            _class = process.__class__
+            nx_process["class_instance"] = ".".join(
+                (_class.__module__, _class.__name__)
+            )
+
+        # dump result
+        if results is not None:
+            h5path = "/".join((get_process_path(), "results"))
+            dicttoh5(
+                results,
+                h5file=process_file,
+                h5path=h5path,
+                update_mode="modify",
+                mode="a",
+            )
+            for interpretation_key, interpretation in interpretations.items():
+                try:
+                    with HDF5File(process_file, mode="a") as h5f:
+                        node = h5f["/".join((get_process_path(), "results"))]
+                        node[interpretation_key].attrs[
+                            "interpretation"
+                        ] = interpretation
+                except KeyError:
+                    _logger.warning(
+                        "Invalid interpretation - no result store"
+                        " for %s" % interpretation_key
+                    )
+
+        # dump configuration
+        if configuration is not None:
+            h5path = "/".join((get_process_path(), "configuration"))
+            dicttoh5(
+                configuration,
+                h5file=process_file,
+                h5path=h5path,
+                update_mode="modify",
+                mode="a",
+            )
 
             with HDF5File(process_file, mode="a") as h5f:
                 nx_process = h5f.require_group(get_process_path())
-                if "NX_class" not in nx_process.attrs:
-                    nx_process.attrs["NX_class"] = "NXprocess"
-                if overwrite:
-                    for key in (
-                        "program",
-                        "results",
-                        "version",
-                        "date",
-                        "sequence_index",
-                        "class_instance",
-                    ):
-                        if key in nx_process:
-                            del nx_process[key]
-
-                # write process information
-                nx_process["program"] = process.program_name()
-                nx_process["version"] = process.program_version()
-                nx_process["date"] = datetime.now().replace(microsecond=0).isoformat()
-                nx_process["sequence_index"] = numpy.int32(process_index)
-                _class = process.__class__
-                nx_process["class_instance"] = ".".join(
-                    (_class.__module__, _class.__name__)
-                )
-
-            # dump result
-            if results is not None:
-                h5path = "/".join((get_process_path(), "results"))
-                if silx.version >= "0.15":
-                    dicttoh5(
-                        results,
-                        h5file=process_file,
-                        h5path=h5path,
-                        update_mode="modify",
-                        mode="a",
-                    )
-                else:
-                    dicttoh5(
-                        results,
-                        h5file=process_file,
-                        h5path=h5path,
-                        overwrite_data=True,
-                        mode="a",
-                    )
-                for interpretation_key, interpretation in interpretations.items():
-                    try:
-                        with HDF5File(process_file, mode="a") as h5f:
-                            node = h5f["/".join((get_process_path(), "results"))]
-                            node[interpretation_key].attrs[
-                                "interpretation"
-                            ] = interpretation
-                    except KeyError:
-                        _logger.warning(
-                            "Invalid interpretation - no result store"
-                            " for %s" % interpretation_key
-                        )
-
-            # dump configuration
-            if configuration is not None:
-                h5path = "/".join((get_process_path(), "configuration"))
-                if silx.version >= "0.15":
-                    dicttoh5(
-                        configuration,
-                        h5file=process_file,
-                        h5path=h5path,
-                        update_mode="modify",
-                        mode="a",
-                    )
-                else:
-                    dicttoh5(
-                        configuration,
-                        h5file=process_file,
-                        h5path=h5path,
-                        overwrite_data=True,
-                        mode="a",
-                    )
-
-                with HDF5File(process_file, mode="a") as h5f:
-                    nx_process = h5f.require_group(get_process_path())
-                    nx_process["configuration"].attrs["NX_class"] = "NXcollection"
+                nx_process["configuration"].attrs["NX_class"] = "NXcollection"
 
     @staticmethod
     def _get_process_nodes(
@@ -415,50 +376,88 @@ class BaseProcessInfo:
         :return: list of _process_desc(sequence_index, configuration, results)
         :rtype: list
         """
-        processes = []
-        with HDF5File(process_file, mode="r", swmr=True) as h5f:
+        # retrieve process to load
+        with open_hdf5(process_file) as h5f:
             if entry is None:
                 if len(h5f.keys()) > 0:
                     root = h5f[list(h5f.keys())[0]]
                 else:
                     _logger.warning("no process find")
+                    return []
             else:
                 root = h5f[entry]
+
+            processes_to_load = {}
             for process in root.keys():
                 nx_process = root[process]
                 if (
                     h5py_read_dataset(nx_process["program"])
                     == process_type.program_name()
                 ):
-                    p_order = h5py_read_dataset(nx_process["sequence_index"])
-                    config = h5todict(
-                        process_file, "/".join((nx_process.name, "configuration"))
-                    )
-                    results = h5todict(
-                        process_file, "/".join((nx_process.name, "results"))
-                    )
-                    processes.append(
-                        _process_desc(
-                            process_order=p_order, configuration=config, results=results
-                        )
+                    processes_to_load[nx_process.name] = h5py_read_dataset(
+                        nx_process["sequence_index"]
                     )
+
+        # load process
+        processes = []
+        for process_name, p_order in processes_to_load.items():
+            config = h5todict(process_file, "/".join((process_name, "configuration")))
+            results = h5todict(process_file, "/".join((process_name, "results")))
+            processes.append(
+                _process_desc(
+                    process_order=p_order, configuration=config, results=results
+                )
+            )
         return processes
 
-    def _set_return_dict(self, return_dict):
-        """
 
-        :param bool return_dict: if True, force the process to return a dict
-                                 instead of a `.TomoBase` object
-        """
-        self._return_dict = return_dict
-
-
-class Task(EwoksTask, BaseProcessInfo):
+class TaskWithProgress(_EwoksTaskWithProgress, BaseProcessInfo):
     """Class from which all tomwer process should inherit
 
     :param logger: the logger used by the class
     """
 
-    def __init__(self, varinfo=None, inputs=None, label=None):
+    def __init__(
+        self,
+        varinfo=None,
+        inputs=None,
+        node_id=None,
+        node_attrs=None,
+        execinfo=None,
+        progress=None,
+    ):
         BaseProcessInfo.__init__(self, inputs=inputs)
-        EwoksTask.__init__(self, varinfo=varinfo, inputs=inputs, label=label)
+        _EwoksTaskWithProgress.__init__(
+            self,
+            varinfo=varinfo,
+            inputs=inputs,
+            node_id=node_id,
+            node_attrs=node_attrs,
+            execinfo=execinfo,
+            progress=progress,
+        )
+
+
+class Task(_EwoksTask, BaseProcessInfo):
+    """Class from which all tomwer process should inherit
+
+    :param logger: the logger used by the class
+    """
+
+    def __init__(
+        self,
+        varinfo=None,
+        inputs=None,
+        node_id=None,
+        node_attrs=None,
+        execinfo=None,
+    ):
+        BaseProcessInfo.__init__(self, inputs=inputs)
+        _EwoksTask.__init__(
+            self,
+            varinfo=varinfo,
+            inputs=inputs,
+            node_id=node_id,
+            node_attrs=node_attrs,
+            execinfo=execinfo,
+        )
