@@ -28,26 +28,25 @@ __license__ = "MIT"
 __date__ = "05/04/2019"
 
 
+import os
 import shutil
 import tempfile
 import unittest
-import numpy
-import os
+
 import h5py
-from tomwer.test.utils import UtilsTest
-from tomwer.core.utils.scanutils import MockEDF, MockHDF5
-from ..reconstruction.axis.axis import AxisProcess
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.process.reconstruction.axis.params import AxisRP
-from tomwer.core.process.reconstruction.axis.mode import AxisMode
-from tomwer.core.scan.edfscan import EDFTomoScan
-from tomwer.core.process.task import Task
+from tomoscan.io import get_swmr_mode
+import numpy
 from silx.io.utils import h5py_read_dataset
 
+from tomwer.core.process.reconstruction.axis.mode import AxisMode
+from tomwer.core.process.reconstruction.axis.params import AxisRP
+from tomwer.core.process.task import Task
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.utils.scanutils import MockEDF, MockNXtomo
+from tomwer.tests.datasets import TomwerCIDatasets
 
-class _AxisDataset:
-    def __init__(self, name):
-        self.name = name
+from ..reconstruction.axis.axis import AxisTask
 
 
 class TestAxisIO(unittest.TestCase):
@@ -78,30 +77,30 @@ class TestAxisIO(unittest.TestCase):
     def testInputOutput(self):
         """Test that io using TomoBase instance work"""
         for input_type in (dict, TomwerScanBase):
-            for return_dict in (True, False):
+            for serialize_output_data in (True, False):
                 with self.subTest(
-                    return_dict=return_dict,
+                    serialize_output_data=serialize_output_data,
                     input_type=input_type,
                 ):
                     input_obj = self.scan
                     if input_obj is dict:
                         input_obj = input_obj.to_dict()
-                    axis_process = AxisProcess(
+                    axis_process = AxisTask(
                         inputs={
                             "axis_params": self.recons_params,
                             "data": input_obj,
-                            "return_dict": return_dict,
+                            "serialize_output_data": serialize_output_data,
                         }
                     )
 
                     # patch the axis process
-                    axis_process._CALCULATIONS_METHODS[
-                        AxisMode.centered
-                    ] = TestAxisIO._random_calc
+                    axis_process._CALCULATIONS_METHODS[AxisMode.centered] = (
+                        TestAxisIO._random_calc
+                    )
 
                     axis_process.run()
                     out = axis_process.outputs.data
-                    if return_dict:
+                    if serialize_output_data:
                         self.assertTrue(isinstance(out, dict))
                     else:
                         self.assertTrue(isinstance(out, TomwerScanBase))
@@ -122,17 +121,18 @@ class TestAxis(unittest.TestCase):
         mock = MockEDF(scan_path=self.tempdir, n_radio=10, n_ini_radio=10)
         scan = EDFTomoScan(mock.scan_path)
         self.recons_params.mode = "centered"
-        axis_process = AxisProcess(
+        axis_process = AxisTask(
             inputs={
                 "data": scan,
                 "axis_params": self.recons_params,
+                "serialize_output_data": False,
             }
         )
 
         axis_process.run()
         self.assertTrue(os.path.exists(scan.process_file))
 
-        with h5py.File(scan.process_file, "r", swmr=True) as h5f:
+        with h5py.File(scan.process_file, "r", swmr=get_swmr_mode()) as h5f:
             self.assertTrue("entry" in h5f)
             self.assertTrue("tomwer_process_0" in h5f["entry"])
             group_axis = h5f["entry"]["tomwer_process_0"]
@@ -143,7 +143,7 @@ class TestAxis(unittest.TestCase):
             axis_value = h5py_read_dataset(group_axis["results"]["center_of_rotation"])
 
         processes = Task.get_processes_frm_type(
-            process_file=scan.process_file, process_type=AxisProcess, entry="entry"
+            process_file=scan.process_file, process_type=AxisTask, entry="entry"
         )
         self.assertEqual(len(processes), 1)
         self.assertEqual(processes[0].results["center_of_rotation"], axis_value)
@@ -164,7 +164,7 @@ class TestAxis(unittest.TestCase):
         """
         self.tempdir = tempfile.mkdtemp()
         dim = 10
-        mock = MockHDF5(
+        mock = MockNXtomo(
             scan_path=self.tempdir, n_proj=10, n_ini_proj=10, scan_range=180, dim=dim
         )
         mock.add_alignment_radio(index=10, angle=90)
@@ -173,17 +173,18 @@ class TestAxis(unittest.TestCase):
         self.recons_params.mode = "centered"
 
         # check data url take
-        axis_process = AxisProcess(
+        axis_process = AxisTask(
             inputs={
                 "data": scan,
                 "axis_params": self.recons_params,
+                "serialize_output_data": False,
             }
         )
         axis_process.run()
         # make sure center of position has been computed
         self.assertTrue(os.path.exists(scan.process_file))
 
-        with h5py.File(scan.process_file, "r", swmr=True) as h5f:
+        with h5py.File(scan.process_file, "r", swmr=get_swmr_mode()) as h5f:
             self.assertTrue("entry" in h5f)
             self.assertTrue("tomwer_process_0" in h5f["entry"])
             group_axis = h5f["entry"]["tomwer_process_0"]
@@ -194,7 +195,7 @@ class TestAxis(unittest.TestCase):
             axis_value = h5py_read_dataset(group_axis["results"]["center_of_rotation"])
         self.assertTrue(-dim / 2 <= axis_value <= dim / 2)
         processes = Task.get_processes_frm_type(
-            process_file=scan.process_file, process_type=AxisProcess
+            process_file=scan.process_file, process_type=AxisTask
         )
         self.assertEqual(len(processes), 1)
         self.assertEqual(processes[0].results["center_of_rotation"], axis_value)
@@ -234,8 +235,8 @@ class TestAxisRP(unittest.TestCase):
             self.axis_rp.set_position_frm_par_file(existing_empty_file, force=False)
         self.assertEqual(self.axis_rp.value_ref_tomwer, old_value)
 
-        valid_par_file = os.path.join(
-            UtilsTest.getEDFDataset("scan_3_"), "scan_3_slice.par"
+        valid_par_file = TomwerCIDatasets.get_dataset(
+            "edf_datasets/scan_3_/scan_3_slice.par"
         )
         assert os.path.isfile(valid_par_file)
         self.axis_rp.set_position_frm_par_file(valid_par_file, force=False)
@@ -255,12 +256,12 @@ class TestSinogramAlgorithm(unittest.TestCase):
     def setUp(self) -> None:
         self.tmp_folder = tempfile.mkdtemp()
         dim = 512
-        self.scan = MockHDF5(
+        self.scan = MockNXtomo(
             scan_path=self.tmp_folder,
             n_proj=10,
             n_ini_proj=10,
             create_ini_dark=True,
-            create_final_ref=True,
+            create_final_flat=True,
             scan_range=360,
             dim=dim,
         ).scan
@@ -270,20 +271,24 @@ class TestSinogramAlgorithm(unittest.TestCase):
 
     def test_growing_window_sinogram(self):
         recons_params = AxisRP()
-        axis_process = AxisProcess(
+        axis_process = AxisTask(
             inputs={"axis_params": recons_params, "data": self.scan}
         )
-        recons_params.mode = AxisMode.growing_window
+        recons_params.mode = AxisMode.growing_window_sinogram
         recons_params.use_sinogram = True
         recons_params.sinogram_subsampling = 1
         axis_process.run()
 
     def test_sliding_window(self):
         recons_params = AxisRP()
-        axis_process = AxisProcess(
-            inputs={"axis_params": recons_params, "data": self.scan}
+        axis_process = AxisTask(
+            inputs={
+                "axis_params": recons_params,
+                "data": self.scan,
+                "serialize_output_data": False,
+            }
         )
-        recons_params.mode = AxisMode.sliding_window
+        recons_params.mode = AxisMode.sliding_window_sinogram
         recons_params.side = "left"
         recons_params.sinogram_subsampling = 1
         recons_params.use_sinogram = True
@@ -291,8 +296,12 @@ class TestSinogramAlgorithm(unittest.TestCase):
 
     def test_sino_coarse_to_fine(self):
         recons_params = AxisRP()
-        axis_process = AxisProcess(
-            inputs={"axis_params": recons_params, "data": self.scan}
+        axis_process = AxisTask(
+            inputs={
+                "axis_params": recons_params,
+                "data": self.scan,
+                "serialize_output_data": False,
+            }
         )
         recons_params.mode = AxisMode.sino_coarse_to_fine
         recons_params.sinogram_subsampling = 1
