@@ -32,16 +32,19 @@ import os
 import shutil
 import tempfile
 import unittest
+
 import numpy
-import time
-from tomwer.core.utils.scanutils import MockEDF, MockHDF5
-from tomwer.core.process.reconstruction.darkref.params import DKRFRP, Method as cMethod
+from tomoscan.esrf.scan.utils import get_data
+
+from tomwer.core.process.reconstruction.darkref.params import DKRFRP
+from tomwer.core.process.reconstruction.darkref.params import ReduceMethod as cMethod
+from tomwer.core.scan.nxtomoscan import NXtomoScan
 from tomwer.core.scan.scanbase import TomwerScanBase
-from ..reconstruction.darkref.darkrefs import DarkRefs
+from tomwer.core.utils.scanutils import MockEDF, MockNXtomo
+from tomwer.tests.datasets import TomwerCIDatasets
+
+from ..reconstruction.darkref.darkrefs import DarkRefsTask
 from ..reconstruction.darkref.darkrefscopy import DarkRefsCopy
-from tomwer.test.utils import UtilsTest
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from silx.io.utils import get_data
 
 
 class TestDarkRefIO(unittest.TestCase):
@@ -53,7 +56,9 @@ class TestDarkRefIO(unittest.TestCase):
         self.scan_edf = MockEDF.mockScan(
             scanID=self.scan_folder, nRadio=10, nRecons=1, nPagRecons=4, dim=10
         )
-        self.mock_hdf5 = MockHDF5(scan_path=self.scan_folder, n_proj=10, n_pag_recons=0)
+        self.mock_hdf5 = MockNXtomo(
+            scan_path=self.scan_folder, n_proj=10, n_pag_recons=0
+        )
         self.scan_hdf5 = self.mock_hdf5.scan
 
         self.recons_params = DKRFRP()
@@ -66,25 +71,25 @@ class TestDarkRefIO(unittest.TestCase):
             (self.scan_edf, self.scan_hdf5), ("edf scan", "hdf5 scan")
         ):
             for input_type in (dict, TomwerScanBase):
-                for return_dict in (True, False):
+                for serialize_output_data in (True, False):
                     with self.subTest(
-                        return_dict=return_dict,
+                        return_dict=serialize_output_data,
                         input_type=input_type,
                         scan_type=scan_type,
                     ):
                         input_obj = scan
                         if input_obj is dict:
                             input_obj = input_obj.to_dict()
-                        process = DarkRefs(
+                        process = DarkRefsTask(
                             inputs={
                                 "dark_ref_params": self.recons_params,
                                 "data": input_obj,
-                                "return_dict": return_dict,
+                                "serialize_output_data": serialize_output_data,
                             }
                         )
                         process.run()
                         out = process.outputs.data
-                        if return_dict:
+                        if serialize_output_data:
                             self.assertTrue(isinstance(out, dict))
                         else:
                             self.assertTrue(isinstance(out, TomwerScanBase))
@@ -99,7 +104,7 @@ class TestDarkRefCopyIO(unittest.TestCase):
         self.scan_edf = MockEDF.mockScan(
             scanID=self.scan_folder, nRadio=10, nRecons=1, nPagRecons=4, dim=10
         )
-        self.scan_hdf5 = MockHDF5(
+        self.scan_hdf5 = MockNXtomo(
             scan_path=self.scan_folder, n_proj=10, n_pag_recons=0
         ).scan
         self.recons_params = DKRFRP()
@@ -113,9 +118,9 @@ class TestDarkRefCopyIO(unittest.TestCase):
             (self.scan_edf, self.scan_hdf5), ("edf scan", "hdf5 scan")
         ):
             for input_type in (dict, TomwerScanBase):
-                for return_dict in (True, False):
+                for serialize_output_data in (True, False):
                     with self.subTest(
-                        return_dict=return_dict,
+                        return_dict=serialize_output_data,
                         input_type=input_type,
                         scan_type=scan_type,
                     ):
@@ -126,13 +131,13 @@ class TestDarkRefCopyIO(unittest.TestCase):
                             inputs={
                                 "dark_ref_params": self.recons_params,
                                 "data": input_obj,
-                                "return_dict": return_dict,
+                                "serialize_output_data": serialize_output_data,
                                 "save_dir": self._save_dir,
                             }
                         )
                         dkrf_process.run()
                         out = dkrf_process.outputs.data
-                        if return_dict:
+                        if serialize_output_data:
                             self.assertTrue(isinstance(out, dict))
                         else:
                             self.assertTrue(isinstance(out, TomwerScanBase))
@@ -145,11 +150,13 @@ class TestDarkRefEdf(unittest.TestCase):
 
     def setUp(self) -> None:
         self.scan_folder = tempfile.mkdtemp()
-        self.dark_data = numpy.array(
-            numpy.random.random(4 * 200 * 200) * 100, numpy.uint16
+        self.dark_data = numpy.array(  # pylint: disable=E1121
+            numpy.random.random(4 * 200 * 200) * 100,
+            dtype=numpy.uint16,
         ).reshape(4, 200, 200)
-        self.flat_data = numpy.array(
-            numpy.random.random(4 * 200 * 200) * 100, numpy.uint32
+        self.flat_data = numpy.array(  # pylint: disable=E1121
+            numpy.random.random(4 * 200 * 200) * 100,
+            dtype=numpy.uint32,
         ).reshape(4, 200, 200)
         self.scan = MockEDF.mockScan(
             scanID=self.scan_folder,
@@ -175,10 +182,10 @@ class TestDarkRefEdf(unittest.TestCase):
         Test darks are computed when only dark are requested
         """
         method_to_test = (
-            cMethod.median,
-            cMethod.average,
-            cMethod.first,
-            cMethod.last,
+            cMethod.MEDIAN,
+            cMethod.MEAN,
+            cMethod.FIRST,
+            cMethod.LAST,
         )
         th_results = (
             numpy.median(self.dark_data, axis=0),
@@ -190,23 +197,19 @@ class TestDarkRefEdf(unittest.TestCase):
             with self.subTest(method=method):
                 if os.path.exists(self.scan.process_file):
                     os.remove(self.scan.process_file)
-                process = DarkRefs(
+                process = DarkRefsTask(
                     inputs={
                         "dark_ref_params": self.recons_params,
                         "data": self.scan,
+                        "serialize_output_data": False,
                     }
                 )
 
                 self.recons_params.dark_calc_method = method
-                self.recons_params.ref_calc_method = cMethod.none
+                self.recons_params.flat_calc_method = cMethod.NONE
                 process.run()
-                self.assertTrue(os.path.exists(self.scan.process_file))
-                darks = DarkRefs.get_darks_frm_process_file(
-                    self.scan.process_file, entry="entry"
-                )
-                self.assertEqual(len(darks), 1)
                 numpy.testing.assert_array_almost_equal(
-                    self.scan.normed_darks[0], th_res.astype(numpy.uint16)
+                    self.scan.reduced_darks[0], th_res.astype(numpy.uint16)
                 )
 
     def testFlat(self):
@@ -214,10 +217,10 @@ class TestDarkRefEdf(unittest.TestCase):
         Test flats are computed when only flat are requested
         """
         method_to_test = (
-            cMethod.median,
-            cMethod.average,
-            cMethod.first,
-            cMethod.last,
+            cMethod.MEDIAN,
+            cMethod.MEAN,
+            cMethod.FIRST,
+            cMethod.LAST,
         )
         th_results = (
             numpy.median(self.flat_data, axis=0),
@@ -229,22 +232,18 @@ class TestDarkRefEdf(unittest.TestCase):
             with self.subTest(method=method):
                 if os.path.exists(self.scan.process_file):
                     os.remove(self.scan.process_file)
-                self.recons_params.dark_calc_method = cMethod.none
-                self.recons_params.ref_calc_method = method
-                process = DarkRefs(
+                self.recons_params.dark_calc_method = cMethod.NONE
+                self.recons_params.flat_calc_method = method
+                process = DarkRefsTask(
                     inputs={
                         "dark_ref_params": self.recons_params,
                         "data": self.scan,
+                        "serialize_output_data": False,
                     }
                 )
                 process.run()
-                self.assertTrue(os.path.exists(self.scan.process_file))
-                flats = DarkRefs.get_flats_frm_process_file(
-                    self.scan.process_file, entry="entry"
-                )
-                self.assertEqual(len(flats), 1)
                 numpy.testing.assert_array_almost_equal(
-                    self.scan.normed_flats[0], th_res.astype(numpy.uint16)
+                    self.scan.reduced_flats[0], th_res.astype(numpy.uint16)
                 )
 
 
@@ -259,9 +258,10 @@ class TestDarkRefNx(unittest.TestCase):
         self.scan_folder = tempfile.mkdtemp()
         self._file_path = os.path.join(self.scan_folder, dataset_name)
         shutil.copyfile(
-            src=UtilsTest.getH5Dataset(folderID=dataset_name), dst=self._file_path
+            src=TomwerCIDatasets.get_dataset(f"h5_datasets/{dataset_name}"),
+            dst=self._file_path,
         )
-        self.scan = HDF5TomoScan(scan=self._file_path, entry="entry0000")
+        self.scan = NXtomoScan(scan=self._file_path, entry="entry0000")
         self.assertFalse(os.path.exists(self.scan.process_file))
         self.recons_params = DKRFRP()
         self.recons_params.overwrite_dark = True
@@ -277,33 +277,24 @@ class TestDarkRefNx(unittest.TestCase):
         """
         darks = self.scan.darks
         self.assertEqual(len(darks), 1)
-        dark_data = get_data(list(darks.values())[0])
 
-        method_to_test = (cMethod.average, cMethod.median, cMethod.last, cMethod.last)
+        method_to_test = (cMethod.MEAN, cMethod.MEDIAN, cMethod.FIRST, cMethod.LAST)
         for method in method_to_test:
             with self.subTest(method=method):
                 self.recons_params.dark_calc_method = method
-                self.recons_params.ref_calc_method = cMethod.none
-                process = DarkRefs(
+                self.recons_params.flat_calc_method = cMethod.NONE
+                process = DarkRefsTask(
                     inputs={
                         "dark_ref_params": self.recons_params,
                         "data": self.scan,
+                        "serialize_output_data": False,
                     }
                 )
-                if os.path.exists(self.scan.process_file):
-                    os.remove(self.scan.process_file)
-
                 process.run()
-                self.assertTrue(os.path.exists(self.scan.process_file))
-                darks = DarkRefs.get_darks_frm_process_file(
-                    self.scan.process_file, entry=self.scan.entry
-                )
-                self.assertEqual(len(darks), 1)
-                self.assertTrue(numpy.allclose(darks[0], dark_data))
-                flats = DarkRefs.get_flats_frm_process_file(
-                    self.scan.process_file, entry=self.scan.entry
-                )
+                flats = self.scan.load_reduced_flats()
                 self.assertEqual(len(flats), 0)
+                darks = self.scan.load_reduced_darks()
+                self.assertEqual(len(darks), 1)
 
     def testFlat(self):
         """
@@ -321,41 +312,38 @@ class TestDarkRefNx(unittest.TestCase):
         self.recons_params.overwrite_dark = True
         self.recons_params.overwrite_ref = True
 
-        method_to_test = (cMethod.average, cMethod.median, cMethod.last, cMethod.last)
+        method_to_test = (cMethod.MEAN, cMethod.MEDIAN, cMethod.FIRST, cMethod.LAST)
         for method in method_to_test:
             with self.subTest(method=method):
-                if method is cMethod.median:
+                if method is cMethod.MEDIAN:
                     expected_res_s1 = numpy.median(data_flat_serie_1, axis=0)
                     expected_res_s2 = numpy.median(data_flat_serie_2, axis=0)
-                elif method is cMethod.average:
+                elif method is cMethod.MEAN:
                     expected_res_s1 = numpy.mean(data_flat_serie_1, axis=0)
                     expected_res_s2 = numpy.mean(data_flat_serie_2, axis=0)
-                elif method is cMethod.first:
+                elif method is cMethod.FIRST:
                     expected_res_s1 = data_flat_serie_1[0]
                     expected_res_s2 = data_flat_serie_2[0]
-                elif method is cMethod.last:
+                elif method is cMethod.LAST:
                     expected_res_s1 = data_flat_serie_1[-1]
                     expected_res_s2 = data_flat_serie_2[-1]
                 else:
                     raise ValueError("method not managed")
 
-                self.recons_params.dark_calc_method = cMethod.none
-                self.recons_params.ref_calc_method = method
-                process = DarkRefs(
+                self.recons_params.dark_calc_method = cMethod.NONE
+                self.recons_params.flat_calc_method = method
+                process = DarkRefsTask(
                     inputs={
                         "dark_ref_params": self.recons_params,
                         "data": self.scan,
+                        "serialize_output_data": False,
                     }
                 )
                 process.run()
-                self.assertTrue(os.path.exists(self.scan.process_file))
-                darks = DarkRefs.get_darks_frm_process_file(
-                    self.scan.process_file, entry=self.scan.entry
-                )
+
+                darks = self.scan.load_reduced_darks()
                 self.assertEqual(len(darks), 0)
-                flats = DarkRefs.get_flats_frm_process_file(
-                    self.scan.process_file, entry=self.scan.entry
-                )
+                flats = self.scan.load_reduced_flats()
                 self.assertEqual(len(flats), 2)
                 self.assertTrue(1 in flats)
                 self.assertTrue(1521 in flats)
@@ -375,29 +363,25 @@ class TestDarkRefNx(unittest.TestCase):
         data_flat_serie_1 = [get_data(url) for url in url_flat_serie_1]
         data_flat_serie_2 = [get_data(url) for url in url_flat_serie_2]
 
-        self.recons_params.dark_calc_method = cMethod.average
-        self.recons_params.ref_calc_method = cMethod.median
+        self.recons_params.dark_calc_method = cMethod.MEAN
+        self.recons_params.flat_calc_method = cMethod.MEDIAN
         darks = self.scan.darks
         self.assertEqual(len(darks), 1)
         dark_data = get_data(list(darks.values())[0])
 
         expected_flats_s1 = numpy.median(data_flat_serie_1, axis=0)
         expected_flats_s2 = numpy.median(data_flat_serie_2, axis=0)
-        process = DarkRefs(
+        process = DarkRefsTask(
             inputs={
                 "dark_ref_params": self.recons_params,
                 "data": self.scan,
+                "serialize_output_data": False,
             }
         )
         process.run()
-        self.assertTrue(os.path.exists(self.scan.process_file))
-        darks = DarkRefs.get_darks_frm_process_file(
-            self.scan.process_file, entry=self.scan.entry
-        )
+        darks = self.scan.load_reduced_darks()
+        flats = self.scan.load_reduced_flats()
         self.assertEqual(len(darks), 1)
-        flats = DarkRefs.get_flats_frm_process_file(
-            self.scan.process_file, entry=self.scan.entry
-        )
         self.assertEqual(len(flats), 2)
         self.assertTrue(0 in darks)
         self.assertTrue(1 in flats)
@@ -405,140 +389,3 @@ class TestDarkRefNx(unittest.TestCase):
         self.assertTrue(numpy.allclose(flats[1], expected_flats_s1))
         self.assertTrue(numpy.allclose(flats[1521], expected_flats_s2))
         self.assertTrue(numpy.allclose(darks[0], dark_data))
-
-    def testReprocessing(self):
-        """
-        Make sure computation is executed only when necessary
-        """
-        self.assertEqual(self.scan.normed_darks, None)
-        self.assertEqual(self.scan.normed_flats, None)
-        process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "data": self.scan,
-            }
-        )
-        process.run()
-        self.assertEqual(len(self.scan.normed_darks), 1)
-        self.assertEqual(len(self.scan.normed_flats), 2)
-
-        # test behavior if overwrite is False
-        self.recons_params.overwrite_dark = False
-        self.recons_params.overwrite_ref = False
-        self.scan.set_normed_darks(None)
-        self.scan.set_normed_flats(None)
-        timestamp_tomwer_processes = os.path.getmtime(self.scan.process_file)
-        time.sleep(1)
-        process.run()
-        self.assertEqual(
-            timestamp_tomwer_processes, os.path.getmtime(self.scan.process_file)
-        )
-        self.assertEqual(len(self.scan.normed_darks), 1)
-        self.assertEqual(len(self.scan.normed_flats), 2)
-
-        # test behavior if overwrite is True
-        self.recons_params.overwrite_dark = True
-        self.recons_params.overwrite_ref = True
-        self.scan.set_normed_darks(None)
-        self.scan.set_normed_flats(None)
-
-        timestamp_tomwer_processes = os.path.getmtime(self.scan.process_file)
-        time.sleep(1)
-        process.run()
-        self.assertNotEqual(
-            timestamp_tomwer_processes, os.path.getmtime(self.scan.process_file)
-        )
-        self.assertEqual(len(self.scan.normed_darks), 1)
-        self.assertEqual(len(self.scan.normed_flats), 2)
-
-
-class TestDarkRefCopyNx(unittest.TestCase):
-    """
-    Test dark ref is correctly processing with h5 refs
-    """
-
-    def setUp(self) -> None:
-        dataset_name = "frm_edftomomill_twoentries.nx"
-        self.scan_folder = tempfile.mkdtemp()
-        self._file_path = os.path.join(self.scan_folder, dataset_name)
-        shutil.copyfile(
-            src=UtilsTest.getH5Dataset(folderID=dataset_name), dst=self._file_path
-        )
-        self.scan_with_ref = HDF5TomoScan(scan=self._file_path, entry="entry0000")
-        self.recons_params = DKRFRP()
-        self.assertFalse(os.path.exists(self.scan_with_ref.process_file))
-
-        mock = MockHDF5(
-            scan_path=self.scan_folder,
-            n_proj=10,
-            n_ini_proj=2,
-            n_pag_recons=0,
-            create_ini_dark=False,
-            create_ini_ref=False,
-            dim=20,
-        )
-        self.scan_without_ref = mock.scan
-        # make sure no ref and no dark registered
-        assert len(self.scan_without_ref.darks) == 0
-        assert len(self.scan_with_ref.darks) > 0
-        assert len(self.scan_without_ref.flats) == 0
-        assert len(self.scan_with_ref.flats) > 0
-        assert self.scan_without_ref.normed_flats is None
-        assert self.scan_with_ref.normed_flats is None
-        assert self.scan_without_ref.normed_darks is None
-        assert self.scan_with_ref.normed_darks is None
-        self._save_dir = tempfile.mkdtemp()
-
-    def tearDown(self) -> None:
-        shutil.rmtree(self.scan_folder)
-
-    def testCopyAuto(self):
-        """Test copy in mode auto"""
-
-        # step 1: check that ref will be registered
-        copy_process = DarkRefsCopy(
-            inputs={
-                "dar_ref_params": self.recons_params,
-                "data": self.scan_with_ref,
-                "mode_auto": True,
-                "save_dir": self._save_dir,
-            }
-        )
-        self.assertTrue(copy_process.is_on_mode_auto)
-        copy_process.run()
-        self.assertTrue(copy_process.is_on_mode_auto)
-
-        self.assertFalse(self.scan_with_ref.normed_flats is None)
-        self.assertEqual(len(self.scan_with_ref.normed_flats), 2)
-        self.assertFalse(self.scan_with_ref.normed_darks is None)
-        self.assertEqual(len(self.scan_with_ref.normed_darks), 1)
-        self.assertEqual(copy_process.has_dark_stored(), True)
-        self.assertEqual(copy_process.has_flat_stored(), True)
-        self.assertTrue(os.path.exists(copy_process._dark_save_file))
-        self.assertTrue(os.path.exists(copy_process._ref_save_file))
-
-        # step 2: check that registered ref will be copied
-        copy_process = DarkRefsCopy(
-            inputs={
-                "dar_ref_params": self.recons_params,
-                "data": self.scan_without_ref,
-                "mode_auto": True,
-                "save_dir": self._save_dir,
-            }
-        )
-        self.assertTrue(os.path.exists(copy_process._dark_save_file))
-        self.assertTrue(os.path.exists(copy_process._ref_save_file))
-        copy_process.run()
-        self.assertFalse(self.scan_without_ref.normed_flats is None)
-        self.assertEqual(len(self.scan_without_ref.normed_flats), 2)
-        self.assertFalse(self.scan_without_ref.normed_darks is None)
-        self.assertEqual(len(self.scan_without_ref.normed_darks), 1)
-        self.assertTrue(os.path.isfile(self.scan_with_ref.process_file))
-        darks = DarkRefs.get_darks_frm_process_file(
-            self.scan_without_ref.process_file, entry=self.scan_without_ref.entry
-        )
-        self.assertEqual(len(darks), 1)
-        flats = DarkRefs.get_flats_frm_process_file(
-            self.scan_without_ref.process_file, entry=self.scan_without_ref.entry
-        )
-        self.assertEqual(len(flats), 2)
