@@ -28,26 +28,31 @@ __license__ = "MIT"
 __date__ = "05/04/2019"
 
 
-import unittest
-import tempfile
-import shutil
-import time
+import gc
 import os
+import shutil
+import tempfile
+import time
+import unittest
+
 import h5py
 import numpy
-
-from tomoscan.io import HDF5File
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.utils.scanutils import MockEDF
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.process.control.scantransfer import ScanTransfer
-from tomwer.test.utils import UtilsTest
-from tomwer.synctools.rsyncmanager import RSyncManager
-from tomwer.core.process.control.datalistener import DataListener
-from nxtomomill.converter import h5_to_nx
 import pytest
+from nxtomomill.converter import from_h5_to_nx
+from nxtomomill.io.config.hdf5config import TomoHDF5Config
+from nxtomo.nxobject.nxdetector import ImageKey
+
+from silx.io.utils import open as open_hdf5
+from tomoscan.io import HDF5File
 from tomoscan.validator import is_valid_for_reconstruction
-from tomoscan.esrf.hdf5scan import ImageKey
+
+from tomwer.core.process.control.datalistener import DataListener
+from tomwer.core.process.control.scantransfer import ScanTransferTask
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.utils.scanutils import MockEDF
+from tomwer.synctools.rsyncmanager import RSyncManager
+from tomwer.tests.datasets import TomwerCIDatasets
 
 
 class TestDataTransferIO(unittest.TestCase):
@@ -67,9 +72,9 @@ class TestDataTransferIO(unittest.TestCase):
     def testInputOutput(self):
         """Test that io using TomoBase instance work"""
         for input_type in (dict, TomwerScanBase):
-            for return_dict in (True, False):
+            for serialize_output_data in (True, False):
                 with self.subTest(
-                    return_dict=return_dict,
+                    return_dict=serialize_output_data,
                     input_type=input_type,
                 ):
                     output_folder = tempfile.mkdtemp()
@@ -88,11 +93,11 @@ class TestDataTransferIO(unittest.TestCase):
                         dim=10,
                     )
 
-                    transfert_process = ScanTransfer(
+                    transfert_process = ScanTransferTask(
                         inputs={
                             "data": scan,
                             "dest_dir": output_folder,
-                            "return_dict": return_dict,
+                            "serialize_output_data": serialize_output_data,
                         }
                     )
 
@@ -102,7 +107,7 @@ class TestDataTransferIO(unittest.TestCase):
                         input_obj = input_obj.to_dict()
                     transfert_process.run()
                     out = transfert_process.outputs.data
-                    if return_dict:
+                    if serialize_output_data:
                         self.assertTrue(isinstance(out, dict))
                     else:
                         self.assertTrue(isinstance(out, TomwerScanBase))
@@ -116,7 +121,7 @@ class TestBlissDataTransfer(unittest.TestCase):
         self.input_dir = tempfile.mkdtemp()
         self.output_dir = tempfile.mkdtemp()
         shutil.copytree(
-            UtilsTest.getBlissDataset(folderID="sample"),
+            TomwerCIDatasets.get_dataset("bliss/sample"),
             os.path.join(self.input_dir, "sample"),
         )
 
@@ -145,6 +150,7 @@ class TestBlissDataTransfer(unittest.TestCase):
     def tearDown(self):
         shutil.rmtree(self.input_dir)
         shutil.rmtree(self.output_dir)
+        gc.collect()
 
     def testDataTransfer(self):
         """Make sure the data transfer is able to retrieve the scan,
@@ -152,10 +158,6 @@ class TestBlissDataTransfer(unittest.TestCase):
         Check that only the specific scan folders will be copy and removed
         and the other won't be affected.
         """
-        out_nx = os.path.join(
-            self.output_dir, "sample_29042021", "sample_29042021_1_1.nx"
-        )
-        self.assertFalse(os.path.exists(out_nx))
         out_proposal = os.path.join(self.output_dir, "ihpayno_sample.h5")
         self.assertFalse(os.path.exists(out_proposal))
         out_sample_file = os.path.join(
@@ -175,7 +177,7 @@ class TestBlissDataTransfer(unittest.TestCase):
         for scan_path in out_not_included_scans:
             self.assertFalse(os.path.exists(scan_path))
 
-        process = ScanTransfer(
+        process = ScanTransferTask(
             inputs={
                 "data": self.scan,
                 "block": True,
@@ -185,8 +187,7 @@ class TestBlissDataTransfer(unittest.TestCase):
         process.run()
 
         time.sleep(1)
-        self.assertTrue(os.path.exists(out_nx), f"{out_nx} does not exists")
-        # self.assertTrue(os.path.exists(out_proposal), f"{out_proposal} does not exists")
+        self.assertTrue(os.path.exists(out_proposal), f"{out_proposal} does not exists")
         self.assertTrue(
             os.path.exists(out_sample_file), f"{out_sample_file} does not exists"
         )
@@ -195,17 +196,17 @@ class TestBlissDataTransfer(unittest.TestCase):
         for scan_path in out_not_included_scans:
             self.assertFalse(os.path.exists(scan_path))
 
-        # insure we can convert it using nxtomomill once converted
+        # ensure we can convert it using nxtomomill once transferred
         output_file = out_sample_file.replace("29042021.h5", "29042021.nx")
         assert output_file != out_sample_file
-        h5_to_nx(
-            input_file_path=out_sample_file,
-            output_file=output_file,
-            single_file=True,
-            file_extension=".nx",
-        )
+        configuration = TomoHDF5Config()
+        configuration.input_file = out_sample_file
+        configuration.output_file = output_file
+        configuration.single_file = True
+        configuration.file_extension = ".nx"
+        from_h5_to_nx(configuration, input_callback=lambda *arg: 0)
         assert os.path.exists(output_file)
-        assert is_valid_for_reconstruction(HDF5TomoScan(output_file, entry="entry0000"))
+        assert is_valid_for_reconstruction(NXtomoScan(output_file, entry="entry0000"))
 
 
 @pytest.mark.skipif(not RSyncManager().has_rsync(), reason="requires rsync")
@@ -291,7 +292,7 @@ class NXTomoDataTransferBase(unittest.TestCase):
             entry_node["x_pixel_size"] = 3.25e-7
             entry_node["y_pixel_size"] = 3.25e-7
 
-            # write beam: add an absolute link for energy to insure this is style valid and that relative files won't be moved
+            # write beam: add an absolute link for energy to ensure this is style valid and that relative files won't be moved
             file_with_energy = os.path.join(self._src_dir, "energy.h5")
             with HDF5File(file_with_energy, mode="w") as energy_file:
                 energy_file["energy"] = 19
@@ -319,10 +320,10 @@ class NXTomoDataTransferBase(unittest.TestCase):
         shutil.rmtree(self._dst_dir)
         return super().tearDown()
 
-    def test(self):
-        src_scan = HDF5TomoScan(self.nexus_file_path, entry="entry0000")
+    def _test(self):
+        src_scan = NXtomoScan(self.nexus_file_path, entry="entry0000")
         assert is_valid_for_reconstruction(src_scan)
-        process = ScanTransfer(
+        process = ScanTransferTask(
             inputs={
                 "data": src_scan,
                 "turn_off_print": True,
@@ -335,23 +336,23 @@ class NXTomoDataTransferBase(unittest.TestCase):
         dst_scan = process.outputs.data
         assert dst_scan.master_file != self.nexus_file_path
         assert is_valid_for_reconstruction(dst_scan, check_values=True)
-        with HDF5File(src_scan.master_file, mode="r") as h5s_src:
+        with open_hdf5(src_scan.master_file) as h5s_src:
             src_dataset = h5s_src["entry0000/detector/data"][...]
-        with HDF5File(dst_scan.master_file, mode="r") as h5s_dst:
+        with open_hdf5(dst_scan.master_file) as h5s_dst:
             dst_dataset = h5s_dst["entry0000/detector/data"][...]
         assert numpy.array_equal(src_dataset, dst_dataset)
 
 
 @pytest.mark.skipif(not RSyncManager().has_rsync(), reason="requires rsync")
 class TestNXtomoTransferRelativeLink(NXTomoDataTransferBase):
-    """Insure NXtomo conversion will work if it contains relative links"""
+    """Ensure NXtomo conversion will work if it contains relative links"""
 
     def setUp(self) -> None:
         super().setUp()
         self.create_nexus_file(relative_links=True)
 
     def test(self):
-        super().test()
+        super()._test()
         # make sure there is the .nx file there and the folder containing file linked the relative
         assert (
             len(
@@ -363,14 +364,14 @@ class TestNXtomoTransferRelativeLink(NXTomoDataTransferBase):
 
 @pytest.mark.skipif(not RSyncManager().has_rsync(), reason="requires rsync")
 class TestNXtomoTransferAbsoluteLink(NXTomoDataTransferBase):
-    """Insure NXtomo conversion will work if it contains absolute links"""
+    """Ensure NXtomo conversion will work if it contains absolute links"""
 
     def setUp(self) -> None:
         super().setUp()
         self.create_nexus_file(relative_links=False)
 
     def test(self):
-        super().test()
+        super()._test()
         # make sure there is only the .nx file there
         assert (
             len(
