@@ -28,50 +28,55 @@ __license__ = "MIT"
 __date__ = "05/04/2019"
 
 
-import shutil
-import tempfile
-import unittest
 import os
 import time
+
 import pytest
-from tomwer.core.utils.scanutils import MockEDF
+
 from tomwer.core.process.control.datawatcher import DataWatcher
+from tomwer.core.process.control.datawatcher.status import DET_END_XML, PARSE_INFO_FILE
+from tomwer.core.utils.scanutils import MockEDF
+from tomwer.core.utils.threads import LoopThread
 
 
-class TestDataWatcherIO(unittest.TestCase):
+@pytest.mark.skipif(os.name != "posix", reason="not tested under windows yet")
+@pytest.mark.parametrize("det_method", (PARSE_INFO_FILE, DET_END_XML))
+def test_data_watcher_io(tmp_path, det_method):
     """Test inputs and outputs types of the handler functions"""
+    scan_folder = tmp_path / "folder" / "my_scan"
+    data_watcher_process = DataWatcher()
+    data_watcher_process.setWaitTimeBtwLoop(1)
+    data_watcher_process.setObsMethod(det_method)
 
-    def setUp(self):
-        self.scan_folder = tempfile.mkdtemp()
-        self.data_watcher_process = None
-        self.scan = MockEDF.mockScan(
-            scanID=self.scan_folder, nRadio=10, nRecons=1, nPagRecons=4, dim=10
-        )
+    MockEDF.mockScan(
+        scanID=str(scan_folder), nRadio=10, nRecons=1, nPagRecons=4, dim=10
+    )
+    data_watcher_process.setFolderObserved(str(tmp_path / "folder"))
+    data_watcher_process.clear_output_values()
+    data_watcher_process.set_serialize_output_data(True)
+    LoopThread.quitEvent.clear()
+    data_watcher_process.start()
 
-    def tearDown(self):
-        if self.data_watcher_process is not None:
-            self.data_watcher_process.stop()
-        shutil.rmtree(self.scan_folder)
+    # note: this quitEvent is nasty. But we don't want to spend time on the (EDF) data watcher which is not the future
 
-    @pytest.mark.skipif(os.name != "posix", reason="not tested under windows yet")
-    def testOutputDict(self):
-        self.data_watcher_process = DataWatcher()
-        self.data_watcher_process.setFolderObserved(self.scan_folder)
-        self.data_watcher_process.setWaitTimeBtwLoop(1)
+    timeout = 8
+    while data_watcher_process.get_output_value("data") is None and timeout > 0:
+        _delta = 0.5
+        time.sleep(_delta)
+        timeout -= _delta
 
-        self.data_watcher_process.clear_output_values()
-        self.data_watcher_process._set_return_dict(True)
-        self.data_watcher_process.start()
+    def quit_test():
+        data_watcher_process.stop()
+        data_watcher_process.waitForObservationFinished()
+        import gc
 
-        timeout = 8
-        while (
-            self.data_watcher_process.get_output_value("data") is None and timeout > 0
-        ):
-            _delta = 0.5
-            time.sleep(_delta)
-            timeout -= _delta
+        gc.collect()
+        LoopThread.quitEvent.clear()
 
-        if timeout <= 0:
-            self.fail("timeout expire")
-        out = self.data_watcher_process.get_output_value("data")
-        self.assertTrue(isinstance(out, dict))
+    if timeout <= 0:
+        quit_test()
+        raise TimeoutError("timeout expire")
+
+    out = data_watcher_process.get_output_value("data")
+    assert isinstance(out, dict)
+    quit_test()
