@@ -30,17 +30,18 @@ __date__ = "14/02/2020"
 import os
 import shutil
 import tempfile
-from glob import glob
 import unittest
-from tomwer.test.utils import UtilsTest
-from tomwer.core.process.reconstruction.nabu import nabuslices as nabu
-from tomwer.core.process.reconstruction.nabu import utils as nabuutils
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSliceMode
-from tomwer.core.utils.scanutils import MockEDF
-from tomwer.core.scan.edfscan import EDFTomoScan
+from glob import glob
+
 from tomwer.core.process.reconstruction.axis import AxisRP
+from tomwer.core.process.reconstruction.nabu import nabuslices as nabu
 from tomwer.core.process.reconstruction.nabu import settings as nabu_settings
+from tomwer.core.process.reconstruction.nabu import utils as nabuutils
+from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSliceMode
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils.scanutils import MockEDF
+from tomwer.tests.datasets import TomwerCIDatasets
 
 
 class TestNabuDescUtils(unittest.TestCase):
@@ -50,7 +51,7 @@ class TestNabuDescUtils(unittest.TestCase):
         unittest.TestCase.setUp(self)
         self.top_dir = tempfile.mkdtemp()
         self.dataset = "scan_3_"
-        data_test_dir = UtilsTest.getEDFDataset(self.dataset)
+        data_test_dir = TomwerCIDatasets.get_dataset(f"edf_datasets/{self.dataset}")
         scan_folder = os.path.join(self.top_dir, self.dataset)
         shutil.copytree(data_test_dir, scan_folder)
         self.scan = ScanFactory.create_scan_object(scan_folder)
@@ -72,7 +73,7 @@ class TestNabuIni(unittest.TestCase):
     def setUp(self) -> None:
         self.dataset = "scan_3_"
         self._root_dir = tempfile.mkdtemp()
-        data_test_dir = UtilsTest.getEDFDataset(self.dataset)
+        data_test_dir = TomwerCIDatasets.get_dataset("edf_datasets/scan_3_")
         scan_folder = os.path.join(self._root_dir, self.dataset)
         shutil.copytree(data_test_dir, scan_folder)
         self.scan = EDFTomoScan(scan_folder)
@@ -126,7 +127,7 @@ class TestNabuIni(unittest.TestCase):
         # check volume and slice .cfg files for nabu reconstruction are here
         for file_name in (
             os.path.basename(self.scan.path) + ".cfg",
-            os.path.basename(self.scan.path) + "slice_0001" + ".cfg",
+            os.path.basename(self.scan.path) + "slice_000001_plane_XY" + ".cfg",
         ):
             with self.subTest(file_name=file_name):
                 self.assertTrue(
@@ -150,7 +151,9 @@ class TestNabuIni(unittest.TestCase):
         # check volume and slice .cfg files for nabu reconstruction are here
         for file_name in (
             os.path.basename(self.scan.path) + "pag_db0100.cfg",
-            os.path.basename(self.scan.path) + "slice_pag_0001_db0100" + ".cfg",
+            os.path.basename(self.scan.path)
+            + "slice_pag_000001_db0100_plane_XY"
+            + ".cfg",
         ):
             with self.subTest(file_name=file_name):
                 self.assertTrue(
@@ -177,11 +180,14 @@ class TestNabuIni(unittest.TestCase):
             ini_files.append(
                 os.path.basename(self.scan.path)
                 + "slice_"
-                + str(slice_index).zfill(4)
+                + str(slice_index).zfill(6)
+                + "_plane_XY"
                 + ".cfg"
             )
 
+        print("nabu_files are", nabu_files)
         for ini_file in ini_files:
+            print("check ini_file", ini_file)
             with self.subTest(file_name=ini_file):
                 self.assertTrue(
                     os.path.join(self._nabu_cfg_folder, ini_file) in nabu_files
@@ -219,10 +225,12 @@ class TestNabuIni(unittest.TestCase):
                 + ".cfg"
             )
             # add slice ini file
+            # note: plane is only specify for slice.
             ini_files.append(
                 os.path.basename(self.scan.path)
-                + "slice_pag_0001_db"
+                + "slice_pag_000001_db"
                 + str(int(pag_db)).zfill(4)
+                + "_plane_XY"
                 + ".cfg"
             )
 
@@ -264,9 +272,10 @@ class TestNabuIni(unittest.TestCase):
                 ini_files.append(
                     os.path.basename(self.scan.path)
                     + "slice_pag_"
-                    + str(slice_index).zfill(4)
+                    + str(slice_index).zfill(6)
                     + "_db"
                     + str(pag_db).zfill(4)
+                    + "_plane_XY"
                     + ".cfg"
                 )
 
@@ -300,9 +309,9 @@ class TestTomwerConfForNabu(unittest.TestCase):
 
     def testSlices(self):
         # check if single value
-        for slice in ("1", NabuSliceMode.MIDDLE):
-            with self.subTest(slice=slice):
-                self.conf["tomwer_slices"] = slice
+        for data_slice in ("1", NabuSliceMode.MIDDLE):
+            with self.subTest(data_slice=data_slice):
+                self.conf["tomwer_slices"] = data_slice
 
                 sub_confs = nabu.interpret_tomwer_configuration(
                     self.conf, scan=self.scan
@@ -312,9 +321,9 @@ class TestTomwerConfForNabu(unittest.TestCase):
                 self.assertEqual(len(sub_confs), 1 + 1)
 
         # check single values or list of values
-        for slice in ("[2, 3, 7]", "(6, 9,56)", "5;6; 9"):
-            with self.subTest(slice=slice):
-                self.conf["tomwer_slices"] = slice
+        for data_slice in ("[2, 3, 7]", "(6, 9,56)", "5;6; 9"):
+            with self.subTest(slice=data_slice):
+                self.conf["tomwer_slices"] = data_slice
 
                 sub_confs = nabu.interpret_tomwer_configuration(
                     self.conf, scan=self.scan
@@ -355,7 +364,7 @@ class TestNabuAndAxis(unittest.TestCase):
     def setUp(self) -> None:
         self.dataset = "scan_3_"
         self._root_dir = tempfile.mkdtemp()
-        data_test_dir = UtilsTest.getEDFDataset(self.dataset)
+        data_test_dir = TomwerCIDatasets.get_dataset("edf_datasets/scan_3_")
         scan_folder = os.path.join(self._root_dir, self.dataset)
         shutil.copytree(data_test_dir, scan_folder)
         self.scan = EDFTomoScan(scan_folder)
@@ -364,9 +373,14 @@ class TestNabuAndAxis(unittest.TestCase):
         # initial configuration
         self.conf = get_nabu_default_config(output_dir=self.scan.path)
 
-        self.nabu_process = nabu.NabuSlices(inputs={"data": self.scan})
-        self.nabu_process.set_dry_run(True)
-        self.nabu_process.set_configuration(self.conf)
+        self.nabu_process = nabu.NabuSlicesTask(
+            inputs={
+                "data": self.scan,
+                "nabu_params": self.conf,
+                "dry_run": True,
+                "serialize_output_data": False,
+            },
+        )
         self.scan._axis_params = AxisRP()
         self.nabu_cfg_folder = os.path.join(
             self.scan.path, nabu_settings.NABU_CFG_FILE_FOLDER
@@ -440,10 +454,10 @@ def get_nabu_default_config(output_dir):
             "log_min_clip": 1e-06,
             "ccd_filter_enabled": 0,
             "log_max_clip": 10.0,
-            "flatfield_enabled": 1,
+            "flatfield": 1,
         },
         "dataset": {
-            "project_name": "hair_A1_50nm_tomo3_1_",
+            "job_name": "hair_A1_50nm_tomo3_1_",
             "location": "/tmp_14_days/payno/hair_A1_50nm_tomo3_1_",
             "flat_file_prefix": "refHST",
             "projections_subsampling": 1,
