@@ -28,52 +28,22 @@ __license__ = "MIT"
 __date__ = "04/03/2019"
 
 
-from collections import OrderedDict
 import datetime
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.blissscan import BlissScan
+import logging
+import typing
+from collections import OrderedDict
+
+import h5py
 from silx.io.url import DataUrl
 from silx.io.utils import h5py_read_dataset
+
 from tomwer.core.process.reconstruction.scores import ComputedScore
-import h5py
-import typing
-import logging
+from tomwer.core.scan.blissscan import BlissScan
+from tomwer.core.scan.scanbase import TomwerScanBase
 
 _logger = logging.getLogger(__name__)
 
 
-def _assert_param_instance(param, valid_instances):
-    """check the parameter value, can be an instance of valid_instance or
-    an iterable of valid_instance"""
-    if isinstance(param, valid_instances):
-        return
-    else:
-        try:
-            for unique_param in param:
-                if not isinstance(unique_param, valid_instances):
-                    assert isinstance(unique_param, valid_instances)
-        except Exception:
-            raise ValueError("%s has an invalid type" % param)
-
-
-def _assert_cast_to_boolean(value):
-    """
-
-    :param value: value we want to cast to a boolean
-    """
-    try:
-        bool(int(value))
-    except Exception:
-        raise ValueError("%s cannot be cast to boolean" % value)
-
-
-def _assert_cast_to_int(value):
-    try:
-        int(value)
-    except Exception:
-        raise ValueError("%s cannot be cast to int" % value)
-
-
 class LastReceivedScansDict(OrderedDict):
     """List the received scan from the first received to the last received"""
 
@@ -138,9 +108,7 @@ def get_scores(node: h5py.Group) -> typing.Union[None, dict]:
                 )
                 scores[float(res_item)] = (url, score)
             except Exception as e:
-                _logger.warning(
-                    "Unable to load {}. Reason is {}".format(score_node.name, str(e))
-                )
+                _logger.warning(f"Unable to load {score_node.name}. Reason is {e}")
         scores = dict(sorted(scores.items(), key=lambda item: item[0]))
 
         return scores
