@@ -22,10 +22,6 @@
 #
 #############################################################################*/
 
-"""This module analyze headDir data directory
-   to detect scan to be reconstructed
-"""
-
 __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "23/08/2021"
@@ -35,5 +31,9 @@ from ewokscore.task import Task as EwoksTask
 
 
 class _RadioStackPlaceHolder(EwoksTask, input_names=("data",)):
+    """
+    Task to aggregate radios of several scans and allow to browse along them all
+    """
+
     def run(self):
         pass
