@@ -22,9 +22,6 @@
 #
 #############################################################################*/
 
-"""This module analyze headDir data directory
-   to detect scan to be reconstructed
-"""
 
 __authors__ = ["H. Payno"]
 __license__ = "MIT"
@@ -35,5 +32,9 @@ from ewokscore.task import Task as EwoksTask
 
 
 class _SliceStackPlaceHolder(EwoksTask, input_names=("data",)):
+    """
+    Task to aggregate reconstructed slices of several scans and allow users to along them
+    """
+
     def run(self):
         pass
