@@ -29,6 +29,7 @@ __date__ = "09/08/2018"
 
 
 import os
+
 import h5py
 
 try:
@@ -39,14 +40,21 @@ try:
     from nxtomomill.settings import Tomo
 
     H5_ZSERIE_INIT_TITLES = Tomo.H5.ZSERIE_INIT_TITLES
+
+    H5_PCOTOMO_INIT_TITLES = Tomo.H5.PCOTOMO_INIT_TITLES
+
 except ImportError:
     # old settings names
+    # TODO: remove in 1.1
     from nxtomomill.settings import H5_INIT_TITLES
     from nxtomomill.settings import H5_ZSERIE_INIT_TITLES
-from silx.io.utils import h5py_read_dataset
-import logging
-from tomoscan.io import HDF5File
+    from nxtomomill.settings import H5_PCOTOMO_INIT_TITLES
 
+import logging
+from typing import Optional
+
+from silx.io.utils import h5py_read_dataset
+from silx.io.utils import open as open_hdf5
 
 _logger = logging.getLogger(__name__)
 
@@ -130,15 +138,59 @@ class BlissScan:
         return "@".join((str(entry), master_file))
 
     def _deduce_transfert_scan(self, output_dir):
-        new_master_file = os.path.join(output_dir, os.path.basename(self.master_file))
-        return BlissScan(master_file=new_master_file, entry=self.entry)
+        new_master_file = os.path.join(
+            output_dir,
+            os.path.basename(os.path.dirname(self.master_file)),
+            os.path.basename(self.master_file),
+        )
+        new_proposal_file = os.path.join(
+            output_dir, os.path.basename(self.proposal_file)
+        )
+        return BlissScan(
+            master_file=new_master_file,
+            proposal_file=new_proposal_file,
+            entry=self.entry,
+        )
 
     @staticmethod
     def is_bliss_file(file_path):
         return len(BlissScan.get_valid_entries(file_path)) > 0
 
+    def is_abort(self, *args, **kwargs):
+        # for now there is not real way to know if a scan is abort or not
+        return False
+
+    def scan_dir_name(self) -> Optional[str]:
+        """for 'this/is/my/acquisition' returns 'acquisition'"""
+        if self.path is not None:
+            return self.path.split(os.sep)[-1]
+        else:
+            return None
+
+    def scan_basename(self) -> Optional[str]:
+        """for 'this/is/my/acquisition' returns 'acquisition'"""
+        if self.path is not None:
+            return self.path
+        else:
+            return None
+
+    def scan_parent_dir_basename(self) -> Optional[str]:
+        if self.path is not None:
+            try:
+                return os.path.dirname(self.path)
+            except Exception:
+                return None
+        else:
+            return None
+
     @staticmethod
     def is_bliss_valid_entry(file_path: str, entry: str):
+        def is_pcotmo_tomo(current_title: str):
+            for title in H5_PCOTOMO_INIT_TITLES:
+                if current_title.startswith(title):
+                    return True
+            return False
+
         def is_standard_tomo(current_title: str):
             for title in H5_INIT_TITLES:
                 if current_title.startswith(title):
@@ -151,7 +203,7 @@ class BlissScan:
                     return True
             return False
 
-        with HDF5File(file_path, mode="r", swmr=True) as h5s:
+        with open_hdf5(file_path) as h5s:
             if not isinstance(h5s, h5py.Group) or not isinstance(
                 h5s[entry], h5py.Group
             ):
@@ -159,7 +211,11 @@ class BlissScan:
             if "title" in h5s[entry]:
                 title = h5py_read_dataset(h5s[entry]["title"])
                 try:
-                    return is_standard_tomo(title) or is_zseries_tomo(title)
+                    return (
+                        is_standard_tomo(title)
+                        or is_zseries_tomo(title)
+                        or is_pcotmo_tomo(title)
+                    )
                 except Exception:
                     pass
             return False
@@ -171,7 +227,7 @@ class BlissScan:
             return tuple()
         else:
             res = []
-            with HDF5File(file_path, mode="r", swmr=True) as h5s:
+            with open_hdf5(file_path) as h5s:
                 for entry in h5s:
                     if BlissScan.is_bliss_valid_entry(file_path=file_path, entry=entry):
                         res.append(entry)
