@@ -28,28 +28,36 @@ __license__ = "MIT"
 __date__ = "09/08/2018"
 
 
+import functools
+import logging
 import os
 import re
 from collections import OrderedDict
+from glob import glob
 from math import ceil
+from typing import Optional
+import pathlib
+from datetime import datetime
+
 import fabio
 import numpy
-import functools
-import logging
-from tomoscan.scanbase import FOV
+from processview.core.dataset import DatasetIdentifier
+from silx.io.url import DataUrl
+from tomoscan.esrf.identifier.edfidentifier import (
+    EDFTomoScanIdentifier as _EDFTomoScanIdentifier,
+)
+from tomoscan.esrf.scan.edfscan import EDFTomoScan as _tsEDFTomoScan
+from tomoscan.esrf.scan.utils import get_data
+from nxtomo.nxobject.nxdetector import FOV
+
 from tomwer.core.process.reconstruction.darkref.settings import (
-    REFHST_PREFIX,
     DARKHST_PREFIX,
+    REFHST_PREFIX,
 )
 from tomwer.core.progress import Progress
-from silx.io.url import DataUrl
-import silx.io.utils
-from glob import glob
-from .scanbase import TomwerScanBase
-from tomoscan.esrf.edfscan import EDFTomoScan as _tsEDFTomoScan
 from tomwer.utils import docstring
-from processview.core.dataset import DatasetIdentifier
-from typing import Optional
+
+from .scanbase import TomwerScanBase
 
 _logger = logging.getLogger(__name__)
 
@@ -58,26 +66,27 @@ global counter_rand
 counter_rand = 1  # used to be sure to return a unique index on recons slices
 
 
-class _EDFTomoScanIdentifier(DatasetIdentifier):
-    def __init__(self, dataset, edf_folder):
-        super().__init__(dataset)
-        self.edf_folder = os.path.realpath(os.path.abspath(edf_folder))
+class EDFTomoScanIdentifier(_EDFTomoScanIdentifier, DatasetIdentifier):
+    def __init__(self, object, folder, file_prefix, metadata=None):
+        super().__init__(object, folder, file_prefix)
+        DatasetIdentifier.__init__(self, EDFTomoScan.from_identifier, metadata=metadata)
+
+    @staticmethod
+    def from_str(identifier):
+        from tomoscan.esrf.scan.edfscan import EDFTomoScan
+
+        return _EDFTomoScanIdentifier._from_str_to_single_frame_identifier(
+            identifier=identifier,
+            SingleFrameIdentifierClass=EDFTomoScanIdentifier,
+            ObjClass=EDFTomoScan,
+        )
 
-    @docstring(DatasetIdentifier)
     def long_description(self) -> str:
-        return self.edf_folder
+        """used for processview header tooltip for now"""
+        return self.to_str()
 
-    def __str__(self):
-        return os.path.basename(self.edf_folder)
-
-    def __eq__(self, other):
-        if isinstance(other, _EDFTomoScanIdentifier):
-            return self.edf_folder == other.edf_folder
-        else:
-            return False
-
-    def __hash__(self):
-        return hash(self.edf_folder)
+    def short_description(self) -> str:
+        return f"scan: {os.path.basename(os.path.basename(self.folder))} (EDF)"
 
 
 class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
@@ -87,16 +96,66 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
     :param Union[str, None] scan: path of the scan
     """
 
-    def __init__(self, scan, overwrite_proc_file=False):
+    def __init__(self, scan, overwrite_proc_file=False, update=True):
         _tsEDFTomoScan.__init__(self, scan=scan)
         TomwerScanBase.__init__(self)
+        # register at least the 'default' working directory as a possible reconstruction path
+        self.add_reconstruction_path(self.path)
 
         self._dark = None
         self._process_file = self.get_process_file_name(self)
         self._init_index_process_file(overwrite_proc_file=overwrite_proc_file)
 
-        if scan is not None:
-            self.update()
+        if scan is not None and update:
+            try:
+                self.update()
+            except IOError:
+                # fabio can raise some empty file error when data is not on disk (can be the case for the datawatcher when acquisiton is on going)
+                pass
+        try:
+            reduced_darks, metadata = self.load_reduced_darks(return_info=True)
+        except (KeyError, OSError, ValueError):
+            # file or key does not exists
+            pass
+        else:
+            self.set_reduced_darks(reduced_darks, darks_infos=metadata)
+
+        try:
+            reduced_flats, metadata = self.load_reduced_flats(return_info=True)
+        except (KeyError, OSError, ValueError):
+            pass
+        else:
+            self.set_reduced_flats(reduced_flats, flats_infos=metadata)
+
+    @staticmethod
+    def is_tomoscan_dir(
+        directory: str,
+        dataset_basename: Optional[str] = None,
+        src_pattern=None,
+        dest_pattern=None,
+        **kwargs,
+    ) -> bool:
+        info_file = EDFTomoScan.get_info_file(
+            directory=directory, dataset_basename=dataset_basename, kwargs=kwargs
+        )
+        if src_pattern is not None and dest_pattern is not None:
+            infofilenice = info_file.replace(src_pattern, dest_pattern, 1)
+            return os.path.isfile(infofilenice) or os.path.isfile(info_file)
+        else:
+            return os.path.isfile(info_file)
+
+    @property
+    def working_directory(self):
+        return self.path
+
+    @staticmethod
+    def from_identifier(identifier):
+        """Return the Dataset from a identifier"""
+        if not isinstance(identifier, EDFTomoScanIdentifier):
+            raise TypeError(
+                f"identifier should be an instance of {EDFTomoScanIdentifier} not {type(identifier)}"
+            )
+        return EDFTomoScan(scan=identifier.folder)
 
     @staticmethod
     def get_process_file_name(scan):
@@ -110,7 +169,6 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
     def clear_caches(self):
         _tsEDFTomoScan.clear_caches(self)
         TomwerScanBase.clear_caches(self)
-        self._dark = None
 
     @staticmethod
     def directory_contains_scan(directory, src_pattern, dest_pattern):
@@ -173,8 +231,8 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
             return a dict with the index of the slice reconstructed.
         """
 
-        def containsDigits(input):
-            return any(char.isdigit() for char in input)
+        def containsDigits(input_):
+            return any(char.isdigit() for char in input_)
 
         if (scanID is None) or (not os.path.isdir(scanID)):
             if withIndex is True:
@@ -206,6 +264,10 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
                                 indexStr = local_str.split("slice_pag_")[-1].split("_")[
                                     0
                                 ]
+                            elif "slice_ctf_" in local_str:
+                                indexStr = local_str.split("slice_ctf_")[-1].split("_")[
+                                    0
+                                ]
                             else:
                                 indexStr = local_str.split("slice_")[-1].split("_")[0]
                             if containsDigits(indexStr):
@@ -219,9 +281,9 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
                                     files.append(gfile)
                     if f.endswith(".vol"):
                         if withIndex is True:
-                            files[
-                                EDFTomoScan.get_index_reconstructed(f, scanID)
-                            ] = os.path.join(scanID, f)
+                            files[EDFTomoScan.get_index_reconstructed(f, scanID)] = (
+                                os.path.join(scanID, f)
+                            )
                         else:
                             files.append(os.path.join(scanID, f))
             return files
@@ -241,7 +303,7 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
 
     @docstring(TomwerScanBase.get_sinogram)
     @functools.lru_cache(maxsize=16, typed=True)
-    def get_sinogram(self, line, subsampling=1):
+    def get_sinogram(self, line, subsampling=1, norm_method=None, **kwargs):
         """
 
         extract the sinogram from projections
@@ -254,8 +316,7 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
         :rtype: numpy.array
         """
         _logger.info(
-            "compute sinogram for line %s of %s (subsampling: %s)"
-            % (line, self.path, subsampling)
+            f"compute sinogram for line {line} of {self.path} (subsampling: {subsampling})"
         )
         assert isinstance(line, int)
         if self.tomo_n is not None and self.dim_2 is not None and line > self.dim_2:
@@ -269,14 +330,13 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
             proj_sort = list(filter(lambda x: not isinstance(x, str), proj_sort))
             proj_sort.sort()
             advancement = Progress(
-                name="compute sinogram for %s, line=%s,"
-                "sampling=%s" % (os.path.basename(self.path), line, subsampling)
+                name=f"compute sinogram for {os.path.basename(self.path)}, line={line}, sampling={subsampling}"
             )
             advancement.setMaxAdvancement(len(proj_sort))
             for i_proj, proj in enumerate(proj_sort):
                 if i_proj % subsampling == 0:
                     url = proj_urls[proj]
-                    radio = silx.io.utils.get_data(url)
+                    radio = get_data(url)
                     radio = self.data_flat_field_correction(radio)
                     sinogram[i_proj // subsampling] = radio[line]
                 advancement.increaseAdvancement(1)
@@ -304,12 +364,14 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
         return None
 
     @docstring(TomwerScanBase.get_proj_angle_url)
-    def get_proj_angle_url(self, use_cache: bool = True):
+    def get_proj_angle_url(self, use_cache: bool = True, *args, **kwargs):
         if not use_cache:
             self._cache_proj_urls = None
 
         if self._cache_proj_urls is None:
-            self._cache_proj_urls = _tsEDFTomoScan.get_proj_angle_url(self)
+            self._cache_proj_urls = _tsEDFTomoScan.get_proj_angle_url(
+                self, *args, **kwargs
+            )
         return self._cache_proj_urls
 
     @functools.lru_cache()
@@ -339,7 +401,7 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
             return None
         path = os.path.join(self.path, f)
         if os.path.exists(path):
-            _logger.info("Getting %s from %s" % (what, f))
+            _logger.info(f"Getting {what} from {f}")
             try:
                 data = fabio.open(path).data
             except Exception:
@@ -402,8 +464,7 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
 
             if len(oProj) > 2:
                 _logger.info(
-                    "Only bordering projections (%s and %s) will "
-                    "be used for extracting %s" % (lowPI, uppPI, what)
+                    f"Only bordering projections ({lowPI} and {uppPI}) will be used for extracting {what}"
                 )
 
             uppPI = uppPI
@@ -414,15 +475,13 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
             if (index >= lowPI) is False:
                 index = lowPI
                 _logger.warning(
-                    "ProjectionI not in the files indexes"
-                    "range (projectionI >= lowerProjIndex)"
+                    "ProjectionI not in the files indexes range (projectionI >= lowerProjIndex)"
                 )
 
             if (index <= uppPI) is False:
                 index = uppPI
                 _logger.warning(
-                    "ProjectionI not in the files indexes"
-                    "range upperProjIndex >= projectionI"
+                    "ProjectionI not in the files indexes range upperProjIndex >= projectionI"
                 )
 
             # simple interpolation
@@ -470,29 +529,37 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
     def _deduce_transfert_scan(self, output_dir):
         if os.path.basename(output_dir) != os.path.basename(self.path):
             raise ValueError(
-                "Transfert to a new EDFTomoScan requires an "
-                "equal basename. Current path is {}, requested "
-                "one is {}".format(self.path, output_dir)
+                "Transfert to a new EDFTomoScan requires an equal basename. "
+                f"Current path is {self.path}, requested one is {output_dir}"
             )
-        return EDFTomoScan(scan=output_dir)
+        # here: avoir reloading the metadata because not sure there will be some processing after the 'data transfert'. And can be long with EDF + GPFS at esrf
+        return EDFTomoScan(scan=output_dir, update=False)
 
     def __str__(self):
         return self.path
 
     # Dataset implementation
 
-    @staticmethod
-    def from_dataset_identifier(identifier):
-        """Return the Dataset from a identifier"""
-        if not isinstance(identifier, _EDFTomoScanIdentifier):
-            raise TypeError(
-                "identifier should be an instance of " "_HDF5TomoScanIdentifier"
-            )
-        return EDFTomoScan(scan=identifier.edf_folder)
-
-    @docstring(TomwerScanBase)
-    def get_dataset_identifier(self) -> DatasetIdentifier:
-        return _EDFTomoScanIdentifier(dataset=self, edf_folder=self.path)
+    @docstring(_tsEDFTomoScan)
+    def get_identifier(self):
+        try:
+            stat = pathlib.Path(self.path).stat()
+        except Exception:
+            stat = None
+        return EDFTomoScanIdentifier(
+            object=self,
+            folder=self.path,
+            file_prefix=self.dataset_basename,
+            metadata={
+                "name": self.path,
+                "creation_time": (
+                    datetime.fromtimestamp(stat.st_ctime) if stat else None
+                ),
+                "modification_time": (
+                    datetime.fromtimestamp(stat.st_ctime) if stat else None
+                ),
+            },
+        )
 
     @docstring(TomwerScanBase)
     def get_nabu_dataset_info(self, binning=1, binning_z=1, proj_subsampling=1):
@@ -516,7 +583,7 @@ class EDFTomoScan(_tsEDFTomoScan, TomwerScanBase):
             else:
                 angles = numpy.linspace(0, numpy.pi, n_angles, False)
 
-        analyzer.rotation_angles = angles
+            analyzer.rotation_angles = angles
         return analyzer
 
     @docstring(TomwerScanBase)
