@@ -28,24 +28,37 @@ __license__ = "MIT"
 __date__ = "09/08/2018"
 
 
-import numpy
-import logging
-import typing
-import json
 import io
+import json
+import logging
 import os
+import typing
 from glob import glob
-from silx.io.url import DataUrl
-from tomwer.core.utils.locker import FileLockerManager, FileLockerContext
-from tomwer.core.utils.ftseriesutils import orderFileByLastLastModification
-from tomoscan.io import HDF5File
-from processview.core.dataset import Dataset
 from typing import Optional
+import functools
+
+import numpy
+from silx.io.url import DataUrl
+from silx.utils.enum import Enum as _Enum
+from silx.io.utils import open as open_hdf5
+
+from tomoscan.identifier import VolumeIdentifier
+from tomoscan.normalization import IntensityNormalization
+from tomoscan.volumebase import VolumeBase
+from tomoscan.identifier import BaseIdentifier
+
+from tomwer.core.tomwer_object import TomwerObject
+from tomwer.core.utils.ftseriesutils import orderFileByLastLastModification
+from tomwer.core.utils.locker import FileLockerManager
+from tomwer.core.volume.edfvolume import EDFVolume
+from tomwer.core.volume.hdf5volume import HDF5Volume
+from tomwer.core.volume.jp2kvolume import JP2KVolume
+from tomwer.core.volume.tiffvolume import TIFFVolume
 
 logger = logging.getLogger(__name__)
 
 
-class TomwerScanBase(Dataset):
+class TomwerScanBase(TomwerObject):
     """
     Simple interface to extend the tomoscan.TomwerScanBase with
     specific functions
@@ -53,10 +66,6 @@ class TomwerScanBase(Dataset):
 
     _DICT_DARK_REF_KEYS = "dark_ref_params"
 
-    _DICT_PYHST2_KEYS = "pyhst2_params"
-
-    _DICT_LAMINO_RP_KEY = "lamino_params"
-
     _DICT_NABU_RP_KEY = "nabu_params"
 
     _DICT_AXIS_KEYS = "axis_params"
@@ -67,17 +76,17 @@ class TomwerScanBase(Dataset):
 
     _DICT_PROCESS_INDEX_KEY = "next_process_index"
 
-    VALID_RECONS_EXTENSION = ".edf", ".npy", ".npz", ".hdf5", ".tiff", ".jp2"
+    _DICT_NORMALIZATION_KEY = "norm_params"
+
+    VALID_RECONS_EXTENSION = ".edf", ".npy", ".npz", ".hdf5", ".tiff", ".jp2", ".vol"
 
     def __init__(self, overwrite_proc_file=False):
+        super().__init__()
+        self._stitching_metadata = None
         self._reconstructions = []
 
         self._nabu_params = None
         """nabu reconstruction parameters"""
-        self._ftseries_params = None
-        """pyhst reconstruction parameters"""
-        self._lamino_params = None
-        """Set of reconstructions parameters for laminography"""
         self._axis_params = None
         """Axis parameters"""
         self._saaxis_params = None
@@ -94,18 +103,28 @@ class TomwerScanBase(Dataset):
         self._cache_radio_axis = {}
         """cache for the radio axis. Key is tuple (mode, nearest), value is
         (url1, url2)"""
-        self._normed_darks = None
-        """darks normed. Dict: key is the index in the sequence and value is
-        the mean or median dark"""
-        self._normed_flats = None
-        """flats normed. Dict: key is the index in the sequence and value is
-        the mean or median dark"""
         self._notify_ffc_rsc_missing = True
         """Should we notify the user if ffc fails because cannot find dark or
         flat. Used to avoid several warnings. Only display one"""
         self._latest_reconstructions = []
-        """list of url related to latest reconstruction from nabu or pyhst"""
+        "list of url related to latest slice reconstruction from nabu"
         self._latest_vol_reconstructions = []
+        """list of url related to latest volume reconstruction from nabu"""
+        self._reconstruction_paths = set()
+        self._proposal_name = None
+
+    def _clear_heavy_cache(self):
+        """For scan for now we don't want to remove any information from the cache.
+        Mor eusefull for volume use case
+        """
+        pass
+
+    @property
+    def working_directory(self) -> Optional[str]:
+        """
+        working directory to use for this scan (for launching reconstruction for example)
+        """
+        raise NotImplementedError("Base class")
 
     def _init_index_process_file(self, overwrite_proc_file=False):
         if (
@@ -113,7 +132,7 @@ class TomwerScanBase(Dataset):
             and self.process_file is not None
             and os.path.exists(self.process_file)
         ):
-            with HDF5File(self.process_file, mode="r", swmr=True) as h5s:
+            with open_hdf5(self.process_file) as h5s:
                 self._process_index = len(h5s.items())
         else:
             self._process_index = 0
@@ -121,25 +140,12 @@ class TomwerScanBase(Dataset):
     def clear_caches(self):
         self._cache_proj_urls = None
         self._notify_ffc_rsc_missing = True
+        super().clear_caches()
 
     @staticmethod
     def get_process_file_name(scan):
         raise NotImplementedError("Base class")
 
-    @property
-    def normed_darks(self):
-        return self._normed_darks
-
-    def set_normed_darks(self, darks):
-        self._normed_darks = darks
-
-    @property
-    def normed_flats(self):
-        return self._normed_flats
-
-    def set_normed_flats(self, flats):
-        self._normed_flats = flats
-
     def _flat_field_correction(
         self,
         data,
@@ -159,7 +165,7 @@ class TomwerScanBase(Dataset):
 
         if dark is None:
             if self._notify_ffc_rsc_missing:
-                logger.error("cannot make flat field correction, dark not found")
+                logger.warning("cannot make flat field correction, dark not found")
             can_process = False
 
         if dark is not None and dark.ndim != 2:
@@ -170,7 +176,7 @@ class TomwerScanBase(Dataset):
 
         if flat1 is None:
             if self._notify_ffc_rsc_missing:
-                logger.error("cannot make flat field correction, flat not found")
+                logger.warning("cannot make flat field correction, flat not found")
             can_process = False
         else:
             if flat1.ndim != 2:
@@ -218,8 +224,7 @@ class TomwerScanBase(Dataset):
         process file"""
         if self.process_file is None:
             raise ValueError("No processing file defined")
-        file_locker = FileLockerManager().get_locker(file_=self.process_file)
-        return FileLockerContext(file_locker)
+        return FileLockerManager().get_lock(file_name=self.process_file)
 
     @property
     def reconstructions(self):
@@ -231,12 +236,11 @@ class TomwerScanBase(Dataset):
         self._reconstructions = reconstructions
 
     @property
-    def ftseries_recons_params(self):
-        return self._ftseries_params
+    def reconstruction_paths(self):
+        return self._reconstruction_paths
 
-    @ftseries_recons_params.setter
-    def ftseries_recons_params(self, recons_params):
-        self._ftseries_params = recons_params
+    def add_reconstruction_path(self, path: str):
+        self._reconstruction_paths.add(path)
 
     @property
     def nabu_recons_params(self):
@@ -246,14 +250,6 @@ class TomwerScanBase(Dataset):
     def nabu_recons_params(self, recons_params):
         self._nabu_params = recons_params
 
-    @property
-    def lamino_recons_params(self):
-        return self._lamino_params
-
-    @lamino_recons_params.setter
-    def lamino_recons_params(self, recons_params):
-        self._lamino_params = recons_params
-
     @property
     def axis_params(self):
         return self._axis_params
@@ -303,9 +299,8 @@ class TomwerScanBase(Dataset):
         self._process_index += 1
         return process_index
 
-    def getRadiosForAxisCalc(
-        self, mode, nearest: bool = True, use_cache: bool = True
-    ) -> tuple:
+    @functools.lru_cache(maxsize=3)
+    def get_opposite_projections(self, mode) -> tuple:
         """
         Return the radios for axis calculation and the requested mode.
 
@@ -318,69 +313,54 @@ class TomwerScanBase(Dataset):
         :type: bool
         :return: couple of `opposite` radios that can be used for axis
                  calculation.
-        :rtype: tuple(AxisResource, AxisResource)
+        :rtype: tuple(Optional[AxisResource], Optional[AxisResource])
         """
-        if not use_cache:
-            del self._cache_radio_axis[(mode, nearest)]
+        radios_with_angle = self.projections_with_angle()
+        angles = numpy.array(
+            tuple(
+                filter(
+                    lambda a: numpy.issubdtype(type(a), numpy.number),
+                    radios_with_angle.keys(),
+                )
+            )
+        )
 
-        def compute():
-            from ..process.reconstruction.axis.params import (
-                AxisResource,
-            )  # avoid cyclic import
-            from ..process.reconstruction.axis.anglemode import CorAngleMode
+        from ..process.reconstruction.axis.anglemode import CorAngleMode
+        from ..process.reconstruction.axis.params import (
+            AxisResource,
+        )  # avoid cyclic import
 
-            _mode = CorAngleMode.from_value(mode)
-            if self.path is None:
-                return None, None
+        mode = CorAngleMode.from_value(mode)
 
-            radios_with_angle = self.projections_with_angle()
-
-            if _mode is CorAngleMode.use_0_180:
-                couples = ((0, 180),)
-            elif _mode is CorAngleMode.use_90_270:
-                couples = ((90, 270),)
-            else:
-                couples = ((0, 180), (90, 270))
-
-            for couple in couples:
-                if (
-                    couple[0] in radios_with_angle.keys()
-                    and couple[1] in radios_with_angle.keys()
-                ):
-                    radio_0 = AxisResource(radios_with_angle[couple[0]])
-                    radio_1 = AxisResource(radios_with_angle[couple[1]])
-                    return radio_0, radio_1
-
-            def find_nearest(angles, angle):
-                if len(angles) == 0:
-                    return None
-                angles = numpy.asarray(angles)
-                dist = numpy.abs(angles - angle)
-                idx = dist.argmin()
-                if isinstance(idx, numpy.ndarray):
-                    idx = idx[0]
-                return angles[idx]
-
-            if nearest is True:
-                angles = []
-                # filter str value (0(1)...)
-                [
-                    angles.append(value)
-                    for value in radios_with_angle.keys()
-                    if numpy.issubdtype(type(value), numpy.number)
-                ]
-                nearest_c1 = find_nearest(angles=angles, angle=couples[0][0])
-                nearest_c2 = find_nearest(angles=angles, angle=couples[0][1])
-                if nearest_c1 is not None and nearest_c2 is not None:
-                    radio_0 = AxisResource(radios_with_angle[nearest_c1])
-                    radio_1 = AxisResource(radios_with_angle[nearest_c2])
-                    return radio_0, radio_1
+        if len(angles) < 2:
+            logger.warning("less than two angles found. Unable to get opposite angles")
             return None, None
 
-        if (mode, nearest) not in self._cache_radio_axis:
-            radio_0, radio_1 = compute()
-            self._cache_radio_axis[(mode, nearest)] = (radio_0, radio_1)
-        return self._cache_radio_axis[(mode, nearest)]
+        initial_angle = angles[0]
+        if mode in (CorAngleMode.use_0_180, CorAngleMode.manual_selection):
+            couples = (initial_angle, initial_angle + 180.0)
+        elif mode is CorAngleMode.use_90_270:
+            couples = (initial_angle + 90.0, initial_angle + 270.0)
+        else:
+            raise ValueError(f"{mode} is not handle")
+
+        def find_nearest(angles: numpy.ndarray, angle: float):
+            if len(angles) == 0:
+                return None
+            dist = numpy.abs(angles - angle)
+            idx = dist.argmin()
+            if isinstance(idx, numpy.ndarray):
+                idx = idx[0]
+            return angles[idx]
+
+        nearest_c1 = find_nearest(angles=angles, angle=couples[0])
+        nearest_c2 = find_nearest(angles=angles, angle=couples[1])
+        if nearest_c1 is not None and nearest_c2 is not None:
+            radio_0 = AxisResource(radios_with_angle[nearest_c1])
+            radio_1 = AxisResource(radios_with_angle[nearest_c2])
+            return radio_0, radio_1
+        else:
+            return None, None
 
     def data_flat_field_correction(self, data, index=None):
         """Apply flat field correction on the given data
@@ -415,20 +395,18 @@ class TomwerScanBase(Dataset):
 
         :return: DataUrl of the process file
         """
-        return DataUrl(
-            file_path=self._process_file, data_path=self.entry or "entry", scheme="silx"
-        )
+        entry = self.entry if hasattr(self, "entry") else "entry"
+        return DataUrl(file_path=self._process_file, data_path=entry, scheme="silx")
 
     def to_dict(self):
         res = {}
-        # ftseries reconstruction parameters
-        if self.ftseries_recons_params:
-            res[self._DICT_PYHST2_KEYS] = self.ftseries_recons_params.to_dict()
-        else:
-            res[self._DICT_PYHST2_KEYS] = None
         # nabu reconstruction parameters
         if self._nabu_params:
-            res[self._DICT_NABU_RP_KEY] = self.nabu_recons_params.to_dict()
+            res[self._DICT_NABU_RP_KEY] = (
+                self.nabu_recons_params
+                if isinstance(self.nabu_recons_params, dict)
+                else self.nabu_recons_params.to_dict()
+            )
         else:
             res[self._DICT_NABU_RP_KEY] = None
         # axis reconstruction parameters
@@ -451,17 +429,17 @@ class TomwerScanBase(Dataset):
             res[self._DICT_DARK_REF_KEYS] = None
         else:
             res[self._DICT_DARK_REF_KEYS] = self._dark_ref_params.to_dict()
+        # normalization
+        if self.intensity_normalization is None:
+            res[self._DICT_NORMALIZATION_KEY] = None
+        else:
+            res[self._DICT_NORMALIZATION_KEY] = self.intensity_normalization.to_dict()
         # process index
         res[self._DICT_PROCESS_INDEX_KEY] = self._process_index
 
-        # lamino reconstruction parameters
-        res[self._DICT_LAMINO_RP_KEY] = self.lamino_recons_params
         return res
 
     def load_from_dict(self, desc):
-        from tomwer.core.process.reconstruction.ftseries.params import (
-            ReconsParams,
-        )  # avoid cyclic import
         from tomwer.core.process.reconstruction.axis.params import (
             AxisRP,
         )  # avoid cyclic import
@@ -470,16 +448,13 @@ class TomwerScanBase(Dataset):
             data = json.load(desc)
         else:
             data = desc
-        if not (self.DICT_PATH_KEY in data and data[self.DICT_TYPE_KEY] == self._TYPE):
+        if not (
+            self.DICT_PATH_KEY in data  # pylint: disable=E1101
+            and data[self.DICT_TYPE_KEY] == self._TYPE  # pylint: disable=E1101
+        ):
             raise ValueError("Description is not an EDFScan json description")
 
-        assert self.DICT_PATH_KEY in data
-        assert self._DICT_LAMINO_RP_KEY in data
-        # load pyhst2 parameters
-        recons_param_data = data[self._DICT_PYHST2_KEYS]
-        if recons_param_data is not None:
-            self.ftseries_recons_params = ReconsParams.from_dict(recons_param_data)
-        self.lamino_recons_params = data[self._DICT_LAMINO_RP_KEY]
+        assert self.DICT_PATH_KEY in data  # pylint: disable=E1101
         # load axis reconstruction parameters
         axis_params = data.get(self._DICT_AXIS_KEYS, None)
         if axis_params is not None:
@@ -493,6 +468,12 @@ class TomwerScanBase(Dataset):
             from tomwer.core.process.reconstruction.darkref.params import DKRFRP
 
             self._dark_ref_params = DKRFRP.from_dict(dark_ref_params)
+        # load normalization
+        intensity_normalization = data.get(self._DICT_NORMALIZATION_KEY, None)
+        if intensity_normalization is not None:
+            self.intensity_normalization = IntensityNormalization.from_dict(
+                intensity_normalization
+            )
         # load saaxis parameters
         saaxis_params = data.get(self._DICT_SA_AXIS_KEYS, None)
         if saaxis_params is not None:
@@ -523,24 +504,39 @@ class TomwerScanBase(Dataset):
         return (
             isinstance(other, self.__class__)
             or isinstance(self, other.__class__)
-            and self.type == other.type
-            and self.ftseries_recons_params == other.ftseries_recons_params
-            and self.lamino_recons_params == other.lamino_recons_params
+            and self.type == other.type  # pylint: disable=E1101
             and self.nabu_recons_params == other.nabu_recons_params
-            and self.path == other.path
+            and self.path == other.path  # pylint: disable=E1101
         )
 
-    def get_sinogram(self, line, progress=None):
+    def get_sinogram(self, line, subsampling=1, norm_method=None, **kwargs):
         """
         extract the sinogram from projections
 
         :param line: which sinogram we want
         :type: int
+        :param subsampling: subsampling to apply on the sinogram
         :return: sinogram from the radio lines
         :rtype: numpy.array
         """
         raise NotImplementedError("Base class")
 
+    def get_normed_sinogram(self, line, subsampling=1):
+        """
+        Util to get the sinogram normed with settings currently defined
+        on the 'intensity_normalization' property
+
+        :param line:
+        :param subsampling:
+        :return:
+        """
+        return self.get_sinogram(
+            line=line,
+            subsampling=subsampling,
+            norm_method=self.intensity_normalization.method,
+            **self.intensity_normalization.get_extra_infos(),
+        )
+
     def __str__(self):
         raise NotImplementedError("Base class")
 
@@ -572,7 +568,7 @@ class TomwerScanBase(Dataset):
         """
         raise NotImplementedError("Base class")
 
-    def get_proj_angle_url(self, use_cache: bool = True):
+    def get_proj_angle_url(self, use_cache: bool = True, *args, **kwargs):
         """
         retrieve the url for each projections (including the alignement /
         return one) and associate to each (if possible) the angle.
@@ -587,117 +583,92 @@ class TomwerScanBase(Dataset):
     def get_reconstructions_urls(self):
         """
 
-        :return: list of urls that contains reconstruction from nabu or pyhst
+        :return: list of urls that contains reconstruction from nabu
         :rtype: list
         """
+        from tomwer.core.process.reconstruction.output import (
+            PROCESS_FOLDER_NAME,
+        )  # avoid cyclic import
 
-        def containsDigits(input):
-            return any(char.isdigit() for char in input)
-
-        if (self.path is None) or (not os.path.isdir(self.path)):
-            return []
-        results = []
-        # parse .npy, .npz and .edf, .vol, *.hdf5 files
-        folderBasename = os.path.basename(self.path)
-        pyhst_files = TomwerScanBase.get_pyhst_recons_file(self.path)
-        if pyhst_files is not None:
-            results.extend(
-                TomwerScanBase.getReconstructedFilesFromParFile(
-                    pyhst_files, with_index=False
+        all_recons_urls = set()
+        recons_paths = self.reconstruction_paths
+        if self.working_directory:
+            recons_paths.add(
+                os.path.join(
+                    self.working_directory,
+                    PROCESS_FOLDER_NAME,
                 )
             )
+        for path in recons_paths:
+            all_recons_urls.update(self.get_reconstructions_urls_from_path(path))
+        return all_recons_urls
 
-        def get_recons_url_from_folder(folder, check_url=False):
-            """
-            retrieve all the url from a folder.
+    def get_reconstructions_urls_from_path(self, path, check_url=False):
+        if path is None or not os.path.isdir(path):
+            return []
+        else:
 
-            :param str folder:
-            :param bool check_url: if True before adding an url try to open it
-                                   and insure is valid
-            :return:
-            """
-            res = []
-            for f in os.listdir(folder):
-                full_file_path = os.path.join(folder, f)
-                if f.startswith(folderBasename) and "slice_" in f:
-                    if os.path.isdir(full_file_path):
-                        res.extend(get_recons_url_from_folder(full_file_path))
-                    elif f.endswith(TomwerScanBase.VALID_RECONS_EXTENSION):
-                        local_str = f
-                        for extension in TomwerScanBase.VALID_RECONS_EXTENSION:
-                            if local_str.endswith(extension):
-                                local_str = local_str.rsplit(extension, 1)[0]
-                                break
-                        if "slice_pag_" in local_str:
-                            indexStr = local_str.split("slice_pag_")[-1].split("_")[0]
-                        else:
-                            indexStr = local_str.split("slice_")[-1].split("_")[0]
-                        if containsDigits(indexStr):
-                            gfile = os.path.join(folder, f)
-                            assert os.path.isfile(gfile)
-                            if gfile.lower().endswith(".edf"):
-                                url = DataUrl(file_path=full_file_path, scheme="fabio")
-                                res.append(url)
+            from tomwer.core.process.reconstruction.saaxis.saaxis import (
+                DEFAULT_RECONS_FOLDER as MULTI_COR_DEFAULT_FOLDER,
+            )
+            from tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta import (
+                DEFAULT_RECONS_FOLDER as MULTI_DB_DEFAULT_FOLDER,
+            )
+            from tomwer.core.process.reconstruction.nabu.settings import (
+                NABU_CFG_FILE_FOLDER,
+            )
 
-                        if f.lower().endswith((".npy", "npz")):
-                            url = DataUrl(file_path=full_file_path, scheme="numpy")
-                            res.append(url)
-                        elif f.lower().endswith((".jp2", ".jp2", ".tiff")):
-                            url = DataUrl(file_path=full_file_path, scheme="tomwer")
-                            res.append(url)
-                        elif f.lower().endswith(
-                            (".h5", ".hdf5", ".nx", ".nexus", ".nx5", ".hdf")
-                        ):
-                            entry = getattr(self, "entry", "entry")
-                            url = DataUrl(
-                                file_path=full_file_path,
-                                scheme="silx",
-                                data_path="/".join(
-                                    (entry, "reconstruction", "results", "data")
-                                ),
-                            )
-                            if check_url is True:
-                                try:
-                                    with HDF5File(
-                                        full_file_path, "r", swmr=True
-                                    ) as h5f:
-                                        if entry not in h5f:
-                                            logger.info(
-                                                "{} does not exists".format(url)
-                                            )
-                                            continue
-                                except Exception:
-                                    logger.info("unable to check {}".format(url))
-                                else:
-                                    logger.info("{} checked".format(url))
-                                    res.append(url)
-                            else:
-                                res.append(url)
-                if f.endswith(".vol"):
-                    url = DataUrl(file_path=os.path.join(self.path, f), scheme="tomwer")
-                    res.append(url)
+            res = set()
+            for f in os.listdir(path):
+                current_path = os.path.join(path, f)
+                if os.path.isfile(current_path):
+                    volume = _get_reconstructed_single_file_volume(
+                        current_path, scan=self
+                    )
+                    if volume is not None:
+                        res.add(volume)
+                # avoid looking deeper if this doesn't look like a folder that could contain slices
+                elif (
+                    f
+                    in (
+                        "matlab",
+                        "steps_file_basename_nabu_sinogram_save_step",
+                        MULTI_COR_DEFAULT_FOLDER,
+                        MULTI_DB_DEFAULT_FOLDER,
+                        NABU_CFG_FILE_FOLDER,
+                    )
+                    or f.endswith("_vol")
+                    or "slice" not in f
+                ):
+                    pass
+                else:
+                    res.update(
+                        self.get_reconstructions_urls_from_path(
+                            path=current_path, check_url=check_url
+                        )
+                    )
             return res
 
-        if os.path.isdir(self.path):
-            results.extend(get_recons_url_from_folder(self.path))
-
-        return results
-
     @property
     def latest_reconstructions(self):
-        """List of latest reconstructions"""
+        """List of latest slices reconstructions (as VolumeIdentifier) - single slice volume"""
         return self._latest_reconstructions
 
     @property
     def latest_vol_reconstructions(self):
-        """List of latest reconstructions"""
+        """List of latest volume reconstructions (as VolumeIdentifier)"""
         return self._latest_vol_reconstructions
 
     def clear_latest_reconstructions(self):
         self._latest_reconstructions = []
 
     def set_latest_reconstructions(self, urls: typing.Iterable):
-        self._latest_reconstructions = urls
+        if urls is None:
+            self._latest_reconstructions = None
+        else:
+            self._latest_reconstructions = list(
+                [self._process_volume_url(url) for url in urls]
+            )
 
     def add_latest_reconstructions(self, urls: typing.Iterable):
         self._latest_reconstructions.extend(urls)
@@ -705,24 +676,56 @@ class TomwerScanBase(Dataset):
     def clear_latest_vol_reconstructions(self):
         self._latest_vol_reconstructions = []
 
-    def set_latest_vol_reconstructions(self, urls):
-        self._latest_vol_reconstructions = urls
-
-    def add_latest_vol_reconstructions(self, urls):
-        self._latest_vol_reconstructions.extend(urls)
-
-    def _update_latest_recons_urls(self, old_path, new_path):
-        new_urls = []
-        for recons_url in self._latest_reconstructions:
-            new_urls.append(
-                DataUrl(
-                    file_path=recons_url.path().replace(old_path, new_path, 1),
-                    data_path=recons_url.data_path(),
-                    data_slice=recons_url.data_slice(),
-                    scheme=recons_url.scheme(),
-                )
+    @staticmethod
+    def _process_volume_url(url):
+        if isinstance(url, str):
+            return VolumeIdentifier.from_str(url)
+        elif isinstance(url, VolumeIdentifier):
+            return url
+        elif isinstance(url, VolumeBase):
+            return url.get_identifier()
+        else:
+            raise TypeError(
+                f"url should be a {VolumeIdentifier} or a string reprenseting a {VolumeIdentifier}. {type(url)} provided instead"
             )
-        self._latest_reconstructions = new_urls
+
+    def set_latest_vol_reconstructions(self, volume_identifiers: typing.Iterable):
+        if volume_identifiers is None:
+            self._latest_vol_reconstructions = None
+        else:
+            self._latest_vol_reconstructions = list(
+                [self._process_volume_url(url) for url in volume_identifiers]
+            )
+
+    def add_latest_vol_reconstructions(self, volume_identifiers: tuple):
+        assert isinstance(
+            volume_identifiers, tuple
+        ), "volume_identifiers is expected to be a tuple"
+        self._latest_vol_reconstructions.extend(
+            self._process_volume_url(volume_identifier)
+            for volume_identifier in volume_identifiers
+        )
+
+    def _update_latest_recons_identifiers(self, old_path, new_path):
+        def update_identifier(identifier):
+            assert isinstance(
+                identifier, BaseIdentifier
+            ), f"identifier is expected to be an instance of {BaseIdentifier}"
+            # small hack as this is not much used: simply try to replace a path by another. this is only used by the data transfer and EDF / SPEC
+            # this time is almost over
+            # FIXME: correct way to do this would be to recreate the volume, modify file or folder path and
+            # recreate the new identifier
+            identifier.replace(old_path, new_path, 1)
+
+        self._latest_reconstructions = [
+            update_identifier(identifier=identifier)
+            for identifier in self._latest_reconstructions
+        ]
+
+        self._latest_vol_reconstructions = [
+            update_identifier(identifier=identifier)
+            for identifier in self._latest_vol_reconstructions
+        ]
 
     def get_url_proj_index(self, url):
         """Return the index in the acquisition from the url"""
@@ -735,8 +738,10 @@ class TomwerScanBase(Dataset):
                     res[value.path()] = key
             return res
 
-        proj_inv_url_to_index = invert_dict(self.projections)
-        alig_inv_url_to_index = invert_dict(self.alignment_projections)
+        proj_inv_url_to_index = invert_dict(self.projections)  # pylint: disable=E1101
+        alig_inv_url_to_index = invert_dict(
+            self.alignment_projections  # pylint: disable=E1101
+        )
         if url.path() in proj_inv_url_to_index:
             return proj_inv_url_to_index[url.path()]
         elif url.path() in alig_inv_url_to_index:
@@ -752,7 +757,7 @@ class TomwerScanBase(Dataset):
         from tomwer.core.process.task import Task
 
         if os.path.exists(self.process_file):
-            with HDF5File(self.process_file, mode="r") as h5s:
+            with open_hdf5(self.process_file) as h5s:
                 if not hasattr(self, "entry"):
                     entry = "entry"
                 else:
@@ -766,8 +771,7 @@ class TomwerScanBase(Dataset):
                     if len(indexes) > 0:
                         self._process_index = max(indexes) + 1
                         logger.debug(
-                            "set process_index from tomwer process file"
-                            "to {}".format(self._process_index)
+                            f"set process_index from tomwer process file to {self._process_index}"
                         )
 
     def get_nabu_dataset_info(self, binning=1, binning_z=1, proj_subsampling=1):
@@ -782,6 +786,12 @@ class TomwerScanBase(Dataset):
         """Return the equivalent DatasetAnalyzer for nabu"""
         raise NotImplementedError("Base class")
 
+    def get_proposal_name(self) -> Optional[str]:
+        return self._proposal_name
+
+    def set_proposal_name(self, proposal_name: str) -> None:
+        self._proposal_name = proposal_name
+
 
 class _TomwerBaseDock(object):
     """
@@ -795,3 +805,79 @@ class _TomwerBaseDock(object):
     @property
     def instance(self):
         return self.__instance
+
+
+def _containsDigits(input_):
+    return any(char.isdigit() for char in input_)
+
+
+def _get_reconstructed_single_file_volume(
+    file_, scan: TomwerScanBase, check_url=False
+) -> Optional[VolumeBase]:
+    scan_basename = scan.get_process_file_name
+    scan_basename = scan.get_dataset_basename()
+    if scan_basename is None:
+        logger.error("TODO")
+
+    file_base_name = os.path.basename(file_)
+    if file_base_name.startswith(scan_basename) and "slice_" in file_base_name:
+        if file_.endswith(TomwerScanBase.VALID_RECONS_EXTENSION):
+            local_str = file_
+
+            for extension in TomwerScanBase.VALID_RECONS_EXTENSION:
+                if local_str.endswith(extension):
+                    local_str = local_str.rsplit(extension, 1)[0]
+                    break
+            if "slice_pag_" in local_str:
+                indexStr = local_str.split("slice_pag_")[-1].split("_")[0]
+                volume_basename = local_str.split("slice_pag_")[0] + "slice_pag_"
+            else:
+                indexStr = local_str.split("slice_")[-1].split("_")[0]
+                volume_basename = local_str.split("slice_")[0] + "slice_"
+
+            if _containsDigits(indexStr):
+                if extension == ".edf":
+                    return EDFVolume(
+                        folder=os.path.dirname(file_),
+                        volume_basename=volume_basename,
+                    )
+
+            if extension in (".jp2", ".jp2"):
+                return JP2KVolume(
+                    folder=os.path.dirname(file_),
+                    volume_basename=volume_basename,
+                    data_extension=extension,
+                )
+            elif extension in (".tiff", ".tif"):
+                return TIFFVolume(
+                    folder=os.path.dirname(file_),
+                    volume_basename=volume_basename,
+                    data_extension=extension,
+                )
+            elif extension in (
+                (".h5", ".hdf5", ".nx", ".nxs", ".nexus", ".nx5", ".hdf")
+            ):
+                entry = getattr(scan, "entry", "entry")
+                volume = HDF5Volume(
+                    file_path=os.path.abspath(file_),
+                    data_path="/".join([entry, "reconstruction"]),
+                )
+
+                if check_url is True:
+                    try:
+                        with open_hdf5(os.path.abspath(file_)) as h5f:
+                            if entry not in h5f:
+                                logger.info("{volume} does not exists")
+                                return None
+                    except Exception:
+                        logger.info("unable to check {volume}")
+                        return None
+                    else:
+                        logger.info("{volume} checked")
+                return volume
+
+
+class NormMethod(_Enum):
+    MANUAL_ROI = "manual ROI"
+    AUTO_ROI = "automatic ROI"
+    DATASET = "from dataset"
