@@ -30,31 +30,32 @@ __date__ = "04/11/2021"
 
 
 import asyncio
-from tomwer.core.scan.futurescan import FutureTomwerScan
-from tomwer.core.utils.scanutils import MockHDF5
 import os
 
+from tomwer.core.futureobject import FutureTomwerObject
+from tomwer.core.utils.scanutils import MockNXtomo
 
-def test_simple_future_scan(tmpdir):
+
+def test_simple_future_tomo_obj(tmpdir):
     """Simple test of the FutureTomwerScan API"""
-    scan = MockHDF5(
+    scan = MockNXtomo(
         scan_path=os.path.join(tmpdir, "scan_test"),
         n_proj=10,
         n_ini_proj=10,
         create_ini_dark=False,
-        create_ini_ref=False,
+        create_ini_flat=False,
         dim=10,
     ).scan
     future = asyncio.Future()
     future.set_result(None)
 
-    future_scan = FutureTomwerScan(
-        scan=scan,
+    future_tomo_obj = FutureTomwerObject(
+        tomo_obj=scan,
         futures=[
             future,
         ],
     )
 
-    future_scan.results()
-    assert future_scan.exceptions() is None
-    assert future_scan.cancelled() is False
+    future_tomo_obj.results()
+    assert future_tomo_obj.exceptions() is None
+    assert future_tomo_obj.cancelled() is False
