@@ -28,14 +28,14 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "16/09/2019"
 
-import unittest
-import shutil
 import os
+import shutil
 import tempfile
-from tomwer.test.utils import UtilsTest
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefs
-from tomwer.core.utils.scanutils import MockHDF5
+import unittest
+
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.utils.scanutils import MockNXtomo
+from tomwer.tests.datasets import TomwerCIDatasets
 
 
 class TestHDF5Scan(unittest.TestCase):
@@ -46,60 +46,51 @@ class TestHDF5Scan(unittest.TestCase):
         self._tmp_dir = tempfile.mkdtemp()
         self.dataset_file = os.path.join(self._tmp_dir, "frm_edftomomill_twoentries.nx")
         shutil.copyfile(
-            UtilsTest.getH5Dataset(folderID="frm_edftomomill_twoentries.nx"),
+            TomwerCIDatasets.get_dataset(
+                "h5_datasets/frm_edftomomill_twoentries.nx",
+            ),
             self.dataset_file,
         )
         assert os.path.isfile(self.dataset_file)
-        self.scan = HDF5TomoScan(scan=self.dataset_file, entry="entry0000")
+        self.scan = NXtomoScan(scan=self.dataset_file, entry="entry0000")
 
     def test_scan_dir(self):
         assert self.scan.scan_dir_name() == self._tmp_dir.split(os.sep)[-1]
 
-    def testSinogram(self):
-        """some tomwer specific unittest of HDF5Scan"""
-        tomo_n = len(self.scan.projections)
-        # apply dark and ref to get flat and dark
-        dark_flat_process = DarkRefs(
-            inputs={
-                "data": self.scan,
-                "dark_ref_params": None,
-            }
-        )
-        dark_flat_process.run()
-        self.assertEqual(
-            self.scan.get_sinogram(line=2, subsampling=1).shape, (tomo_n, 20)
-        )
-        self.assertEqual(
-            self.scan.get_sinogram(line=2, subsampling=2).shape, (tomo_n // 2, 20)
-        )
-        self.assertEqual(
-            self.scan.get_sinogram(line=2, subsampling=3).shape, (tomo_n // 3, 20)
-        )
-
     def testFFInterval(self):
         """test the call to ff_interval"""
         scan_path = os.path.join(self._tmp_dir, "my_scan_1")
-        scan_1 = MockHDF5(
+        scan_1 = MockNXtomo(
             scan_path=scan_path,
             n_ini_proj=20,
             n_proj=20,
             n_alignement_proj=2,
-            create_final_ref=True,
+            create_final_flat=True,
             create_ini_dark=True,
-            create_ini_ref=True,
+            create_ini_flat=True,
             n_refs=5,
         ).scan
         self.assertEqual(scan_1.ff_interval, 20)
 
         scan_path2 = os.path.join(self._tmp_dir, "my_scan_2")
-        scan_2 = MockHDF5(
+        scan_2 = MockNXtomo(
             scan_path=scan_path2,
             n_ini_proj=10,
             n_proj=10,
             n_alignement_proj=2,
-            create_final_ref=False,
+            create_final_flat=False,
             create_ini_dark=True,
-            create_ini_ref=True,
+            create_ini_flat=True,
             n_refs=1,
         ).scan
         self.assertEqual(scan_2.ff_interval, 0)
+
+
+def test_working_directory():
+    """test behavior of the working directory function"""
+    scan = NXtomoScan(scan=None, entry="my_entry")
+    assert scan.working_directory is None
+    scan = NXtomoScan(scan="/full/path/my_file.sh", entry="my_entry")
+    assert scan.working_directory == os.path.abspath("/full/path")
+    scan = NXtomoScan(scan="my_file.sh", entry="my_entry")
+    assert scan.working_directory == os.path.abspath(".")
