@@ -29,6 +29,7 @@ __date__ = "02/06/2017"
 
 
 import os
+from silx.utils.enum import Enum as _Enum
 
 
 __LBSRAM_PATH = '/lbsram'
@@ -85,18 +86,49 @@ def _set_dest_path(path):
     __DEST_PATH = path
 
 
+class SlurmSettingsMode(_Enum):
+    MANUAL = "manual"
+    GENERIC = "generic"
+    CAST_VOLUME = "cast_volume"
+    SLICE_RECONSTRUCTION = "slice_reconstruction"
+    VOLUME_RECONSTRUCTION = "volume_reconstruction"
+    STITCHING = "stitching"
+
+    @staticmethod
+    def get_settings_class(mode):
+        assert isinstance(mode, SlurmSettingsMode)
+        if mode is SlurmSettingsMode.MANUAL:
+            return None
+        elif mode is SlurmSettingsMode.GENERIC:
+            return SlurmSettings
+        elif mode is SlurmSettingsMode.CAST_VOLUME:
+            return DefaultSlurmSettingsCastVolume
+        elif mode is SlurmSettingsMode.SLICE_RECONSTRUCTION:
+            return DefaultSlurmSettingsSliceReconstruction
+        elif mode is SlurmSettingsMode.VOLUME_RECONSTRUCTION:
+            return DefaultSlurmSettingsVolumeReconstruction
+        elif mode is SlurmSettingsMode.STITCHING:
+            return DefaultSlurmSettingsStitching
+        else:
+            raise ValueError(f"{mode} not handled")
+
+
 class SlurmSettings:
     # Default slurm cluster configuration
-    N_CORES_PER_WORKER = 4
+
+    N_CORES_PER_TASK = 4
     """Number of CPU per worker"""
 
-    N_WORKERS = 1
+    N_TASKS = 1
     """Number of worker"""
 
-    MEMORY_PER_WORKER = 50  # memory in GB
+    N_JOBS = 1
+    """on how many job we want to split the EwoksTask"""
+
+    MEMORY_PER_WORKER = 128  # memory in GB
     """Amount of memory per worker"""
 
-    QUEUE = "p9gpu-long"
+    PARTITION = "p9gpu"
     """Queue / partition to use"""
 
     DEFAULT_WALLTIME = "01:00:00"  # None if the default wall time
@@ -104,7 +136,7 @@ class SlurmSettings:
     N_GPUS_PER_WORKER = 1
     """number of gpu per worker"""
 
-    PYTHON_VENV = "/scisoft/tomotools_env/tomwer/ubuntu20_pow9/latest_gpu_stable"
+    PYTHON_VENV = None
     """Python executable to take. Useful if compute nodes have a different environment from the front end.
     """
 
@@ -112,8 +144,50 @@ class SlurmSettings:
     """Slurm cluster project name. `scan`, `process` and `info` will be format.
     """
 
-    PORT_RANGE = 36087, 38000, 1
-    "TCP Port for the dask-distributed scheduler. Provided as start, stop, step"
+    MODULES_TO_LOAD = ("tomotools/stable", )
 
-    DASHBOARD_PORT = 36088
-    "TCP Port for the dashboard"
+    SBATCH_EXTRA_PARAMS = {
+        "export": "NONE",  # value to provide to sbatch --export={}
+    }
+
+
+class DefaultSlurmSettingsCastVolume(SlurmSettings):
+    """
+    default proposed configuration for casting a volume remotly
+    """
+    N_GPUS_PER_WORKER = 0
+
+    N_CORES_PER_TASK = 8
+
+    MEMORY_PER_WORKER = 128  # memory in GB
+
+    PARTITION = "nice*"
+
+
+class DefaultSlurmSettingsSliceReconstruction(SlurmSettings):
+    """
+    default proposed configuration for reconstructing a single slice remotly
+    """
+    MEMORY_PER_WORKER = 256  # memory in GB
+
+    PARTITION = None
+
+
+class DefaultSlurmSettingsVolumeReconstruction(SlurmSettings):
+    """
+    default proposed configuration for reconstructing a full volume remotly
+    """
+    MEMORY_PER_WORKER = 256  # memory in GB
+
+    PARTITION = None
+
+
+class DefaultSlurmSettingsStitching(SlurmSettings):
+    """*
+    default proposed configuration for stitching
+    """
+    N_JOBS = 15
+
+    PARTITION = "nice-long"
+
+    N_GPUS_PER_WORKER = 0
