@@ -30,16 +30,22 @@ __license__ = "MIT"
 __date__ = "11/10/2021"
 
 
-import subprocess
+import uuid
+from datetime import datetime
+
+from sluurp.utils import has_sbatch_available
 
 
 def is_slurm_available() -> bool:
     """
     Return True if the environment knows about slurm command (sbatch)
     """
-    try:
-        subprocess.call(["sbatch", "--version"], stdout=subprocess.PIPE)
-    except OSError:
-        return False
+    return has_sbatch_available()
+
+
+def get_slurm_script_name(prefix="") -> str:
+    now_str = str(datetime.now()).replace(" ", "_")
+    if prefix in ("", None):
+        return str(uuid.uuid4()).split("-")[0] + f"_{now_str}.sh"
     else:
-        return True
+        return f"{prefix}_" + str(uuid.uuid4()).split("-")[0] + f"_{now_str}.sh"
