@@ -28,10 +28,12 @@ __license__ = "MIT"
 __date__ = "14/10/2021"
 
 
-from silx.gui import qt
-from tomwer.core.scan.futurescan import FutureTomwerScan
-from collections import OrderedDict
 import functools
+from collections import OrderedDict
+
+from silx.gui import qt
+
+from tomwer.core.futureobject import FutureTomwerObject
 
 
 class FutureTomwerScanObserverWidget(qt.QWidget):
@@ -86,11 +88,6 @@ class FutureTomwerScanObserverWidget(qt.QWidget):
 
         # set up
         self._convertWhenFinished.setChecked(True)
-        h_header = self.observationTable.horizontalHeader()
-        for column_index in (0, 2, 3, 4):
-            h_header.setSectionResizeMode(column_index, qt.QHeaderView.Interactive)
-        for column_index in (1,):
-            h_header.setSectionResizeMode(column_index, qt.QHeaderView.Stretch)
 
         # connect signal / slot
         self.filterWidget.sigChanged.connect(self._filterUpdated)
@@ -100,39 +97,41 @@ class FutureTomwerScanObserverWidget(qt.QWidget):
 
     def _convertSelected(self, *args, **kwargs):
         for cell in self.observationTable.selectionModel().selectedRows():
-            future_scan = self.observationTable.model().futureScans.get(
+            future_tomo_obj = self.observationTable.model().futureTomoObjs.get(
                 cell.row(), None
             )
-            if future_scan is not None:
-                self.sigConversionRequested.emit(future_scan)
-        self.observationTable.model().endResetModel()
+            if future_tomo_obj is not None:
+                self.sigConversionRequested.emit(future_tomo_obj)
+        self.observationTable.model().updateIndices()
+        self.observationTable.model().layoutChanged.emit()
 
     def _cancelSelected(self, *args, **kwargs):
         for cell in self.observationTable.selectionModel().selectedRows():
-            future_scan = self.observationTable.model().futureScans.get(
+            future_tomo_obj = self.observationTable.model().futureTomoObjs.get(
                 cell.row(), None
             )
-            if future_scan is not None:
-                future_scan.cancel()
-        self.observationTable.model().endResetModel()
+            if future_tomo_obj is not None:
+                future_tomo_obj.cancel()
+        self.observationTable.model().layoutChanged.emit()
 
-    def addFutureScan(self, future_scan: FutureTomwerScan):
-        self.observationTable.model().addFutureScan(future_scan)
+    def addFutureTomoObj(self, future_tomo_obj: FutureTomwerObject):
+        self.observationTable.model().addFutureTomoObj(future_tomo_obj)
 
-    def removeFutureScan(self, future_scan: FutureTomwerScan):
-        self.observationTable.model().removeFutureScan(future_scan)
+    def removeFutureTomoObj(self, future_tomo_obj: FutureTomwerObject):
+        self.observationTable.model().removeFutureTomoObj(future_tomo_obj)
 
     def _filterUpdated(self):
         self.observationTable.model().filtered_status = (
             self.filterWidget.getFilteredStatus()
         )
+        self.observationTable.model().endResetModel()
 
     def updateView(self):
         """
         Update scan view and each future scan status
         """
-        self.observationTable.model().updateStatusAllFutureScans()
-        self.observationTable.model().endResetModel()
+        self.observationTable.model().updateStatusAllFutureTomoObjs()
+        self.observationTable.model().layoutChanged.emit()
 
     def convertWhenFinished(self):
         return self._convertWhenFinished.isChecked()
@@ -140,8 +139,8 @@ class FutureTomwerScanObserverWidget(qt.QWidget):
     def _conversionPolicyChanged(self, *args, **kwargs):
         self.sigConversionPolicyChanged.emit()
 
-    def futureScans(self):
-        return self.observationTable.model().futureScans
+    def futureTomoObjs(self):
+        return self.observationTable.model().futureTomoObjs
 
     def setConvertWhenFinished(self, convert):
         self._convertWhenFinished.setChecked(convert)
@@ -154,11 +153,11 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
     def __init__(self, parent, header, *args):
         qt.QAbstractTableModel.__init__(self, parent, *args)
         self.header = header
-        self.futureScans = OrderedDict()
+        self.futureTomoObjs = OrderedDict()
         # key is index, value is scan
-        self._filteredFutureScans = OrderedDict()
+        self._filteredFutureTomoObjs = OrderedDict()
         # key is index, value is scan but only for the scan we want to display
-        self._scansStatus = dict()
+        self._tomoObjStatus = dict()
         # key is the future scan, value is the status of the scan
         self._filteredStatus = tuple()
         # list of status to filter
@@ -174,36 +173,37 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
         self._headersI = {v: k for k, v in self._headers.items()}
         assert len(self._headers) == len(self._headersI), "insure values are unique"
 
-    def _computeUnfilteredFutureScans(self):
-        def is_filtered(scan):
-            return scan.status in self._filteredStatus
+    def is_filtered(self, future_scan):
+        return future_scan.status in self._filteredStatus
 
+    def _computeUnfilteredFutureTomoObjs(self):
         i_row = 0
-        self._filteredFutureScans = OrderedDict()
+        self._filteredFutureTomoObjs = OrderedDict()
 
-        for _, scan in self.futureScans.items():
-            if not is_filtered(scan):
-                self._filteredFutureScans[i_row] = scan
+        for _, scan in self.futureTomoObjs.items():
+            if not self.is_filtered(scan):
+                self._filteredFutureTomoObjs[i_row] = scan
                 i_row += 1
-        return self._filteredFutureScans
+        return self._filteredFutureTomoObjs
 
     def endResetModel(self, *args, **kwargs):
-        self._computeUnfilteredFutureScans()
+        self._computeUnfilteredFutureTomoObjs()
         super().endResetModel(*args, **kwargs)
 
-    def addFutureScan(self, future_scan):
-        if not isinstance(future_scan, FutureTomwerScan):
+    def addFutureTomoObj(self, future_tomo_obj):
+        if not isinstance(future_tomo_obj, FutureTomwerObject):
             raise TypeError(
-                f"future_scan is expected to be an instance of {FutureTomwerScan} and not {type(future_scan)}"
+                f"future_tomo_obj is expected to be an instance of {FutureTomwerObject} and not {type(future_tomo_obj)}"
             )
-        self.futureScans[len(self.futureScans)] = future_scan
-        self._scansStatus[future_scan] = "running"
-        for future in future_scan.futures:
+        self.futureTomoObjs[len(self.futureTomoObjs)] = future_tomo_obj
+        self._tomoObjStatus[future_tomo_obj] = "pending"
+        for future in future_tomo_obj.futures:
             callback = functools.partial(
                 self._updateStatus,
-                future_scan=future_scan,
+                future_tomo_obj=future_tomo_obj,
             )
 
+            # TODO: remove this since we are not using distributed anymore...
             class CallBack:
                 # we cannot create a future directly because distributed enforce
                 # the callback to have a function signature with only the future
@@ -215,36 +215,70 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
                     self.f_partial()
 
             future.add_done_callback(CallBack(callback).process)
-        self.endResetModel()
+        obj_index = len(self._filteredFutureTomoObjs)
 
-    def updateStatusAllFutureScans(self):
-        for future_scan in self._scansStatus:
-            self._scansStatus[future_scan] = future_scan.status
+        if not self.is_filtered(future_tomo_obj):
+            self._filteredFutureTomoObjs[obj_index] = future_tomo_obj
 
-    def _updateStatus(self, future_scan):
-        if future_scan is None:
+        self.dataChanged.emit(
+            self.createIndex(obj_index, 0),
+            self.createIndex(obj_index, len(self._headers) - 1),
+        )
+        self.layoutChanged.emit()
+
+    def updateStatusAllFutureTomoObjs(self):
+        for future_tomo_obj in self._tomoObjStatus:
+            self._tomoObjStatus[future_tomo_obj] = future_tomo_obj.status
+
+    def _updateStatus(self, future_tomo_obj):
+        if future_tomo_obj is None:
             return
-        status = future_scan.status
-        if (
-            future_scan not in self._scansStatus
-            or status != self._scansStatus[future_scan]
-        ):
-            self._scansStatus[future_scan] = status
-            self.sigStatusUpdated.emit(future_scan, status)
-            self.endResetModel()
+        status = future_tomo_obj.status
 
-    def removeFutureScan(self, future_scan):
-        for container in (self.futureScans, self._filteredFutureScans):
-            futureScansI = {v: k for k, v in container.items()}
-            idx = futureScansI.get(future_scan, None)
+        trigger_update = (
+            future_tomo_obj not in self._tomoObjStatus
+            or status != self._tomoObjStatus[future_tomo_obj]
+            or status
+            in (
+                "finished",
+                "completed",
+            )  # hack: when several finish at the same time, ensure the conversion is done :()
+        )
+        if trigger_update:
+            self._tomoObjStatus[future_tomo_obj] = status
+            obj_index = list(self._tomoObjStatus.keys()).index(future_tomo_obj)
+            try:
+                self.sigStatusUpdated.emit(future_tomo_obj, status)
+            except Exception:
+                pass
+            else:
+                self.dataChanged.emit(
+                    self.createIndex(obj_index, 0),
+                    self.createIndex(obj_index, len(self._headers) - 1),
+                )
+
+    def removeFutureTomoObj(self, future_tomo_obj):
+        for container in (self.futureTomoObjs, self._filteredFutureTomoObjs):
+            futureTomoObjsI = {v: k for k, v in container.items()}
+            idx = futureTomoObjsI.get(future_tomo_obj, None)
             if idx is not None:
                 del container[idx]
-        if future_scan in self._scansStatus:
-            del self._scansStatus[future_scan]
-        self.endResetModel()
+        if future_tomo_obj in self._tomoObjStatus:
+            del self._tomoObjStatus[future_tomo_obj]
+        self.layoutChanged.emit()
 
-    def getUnfilteredFutureScans(self):
-        return self._filteredFutureScans
+    def updateIndices(self):
+        """
+        Update tomo object indices when a tomo object has been removed (during a convertion for example)
+        """
+        tomo_objs = list(self.futureTomoObjs.values())
+        self.futureTomoObjs.clear()
+        for i_tomo_obj, tomo_obj in enumerate(tomo_objs):
+            self.futureTomoObjs[i_tomo_obj] = tomo_obj
+        self._computeUnfilteredFutureTomoObjs()
+
+    def getUnfilteredFutureTomoObjs(self):
+        return self._filteredFutureTomoObjs
 
     @property
     def filtered_status(self):
@@ -253,15 +287,15 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
     @filtered_status.setter
     def filtered_status(self, filters):
         self._filteredStatus = filters
-        self.endResetModel()
+        self.layoutChanged.emit()
 
     def clear(self):
-        self.futureScans = OrderedDict()
-        self._filteredFutureScans = OrderedDict()
-        self.endResetModel()
+        self.futureTomoObjs = OrderedDict()
+        self._filteredFutureTomoObjs = OrderedDict()
+        self.layoutChanged.emit()
 
     def rowCount(self, parent=None):
-        return len(self.getUnfilteredFutureScans())
+        return len(self.getUnfilteredFutureTomoObjs())
 
     def columnCount(self, parent=None):
         return len(self._headers)
@@ -274,22 +308,14 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
                 return None
 
     def data(self, index, role):
-        if index.isValid() is False:
+        if not index.isValid():
             return None
 
         if role not in (qt.Qt.DisplayRole, qt.Qt.ToolTipRole, qt.Qt.BackgroundRole):
             return None
 
-        f_scan = self._filteredFutureScans[index.row()]
-
-        def get_client_has_what_info():
-            if f_scan.clients is not None:
-                try:
-                    return "\n".join([client.has_what() for client in f_scan.clients])
-                except Exception:
-                    return str()
-            else:
-                return str()
+        key = list(self._filteredFutureTomoObjs.keys())[index.row()]
+        f_scan = self._filteredFutureTomoObjs[key]
 
         if role == qt.Qt.BackgroundRole:
             if index.column() in (self._headersI["status"],):
@@ -303,7 +329,7 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
                     return qt.QColor("#fcba03")
                 elif f_scan.status == "running":
                     return qt.QColor("#839684")
-                elif f_scan.status == "finished":
+                elif f_scan.status in ("finished", "completed"):
                     return qt.QColor("#068c0c")
 
         elif role == qt.Qt.DisplayRole:
@@ -311,7 +337,7 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
             if col_type == "start-time":
                 return f_scan.start_time.strftime("%H:%M:%S")
             elif col_type == "dataset":
-                return str(f_scan.scan.get_dataset_identifier())
+                return str(f_scan.tomo_obj.get_identifier())
             elif col_type == "requester":
                 from processview.core.manager import ProcessManager
 
@@ -324,31 +350,31 @@ class _DatasetProcessModel(qt.QAbstractTableModel):
             elif col_type == "status":
                 return f_scan.status
             elif col_type == "slurm-job":
-                infos_job = ""
-                if f_scan.clients is not None:
-                    for client in f_scan.clients:
-                        job_ids = [str(job_id) for job_id in client.job_ids.values()]
-                        infos_job += ", ".join(job_ids)
+                infos_job = ",".join(
+                    [
+                        str(future.job_id) if hasattr(future, "job_id") else ""
+                        for future in f_scan.futures
+                    ]
+                )
                 return infos_job
-            elif col_type == "info":
-                return get_client_has_what_info()
-
-        elif role == qt.Qt.ToolTipRole:
-            return get_client_has_what_info()
 
 
 class ObservationTable(qt.QTableView):
-    def __init__(self, parent):
-        qt.QTableView.__init__(self, parent)
-
-        # QMenu
-        self.menu = qt.QMenu(self)
-        self._infoAction = qt.QAction("info")
-        self._infoAction.triggered.connect(self._requestInfo)
-        self.menu.addAction(self._infoAction)
-
-    def _requestInfo(self, *args, **kwargs):
-        raise NotImplementedError
+    def sizeHintForColumn(self, column):
+        # warning: to be called we need to call resizeColumnsToContents upper
+        if column == 1:  # dataset column
+            return 400
+        elif column in (
+            0,
+            2,
+            3,
+            4,
+        ):  # start-time, status, requester, slurm-job
+            return 55
+        elif column == 5:
+            return 200
+        else:
+            return super().sizeHintForColumn(column)
 
 
 class FilterWidget(qt.QGroupBox):
@@ -360,7 +386,6 @@ class FilterWidget(qt.QGroupBox):
     """signal emit when filter change"""
 
     def __init__(self, parent=None):
-
         super().__init__("show job with status", parent)
         self.setLayout(qt.QHBoxLayout())
         # spacer
