@@ -27,12 +27,18 @@ __license__ = "MIT"
 __date__ = "19/10/2021"
 
 
-from tomwer.gui.reconstruction.nabu.slurm import SlurmSettingsWidget
-from tomwer.core.settings import SlurmSettings
-from silx.gui.utils.testutils import TestCaseQt
-from silx.gui import qt
-from tomwer.test.utils import skip_gui_test
 import pytest
+from silx.gui import qt
+from silx.gui.utils.testutils import TestCaseQt
+
+from tomwer.core.settings import SlurmSettings
+from tomwer.gui.cluster.slurm import (
+    SlurmSettingsMode,
+    SlurmSettingsWidget,
+    SlurmSettingsWindow,
+)
+from tomwer.tests.utils import skip_gui_test
+from tomwer.tests.conftest import qtapp  # noqa F401
 
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
@@ -45,29 +51,38 @@ class TestSlurmWidget(TestCaseQt):
         self.slurmWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
         self.slurmWidget.close()
         self.slurmWidget = None
-        TestCaseQt.tearDown(self)
 
     def testGetConfiguration(self):
-        self.slurmWidget._slurmCB.setChecked(False)
         dict_res = self.slurmWidget.getConfiguration()
         expected_dict = {
-            "active_slurm": False,
-            "cores": SlurmSettings.N_CORES_PER_WORKER,
-            "n_workers": SlurmSettings.N_WORKERS,
+            "cpu-per-task": SlurmSettings.N_CORES_PER_TASK,
+            "n_tasks": SlurmSettings.N_TASKS,
             "memory": SlurmSettings.MEMORY_PER_WORKER,
-            "queue": SlurmSettings.QUEUE,
+            "partition": "",
             "n_gpus": SlurmSettings.N_GPUS_PER_WORKER,
+            "job_name": "tomwer_{scan}_-_{process}_-_{info}",
+            "n_jobs": 1,
+            "modules": ("tomotools/stable",),
+            "walltime": "01:00:00",
+            "sbatch_extra_params": {
+                "export": "NONE",
+                "gpu_card": None,
+            },
         }
         assert dict_res == expected_dict, f"{dict_res} vs {expected_dict}"
 
     def testSetConfiguration(self):
         self.slurmWidget.setConfiguration(
             {
-                "cores": 2,
-                "n_workers": 3,
+                "cpu-per-task": 2,
+                "n_tasks": 3,
                 "memory": 156,
-                "queue": "test-queue",
+                "partition": "test-queue",
                 "n_gpus": 5,
+                "modules": "mymodule, mysecond/10.3",
+                "sbatch_extra_params": {
+                    "export": "ALL",
+                },
             }
         )
 
@@ -76,3 +91,17 @@ class TestSlurmWidget(TestCaseQt):
         assert self.slurmWidget.getMemory() == 156
         assert self.slurmWidget.getQueue() == "test-queue"
         assert self.slurmWidget.getNGPU() == 5
+        assert self.slurmWidget.getSBatchExtraParams() == {
+            "export": "ALL",
+            "gpu_card": None,
+        }
+
+
+def test_SlurmSettingsWindow(qtapp):  # noqa F811
+    """
+    test that slurm SettingsWindow can load all the different slurm settings modes
+    """
+    widget = SlurmSettingsWindow()
+    for mode in SlurmSettingsMode:
+        widget.setCurrentSettingsMode(mode)
+        widget.getConfiguration()
