@@ -30,11 +30,13 @@ __date__ = "08/11/2021"
 import asyncio
 import os
 import shutil
+import tempfile
+
 from silx.gui import qt
 from silx.gui.utils.testutils import TestCaseQt
-import tempfile
-from tomwer.core.utils.scanutils import MockHDF5
-from tomwer.core.scan.futurescan import FutureTomwerScan
+
+from tomwer.core.futureobject import FutureTomwerObject
+from tomwer.core.utils.scanutils import MockNXtomo
 from tomwer.gui.cluster.supervisor import FutureTomwerScanObserverWidget
 
 
@@ -47,15 +49,15 @@ class TestSupervisor(TestCaseQt):
 
         # set up scans
         self._scans = []
-        self._future_scans = []
+        self._future_tomo_objs = []
         for i in range(5):
             # create scan
-            scan = MockHDF5(
+            scan = MockNXtomo(
                 scan_path=os.path.join(self.tempdir, f"scan_test{i}"),
                 n_proj=10,
                 n_ini_proj=10,
                 create_ini_dark=False,
-                create_ini_ref=False,
+                create_ini_flat=False,
                 dim=10,
             ).scan
             self._scans.append(scan)
@@ -64,9 +66,9 @@ class TestSupervisor(TestCaseQt):
             future = asyncio.Future()
             if i == 1:
                 future.set_result(None)
-            self._future_scans.append(
-                FutureTomwerScan(
-                    scan=scan,
+            self._future_tomo_objs.append(
+                FutureTomwerObject(
+                    tomo_obj=scan,
                     futures=(future,),
                 )
             )
@@ -80,10 +82,10 @@ class TestSupervisor(TestCaseQt):
         self._supervisor = None
         shutil.rmtree(self.tempdir)
         self._scans.clear()
-        self._future_scans.clear()
+        self._future_tomo_objs.clear()
         super().tearDown()
 
     def test(self):
-        for futureScan in self._future_scans:
-            self._supervisor.addFutureScan(future_scan=futureScan)
+        for future_tomo_obj in self._future_tomo_objs:
+            self._supervisor.addFutureTomoObj(future_tomo_obj=future_tomo_obj)
         self._supervisor.show()
