@@ -29,19 +29,23 @@ __date__ = "16/03/2020"
 
 
 import os
-import weakref
 import socket
+import weakref
+
 from silx.gui import qt
+
 from tomwer.core import settings
 from tomwer.core.process.utils import LastReceivedScansDict
+from tomwer.core.scan.blissscan import BlissScan
 from tomwer.gui import icons as tomwericons
 from tomwer.gui.control import datareacheractions as actions
 from tomwer.gui.control.history import ScanHistory
 from tomwer.gui.control.observations import ScanObservation
-from tomwer.core.scan.blissscan import BlissScan
+from tomwer.gui.utils.inputwidget import (
+    HDF5ConfigFileSelector,
+    NXTomomillOutputDirSelector,
+)
 from tomwer.synctools.rsyncmanager import BlissSequenceRSyncWorker
-from tomwer.gui.utils.inputwidget import ConfigFileSelector
-from tomwer.gui.utils.inputwidget import NXTomomillOutputDirSelector
 
 
 class DataListenerWidget(qt.QMainWindow):
@@ -121,10 +125,10 @@ class DataListenerWidget(qt.QMainWindow):
 
         # signal / slot connection
         self._actionGroup.triggered.connect(self._updateCentralWidget)
-        self._controlWidget.sigActivated.connect(self._activated)
-        self._controlWidget.sigDeactivated.connect(self._deactivated)
-        self._configWindow.sigConfigurationChanged.connect(self._configChanged)
-        self._configWindow.sigCFGFileChanged.connect(self._cfgFileChanged)
+        self._controlWidget.sigActivated.connect(self.sigActivate)
+        self._controlWidget.sigDeactivated.connect(self.sigDeactivate)
+        self._configWindow.sigConfigurationChanged.connect(self.sigConfigurationChanged)
+        self._configWindow.sigCFGFileChanged.connect(self.sigCFGFileChanged)
 
         # expose api
         self.activate = self._controlWidget.activate
@@ -167,18 +171,6 @@ class DataListenerWidget(qt.QMainWindow):
         self._listener().sigScanAdded.disconnect(self._acquisitionUpdated)
         self._listener().sigAcquisitionEnded.disconnect(self._acquisitionEnded)
 
-    def _activated(self):
-        self.sigActivate.emit()
-
-    def _deactivated(self):
-        self.sigDeactivate.emit()
-
-    def _configChanged(self):
-        self.sigConfigurationChanged.emit()
-
-    def _cfgFileChanged(self, cfg_file):
-        self.sigCFGFileChanged.emit(cfg_file)
-
     def _updateCentralWidget(self, action_triggered):
         action_to_widget = {
             self._controlAction: self._controlWidget,
@@ -390,7 +382,7 @@ class ConfigurationWidget(qt.QDialog):
         # configuration file to use
         self._cfgLabel = qt.QLabel("config file")
         self.layout().addWidget(self._cfgLabel, 2, 0, 1, 1)
-        self._cfgWidget = ConfigFileSelector(self)
+        self._cfgWidget = HDF5ConfigFileSelector(self)
         self._cfgWidget.setContentsMargins(0, 0, 0, 0)
         self._cfgWidget.layout().setContentsMargins(0, 0, 0, 0)
         self.layout().addWidget(self._cfgWidget, 2, 1, 1, 2)
