@@ -28,18 +28,19 @@ __license__ = "MIT"
 __date__ = "23/07/2020"
 
 
-from silx.gui import qt
-from tomwer.gui.visualization.dataviewer import DataViewer
-from tomwer.core.process.control.scanvalidator import ScanValidator as BaseDataValidator
-from tomwer.gui.utils.waiterthread import QWaiterThread
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.scanbase import _TomwerBaseDock
-from tomwer.gui.qfolderdialog import QScanDialog
-from tomwer.core.scan.scanfactory import ScanFactory
-import weakref
-from contextlib import AbstractContextManager
 import logging
 import time
+import weakref
+from contextlib import AbstractContextManager
+
+from silx.gui import qt
+
+from tomwer.core.process.control.scanvalidator import ScanValidator as BaseDataValidator
+from tomwer.core.scan.scanbase import TomwerScanBase, _TomwerBaseDock
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.gui.qfolderdialog import QDataDialog
+from tomwer.gui.utils.waiterthread import QWaiterThread
+from tomwer.gui.visualization.dataviewer import DataViewer
 
 _logger = logging.getLogger(__file__)
 
@@ -141,10 +142,10 @@ class DataValidator(qt.QMainWindow, BaseDataValidator):
 
         with self.SliderCM(self._scanSelectorSlider):
             current_index = self._scanSelectorSlider.value()
-            self._scanSelectorSlider.setRange(0, len(self._scansToValidate) - 1)
+            self._scanSelectorSlider.setRange(0, len(self._scans) - 1)
             self._scanSelectorSlider.setValue(current_index)
 
-        if str(scan) in self._scansToValidate:
+        if str(scan) in self._scans:
             self.setActiveScan(scan)
 
     def setActiveScan(self, scan: TomwerScanBase):
@@ -155,13 +156,13 @@ class DataValidator(qt.QMainWindow, BaseDataValidator):
         """
         if scan is None:
             self.clear()
-        elif str(scan) not in self._scansToValidate:
+        elif str(scan) not in self._scans:
             _logger.warning("scan has not been registered")
         else:
             with self.SliderCM(self._scanSelectorSlider):
                 self._centralWidget.setScan(scan)
                 self._controlButtons.setScan(scan)
-                index = list(self._scansToValidate.keys()).index(str(scan))
+                index = list(self._scans.keys()).index(str(scan))
                 self._scanSelectorSlider.setValue(index)
 
     def _updateCurrentScan(self):
@@ -174,11 +175,11 @@ class DataValidator(qt.QMainWindow, BaseDataValidator):
         :return:
         """
         current_scan_index = self._scanSelectorSlider.value()
-        if current_scan_index >= len(self._scansToValidate):
+        if current_scan_index >= len(self._scans):
             return None
         else:
-            scan_id = list(self._scansToValidate.keys())[current_scan_index]
-            scan = self._scansToValidate[scan_id]
+            scan_id = list(self._scans.keys())[current_scan_index]
+            scan = self._scans[scan_id]
             assert isinstance(scan, TomwerScanBase)
             return scan
 
@@ -190,7 +191,7 @@ class DataValidator(qt.QMainWindow, BaseDataValidator):
             BaseDataValidator._validateScan(self, scan)
             with self.SliderCM(self._scanSelectorSlider):
                 current_index = self._scanSelectorSlider.value()
-                self._scanSelectorSlider.setRange(0, len(self._scansToValidate) - 1)
+                self._scanSelectorSlider.setRange(0, len(self._scans) - 1)
                 # note: slider force the value to be in [minimum, maximum]
                 self._scanSelectorSlider.setValue(current_index)
                 self._updateCurrentScan()
@@ -227,7 +228,7 @@ class DataValidator(qt.QMainWindow, BaseDataValidator):
 
     def _addNewScanCallback(self):
         """Callback when we want to add a new scan to the scan validator"""
-        dialog = QScanDialog(self, multiSelection=True)
+        dialog = QDataDialog(self, multiSelection=True)
 
         if not dialog.exec_():
             dialog.close()
@@ -240,9 +241,7 @@ class DataValidator(qt.QMainWindow, BaseDataValidator):
                 scans = ScanFactory.create_scan_objects(scan_path=folder)
             except Exception as e:
                 _logger.error(
-                    "cannot create scan instances from {}. Error is {}".format(
-                        folder, e
-                    )
+                    f"cannot create scan instances from {folder}. Error is {e}"
                 )
             else:
                 for scan in scans:
