@@ -28,19 +28,25 @@ __license__ = "MIT"
 __date__ = "18/02/2018"
 
 
-import os
 import logging
-from silx.gui import qt, icons as silxicons
-from tomwer.io.utils import get_default_directory
+import os
+from typing import Optional
+
+from silx.gui import icons as silxicons
+from silx.gui import qt
+
 from tomwer.core.process.control.datawatcher.datawatcher import _DataWatcher
-from tomwer.gui.utils.waiterthread import QWaiterThread
-from tomwer.gui.control import observations, history, datareacheractions
-from tomwer.gui.control.datawatcher import configuration
-from .datawatcherobserver import _QOngoingObservation
-from .datawatcherobserver import _QDataWatcherObserver
+from tomwer.core.scan.blissscan import BlissScan
+from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.core.scan.scanfactory import ScanFactory
 from tomwer.core.utils import logconfig
-from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.control import datareacheractions, history, observations
+from tomwer.gui.control.datawatcher import configuration
+from tomwer.gui.control.datawatcher.controlwidget import ControlWidget
+from tomwer.gui.utils.waiterthread import QWaiterThread
+from tomwer.io.utils import get_default_directory
+
+from .datawatcherobserver import _QDataWatcherObserver, _QOngoingObservation
 
 logger = logging.getLogger(__name__)
 
@@ -52,9 +58,6 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
     :param parent: the parent widget
     """
 
-    _textStopObservation = "Stop observation"
-    _textStartObservation = "Start observation"
-
     obsStatusToWidgetStatus = {
         "not processing": "Not processed",
         "none found": "Running",
@@ -76,10 +79,9 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
 
     DEFAULT_DIRECTORY = "/lbsram/data/visitor"
 
-    # TODO: status should also pass a TomoBase instance
     sigTMStatusChanged = qt.Signal(str)
-    """Signal emitted when the changed"""
-    sigScanReady = qt.Signal(TomwerScanBase)
+    """Signal emitted when the state changed"""
+    sigScanReady = qt.Signal(object)
     """Signal emitted when a scan is considered as ready"""
     sigFolderObservedChanged = qt.Signal()
     """Signal emitted when the user change the observed folder"""
@@ -87,6 +89,8 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
     """Signal emitted when the observation starts"""
     sigObservationEnd = qt.Signal()
     """Signal emitted when the observation end"""
+    sigFilterFileNamePatternChanged = qt.Signal(str)
+    """Signal emut when the filter pattern change"""
 
     def __init__(self, parent=None):
         qt.QMainWindow.__init__(self, parent)
@@ -103,6 +107,19 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         status ..."""
         _DataWatcher.__init__(self)
         self.setFolderObserved(self.folderObserved)
+        self._setStateIcon(silxicons.getQIcon("remove"))
+
+        # connect signal / slot
+        self._qteFolderSelected.textChanged.connect(self._updateFolderObserved)
+        self._qteFolderSelected.editingFinished.connect(self._restartObservation)
+        self._filterQLE.editingFinished.connect(self._restartObservation)
+        self._filterQLE.editingFinished.connect(self._filterChanged)
+        self._qpbstartstop.released.connect(self._switchObservation)
+
+        # expose api
+        self.sigObservationModeChanged = (
+            self.getConfigWindow()._observationMethod.sigSelectionChanged
+        )
 
     def _initClass(self):
         self.loopObservationThread = None
@@ -131,7 +148,6 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
 
         # set initial path to observe
         self.setFolderObserved(self._getInitPath())
-        self._initStatusView()
 
         # hide all windows by default
         for widget in (
@@ -153,6 +169,10 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         self._historyAction.toggled[bool].connect(self.getHistoryWindow().setVisible)
 
         self._controlAction.setChecked(True)
+        # expose signal
+        self.sigObservationModeChanged = (
+            self.getConfigWindow()._observationMethod.sigSelectionChanged
+        )
 
     def close(self):
         logger.info("closing the datawatacher")
@@ -160,11 +180,14 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         if self.loopObservationThread is not None:
             self.loopObservationThread.blockSignals(True)
             self._disconnectObserverThread()
-            self.loopObservationThread.wait()
+            self.loopObservationThread.wait(4000)
             self.loopObservationThread = None
-        if self.observationThread.isRunning():
-            self.observationThread.blockSignals(True)
-            self.observationThread.wait()
+        if (
+            self.observationThread is not None
+            and self.observationThread.isRunning()  # pylint: disable=E1101
+        ):
+            self.observationThread.blockSignals(True)  # pylint: disable=E1101
+            self.observationThread.wait(4000)
             self.observationThread = None
         super(DataWatcherWidget, self).close()
 
@@ -215,21 +238,7 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
 
     def getControlWindow(self):
         if self.controlWidget is None:
-            self.controlWidget = qt.QWidget(parent=self)
-            self.controlWidget.setLayout(qt.QVBoxLayout())
-            layout = self.controlWidget.layout()
-
-            self.statusBar = qt.QStatusBar(parent=self.controlWidget)
-            self._qlInfo = qt.QLabel(parent=self.controlWidget)
-
-            layout.addWidget(self._getFolderSelection())
-            layout.addWidget(self._qlInfo)
-            layout.addWidget(self._buildStartStopButton())
-            layout.addWidget(self.statusBar)
-
-            spacer = qt.QWidget(self)
-            spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
-            layout.addWidget(spacer)
+            self.controlWidget = ControlWidget(parent=self)
 
         return self.controlWidget
 
@@ -243,32 +252,15 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         self._mainWidget.layout().addWidget(self.getHistoryWindow())
         self.setCentralWidget(self._mainWidget)
 
-    def _buildStartStopButton(self):
-        """
-        Build the start/stop button in a QHLayout with one spacer on the left
-        and one on the right
-        """
-        widget = qt.QWidget(self.controlWidget)
-        layout = qt.QHBoxLayout()
-        widget.setLayout(layout)
+    def _filterChanged(self, *args, **kwargs):
+        self.sigFilterFileNamePatternChanged.emit(self._filterQLE.text())
 
-        # left spacer
-        spacerL = qt.QWidget(widget)
-        spacerL.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum)
-        layout.addWidget(spacerL)
-
-        # button
-        self._qpbstartstop = qt.QPushButton(self._textStartObservation)
-        self._qpbstartstop.setAutoDefault(True)
-        self._qpbstartstop.pressed.connect(self._switchObservation)
-        layout.addWidget(self._qpbstartstop)
-
-        # right spacer
-        spacerR = qt.QWidget(widget)
-        spacerR.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum)
-        layout.addWidget(spacerR)
-
-        return widget
+    def getFilterLinuxFileNamePattern(self) -> Optional[str]:
+        value = self._filterQLE.text().rstrip(" ").lstrip(" ")
+        if value == "":
+            return None
+        else:
+            return value
 
     def _buildLoopTimeBreak(self):
         """
@@ -289,11 +281,35 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
 
         return widget
 
+    @property
+    def _qlInfo(self):
+        return self.getControlWindow()._qlInfo
+
+    @property
+    def _qteFolderSelected(self):
+        return self.getControlWindow()._qteFolderSelected
+
+    @property
+    def _filterQLE(self):
+        return self.getControlWindow()._filterQLE
+
+    @property
+    def _qpbstartstop(self):
+        return self.getControlWindow()._qpbstartstop
+
+    @property
+    def _stateLabel(self):
+        return self.getControlWindow()._stateLabel
+
+    @property
+    def mystatusBar(self):
+        return self.getControlWindow().mystatusBar
+
     def _getInitPath(self):
         initPath = ""
         if "DATADIR" in os.environ:
             initPath = os.environ["DATADIR"]
-            self._qlInfo.setText(
+            self.getControlWindow()._qlInfo.setText(
                 "note : environment variable DATADIR found, "
                 "$DATADIR setted has the root of the observe folder"
             )
@@ -336,8 +352,8 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         if self.observationThread is not None:
             # remove connection
             self._disconnectObserverThread()
-            self.observationThread.blockSignals(True)
-            self.observationThread.wait()
+            self.observationThread.blockSignals(True)  # pylint: disable=E1101
+            self.observationThread.wait(4000)
 
         if sucess is False:
             self._setCurrentStatus(str("not processing"))
@@ -348,7 +364,7 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
                 message = "observation stopped"
                 self.sigObservationEnd.emit()
                 logger.inform(message)
-                self.statusBar.showMessage(message)
+                self.mystatusBar.showMessage(message)
                 self._setCurrentStatus("not processing")
             return True
 
@@ -360,9 +376,9 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
             mean that an observation was already running
         """
         if _DataWatcher.start(self):
-            mess = "start observation on %s" % self.folderObserved
+            mess = f"start observation on {self.folderObserved}"
             logger.inform(mess)
-            self.statusBar.showMessage(mess)
+            self.mystatusBar.showMessage(mess)
             self._setCurrentStatus("started")
             self.sigObservationStart.emit()
             return True
@@ -371,44 +387,12 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
 
     def _setIsObserving(self, b):
         _DataWatcher._setIsObserving(self, b)
-        if _DataWatcher.isObserving(self) is True:
-            self._qpbstartstop.setText(self._textStopObservation)
+        if self.isObserving is True:  # pylint: disable=E1101
+            self._qpbstartstop.setText(self.getControlWindow()._TXT_STOP_OBS)
         else:
-            self._qpbstartstop.setText(self._textStartObservation)
+            self._qpbstartstop.setText(self.getControlWindow()._TXT_START_OBS)
 
-    def _getFolderSelection(self):
-        """
-        Return the widget used for the folder selection
-        """
-        widget = qt.QWidget(self)
-        layout = qt.QHBoxLayout()
-
-        self._qtbSelectFolder = qt.QPushButton("Select folder", parent=widget)
-        self._qtbSelectFolder.setAutoDefault(True)
-        self._qtbSelectFolder.clicked.connect(self._setFolderPath)
-
-        self._qteFolderSelected = qt.QLineEdit("", parent=widget)
-        self._qteFolderSelected.textChanged.connect(self._updateFolderObserved)
-        self._qteFolderSelected.editingFinished.connect(self._restartObservation)
-
-        layout.setContentsMargins(0, 0, 0, 0)
-        layout.setSpacing(0)
-
-        layout.addWidget(self._qteFolderSelected)
-        layout.addWidget(self._qtbSelectFolder)
-
-        self.animated_icon = silxicons.getWaitIcon()
-        self.__stateLabel = qt.QLabel(parent=widget)
-        self.animated_icon.register(self.__stateLabel)
-        self._setStateIcon(silxicons.getQIcon("remove"))
-
-        self.__stateLabel.setFixedWidth(30)
-        layout.addWidget(self.__stateLabel)
-
-        widget.setLayout(layout)
-        return widget
-
-    def _setFolderPath(self):
+    def _setFolderPath(self):  # pragma: no cover
         """
         Ask the user the path to the folder to observe
         """
@@ -462,17 +446,17 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         else:
             return False
 
-    def _messageNotDir(self, dir):
-        super()._messageNotDir(dir=dir)
-        message = "Given path (%s) isn't a directory." % dir
-        self.statusBar.showMessage("!!! " + message + "!!!")
+    def _messageNotDir(self, dir_):
+        super()._messageNotDir(dir_=dir_)
+        message = f"Given path ({dir_}) isn't a directory."
+        self.mystatusBar.showMessage("!!! " + message + "!!!")
 
     def informationReceived(self, info):
-        self.statusBar.showMessage(info)
+        self.mystatusBar.showMessage(info)
 
     def _scanStatusChanged(self, scan, status):
-        mess = "scan %s is observed. Status: %s" % (os.path.basename(scan), status)
-        self.statusBar.showMessage(mess)
+        mess = f"scan {os.path.basename(scan)} is observed. Status: {status}"
+        self.mystatusBar.showMessage(mess)
 
     def _connectObserverThread(self):
         if self.observationThread is not None and self.obsThIsConnected is False:
@@ -490,20 +474,10 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
             self.observationThread.sigScanReady.disconnect(self._signalScanReady)
             self.obsThIsConnected = False
 
-    def _initStatusView(self):
-        """
-        The status view need a thread to update the animated icon when scanning
-        """
-        self.__threadAnimation = QWaiterThread(0.1)
-        self.__threadAnimation.finished.connect(self._updateAnimatedIcon)
-
     def _updateStatusView(self):
         """Update the processing state"""
         if self.currentStatus in self._animatedStates:
-            if not self.__threadAnimation.isRunning():
-                self.__threadAnimation.start()
-            elif self.__threadAnimation is not None:
-                self.__threadAnimation.wait()
+            pass
         elif self.currentStatus == "acquisition ended":
             self._setStateIcon(silxicons.getQIcon("selected"))
         elif self.currentStatus == "failure":
@@ -517,21 +491,9 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
         :param icon:the icon to set"""
         # needed for heritage from DataWatcher
         if icon is None:
-            self.__stateLabel.setPixmap(qt.QIcon().pixmap(30, state=qt.QIcon.On))
+            self._stateLabel.setPixmap(qt.QIcon().pixmap(30, state=qt.QIcon.On))
         else:
-            self.__stateLabel.setPixmap(icon.pixmap(30, state=qt.QIcon.On))
-
-    def _updateAnimatedIcon(self):
-        """Simple function which manage the waiting icon"""
-        if self.currentStatus in self._animatedStates:
-            icon = self.animated_icon.currentIcon()
-            if icon is None:
-                icon = qt.QIcon()
-            self.animated_icon._updateState()
-            self._setStateIcon(icon)
-
-            # get ready for the next animation
-            self.__threadAnimation.start()
+            self._stateLabel.setPixmap(icon.pixmap(30, state=qt.QIcon.On))
 
     def _signalScanReady(self, scan):
         if type(scan) is str:
@@ -539,11 +501,11 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
                 _scan = ScanFactory.create_scan_object(scan_path=scan)
             except Exception as e:
                 logger.error(
-                    "Fail to create a TomoBase instance from", scan, "Reason is", e
+                    f"Fail to create a TomoBase instance from {scan} Reason is {e}"
                 )
                 return
         else:
-            assert isinstance(scan, TomwerScanBase)
+            assert isinstance(scan, (TomwerScanBase, BlissScan))
             _scan = scan
 
         _DataWatcher._signalScanReady(self, _scan)
@@ -566,6 +528,7 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
             obsMethod=self.obsMethod,
             srcPattern=self.srcPattern,
             destPattern=self.destPattern,
+            file_name_pattern=self.linux_filter_file_pattern,
         )
         self._connectObserverThread()
 
@@ -575,7 +538,7 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
 
     def waitForObservationFinished(self):
         if self.observationThread.observations is not None:
-            for dir, thread in self.observationThread.observations.dict.items():
+            for _, thread in self.observationThread.observations.dict.items():
                 thread.wait(10)
 
     def _launchObservation(self):
@@ -584,9 +547,12 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
             return
 
         # manage data watcher observation
-        if self.observationThread is None or not self.observationThread.isRunning():
+        if (
+            self.observationThread is None
+            or not self.observationThread.isRunning()  # pylint: disable=E1101
+        ):
             if self._initObservation() is False:
-                self.currentStatus = self._setCurrentStatus("failure")
+                self._setCurrentStatus("failure")
                 logger.info(
                     "failed on observation",
                     extra={logconfig.DOC_TITLE: self._scheme_title},
@@ -608,11 +574,7 @@ class DataWatcherWidget(_DataWatcher, qt.QMainWindow):
             self.loopObservationThread.start()
 
     def _observation_thread_running(self):
-        return self.observationThread is not None and self.observationThread.isRunning()
-
-
-if __name__ == "__main__":
-    qapp = qt.QApplication([])
-    widget = DataWatcherWidget(parent=None)
-    widget.show()
-    qapp.exec_()
+        return (
+            self.observationThread is not None
+            and self.observationThread.isRunning()  # pylint: disable=E1101
+        )
