@@ -28,20 +28,21 @@ __license__ = "MIT"
 __date__ = "10/01/2019"
 
 
+import logging
 import os
+from fnmatch import fnmatch
+
+import h5py
 from silx.gui import qt
-from tomwer.core.process.control.datawatcher.datawatcherobserver import (
-    _OngoingObservation,
-)
+
+from tomwer.core.process.control.datawatcher import status as datawatcherstatus
 from tomwer.core.process.control.datawatcher.datawatcherobserver import (
     _DataWatcherObserver_MixIn,
+    _DataWatcherStaticObserverMixIn,
+    _OngoingObservation,
 )
-from tomwer.core.process.control.datawatcher.datawatcherobserver import (
-    _DataWatcherFixObserverMixIn,
-)
+from tomwer.core.scan.blissscan import BlissScan
 from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.process.control.datawatcher import status as datawatcherstatus
-import logging
 from tomwer.core.scan.scanfactory import ScanFactory
 
 logger = logging.getLogger(__name__)
@@ -55,7 +56,7 @@ class _QDataWatcherObserver(_DataWatcherObserver_MixIn, qt.QThread):
     tomwer.core package
     """
 
-    sigScanReady = qt.Signal(TomwerScanBase)
+    sigScanReady = qt.Signal(object)
 
     def __init__(
         self,
@@ -66,6 +67,7 @@ class _QDataWatcherObserver(_DataWatcherObserver_MixIn, qt.QThread):
         srcPattern=None,
         destPattern=None,
         ignoredFolders=None,
+        file_name_pattern=None,
     ):
         qt.QThread.__init__(self)
         _DataWatcherObserver_MixIn.__init__(
@@ -78,15 +80,16 @@ class _QDataWatcherObserver(_DataWatcherObserver_MixIn, qt.QThread):
             destPattern=destPattern,
             ignoredFolders=ignoredFolders,
             time_between_loops=0.2,
+            file_name_pattern=file_name_pattern,
         )
         self.observations.sigScanReady.connect(self._signalScanReady)
 
     def _signalScanReady(self, scan):
-        assert isinstance(scan, TomwerScanBase)
+        assert isinstance(scan, (TomwerScanBase, BlissScan))
         self.sigScanReady.emit(scan)
 
     def _getObserver(self, scanID):
-        return _QDataWatcherFixObserver(
+        return _QDataWatcherStaticObserver(
             scanID=scanID,
             obsMethod=self.obsMethod,
             srcPattern=self.srcPattern,
@@ -96,20 +99,40 @@ class _QDataWatcherObserver(_DataWatcherObserver_MixIn, qt.QThread):
         )
 
     def run(self):
-        def process(directory):
-            if (
-                self.observations.isObserving(directory) is False
-                and self.dataWatcherProcess._isScanDirectory(directory)
-                and directory not in self.observations.ignoredFolders
-            ):
-                self.observe(directory)
+        look_for_hdf5_file = self.obsMethod in (
+            datawatcherstatus.BLISS_SCAN_END,
+            datawatcherstatus.NXtomo_END,
+        )
 
-            try:
-                for f in os.listdir(directory):
-                    if os.path.isdir(os.path.join(directory, f)):
-                        process(os.path.join(directory, f))
-            except Exception:
-                pass
+        def process(file_path):
+            do_observation = (
+                self.observations.isObserving(file_path) is False
+                and (
+                    (look_for_hdf5_file and h5py.is_hdf5(file_path))
+                    or (self.dataWatcherProcess._isScanDirectory(file_path))
+                )
+                and file_path not in self.observations.ignoredFolders
+            )
+            if (
+                do_observation
+                and self.file_name_pattern is not None
+                and not fnmatch(os.path.basename(file_path), self.file_name_pattern)
+            ):
+                do_observation = False
+            if do_observation is True:
+                # do the observation
+                self.observe(file_path)
+
+            if os.path.isdir(file_path):
+                try:
+                    for f in os.listdir(file_path):
+                        full_file_path = os.path.join(file_path, f)
+                        if os.path.isdir(full_file_path) or (
+                            look_for_hdf5_file and h5py.is_hdf5(full_file_path)
+                        ):
+                            process(full_file_path)
+                except Exception:
+                    pass
 
         if not os.path.isdir(self.headDir):
             logger.warning("can't observe %s, not a directory" % self.headDir)
@@ -134,7 +157,7 @@ class _QOngoingObservation(_OngoingObservation, qt.QObject):
     _OngoingObservation with a QObject and signals for each event
     """
 
-    sigScanReady = qt.Signal(TomwerScanBase)
+    sigScanReady = qt.Signal(object)
     """Emitted when a finished acquisition is detected"""
     sigObsAdded = qt.Signal(str)
     """Signal emitted when an observation is added"""
@@ -150,11 +173,14 @@ class _QOngoingObservation(_OngoingObservation, qt.QObject):
     def _acquisition_ended(self, scanID):
         _OngoingObservation._acquisition_ended(self, scanID=scanID)
         try:
-            scan = ScanFactory.create_scan_object(scan_path=scanID)
+            scans = ScanFactory.create_scan_objects(
+                scan_path=scanID, accept_bliss_scan=True
+            )
         except Exception as e:
-            logger.error("Fail to create TomoBase instance from", scanID, "Error is", e)
+            logger.error(f"Fail to create TomoBase instance from {scanID} Error is {e}")
         else:
-            self.sigScanReady.emit(scan)
+            for scan in scans:
+                self.sigScanReady.emit(scan)
 
     def add(self, observer):
         already_observing = self.isObserving(observer.path)
@@ -185,7 +211,7 @@ class _QOngoingObservation(_OngoingObservation, qt.QObject):
         self.dict = {}
 
 
-class _QDataWatcherFixObserver(_DataWatcherFixObserverMixIn, qt.QThread):
+class _QDataWatcherStaticObserver(_DataWatcherStaticObserverMixIn, qt.QThread):
     """
     Implementation of the _DataWatcherFixObserverMixIn with a qt.QThread
     """
@@ -204,7 +230,7 @@ class _QDataWatcherFixObserver(_DataWatcherFixObserverMixIn, qt.QThread):
         observationRegistry,
     ):
         qt.QThread.__init__(self)
-        _DataWatcherFixObserverMixIn.__init__(
+        _DataWatcherStaticObserverMixIn.__init__(
             self,
             scanID=scanID,
             obsMethod=obsMethod,
@@ -215,7 +241,11 @@ class _QDataWatcherFixObserver(_DataWatcherFixObserverMixIn, qt.QThread):
         )
 
     def run(self):
-        if not os.path.isdir(self.path):
+        look_for_hdf5_file = self.obsMethod in (
+            datawatcherstatus.BLISS_SCAN_END,
+            datawatcherstatus.NXtomo_END,
+        )
+        if not look_for_hdf5_file and not os.path.isdir(self.path):
             logger.info("can't observe %s, not a directory" % self.path)
             self.status = "failure"
             self.sigStatusChanged.emit(
@@ -225,37 +255,40 @@ class _QDataWatcherFixObserver(_DataWatcherFixObserverMixIn, qt.QThread):
             return
 
         try:
-            scan = ScanFactory.create_scan_object(scan_path=self.path)
+            scans = ScanFactory.create_scan_objects(
+                scan_path=self.path, accept_bliss_scan=True
+            )
         except ValueError as e:
             logger.error(e)
         else:
-            if scan.is_abort(
-                src_pattern=self.srcPattern, dest_pattern=self.destPattern
-            ):
-                if self.status != "aborted":
-                    logger.info("Acquisition %s has been aborted" % self.path)
-                    self.dataWatcherProcess._removeAcquisition(
-                        scanID=self.path, reason="acquisition aborted by the user"
+            for scan in scans:
+                if scan.is_abort(
+                    src_pattern=self.srcPattern, dest_pattern=self.destPattern
+                ):
+                    if self.status != "aborted":
+                        logger.info("Acquisition %s has been aborted" % self.path)
+                        self.dataWatcherProcess._removeAcquisition(
+                            scanID=self.path, reason="acquisition aborted by the user"
+                        )
+
+                        self.status = "aborted"
+                    self.sigStatusChanged.emit(
+                        datawatcherstatus.OBSERVATION_STATUS[self.status], self.path
                     )
+                    self.validation = -2
+                    return
+                dataComplete = self.dataWatcherProcess.is_data_complete()
 
-                    self.status = "aborted"
-                self.sigStatusChanged.emit(
-                    datawatcherstatus.OBSERVATION_STATUS[self.status], self.path
-                )
-                self.validation = -2
-                return
-            dataComplete = self.dataWatcherProcess.is_data_complete()
-
-            if dataComplete is True:
-                self.status = "acquisition ended"
-                self.sigStatusChanged.emit(
-                    datawatcherstatus.OBSERVATION_STATUS[self.status], self.path
-                )
-                self.validation = 1
-            else:
-                self.status = "waiting for acquisition ending"
-                self.sigStatusChanged.emit(
-                    datawatcherstatus.OBSERVATION_STATUS[self.status], self.path
-                )
-                self.validation = 0
+                if dataComplete is True:
+                    self.status = "acquisition ended"
+                    self.sigStatusChanged.emit(
+                        datawatcherstatus.OBSERVATION_STATUS[self.status], self.path
+                    )
+                    self.validation = 1
+                else:
+                    self.status = "waiting for acquisition ending"
+                    self.sigStatusChanged.emit(
+                        datawatcherstatus.OBSERVATION_STATUS[self.status], self.path
+                    )
+                    self.validation = 0
         return
