@@ -28,12 +28,12 @@ __license__ = "MIT"
 __date__ = "18/03/2020"
 
 
-from silx.gui import qt
-from silx.gui.utils.testutils import TestCaseQt
-from tomwer.test.utils import skip_gui_test
-from tomwer.gui.control.datalistener import ConfigurationWidget
-from silx.gui.utils.testutils import SignalListener
 import pytest
+from silx.gui import qt
+from silx.gui.utils.testutils import SignalListener, TestCaseQt
+
+from tomwer.gui.control.datalistener import ConfigurationWidget
+from tomwer.tests.utils import skip_gui_test
 
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
@@ -54,7 +54,6 @@ class TestDataListenerConfiguration(TestCaseQt):
         self._configWidget.close()
         self.qapp.processEvents()
         self._configWidget = None
-        TestCaseQt.tearDown(self)
 
     def testConfiguration(self):
         self.assertEqual(
