@@ -28,25 +28,25 @@ __license__ = "MIT"
 __date__ = "30/10/2020"
 
 
+import logging
+from typing import Union
+
 import h5py
 from silx.gui import qt
-from silx.io.utils import h5py_read_dataset
-from tomoscan.esrf.hdf5scan import ImageKey
-from silx.io.url import DataUrl
-from tomoscan.io import HDF5File
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
 from silx.gui.dialog.DataFileDialog import DataFileDialog
-from tomwer.io.utils import get_default_directory
-import tomwer.core.utils.nxtomoutils as nxtomo_utils
+from silx.io.url import DataUrl
+from silx.io.utils import h5py_read_dataset
+from silx.io.utils import open as open_hdf5
+from nxtomo.nxobject.nxdetector import ImageKey
 
-from typing import Union
-import logging
+import tomwer.core.utils.nxtomoutils as nxtomo_utils
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.io.utils import get_default_directory
 
 _logger = logging.getLogger(__name__)
 
 
 class _DarkOrFlatUrl(qt.QWidget):
-
     editingFinished = qt.Signal()
     """Signal emit when the url changes"""
 
@@ -58,6 +58,7 @@ class _DarkOrFlatUrl(qt.QWidget):
         self.setLayout(qt.QHBoxLayout())
 
         self._qle = qt.QLineEdit("", self)
+        self._qle.setPlaceholderText("scheme:file_path?data_path")
         self.layout().addWidget(self._qle)
 
         self._optionsCB = qt.QComboBox(self)
@@ -66,11 +67,8 @@ class _DarkOrFlatUrl(qt.QWidget):
 
         # connect signal / slot
         self._qle.editingFinished.connect(self._tryUpdateOptions)
-        self._qle.editingFinished.connect(self._editingFinished)
-        self._optionsCB.currentIndexChanged.connect(self._editingFinished)
-
-    def _editingFinished(self):
-        self.editingFinished.emit()
+        self._qle.editingFinished.connect(self.editingFinished)
+        self._optionsCB.currentIndexChanged.connect(self.editingFinished)
 
     def setUrl(self, url):
         """
@@ -85,7 +83,7 @@ class _DarkOrFlatUrl(qt.QWidget):
         url = self._redirectDataPath(url, logger=_logger)
 
         def dataset_invalid(url):
-            with HDF5File(url.file_path(), mode="r") as h5s:
+            with open_hdf5(url.file_path()) as h5s:
                 if not isinstance(h5s[url.data_path()], h5py.Dataset):
                     return True
             return False
@@ -101,11 +99,11 @@ class _DarkOrFlatUrl(qt.QWidget):
         self._qle.setText(url.path())
         self._updateOptions(url)
         self.blockSignals(old)
-        self._editingFinished()
+        self.editingFinished.emit()
 
     def setSerie(self, serie):
         type_name = self._type.name.lower().replace("_", " ")
-        item_txt = "{} serie n°{}".format(type_name, serie)
+        item_txt = f"{type_name} serie n°{serie}"
         idx = self._optionsCB.findText(item_txt)
         if idx >= 0:
             self._optionsCB.setCurrentIndex(idx)
@@ -115,10 +113,10 @@ class _DarkOrFlatUrl(qt.QWidget):
 
     def _redirectDataPath(self, url, logger=None):
         try:
-            with HDF5File(url.file_path(), mode="r") as h5s:
+            with open_hdf5(url.file_path()) as h5s:
                 node = h5s[url.data_path()]
 
-                if HDF5TomoScan.entry_is_nx_tomo(node):
+                if NXtomoScan.entry_is_nx_tomo(node):
                     if (
                         "detector" in node["instrument"]
                         and "data" in node["instrument"]["detector"]
@@ -132,7 +130,7 @@ class _DarkOrFlatUrl(qt.QWidget):
                             (url.data_path(), "instrument", "detector", "data")
                         )
                         url = DataUrl(file_path=url.file_path(), data_path=data_path)
-                elif HDF5TomoScan.is_nxdetector(node):
+                elif NXtomoScan.is_nxdetector(node):
                     if "data" in node:
                         if logger:
                             _logger.info(
@@ -173,16 +171,16 @@ class _DarkOrFlatUrl(qt.QWidget):
                 type_name = self._type.name.lower().replace("_", " ")
                 series_indexes = list(range(n_serie))
                 for i_serie in series_indexes:
-                    self._optionsCB.addItem("{} serie n°{}".format(type_name, i_serie))
+                    self._optionsCB.addItem(f"{type_name} serie n°{i_serie}")
                 # if this is a end url then set to the last found entry
                 if self._when == "end":
                     opt_idx = self._optionsCB.findText(
-                        "{} serie n°{}".format(type_name, series_indexes[-1])
+                        f"{type_name} serie n°{series_indexes[-1]}"
                     )
                 else:
                     # else set it to the first entry
                     opt_idx = self._optionsCB.findText(
-                        "{} serie n°{}".format(type_name, series_indexes[0])
+                        f"{type_name} serie n°{series_indexes[0]}"
                     )
                 if opt_idx >= 0:
                     self._optionsCB.setCurrentIndex(opt_idx)
@@ -190,10 +188,10 @@ class _DarkOrFlatUrl(qt.QWidget):
     def _getImageKey(self, url):
         # if we are on a 'detector / data dataset' then we can try to reach
         # image_key information
-        with HDF5File(url.file_path(), mode="r") as h5s:
+        with open_hdf5(url.file_path()) as h5s:
             dataset = h5s[url.data_path()]
             grp_parent = dataset.parent
-            if grp_parent is not None and HDF5TomoScan.is_nxdetector(grp_parent):
+            if grp_parent is not None and NXtomoScan.is_nxdetector(grp_parent):
                 if "image_key" in grp_parent:
                     return h5py_read_dataset(grp_parent["image_key"])
         return None
@@ -241,7 +239,7 @@ class _DarkOrFlatUrl(qt.QWidget):
                 scheme="silx",
             )
         except Exception as e:
-            _logger.warning("Fail to create url. Reason is", str(e))
+            _logger.warning(f"Fail to create url. Reason is {e}")
             return None
         else:
             return url
@@ -318,17 +316,14 @@ class DarkRefPatchWidget(qt.QWidget):
         self._efCB.toggled.connect(self._toggleEF)
 
         # connect QLE modifications
-        self._sdQLE.editingFinished.connect(self._configurationChanged)
-        self._sdCB.toggled.connect(self._configurationChanged)
-        self._sfQLE.editingFinished.connect(self._configurationChanged)
-        self._sfCB.toggled.connect(self._configurationChanged)
-        self._efQLE.editingFinished.connect(self._configurationChanged)
-        self._efCB.toggled.connect(self._configurationChanged)
-        self._edQLE.editingFinished.connect(self._configurationChanged)
-        self._edCB.toggled.connect(self._configurationChanged)
-
-    def _configurationChanged(self):
-        self.sigConfigurationChanged.emit()
+        self._sdQLE.editingFinished.connect(self.sigConfigurationChanged)
+        self._sdCB.toggled.connect(self.sigConfigurationChanged)
+        self._sfQLE.editingFinished.connect(self.sigConfigurationChanged)
+        self._sfCB.toggled.connect(self.sigConfigurationChanged)
+        self._efQLE.editingFinished.connect(self.sigConfigurationChanged)
+        self._efCB.toggled.connect(self.sigConfigurationChanged)
+        self._edQLE.editingFinished.connect(self.sigConfigurationChanged)
+        self._edCB.toggled.connect(self.sigConfigurationChanged)
 
     def _selectSDDataset(self):
         self._selectDataset("start dark", self._sdQLE)
