@@ -28,15 +28,17 @@ __license__ = "MIT"
 __date__ = "26/10/2020"
 
 
-from silx.gui import qt
-from tomwer.core.process.edit.imagekeyeditor import IMAGE_KEYS
-from typing import Union
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomoscan.esrf.hdf5scan import ImageKey as _ImageKey
-import weakref
 import functools
-from collections.abc import Iterable
 import logging
+import weakref
+from collections.abc import Iterable
+from typing import Union
+
+from silx.gui import qt
+from nxtomo.nxobject.nxdetector import ImageKey as _ImageKey
+
+from tomwer.core.process.edit.imagekeyeditor import IMAGE_KEYS
+from tomwer.core.scan.nxtomoscan import NXtomoScan
 
 _logger = logging.getLogger(__name__)
 
@@ -91,7 +93,7 @@ class ImageKeyDialog(qt.QDialog):
     def setScan(self, scan):
         """
 
-        :param HDF5TomoScan scan: scan to be edited
+        :param NXtomoScan scan: scan to be edited
         """
         self._widget.setScan(scan)
 
@@ -118,7 +120,7 @@ class ImageKeyWindow(qt.QMainWindow):
     def setScan(self, scan) -> None:
         """
 
-        :param HDF5TomoScan scan: scan to be edited
+        :param NXtomoScan scan: scan to be edited
         """
         self._mainWindow.setScan(scan=scan)
         self._editorControl.setScan(scan=scan)
@@ -154,7 +156,6 @@ class ImageKeyWindow(qt.QMainWindow):
 
 
 class _ImageKeyEditor(qt.QDialog):
-
     sigApply = qt.Signal()
     """Signal emitted when the user request to apply the current modifications
     """
@@ -172,7 +173,7 @@ class _ImageKeyEditor(qt.QDialog):
         # mode selection
         self.layout().addWidget(qt.QLabel("selection mode"), 1, 0, 1, 1)
         self._selectionModeCB = qt.QComboBox(self)
-        for mode in ("slice", "list"):
+        for mode in ("range", "list"):
             self._selectionModeCB.addItem(mode)
         self._selectionModeCB.setCurrentIndex(self._selectionModeCB.findText("list"))
         self.layout().addWidget(self._selectionModeCB, 1, 1, 1, 1)
@@ -212,7 +213,7 @@ class _ImageKeyEditor(qt.QDialog):
         self._selectionModeCB.currentTextChanged.connect(
             self._selectionWidget.setSelectionMode
         )
-        self._buttons.button(qt.QDialogButtonBox.Ok).clicked.connect(self._apply)
+        self._buttons.button(qt.QDialogButtonBox.Ok).clicked.connect(self.sigApply)
 
     def getUpgradeToImagKey(self) -> str:
         """
@@ -225,7 +226,7 @@ class _ImageKeyEditor(qt.QDialog):
     def getSelectionMode(self) -> str:
         """
 
-        :return: frame indexes selection mode. Can be 'slice' or 'list'
+        :return: frame indexes selection mode. Can be 'range' or 'list'
         """
         return self._selectionModeCB.currentText()
 
@@ -235,20 +236,17 @@ class _ImageKeyEditor(qt.QDialog):
         :return: list of indexes to be modified
         :rtype: list
         """
-        if self.getSelectionMode() == "slice":
+        if self.getSelectionMode() == "range":
             return self._selectionWidget.getSlice()
         elif self.getSelectionMode() == "list":
             return self._selectionWidget.getList()
         else:
             raise NotImplementedError("Not implemented")
 
-    def _apply(self):
-        self.sigApply.emit()
-
     def setScan(self, scan):
         self._selectionWidget.setScan(scan=scan)
         if scan is not None:
-            self._scanLabel.setText(str(scan))
+            self._scanLabel.setText(scan.get_identifier().to_str())
         else:
             self._scanLabel.setText("")
 
@@ -305,6 +303,7 @@ class _ListSelection(qt.QWidget):
 
         # list selection
         self._listQLE = qt.QLineEdit("", self)
+        self._listQLE.setPlaceholderText("0, 8, -1, ...")
         self.layout().addWidget(self._listQLE)
         reg_exp = qt.QRegExp(r"(\d+)(,[ ]?\s*\d+)*")
         # list of int with optional space
@@ -321,7 +320,7 @@ class _ListSelection(qt.QWidget):
     def _selectFrames(self):
         if self._scan is None or self._scan() is None:
             _logger.warning(
-                "No scan set or has been moved. Unable to select " "slices from it."
+                "No scan set or has been moved. Unable to select slices from it."
             )
             return
         scan = self._scan()
@@ -375,11 +374,14 @@ class _EditionSelection(qt.QWidget):
     def setSelectionMode(self, mode: str) -> None:
         """
 
-        :param str mode: define indexes selection mode. Can be 'slice' or
+        :param str mode: define indexes selection mode. Can be 'range' or
                          'list'
         """
-        assert mode in ("slice", "list")
-        self._sliceSelection.setVisible(mode == "slice")
+        if mode == "slice":
+            # for backward compatibility (user can have .ows with old name)
+            mode = "range"
+        assert mode in ("range", "list")
+        self._sliceSelection.setVisible(mode == "range")
         self._listSelection.setVisible(mode == "list")
 
     def getList(self) -> list:
@@ -392,6 +394,21 @@ class _EditionSelection(qt.QWidget):
         self._listSelection.setScan(scan)
 
 
+class _ImageKeyComboBox(qt.QComboBox):
+    def __init__(self, parent, image_key):
+        qt.QComboBox.__init__(self, parent)
+        for img_key_type in IMAGE_KEYS:
+            self.addItem(img_key_type)
+        self.setImageKey(image_key)
+
+    def setImageKey(self, image_key):
+        if image_key in _ImageKeyList._IMAGE_KEYS_INV:
+            image_key = _ImageKeyList._IMAGE_KEYS_INV[image_key]
+        idx = self.findText(image_key)
+        assert idx >= 0, "image key not recognized"
+        self.setCurrentIndex(idx)
+
+
 class _ImageKeyList(qt.QTableWidget):
     sigKeyUpdated = qt.Signal(tuple)
     """Signal emitted when a key has been updated. tuple contains
@@ -405,20 +422,6 @@ class _ImageKeyList(qt.QTableWidget):
 
     _IMAGE_KEYS_INV = {v: k for k, v in IMAGE_KEYS.items()}
 
-    class ImageKeyComboBox(qt.QComboBox):
-        def __init__(self, parent, image_key):
-            qt.QComboBox.__init__(self, parent)
-            for img_key_type in IMAGE_KEYS:
-                self.addItem(img_key_type)
-            self.setImageKey(image_key)
-
-        def setImageKey(self, image_key):
-            if image_key in _ImageKeyList._IMAGE_KEYS_INV:
-                image_key = _ImageKeyList._IMAGE_KEYS_INV[image_key]
-            idx = self.findText(image_key)
-            assert idx >= 0, "image key not recognized"
-            self.setCurrentIndex(idx)
-
     def __init__(self, parent):
         qt.QTableWidget.__init__(self, parent)
         self._filterMode = None
@@ -444,7 +447,7 @@ class _ImageKeyList(qt.QTableWidget):
         """
         Update the QTableWidget for the provided frames
 
-        :param Iterable frames: Iterable of tomoscan.esrf.HDF5TomoScan.Frame
+        :param Iterable frames: Iterable of tomoscan.esrf.scan.NXtomoScan.Frame
         :param dict frames_new_keys: dictionary with frame edited. key is frame
                                      index. Value is the new `image_key` value
         """
@@ -477,7 +480,7 @@ class _ImageKeyList(qt.QTableWidget):
                 new_image_key = frame.image_key
             new_image_key = _ImageKey.from_value(new_image_key)
             new_image_key = self._IMAGE_KEYS_INV[new_image_key]
-            newImgKeyItem = self.ImageKeyComboBox(parent=self, image_key=new_image_key)
+            newImgKeyItem = _ImageKeyComboBox(parent=self, image_key=new_image_key)
             self._imageKeyComboboxes[index] = newImgKeyItem
             callback = functools.partial(self.notify_modified, frame.index)
             newImgKeyItem.currentIndexChanged.connect(callback)
@@ -516,6 +519,7 @@ class _ImageKeyList(qt.QTableWidget):
             my_list = modifications
         to_apply = {}
         for frame_index in my_list:
+            assert isinstance(frame_index, int), "frame_index should be an int"
             to_apply[frame_index] = new_image_key
             if frame_index in self._imageKeyComboboxes:
                 self._imageKeyComboboxes[frame_index].setImageKey(new_image_key)
@@ -545,12 +549,13 @@ class _ImageKeyList(qt.QTableWidget):
             else:
                 self.hideRow(i)
 
-    def notify_modified(self, frame_index) -> None:
+    def notify_modified(self, frame_index: int) -> None:
         """
         Update list of modification and emit sigKeyUpdated
 
         :param int frame_index:
         """
+        assert isinstance(frame_index, int), "frame index should be an int"
         frame_row = self._getFrameRowIndex(frame_index)
         if frame_row is not None:
             new_image_key = self._imageKeyComboboxes[frame_index].currentText()
@@ -570,7 +575,6 @@ class _ImageKeyList(qt.QTableWidget):
 
 
 class _ImageKeyListFiltered(qt.QWidget):
-
     MODIFIED_IMG_FILTER = "modified"
     ALL_IMG_FILTER = "all"
 
@@ -622,8 +626,10 @@ class _ImageKeyListFiltered(qt.QWidget):
         return self._filterCB.currentText()
 
     def setScan(self, scan):
-        if not isinstance(scan, HDF5TomoScan):
-            raise TypeError("scan should be an instance of {}" "".format(HDF5TomoScan))
+        if not isinstance(scan, NXtomoScan):
+            raise TypeError(
+                f"scan should be an instance of {NXtomoScan}. {type(scan)} provided"
+            )
         self.__frames_modifications = {}
         self._scan = weakref.ref(scan)
         frames = scan.frames
@@ -656,11 +662,7 @@ class _FrameSelectionDialog(qt.QDialog):
     def setFrames(self, frames):
         item_texts = []
         for frame in frames:
-            item_texts.append(
-                "{index} ({image_key})".format(
-                    index=frame.index, image_key=frame.image_key.value
-                )
-            )
+            item_texts.append(f"{frame.index} ({frame.image_key.value})")
         self._listWidget.addItems(item_texts)
 
     def getSelectedFrameIndexes(self):
@@ -670,3 +672,151 @@ class _FrameSelectionDialog(qt.QDialog):
             return int(my_str.replace(" ", "").split("(")[0])
 
         return [get_index(item.text()) for item in item_selected]
+
+
+class ImageKeyUpgraderList(qt.QTableWidget):
+    """
+    List defining image key upgrade operations
+    so to upgrade all frames with a specific Image_key to another image_key
+    """
+
+    COLUMNS = ("operations", "remove")
+
+    sigOperationsChanged = qt.Signal()
+    """signal emit when the operations changed"""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._operations = {}
+        # operations. dict with image_key 'from' as key and image_key to as value.
+        # this ensure we will not ask for contradictory operations (like upgrading one image_key to two different image_key)
+        self._update()
+
+    def getOperations(self) -> tuple:
+        return self._operations
+
+    def setOperations(self, operations: dict):
+        self._operations.clear()
+        for from_image_key, to_image_key in operations.items():
+            self.addOperation(from_image_key=from_image_key, to_image_key=to_image_key)
+
+    def _update(self):
+        self.setColumnCount(len(self.COLUMNS))
+        self.setRowCount(len(self._operations))
+        self.setHorizontalHeaderLabels(self.COLUMNS)
+        self.horizontalHeader().setSectionResizeMode(qt.QHeaderView.ResizeToContents)
+        self.verticalHeader().hide()
+
+        for i_operation, (from_image_key, to_image_key) in enumerate(
+            self._operations.items()
+        ):
+            item_info = qt.QTableWidgetItem()
+            item_info.setText(
+                f"convert all `{_ImageKeyList._IMAGE_KEYS_INV[from_image_key]}` frames to `{_ImageKeyList._IMAGE_KEYS_INV[to_image_key]}`"
+            )
+            self.setItem(i_operation, 0, item_info)
+
+            remove_button = qt.QPushButton(parent=self)
+            style = qt.QApplication.style()
+            stopIcon = style.standardIcon(qt.QStyle.SP_DialogCloseButton)
+            remove_button.setIcon(stopIcon)
+            remove_button.released.connect(
+                functools.partial(
+                    self.removeOperation,
+                    from_image_key,
+                    to_image_key,
+                )
+            )
+            self.setCellWidget(i_operation, 1, remove_button)
+
+    def addOperation(self, from_image_key: _ImageKey, to_image_key: _ImageKey):
+        from_image_key = _ImageKey.from_value(from_image_key)
+        to_image_key = _ImageKey.from_value(to_image_key)
+
+        self._operations[from_image_key] = to_image_key
+        self._update()
+        self.sigOperationsChanged.emit()
+
+    def removeOperation(self, from_image_key: _ImageKey, to_image_key: _ImageKey):
+        from_image_key = _ImageKey.from_value(from_image_key)
+        to_image_key = _ImageKey.from_value(to_image_key)
+        if (
+            from_image_key not in self._operations
+            or self._operations[from_image_key] != to_image_key
+        ):
+            return
+        self._operations.pop(from_image_key)
+        self._update()
+        self.sigOperationsChanged.emit()
+
+
+class _AddImageKeyUpgradeOperation(qt.QGroupBox):
+    sigOperationAdded = qt.Signal(dict)
+    """Signal emit when a new operation is requested. Dict contains keys 'from_image_key' and 'to_image_key' to define the operation"""
+
+    def __init__(self, title="add operation", *args, **kwargs):
+        super().__init__(title=title, *args, **kwargs)
+        self.setLayout(qt.QHBoxLayout())
+        self.layout().addWidget(qt.QLabel("from"))
+        self._fromCB = _ImageKeyComboBox(parent=self, image_key=_ImageKey.PROJECTION)
+        self.layout().addWidget(self._fromCB)
+        self.layout().addWidget(qt.QLabel("to"))
+        self._toCB = _ImageKeyComboBox(parent=self, image_key=_ImageKey.DARK_FIELD)
+        self.layout().addWidget(self._toCB)
+        self._addButtonPB = qt.QPushButton("add")
+        self.layout().addWidget(self._addButtonPB)
+
+        # signal / slot connection
+        self._addButtonPB.released.connect(self._addButtonCallback)
+
+    def _addButtonCallback(self, *args, **kwargs):
+        self.sigOperationAdded.emit(self._getCurrentOperation())
+
+    def _getCurrentOperation(self):
+        return {
+            "from_image_key": IMAGE_KEYS[self._fromCB.currentText()].value,
+            "to_image_key": IMAGE_KEYS[self._toCB.currentText()].value,
+        }
+
+
+class ImageKeyUpgraderWidget(qt.QWidget):
+    """
+    Main widget to define ImageKey upgrade operations.
+    Will upgrade all image_key within a specific value to another value.
+    """
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.setLayout(qt.QVBoxLayout())
+
+        self._addOperationWidget = _AddImageKeyUpgradeOperation(parent=self)
+        self.layout().addWidget(self._addOperationWidget)
+
+        self._operationList = ImageKeyUpgraderList(parent=self)
+        self.layout().addWidget(self._operationList)
+
+        # connect signal / slot
+        self._addOperationWidget.sigOperationAdded.connect(self._addOperationCallback)
+        # expose signal
+        self.sigOperationsChanged = self._operationList.sigOperationsChanged
+
+    def _addOperationCallback(self, operation_as_dict: dict):
+        return self.addOperation(**operation_as_dict)
+
+    # exposing some API
+
+    def getOperations(self) -> tuple:
+        return self._operationList.getOperations()
+
+    def addOperation(self, from_image_key: _ImageKey, to_image_key: _ImageKey):
+        self._operationList.addOperation(
+            from_image_key=from_image_key, to_image_key=to_image_key
+        )
+
+    def removeOperation(self, from_image_key: _ImageKey, to_image_key: _ImageKey):
+        self._operationList.removeOperation(
+            from_image_key=from_image_key, to_image_key=to_image_key
+        )
+
+    def setOperations(self, operations: dict):
+        return self._operationList.setOperations(operations=operations)
