@@ -31,15 +31,13 @@ __date__ = "06/08/2018"
 
 import functools
 import logging
-import os
 
 from silx.gui import qt
 from silx.io.url import DataUrl
+from silx.io.utils import open as open_hdf5
 
-from tomwer.core.utils import ftseriesutils
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.synctools.imageloaderthread import ImageLoaderThread
-from tomoscan.io import HDF5File
 
 logger = logging.getLogger(__name__)
 
@@ -69,13 +67,11 @@ class ImageFromFile(_Image):
             data_path = None
             if _file.lower().endswith(".edf"):
                 scheme = "fabio"
-            elif _file.lower().endswith((".npy", ".npz")):
-                scheme = "numpy"
             else:
 
                 def get_nabu_entry():
                     try:
-                        with HDF5File(_file, mode="r", swmr=True) as h5s:
+                        with open_hdf5(_file) as h5s:
                             for node in h5s:
                                 if "reconstruction" in h5s[node]:
                                     return "/".join(
@@ -154,37 +150,8 @@ class FileWithImage(object):
         :param bool _load: if True then launch the load of the image
         :return: the list of images contained in the given file.
         """
-        if self.file.endswith(".vol"):
-            return self._dealWithVolFile(self.file, _load)
-        elif self.file.lower().endswith(TomwerScanBase.VALID_RECONS_EXTENSION):
+        if self.file.lower().endswith(TomwerScanBase.VALID_RECONS_EXTENSION):
             return [ImageFromFile(self.file)]
         else:
             logger.error("only deal with .edf and .vol extension")
             return []
-
-    def _dealWithVolFile(self, _file, _load):
-        _imagesFiles = []
-        volInfoFile = _file.replace(".vol", ".vol.info")
-        if not os.path.exists(volInfoFile):
-            mess = (
-                "Can't find description file %s associated with raw "
-                "data file %s " % (volInfoFile, _file)
-            )
-            logger.warning(mess)
-        else:
-            shape = ftseriesutils.get_vol_file_shape(volInfoFile)
-            if shape is not None:
-                for zSlice in range(shape[0]):
-                    if _file.lower().endswith(".edf"):
-                        scheme = "fabio"
-                    else:
-                        scheme = "tomwer"
-                    _imagesFiles.append(
-                        ImageFromFile(
-                            url=DataUrl(
-                                file_path=_file, data_slice=(zSlice,), scheme=scheme
-                            ),
-                            _load=_load,
-                        )
-                    )
-            return _imagesFiles
