@@ -31,16 +31,21 @@ __license__ = "MIT"
 __date__ = "14/10/2019"
 
 
-from silx.gui import qt
-
-from tomwer.core.scan.scanbase import TomwerScanBase
-from .radioaxis import RadioAxisWindow
-from ...utils.scandescription import ScanNameLabel
-from tomwer.core.process.reconstruction.axis.mode import AxisMode
-from tomwer.gui.utils.buttons import PadlockButton
-from tomwer.synctools.axis import QAxisRP
-from silx.utils.deprecation import deprecated
+import numpy
 import logging
+from typing import Optional
+
+from silx.gui import qt
+from silx.utils.deprecation import deprecated
+
+from tomwer.core.process.reconstruction.axis.mode import AxisMode
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.utils.buttons import PadlockButton
+from tomwer.gui.settings import EDITING_BACKGROUND_COLOR
+from tomwer.synctools.axis import QAxisRP
+
+from ...utils.scandescription import ScanNameLabelAndShape
+from .radioaxis import RadioAxisWindow
 
 _logger = logging.getLogger(__file__)
 
@@ -98,8 +103,8 @@ class AxisWindow(qt.QMainWindow):
     sigLockModeChanged = qt.Signal()
     """Signal emitted when the lock on the method change"""
 
-    def __init__(self, axis_params, parent=None):
-        qt.QMainWindow.__init__(self, parent=parent)
+    def __init__(self, axis_params, parent=None, backend=None):
+        super().__init__(parent=parent)
         self._mainWidget = qt.QWidget(self)
         self._mainWidget.setLayout(qt.QVBoxLayout())
         self.setCentralWidget(self._mainWidget)
@@ -107,7 +112,7 @@ class AxisWindow(qt.QMainWindow):
         self._axis_params = axis_params
 
         # add scan name
-        self._scan_label = ScanNameLabel(parent=self)
+        self._scan_label = ScanNameLabelAndShape(parent=self)
         self._mainWidget.layout().addWidget(self._scan_label)
 
         # add selection
@@ -116,7 +121,9 @@ class AxisWindow(qt.QMainWindow):
         self._selectionGB.hide()
 
         # add widget for radio and sinogram axis
-        self._axisWidget = _AxisWidget(parent=self, axis_params=axis_params)
+        self._axisWidget = _AxisWidget(
+            parent=self, axis_params=axis_params, backend=backend
+        )
         self._mainWidget.layout().addWidget(self._axisWidget)
 
         self.setCentralWidget(self._mainWidget)
@@ -130,7 +137,7 @@ class AxisWindow(qt.QMainWindow):
         self._selectionGB._selectionChanged(self._selectionGB.getSelection())
 
         # connect signal / slots
-        self._axisWidget.sigValidateRequest.connect(self._repeatValidateRequest)
+        self._axisWidget.sigValidateRequest.connect(self.sigApply)
         self._axisWidget.sigComputationRequested.connect(self._computationRequested)
         self._axisWidget.sigLockModeChanged.connect(self._lockModeChanged)
         self._selectionGB.sigSelectionChanged.connect(self._axisWidget._axisTypeChanged)
@@ -154,8 +161,13 @@ class AxisWindow(qt.QMainWindow):
         self._setModeLockFrmSettings = self._axisWidget._setModeLockFrmSettings
         self.getRadioMode = self._axisWidget.getRadioMode
         self.getEstimatedCor = self._axisWidget.getEstimatedCor
+        self.setEstimatedCor = self._axisWidget.setEstimatedCor
         self.sigMethodChanged = self._axisWidget.sigModeChanged
 
+    def setAutoUpdateEstimatedCor(self, value: bool):
+        assert isinstance(value, bool)
+        self._axisWidget.setAutoUpdateEstimatedCor(value)
+
     def _disableForProcessing(self, *args, **kwargs):
         self._mainWidget.setEnabled(False)
         self._axisWidget.setEnabled(False)
@@ -192,11 +204,8 @@ class AxisWindow(qt.QMainWindow):
     def _computationReady(self):
         self.sigComputationRequested.emit()
 
-    def _repeatValidateRequest(self):
-        self.sigApply.emit()
-
     def setPosition(self, frm: float, value: float):
-        self._axisWidget.setPosition(value_ref_tomwer=value)
+        self._axisWidget.setPosition(relative_value=value)
 
 
 class _AxisWidget(qt.QMainWindow):
@@ -218,13 +227,15 @@ class _AxisWidget(qt.QMainWindow):
 
     sigSinogramReady = qt.Signal()
 
-    def __init__(self, parent, axis_params):
-        qt.QMainWindow.__init__(self, parent)
+    def __init__(self, parent, axis_params, backend=None):
+        super().__init__(parent)
         self._settingsWidget = None
         self.setDockOptions(qt.QMainWindow.AnimatedDocks)
 
         self._settingsDockWidget = qt.QDockWidget(parent=self)
         self._settingsDockWidget.setFeatures(qt.QDockWidget.DockWidgetMovable)
+        self._settingsDockWidget.layout().setContentsMargins(0, 0, 0, 0)
+        self._settingsDockWidget.layout().setSpacing(0)
         self.addDockWidget(qt.Qt.RightDockWidgetArea, self._settingsDockWidget)
 
         self._controlWidget = _ControlWidget(parent=self)
@@ -232,19 +243,29 @@ class _AxisWidget(qt.QMainWindow):
             qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum
         )
         self._controlDockWidget = qt.QDockWidget(parent=self)
+        self._controlDockWidget.setSizePolicy(
+            qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum
+        )
+        self._controlDockWidget.layout().setContentsMargins(0, 0, 0, 0)
+        self._controlDockWidget.layout().setSpacing(0)
+        self._controlDockWidget.setMaximumHeight(150)
         self._controlDockWidget.setWidget(self._controlWidget)
         self._controlDockWidget.setFeatures(qt.QDockWidget.DockWidgetMovable)
         self.addDockWidget(qt.Qt.RightDockWidgetArea, self._controlDockWidget)
 
         self._axis_params = axis_params
-        self._radioAxis = RadioAxisWindow(parent=self, axis=axis_params)
+        self._radioAxis = RadioAxisWindow(
+            parent=self, axis=axis_params, backend=backend
+        )
         self._plotWidgets = qt.QWidget(self)
         self._plotWidgets.setLayout(qt.QVBoxLayout())
         self.setCentralWidget(self._plotWidgets)
+        self._radioAxis.removeDockWidget(self._radioAxis.getSettingsWidgetDocker())
 
         # expose API
         self.getRadioMode = self._radioAxis.getMode
         self.getEstimatedCor = self._radioAxis.getEstimatedCor
+        self.setEstimatedCor = self._radioAxis.setEstimatedCor
 
         # set up
         if self._axis_params.mode is AxisMode.manual:
@@ -252,27 +273,49 @@ class _AxisWidget(qt.QMainWindow):
                 self._axis_params.relative_cor_value,
                 self._axis_params.absolute_cor_value,
             )
+        self._controlWidget._positionInfo.setAxis(self._axis_params)
 
         # connect signal / slots
-        self._controlWidget.sigComputationRequest.connect(
-            self._repeatComputationRequest
-        )
-        self._controlWidget.sigValidateRequest.connect(self._repeatValidateRequest)
+        self._controlWidget.sigComputationRequest.connect(self.sigComputationRequested)
+        self._controlWidget.sigValidateRequest.connect(self.sigValidateRequest)
         self._controlWidget.sigLockCORValueChanged.connect(self._CORValueLocked)
-        self._radioAxis.sigLockModeChanged.connect(self._lockModeChanged)
+        self._radioAxis.sigLockModeChanged.connect(self.sigLockModeChanged)
         self._radioAxis.sigPositionChanged.connect(self._setPositionFrmTuple)
+        self._controlWidget._positionInfo.sigRelativeValueSet.connect(
+            self._forceRelativePosition
+        )
+        self._controlWidget._positionInfo.sigAbsolueValueSet.connect(
+            self._forceAbsolutePosition
+        )
 
         # set up
         self.addPlot(self._radioAxis)
         self.setActiveWidget(self._radioAxis)
 
+    def _forceRelativePosition(self, value: float):
+        self.setMode(AxisMode.manual)
+        self.setPosition(
+            relative_value=value,
+            absolute_value=None,
+        )
+
+    def _forceAbsolutePosition(self, value: float):
+        self.setMode(AxisMode.manual)
+        self.setPosition(
+            relative_value=None,
+            absolute_value=value,
+        )
+
+    def setAutoUpdateEstimatedCor(self, value):
+        self._radioAxis.setUpdateAutomaticallyEstimatedCor(value)
+
+    def getAutoUpdateEstimatedCor(self):
+        return self._radioAxis.updateAutomaticallyEstimatedCor()
+
     @deprecated(replacement="getRadioMode")
     def getMode(self):
         return self._radioAxis.getMode()
 
-    def _sinoAxisReady(self):
-        self.sigSinogramReady.emit()
-
     def manual_uses_full_image(self, value):
         self._radioAxis.manual_uses_full_image(value)
 
@@ -280,15 +323,6 @@ class _AxisWidget(qt.QMainWindow):
         self.getAxis().mode = self.getMode()
         self.getAxis().use_sinogram = self.useSinogram()
 
-    def _lockModeChanged(self):
-        self.sigLockModeChanged.emit()
-
-    def _repeatComputationRequest(self):
-        self.sigComputationRequested.emit()
-
-    def _repeatValidateRequest(self):
-        self.sigValidateRequest.emit()
-
     def setMode(self, mode):
         mode = AxisMode.from_value(mode)
         self.setActiveWidget(self._radioAxis)
@@ -321,11 +355,9 @@ class _AxisWidget(qt.QMainWindow):
         self._settingsDockWidget.setWidget(widget)
 
     def _setPositionFrmTuple(self, value):
-        self.setPosition(value_ref_tomwer=value[0])
+        self.setPosition(relative_value=value[0])
 
-    def setPosition(
-        self, value_ref_tomwer: float, value_ref_nabu: float = None
-    ) -> None:
+    def setPosition(self, relative_value: float, absolute_value: float = None) -> None:
         """
 
         :param float value_ref_tomwer: center of rotation value on tomwer
@@ -335,19 +367,28 @@ class _AxisWidget(qt.QMainWindow):
         :raises: ValueError if the frm parameter is not recognized
         """
         if (
-            value_ref_nabu is None
+            absolute_value is None
             and self._axis_params.frame_width is not None
-            and value_ref_tomwer is not None
+            and relative_value is not None
         ):
             try:
-                value_ref_nabu = value_ref_tomwer + self._axis_params.frame_width / 2.0
+                absolute_value = relative_value + self._axis_params.frame_width / 2.0
             except TypeError:
-                value_ref_nabu = None
+                absolute_value = None
+        if (
+            relative_value is None
+            and self._axis_params.frame_width is not None
+            and absolute_value is not None
+        ):
+            try:
+                relative_value = absolute_value - self._axis_params.frame_width / 2.0
+            except TypeError:
+                relative_value = None
         self._controlWidget.setPosition(
-            value_ref_tomwer=value_ref_tomwer, value_ref_nabu=value_ref_nabu
+            value_ref_tomwer=relative_value, value_ref_nabu=absolute_value
         )
-        if value_ref_tomwer is not None:
-            self._radioAxis.setXShift(value_ref_tomwer)
+        if relative_value is not None:
+            self._radioAxis.setXShift(relative_value)
         else:
             self._radioAxis.resetShift()
 
@@ -371,10 +412,21 @@ class _AxisWidget(qt.QMainWindow):
             )
         elif set_position is False:
             if scan.dim_1 is not None:
-                self._controlWidget._updateAbsolutePosition(width=scan.dim_1)
-
-    def _applyRequested(self) -> None:
-        self.sigApply.emit()
+                # if absolute position is know and relative position not (in the case the user already define absolute position only):
+                has_absolute_val_already = not self._controlWidget._positionInfo._absolutePositionQLE.text().startswith(
+                    (".", "?")
+                )
+                has_relative_val_already = not self._controlWidget._positionInfo._relativePositionQLE.text().startswith(
+                    (".", "?")
+                )
+                if (
+                    self.getMode() is AxisMode.manual
+                    and has_absolute_val_already
+                    and not has_relative_val_already
+                ):
+                    self._controlWidget._updateRelativePosition(width=scan.dim_1)
+                else:
+                    self._controlWidget._updateAbsolutePosition(width=scan.dim_1)
 
     def _computationRequested(self) -> None:
         self.sigComputationRequested.emit()
@@ -493,12 +545,15 @@ class _ControlWidget(qt.QWidget):
 
     def setPosition(self, value_ref_tomwer, value_ref_nabu):
         self._positionInfo.setPosition(
-            value_ref_tomwer=value_ref_tomwer, value_ref_nabu=value_ref_nabu
+            relative_cor=value_ref_tomwer, abs_cor=value_ref_nabu
         )
 
     def _updateAbsolutePosition(self, width):
         self._positionInfo._updateAbsolutePosition(width=width)
 
+    def _updateRelativePosition(self, width):
+        self._positionInfo._updateRelativePosition(width=width)
+
     def isValueLock(self):
         return self._lockBut.isLocked()
 
@@ -506,11 +561,25 @@ class _ControlWidget(qt.QWidget):
         self._lockBut.setLock(lock)
 
 
+class _PositionInfoValidator(qt.QDoubleValidator):
+    def validate(self, a0: str, a1: int):
+        if "..." in a0 or a0 == "?":
+            return (qt.QDoubleValidator.Acceptable, a0, a1)
+        else:
+            return super().validate(a0, a1)
+
+
 class _PositionInfoWidget(qt.QWidget):
     """Widget used to display information relative to the current position"""
 
+    sigRelativeValueSet = qt.Signal(float)
+    """Emit when the user define manually the relative value"""
+    sigAbsolueValueSet = qt.Signal(float)
+    """Emit when the user define manually the absolute value"""
+
     def __init__(self, parent, axis=None):
         self._axis = None
+        self._defaultBackgroundColor = None
         qt.QWidget.__init__(self, parent)
         self.setLayout(qt.QGridLayout())
         centerLabel = qt.QLabel("center", parent=self)
@@ -522,13 +591,14 @@ class _PositionInfoWidget(qt.QWidget):
         self.layout().addWidget(centerLabel, 0, 0, 1, 1)
         self.layout().addWidget(qt.QLabel(" (relative):"), 0, 1, 1, 1)
 
-        self._tomwerPositionLabel = qt.QLineEdit("", parent=self)
-        self._tomwerPositionLabel.setReadOnly(True)
-        self._tomwerPositionLabel.setSizePolicy(
+        self._relativePositionQLE = qt.QLineEdit("", parent=self)
+        self._positionValidator = _PositionInfoValidator(self)
+        self._relativePositionQLE.setValidator(self._positionValidator)
+        self._relativePositionQLE.setSizePolicy(
             qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum
         )
-        self._tomwerPositionLabel.setStyleSheet("color: red")
-        self.layout().addWidget(self._tomwerPositionLabel, 0, 2, 1, 1)
+        self._relativePositionQLE.setStyleSheet("color: red")
+        self.layout().addWidget(self._relativePositionQLE, 0, 2, 1, 1)
 
         centerLabel = qt.QLabel("center", parent=self)
         centerLabel.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum)
@@ -538,52 +608,126 @@ class _PositionInfoWidget(qt.QWidget):
 
         self.layout().addWidget(centerLabel, 1, 0, 1, 1)
         self.layout().addWidget(qt.QLabel(" (absolute):"), 1, 1, 1, 1)
-        self._nabuPositionLabel = qt.QLineEdit("", parent=self)
-        self._nabuPositionLabel.setReadOnly(True)
-        self._nabuPositionLabel.setSizePolicy(
+        self._absolutePositionQLE = qt.QLineEdit("", parent=self)
+        self._absolutePositionQLE.setValidator(self._positionValidator)
+        self._absolutePositionQLE.setSizePolicy(
             qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum
         )
-        self._nabuPositionLabel.setStyleSheet("color: #ff8c00")
-        self.layout().addWidget(self._nabuPositionLabel, 1, 2, 1, 1)
+        self._absolutePositionQLE.setStyleSheet("color: #ff8c00")
+        self.layout().addWidget(self._absolutePositionQLE, 1, 2, 1, 1)
 
         if axis:
             self.setAxis(axis)
 
+        # connect signal / slot
+        self._relativePositionQLE.textEdited.connect(
+            self._userStartEditingRelativePosition
+        )
+        self._relativePositionQLE.editingFinished.connect(
+            self._userUpdatedRelativePosition
+        )
+        self._absolutePositionQLE.textEdited.connect(
+            self._userStartEditingAbsolutePosition
+        )
+
+        self._absolutePositionQLE.editingFinished.connect(
+            self._userUpdatedAbsolutePosition
+        )
+
+    def _getDefaultBackgroundColor(self):
+        if self._defaultBackgroundColor is None:
+            self._defaultBackgroundColor = self.palette().color(
+                self._absolutePositionQLE.backgroundRole()
+            )
+        return self._defaultBackgroundColor
+
+    def _userStartEditingRelativePosition(self, *args, **kwargs):
+        palette = self.palette()
+        palette.setColor(
+            self._relativePositionQLE.backgroundRole(), EDITING_BACKGROUND_COLOR
+        )
+        self._relativePositionQLE.setPalette(palette)
+
+    def _userStartEditingAbsolutePosition(self, *args, **kwargs):
+        palette = self.palette()
+        palette.setColor(
+            self._absolutePositionQLE.backgroundRole(), EDITING_BACKGROUND_COLOR
+        )
+        self._absolutePositionQLE.setPalette(palette)
+
+    def _userUpdatedRelativePosition(self, *args, **kwargs):
+        palette = self.palette()
+        palette.setColor(
+            self._relativePositionQLE.backgroundRole(),
+            self._getDefaultBackgroundColor(),
+        )
+        self._relativePositionQLE.setPalette(palette)
+        if self._relativePositionQLE.text().startswith((".", "?")):
+            return
+        else:
+            value = float(self._relativePositionQLE.text())
+            # make sure we only emit the signal if the value changed (and when the Qline has been edited).
+            if self._axis.relative_cor_value is None or (
+                self._axis is not None
+                and not numpy.isclose(value, self._axis.relative_cor_value, atol=1e-3)
+            ):
+                self.sigRelativeValueSet.emit(value)
+
+    def _userUpdatedAbsolutePosition(self, *args, **kwargs):
+        palette = self.palette()
+        palette.setColor(
+            self._absolutePositionQLE.backgroundRole(),
+            self._getDefaultBackgroundColor(),
+        )
+        self._absolutePositionQLE.setPalette(palette)
+        if self._absolutePositionQLE.text().startswith((".", "?")):
+            return
+        else:
+            value = float(self._absolutePositionQLE.text())
+            # make sure we only emit the signal if the value changed (and when the Qline has been edited).
+            if self._axis.absolute_cor_value is None or (
+                self._axis is not None
+                and not numpy.isclose(value, self._axis.absolute_cor_value, atol=1e-3)
+            ):
+                self.sigAbsolueValueSet.emit(value)
+
     def setAxis(self, axis):
         assert isinstance(axis, QAxisRP)
         if axis == self._axis:
             return
-        if self._axis is not None:
-            self._axis.sigChanged.disconnect(self._updatePosition)
         self._axis = axis
-        self._axis.sigChanged.connect(self._updatePosition)
-        self._updatePosition()
-
-    def _updatePosition(self):
-        if self._axis:
-            if self._axis.relative_cor_value is None:
-                value = "?"
-                self._tomwerPositionLabel.setText(value)
-                self._nabuPositionLabel.setText(value)
-            else:
-                self._tomwerPositionLabel.setText(str(self._axis.relative_cor_value))
-                self._nabuPositionLabel.setText(str(self._axis.value_ref_nabu))
 
     def getPosition(self):
-        return float(self._tomwerPositionLabel.text())
+        return float(self._relativePositionQLE.text())
 
-    def setPosition(self, value_ref_tomwer, value_ref_nabu):
-        if value_ref_tomwer is None:
-            self._tomwerPositionLabel.setText("?")
+    def setPosition(self, relative_cor: Optional[float], abs_cor: Optional[float]):
+        if relative_cor is None:
+            self._relativePositionQLE.setText("?")
+        elif isinstance(relative_cor, str):
+            self._relativePositionQLE.setText(relative_cor)
         else:
-            self._tomwerPositionLabel.setText(str(value_ref_tomwer))
-        if value_ref_nabu is None:
-            self._nabuPositionLabel.setText("?")
+            self._relativePositionQLE.setText(f"{relative_cor:.3f}")
+        if abs_cor is None:
+            self._absolutePositionQLE.setText("?")
         else:
-            self._nabuPositionLabel.setText(str(value_ref_nabu))
+            self._absolutePositionQLE.setText(f"{abs_cor:.3f}")
 
     def _updateAbsolutePosition(self, width):
-        rel_value = self._tomwerPositionLabel.text()
-        if rel_value.isnumeric() and width is not None:
-            abs_value = float(rel_value) + width / 2.0
-            self._nabuPositionLabel.setText(str(abs_value))
+        if width is not None:
+            try:
+                rel_value = float(self._relativePositionQLE.text())
+            except Exception:
+                return
+            else:
+                abs_value = float(rel_value) + width / 2.0
+                self._absolutePositionQLE.setText(str(abs_value))
+
+    def _updateRelativePosition(self, width):
+        if width is not None:
+            try:
+                abs_value = float(self._absolutePositionQLE.text())
+            except Exception:
+                return
+            else:
+                rel_value = float(abs_value) - width / 2.0
+                self._relativePositionQLE.setText(str(rel_value))
