@@ -28,13 +28,15 @@ __license__ = "MIT"
 __date__ = "10/01/2018"
 
 
-from silx.gui import qt
 import logging
-from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefs
+
+from silx.gui import qt
+
 from tomwer.core.process.reconstruction.darkref import params as dkrf
+from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefsTask
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.gui.utils.sandboxes import RegularExpressionSandBox
-from tomwer.synctools.ftseries import QReconsParams, _QDKRFRP
+from tomwer.synctools.darkref import QDKRFRP
 
 logger = logging.getLogger(__name__)
 
@@ -87,10 +89,7 @@ class DarkRefWidget(qt.QWidget):
         return self.mainWidget.recons_params
 
 
-class QDarkRefs(DarkRefs):
-    def _instanciateReconsParams(self):
-        return QReconsParams(empty=True)
-
+class QDarkRefs(DarkRefsTask):
     def _computationStarts(self):
         pass
 
@@ -104,7 +103,7 @@ class _WhatCheckBox(qt.QWidget):
     (None, median, average) for a what (ref, dark)
     """
 
-    sigChanged = qt.Signal(dkrf.Method)
+    sigChanged = qt.Signal(dkrf.ReduceMethod)
     """Signal emitted when the calculation mode change"""
 
     def __init__(self, parent, text):
@@ -113,27 +112,27 @@ class _WhatCheckBox(qt.QWidget):
         self._checkbox = qt.QCheckBox(text=text, parent=self)
         self.layout().addWidget(self._checkbox)
         self._modeCB = qt.QComboBox(parent=self)
-        for mode in dkrf.Method:
-            if mode is dkrf.Method.none:
+        for mode in dkrf.ReduceMethod:
+            if mode is dkrf.ReduceMethod.NONE:
                 continue
-            self._modeCB.addItem(mode.name.title())
+            self._modeCB.addItem(mode.value)
         self._modeCB.setItemData(
-            self._modeCB.findText(dkrf.Method.median.name.title()),
+            self._modeCB.findText(dkrf.ReduceMethod.MEDIAN.value),
             "Compute the median for each serie",
             qt.Qt.ToolTipRole,
         )
         self._modeCB.setItemData(
-            self._modeCB.findText(dkrf.Method.average.name.title()),
+            self._modeCB.findText(dkrf.ReduceMethod.MEAN.value),
             "Compute the average for each serie",
             qt.Qt.ToolTipRole,
         )
         self._modeCB.setItemData(
-            self._modeCB.findText(dkrf.Method.first.name.title()),
+            self._modeCB.findText(dkrf.ReduceMethod.FIRST.value),
             "Pick the first frame for a serie",
             qt.Qt.ToolTipRole,
         )
         self._modeCB.setItemData(
-            self._modeCB.findText(dkrf.Method.last.name.title()),
+            self._modeCB.findText(dkrf.ReduceMethod.LAST.value),
             "Pick the last frame for a serie",
             qt.Qt.ToolTipRole,
         )
@@ -150,30 +149,28 @@ class _WhatCheckBox(qt.QWidget):
         or not (in the case we are simply reading structure and to avoid
         looping in signals with other QObject)"""
 
-    def getMode(self):
-        if self._checkbox.isChecked() is True:
-            return getattr(dkrf.Method, self._modeCB.currentText().lower())
+    def getMode(self) -> dkrf.ReduceMethod:
+        if self._checkbox.isChecked():
+            return dkrf.ReduceMethod.from_value(self._modeCB.currentText())
         else:
-            return dkrf.Method.none
+            return dkrf.ReduceMethod.NONE
 
     def getModeName(self):
-        return self.getMode().name.split(".")[-1].title()
+        return self.getMode().value
 
     def _modeChange(self, *a, **b):
         self.sigChanged.emit(self.getMode())
 
     def setMode(self, mode):
-        _mode = mode
-        if type(_mode) is str:
-            _mode = getattr(dkrf.Method, mode.lower())
-        assert _mode in dkrf.Method
+        mode = dkrf.ReduceMethod.from_value(mode)
+        assert mode in dkrf.ReduceMethod
         self._checkbox.toggled.disconnect(self._modeChange)
         self._modeCB.currentIndexChanged.disconnect(self._modeChange)
-        self._checkbox.setChecked(_mode is not dkrf.Method.none)
-        if _mode is not dkrf.Method.none:
-            index = self._modeCB.findText(_mode.name.title())
+        self._checkbox.setChecked(mode is not dkrf.ReduceMethod.NONE)
+        if mode is not dkrf.ReduceMethod.NONE:
+            index = self._modeCB.findText(mode.value)
             if index < 0:
-                logger.error("index for %s is not recognized" % _mode)
+                logger.error(f"index for {mode.value} is not recognized")
             else:
                 self._modeCB.setCurrentIndex(index)
         self._checkbox.toggled.connect(self._modeChange)
@@ -201,7 +198,8 @@ class _TabGeneral(qt.QWidget):
         self._grpOptions = qt.QGroupBox("options", parent=self)
         self._grpOptions.setLayout(qt.QVBoxLayout())
         self._rmOptionCB = qt.QCheckBox(
-            parent=self._grpOptions, text="remove raw files when done"
+            parent=self._grpOptions,
+            text="remove raw EDF files when done (for spec only)",
         )
         self.sigRmToggled = self._rmOptionCB.toggled
         self._skipOptionCB = qt.QCheckBox(
@@ -232,17 +230,17 @@ class _TabExpert(qt.QWidget):
         self._patternsWidget.setLayout(qt.QGridLayout())
 
         self._patternsWidget.layout().addWidget(
-            qt.QLabel("dark file pattern", parent=self._patternsWidget), 0, 0
+            qt.QLabel("pyhst dark file pattern", parent=self._patternsWidget), 0, 0
         )
         self._darkLE = qt.QLineEdit(parent=self._patternsWidget)
-        self._darkLE.setToolTip(DarkRefs.getDarkPatternTooltip())
+        self._darkLE.setToolTip(DarkRefsTask.getDarkPatternTooltip())
         self._darkLE.editingFinished.connect(self._darkPatternEdited)
         self._patternsWidget.layout().addWidget(self._darkLE, 0, 1)
         self._patternsWidget.layout().addWidget(
-            qt.QLabel("flat file pattern", parent=self._patternsWidget), 1, 0
+            qt.QLabel("pyhst flat file pattern", parent=self._patternsWidget), 1, 0
         )
         self._refLE = qt.QLineEdit(parent=self._patternsWidget)
-        self._refLE.setToolTip(DarkRefs.getRefPatternTooltip())
+        self._refLE.setToolTip(DarkRefsTask.getRefPatternTooltip())
         self._refLE.editingFinished.connect(self._refPatternEdited)
 
         self._patternsWidget.layout().addWidget(self._refLE, 1, 1)
@@ -306,7 +304,7 @@ class DarkRefTab(qt.QTabWidget):
                 self.recons_params.__class__.dark_calc_method.setter
             )
             self.tabGeneral.sigRefChanged.disconnect(
-                self.recons_params.__class__.ref_calc_method.setter
+                self.recons_params.__class__.flat_calc_method.setter
             )
             self.tabGeneral.sigRmToggled.disconnect(self.recons_params._set_remove_opt)
             self.tabGeneral.sigSkipToggled.disconnect(
@@ -329,7 +327,7 @@ class DarkRefTab(qt.QTabWidget):
             self.recons_params.__class__.dark_calc_method.setter
         )
         self.tabGeneral.sigRefChanged.connect(
-            self.recons_params.__class__.ref_calc_method.setter
+            self.recons_params.__class__.flat_calc_method.setter
         )
         self.tabGeneral.sigRmToggled.connect(self.recons_params._set_remove_opt)
         self.tabGeneral.sigSkipToggled.connect(self.recons_params._set_skip_if_exist)
@@ -338,7 +336,7 @@ class DarkRefTab(qt.QTabWidget):
             self.recons_params.__class__.dark_pattern.setter
         )
         self.tabExpert.sigRefPatternEdited.connect(
-            self.recons_params.__class__.ref_pattern.setter
+            self.recons_params.__class__.flat_pattern.setter
         )
         self.recons_params.sigChanged.connect(self._updateReconsParams)
 
@@ -346,11 +344,9 @@ class DarkRefTab(qt.QTabWidget):
         self.loadStructs(self.recons_params.to_dict())
 
     def setReconsParams(self, recons_params):
-        assert isinstance(recons_params, (QReconsParams, _QDKRFRP))
-        if isinstance(recons_params, _QDKRFRP):
-            _recons_params = recons_params
-        else:
-            _recons_params = recons_params.dkrf
+        if not isinstance(recons_params, QDKRFRP):
+            raise TypeError
+        _recons_params = recons_params
 
         if self.recons_params:
             self._disconnectToReconsParams()
