@@ -28,13 +28,17 @@ __license__ = "MIT"
 __date__ = "11/02/2020"
 
 
+import logging
 import os
 import typing
+
 from silx.gui import qt
-from tomwer.core.process.reconstruction.nabu.nabucommon import NabuOutputFileFormat
-from tomwer.gui.reconstruction.nabu.nabuconfig.base import _NabuStageConfigBase
+
+from tomwer.core.process.reconstruction.output import NabuOutputFileFormat
 from tomwer.core.process.reconstruction.nabu.utils import _NabuStages
-import logging
+from tomwer.core.process.reconstruction.output import ProcessDataOutputDirMode
+from tomwer.gui.qlefilesystem import QLFileSystem
+from tomwer.gui.reconstruction.nabu.nabuconfig.base import _NabuStageConfigBase
 from tomwer.io.utils import get_default_directory
 
 try:
@@ -48,10 +52,25 @@ _logger = logging.getLogger(__name__)
 
 
 class QNabuFileFormatComboBox(qt.QComboBox):
-    def __init__(self, parent: typing.Optional[qt.QWidget] = ...) -> None:
+    def __init__(
+        self, parent: typing.Optional[qt.QWidget] = ..., filter_formats=tuple()
+    ) -> None:
+        """
+        :param filter_format: if provided the given file format won't have an item on the ComboBox
+        """
         super().__init__(parent=parent)
+        if not isinstance(filter_formats, tuple):
+            raise TypeError(
+                f"filter_format should be a tuple. Get {type(filter_formats)} instead"
+            )
+        filter_formats = [
+            NabuOutputFileFormat.from_value(file_format)
+            for file_format in filter_formats
+        ]
 
         for ff in NabuOutputFileFormat:
+            if ff in filter_formats:
+                continue
             if ff is NabuOutputFileFormat.JP2K:
                 if not has_glymur:
                     _logger.warning(
@@ -71,6 +90,144 @@ class QNabuFileFormatComboBox(qt.QComboBox):
                 self.addItem(ff.value)
 
 
+class QNabuFileFormatComboBoxIgnoreWheel(QNabuFileFormatComboBox):
+    def wheelEvent(self, e: qt.QWheelEvent) -> None:
+        pass
+
+
+class NabuOutputLocationWidget(qt.QGroupBox):
+    sigOutputChanged = qt.Signal()
+    """Emit when location changed"""
+
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(title="output folder", *args, **kwargs)
+        self.setLayout(qt.QGridLayout())
+
+        self.layout().setContentsMargins(0, 0, 0, 0)
+        self.layout().setSpacing(0)
+
+        # in scan folder
+        self._inScanFolderRB = qt.QRadioButton(
+            ProcessDataOutputDirMode.IN_SCAN_FOLDER.value, self
+        )
+        self._inScanFolderRB.setToolTip(
+            "Reconstruction will be saved at the same level as the acquisition folder. (near the NXtomo file (.nx) or under the spec acquisition folder)"
+        )
+        self.layout().addWidget(self._inScanFolderRB, 1, 0, 1, 1)
+        # in processed data dir
+        self._processedDataDirRB = qt.QRadioButton(
+            ProcessDataOutputDirMode.PROCESSED_DATA_FOLDER.value, self
+        )
+        self._inScanFolderRB.setToolTip(
+            "Reconstruction will be saved under the PROCESSED_DATA/dataset folder. if exists else under the scan folder."
+        )
+        self.layout().addWidget(self._processedDataDirRB, 2, 0, 1, 1)
+        # raw data dir
+        self._rawDataDirRB = qt.QRadioButton(
+            ProcessDataOutputDirMode.RAW_DATA_FOLDER.value, self
+        )
+        self._rawDataDirRB.setToolTip(
+            "Reconstruction will be saved under the RAW_DATA/dataset folder. if exists else under the scan folder."
+        )
+        self.layout().addWidget(self._rawDataDirRB, 3, 0, 1, 1)
+        # other dir
+        self._otherDirRB = qt.QRadioButton(ProcessDataOutputDirMode.OTHER.value, self)
+        self._otherDirRB.setToolTip(
+            "Reconstruction will be saved under user provided folder."
+        )
+        self.layout().addWidget(self._otherDirRB, 4, 0, 1, 1)
+        self._outputDirQLE = QLFileSystem(
+            "", self, filters=qt.QDir.NoDotAndDotDot | qt.QDir.Dirs
+        )
+        self.layout().addWidget(self._outputDirQLE, 4, 1, 1, 1)
+        style = qt.QApplication.style()
+        icon_opendir = style.standardIcon(qt.QStyle.SP_DirOpenIcon)
+        self._selectOutputPB = qt.QPushButton(icon_opendir, "", self)
+        self._selectOutputPB.setIcon(icon_opendir)
+        general_tooltip = (
+            "You can enter a string with some keywords like {my_keyword}. Those will be interpreted during runtime according to scan metadata. Possible keywords are:"
+            + "\n - 'scan_dir_name': returns name of the directory containing the acquisition (! not a path !)"
+            + "\n - 'scan_basename': returns basename of the directory containing the acquisition"
+            + "\n - 'scan_parent_dir_basename': returns basename of the PARENT directory containing the acquisition"
+        )
+
+        self._selectOutputPB.setToolTip(general_tooltip)
+        self._outputDirQLE.setToolTip(general_tooltip)
+        self.layout().addWidget(self._selectOutputPB, 4, 2, 1, 1)
+
+        # set up:
+        self._inScanFolderRB.setChecked(True)
+
+        # connect signal / slot
+        self._inScanFolderRB.toggled.connect(self._outputModeChanged)
+        self._processedDataDirRB.toggled.connect(self._outputModeChanged)
+        self._rawDataDirRB.toggled.connect(self._outputModeChanged)
+        self._otherDirRB.toggled.connect(self._outputModeChanged)
+        self._selectOutputPB.released.connect(self._selectOutput)
+        self._inScanFolderRB.setChecked(True)
+        self._outputModeChanged()
+
+    def getOutputDirMode(self) -> ProcessDataOutputDirMode:
+        if self._inScanFolderRB.isChecked():
+            return ProcessDataOutputDirMode.IN_SCAN_FOLDER
+        elif self._processedDataDirRB.isChecked():
+            return ProcessDataOutputDirMode.PROCESSED_DATA_FOLDER
+        elif self._rawDataDirRB.isChecked():
+            return ProcessDataOutputDirMode.RAW_DATA_FOLDER
+        elif self._otherDirRB.isChecked():
+            return ProcessDataOutputDirMode.OTHER
+
+    def setOutputDirMode(
+        self, mode: typing.Union[ProcessDataOutputDirMode, str]
+    ) -> None:
+        mode = ProcessDataOutputDirMode.from_value(mode)
+        if mode is ProcessDataOutputDirMode.IN_SCAN_FOLDER:
+            self._inScanFolderRB.setChecked(True)
+        elif mode is ProcessDataOutputDirMode.PROCESSED_DATA_FOLDER:
+            self._processedDataDirRB.setChecked(True)
+        elif mode is ProcessDataOutputDirMode.RAW_DATA_FOLDER:
+            self._rawDataDirRB.setChecked(True)
+        elif mode is ProcessDataOutputDirMode.OTHER:
+            self._otherDirRB.setChecked(True)
+
+    def _outputModeChanged(self, *args, **kwargs):
+        outputMode = self.getOutputDirMode()
+        self._outputDirQLE.setVisible(outputMode is ProcessDataOutputDirMode.OTHER)
+        self._selectOutputPB.setVisible(outputMode is ProcessDataOutputDirMode.OTHER)
+        self.sigOutputChanged.emit()
+
+    def _selectOutput(self):  # pragma: no cover
+        defaultDirectory = self._outputDirQLE.text()
+        if not os.path.isdir(defaultDirectory):
+            defaultDirectory = get_default_directory()
+
+        dialog = qt.QFileDialog(self, directory=defaultDirectory)
+        dialog.setFileMode(qt.QFileDialog.DirectoryOnly)
+
+        if not dialog.exec_():
+            dialog.close()
+            return
+
+        self._outputDirQLE.setText(dialog.selectedFiles()[0])
+
+    def getOutputDir(self):
+        """
+
+        :return: None if the default output directory is selected else
+                 return path to the directory
+        """
+        if self._otherDirRB.isChecked():
+            return self._outputDirQLE.text()
+        else:
+            return None
+
+    def setOutputDir(self, output_dir):
+        if output_dir in (None, ""):
+            pass
+        else:
+            self._outputDirQLE.setText(output_dir)
+
+
 class _NabuOutputConfig(_NabuStageConfigBase, qt.QWidget):
     """
     Widget to define the output configuration of nabu
@@ -87,41 +244,14 @@ class _NabuOutputConfig(_NabuStageConfigBase, qt.QWidget):
         self.setLayout(qt.QGridLayout())
 
         # output dir
-        self._output_dir_widget = qt.QWidget(self)
-        self._outputLabel = qt.QLabel("output:", self._output_dir_widget)
+        self._output_dir_widget = NabuOutputLocationWidget(parent=self)
         self.layout().addWidget(self._output_dir_widget, 0, 0, 1, 4)
-        self._output_dir_widget.setLayout(qt.QHBoxLayout())
-        self._output_dir_widget.layout().setContentsMargins(0, 0, 0, 0)
-        self._output_dir_widget.layout().setSpacing(0)
-        self._output_dir_widget.layout().addWidget(self._outputLabel)
-        self._defaultOutput = qt.QCheckBox(
-            "raw data root level", self._output_dir_widget
-        )
-        self._defaultOutput.setToolTip(
-            "set reconstruction at the the same " "level as raw data"
-        )
-        self._output_dir_widget.layout().addWidget(self._defaultOutput)
-        self._outputDirQLE = qt.QLineEdit("", self._output_dir_widget)
-        self._output_dir_widget.layout().addWidget(self._outputDirQLE)
-        style = qt.QApplication.style()
-        icon_opendir = style.standardIcon(qt.QStyle.SP_DirOpenIcon)
-        self._selectOutputPB = qt.QPushButton(icon_opendir, "", self._output_dir_widget)
-        self._selectOutputPB.setIcon(icon_opendir)
-        self._selectOutputPB.setToolTip(
-            "select output directory. This can interpret the following keyword (that should be provided like '{keyword}'."
-            "\n - 'scan_dir_name': returns name of the directory containing the acquisition (! not a path !)"
-            "\n - 'scan_basename': returns basename of the directory containing the acquisition"
-            "\n - 'scan_parent_dir_basename': returns basename of the PARENT directory containing the acquisition"
-        )
-        self._output_dir_widget.layout().addWidget(self._selectOutputPB)
-        # is required in nabu configuration file but tomwer wan't to force
-        # it to be in expert mode
         self.registerWidget(self._output_dir_widget, "advanced")
 
         # file format
         self._outputFileFormatLabel = qt.QLabel("output file format:", self)
         self.layout().addWidget(self._outputFileFormatLabel, 1, 0, 1, 1)
-        self._fileFormatCB = QNabuFileFormatComboBox(self)
+        self._fileFormatCB = QNabuFileFormatComboBoxIgnoreWheel(self)
         self.layout().addWidget(self._fileFormatCB, 1, 2, 1, 2)
         self.registerWidget(self._outputFileFormatLabel, "optional")
         self.registerWidget(self._fileFormatCB, "optional")
@@ -144,38 +274,16 @@ class _NabuOutputConfig(_NabuStageConfigBase, qt.QWidget):
         self.layout().addWidget(spacer, 200, 0, 1, 1)
 
         # set up
-        self._defaultOutput.setChecked(True)
-        self._hideOutput(True)
         self.setFramePerGroup(100)
         self.setFileformat("hdf5")
 
         # connect signal / slot
-        self._defaultOutput.toggled.connect(self._hideOutput)
-        self._selectOutputPB.released.connect(self._selectOutput)
-        self._outputDirQLE.editingFinished.connect(self._outputDirChanged)
-        self._defaultOutput.toggled.connect(self._outputDirChanged)
+        self._output_dir_widget._outputDirQLE.editingFinished.connect(
+            self._outputDirChanged
+        )
+        self._output_dir_widget.sigOutputChanged.connect(self._outputDirChanged)
         self._fileFormatCB.currentTextChanged.connect(self._fileFormatChanged)
         self._framePerGroup.valueChanged.connect(self._framePerGroupChanged)
-        self._hideOutput(True)
-
-    def _hideOutput(self, hide):
-        self._outputDirQLE.setVisible(not hide)
-        self._selectOutputPB.setVisible(not hide)
-
-    def _selectOutput(self):
-        defaultDirectory = self._outputDirQLE.text()
-        if os.path.isdir(defaultDirectory):
-            defaultDirectory = get_default_directory()
-
-        dialog = qt.QFileDialog(self, directory=defaultDirectory)
-        dialog.setFileMode(qt.QFileDialog.DirectoryOnly)
-
-        if not dialog.exec_():
-            dialog.close()
-            return
-
-        self._outputDirQLE.setText(dialog.selectedFiles()[0])
-        self._outputDirChanged()
 
     def _outputDirChanged(self):
         self.sigConfChanged.emit("location")
@@ -187,22 +295,16 @@ class _NabuOutputConfig(_NabuStageConfigBase, qt.QWidget):
         self.sigConfChanged.emit("frames_per_group")
 
     def getOutputDir(self):
-        """
-
-        :return: None if the default output directory is selected else
-                 return path to the directory
-        """
-        if self._defaultOutput.isChecked():
-            return None
-        else:
-            return self._outputDirQLE.text()
+        return self._output_dir_widget.getOutputDir()
 
     def setOutputDir(self, dir):
-        if dir in (None, ""):
-            self._defaultOutput.setChecked(True)
-        else:
-            self._defaultOutput.setChecked(False)
-            self._outputDirQLE.setText(dir)
+        return self._output_dir_widget.setOutputDir(dir)
+
+    def getOutputdirMode(self):
+        return self._output_dir_widget.getOutputDirMode()
+
+    def setOutputdirMode(self, mode):
+        return self._output_dir_widget.setOutputDirMode(mode=mode)
 
     def getFileFormat(self) -> NabuOutputFileFormat:
         return NabuOutputFileFormat.from_value(self._fileFormatCB.currentText())
@@ -222,21 +324,25 @@ class _NabuOutputConfig(_NabuStageConfigBase, qt.QWidget):
         return {
             "file_format": self.getFileFormat().value,
             "location": self.getOutputDir() or "",
+            "output_dir_mode": self.getOutputdirMode().value,
             # 'frames_per_group': self.getFramePerGroup(),
         }
 
     def setConfiguration(self, config):
         if "file_format" in config:
             self.setFileformat(config["file_format"])
-        if "location" in config:
-            self.setOutputDir(config["location"])
+        location = config.get("location", None)
+        if location == "":
+            location = None
+        if location is not None:
+            self.setOutputDir(location)
         if "frames_per_group" in config:
             self.setFramePerGroup(int(config["frames_per_group"]))
 
-
-if __name__ == "__main__":
-    app = qt.QApplication([])
-    widget = _NabuOutputConfig(parent=None)
-    widget.show()
-    print(widget.getConfiguration())
-    app.exec_()
+        # definition of default_output_dir_mode ensure backward compatibility
+        default_output_dir_mode = (
+            None if location is None else ProcessDataOutputDirMode.OTHER
+        )
+        output_dir_mode = config.get("output_dir_mode", default_output_dir_mode)
+        if output_dir_mode is not None:
+            self.setOutputdirMode(output_dir_mode)
