@@ -27,26 +27,30 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "12/02/2020"
 
-import typing
 import functools
-from tomwer.utils import docstring
+import logging
+import typing
+
 from silx.gui import qt
+from silx.gui.dialog.DataFileDialog import DataFileDialog
+from silx.utils.enum import Enum as _Enum
+
+from tomwer.core.process.reconstruction.nabu.plane import NabuPlane
+from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSliceMode
 from tomwer.core.process.reconstruction.nabu.utils import (
-    _NabuStages,
-    _NabuReconstructionMethods,
     _NabuFBPFilterType,
     _NabuPaddingType,
+    _NabuReconstructionMethods,
+    _NabuStages,
 )
-from tomwer.core.process.reconstruction.nabu.nabuslices import NabuSliceMode
 from tomwer.gui.reconstruction.nabu.nabuconfig.base import _NabuStageConfigBase
 from tomwer.gui.utils.inputwidget import SelectionLineEdit
-from tomwer.gui.utils.scrollarea import QComboBoxIgnoreWheel
-from tomwer.gui.utils.scrollarea import QDoubleSpinBoxIgnoreWheel
-from tomwer.gui.utils.scrollarea import QSpinBoxIgnoreWheel
-from silx.utils.enum import Enum as _Enum
-from silx.gui.dialog.DataFileDialog import DataFileDialog
-
-import logging
+from tomwer.gui.utils.scrollarea import (
+    QComboBoxIgnoreWheel,
+    QDoubleSpinBoxIgnoreWheel,
+    QSpinBoxIgnoreWheel,
+)
+from tomwer.utils import docstring
 
 _logger = logging.getLogger(__name__)
 
@@ -68,7 +72,7 @@ class SliceGroupBox(qt.QGroupBox):
         self.layout().addWidget(self._modeCB)
         self._modeCB.setFocusPolicy(qt.Qt.FocusPolicy.NoFocus)
 
-        self._sliceQLE = SelectionLineEdit(parent=self)
+        self._sliceQLE = SelectionLineEdit(parent=self, allow_negative_indices=True)
         self.layout().addWidget(self._sliceQLE)
 
         spacer = qt.QWidget(parent=self)
@@ -82,16 +86,13 @@ class SliceGroupBox(qt.QGroupBox):
 
         # connect signal / slot
         self._modeCB.currentIndexChanged.connect(self._updateSliceQLEVisibilty)
-        self._modeCB.currentIndexChanged.connect(self._triggerSlicesChanged)
-        self._sliceQLE.editingFinished.connect(self._triggerSlicesChanged)
-        self.toggled.connect(self._triggerSlicesChanged)
+        self._modeCB.currentIndexChanged.connect(self.sigSlicesChanged)
+        self._sliceQLE.editingFinished.connect(self.sigSlicesChanged)
+        self.toggled.connect(self.sigSlicesChanged)
 
     def _updateSliceQLEVisibilty(self, *args, **kwargs):
         self._sliceQLE.setVisible(self.getMode() == NabuSliceMode.OTHER)
 
-    def _triggerSlicesChanged(self):
-        self.sigSlicesChanged.emit()
-
     def setMode(self, mode: NabuSliceMode):
         mode = NabuSliceMode.from_value(mode)
         item_index = self._modeCB.findText(mode.value)
@@ -194,7 +195,7 @@ class TranslationMvtFileWidget(qt.QWidget):
         else:
             return dialog.selectedUrl()
 
-    def _selectTextFile(self):
+    def _selectTextFile(self):  # pragma: no cover
         dialog = qt.QFileDialog(self)
         dialog.setFileMode(qt.QFileDialog.ExistingFile)
 
@@ -236,6 +237,46 @@ class TranslationMvtFileWidget(qt.QWidget):
         self._selectButton.setEnabled(self._checkBox.isChecked())
 
 
+class AnglesFileWidget(qt.QWidget):
+    """widget to retrieve the text file for angles to provide to nabu"""
+
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+        self.setLayout(qt.QHBoxLayout())
+        self._qle = qt.QLineEdit("", self)
+        self.layout().addWidget(self._qle)
+        self._selectPB = qt.QPushButton("select", self)
+        self.layout().addWidget(self._selectPB)
+
+        # connect signal / slot
+        self._selectPB.released.connect(self._selectFile)
+
+    def getFile(self) -> str:
+        return self._qle.text().replace(" ", "")
+
+    def setFile(self, file_path):
+        self._qle.setText(file_path)
+
+    def _selectFile(self) -> str:  # pragma: no cover
+        dialog = qt.QFileDialog()
+        dialog.setFileMode(qt.QFileDialog.ExistingFile)
+        dialog.setNameFilters(
+            [
+                "Text file (*.txt)",
+                "Any file (*)",
+            ]
+        )
+
+        if not dialog.exec_():
+            dialog.close()
+            return
+        elif len(dialog.selectedFiles()) == 0:
+            return
+        else:
+            file_path = dialog.selectedFiles()[0]
+            self._qle.setText(file_path)
+
+
 class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
     """
     Widget to define the configuration of nabu reconstruction processing
@@ -251,7 +292,6 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         _NabuStageConfigBase.__init__(self, stage=_NabuStages.PRE)
         self.setLayout(qt.QGridLayout())
 
-        self.__angle_files = ""
         self.__optimizationIteAlgo = "chambolle-pock"
 
         # slices to be reconstructed online
@@ -261,55 +301,74 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         # come with an issue as we are on a scroll area
         self._slicesWidget.setFocusPolicy(qt.Qt.FocusPolicy.NoFocus)
 
+        # axis
+        self._axisLabel = qt.QLabel("reconstruction plane", self)
+        self._axisLabel.setToolTip("Over which axis the slice must be picked")
+        self.layout().addWidget(self._axisLabel)
+        self._axisQCB = qt.QComboBox(self)
+        axis_tooltips = {
+            NabuPlane.YZ.value: "along axis x (aka axis 2 - expected to be slow)",
+            NabuPlane.XZ.value: "along axis y (aka axis 1 - expected to be slow)",
+            NabuPlane.XY.value: "along axis Z (aka axis 0 - fastest)",
+        }
+        for item, tooltip in axis_tooltips.items():
+            self._axisQCB.addItem(item)
+            self._axisQCB.setItemData(
+                self._axisQCB.findText(item),
+                tooltip,
+                qt.Qt.ToolTipRole,
+            )
+        self.layout().addWidget(self._axisQCB, 1, 1, 1, 1)
+
         # method
         self._methodLabel = qt.QLabel("method", self)
-        self.layout().addWidget(self._methodLabel, 1, 0, 1, 1)
+        self.layout().addWidget(self._methodLabel, 2, 0, 1, 1)
         self._methodQCB = QComboBoxIgnoreWheel(parent=self, scrollArea=scrollArea)
         for method in _NabuReconstructionMethods:
             self._methodQCB.addItem(method.value)
-        self.layout().addWidget(self._methodQCB, 1, 1, 1, 1)
+        self.layout().addWidget(self._methodQCB, 2, 1, 1, 1)
         self.registerWidget(self._methodLabel, "required")
         self.registerWidget(self._methodQCB, "required")
 
         # angle_offset
         self._labelOffsetLabel = qt.QLabel("angle offset (in degree)", self)
-        self.layout().addWidget(self._labelOffsetLabel, 2, 0, 1, 1)
+        self.layout().addWidget(self._labelOffsetLabel, 3, 0, 1, 1)
         self._angleOffsetQDSB = QDoubleSpinBoxIgnoreWheel(self, scrollArea)
         self._angleOffsetQDSB.setMaximum(-180)
         self._angleOffsetQDSB.setMaximum(180)
-        self.layout().addWidget(self._angleOffsetQDSB, 2, 1, 1, 1)
+        self.layout().addWidget(self._angleOffsetQDSB, 3, 1, 1, 1)
         self.registerWidget(self._labelOffsetLabel, "advanced")
         self.registerWidget(self._angleOffsetQDSB, "advanced")
 
         # fbp filter type
         self._fbpFilterCB = qt.QCheckBox("fbp filter", self)
-        self.layout().addWidget(self._fbpFilterCB, 3, 0, 1, 1)
+        self.layout().addWidget(self._fbpFilterCB, 4, 0, 1, 1)
         self._fbpFilterType = QComboBoxIgnoreWheel(self, scrollArea)
         for filter_type in _NabuFBPFilterType:
             self._fbpFilterType.addItem(filter_type.value)
-        self.layout().addWidget(self._fbpFilterType, 3, 1, 1, 1)
+        self.layout().addWidget(self._fbpFilterType, 4, 1, 1, 1)
         self.registerWidget(self._fbpFilterCB, "advanced")
         self.registerWidget(self._fbpFilterType, "advanced")
 
         # padding type
         self._paddingTypeLabel = qt.QLabel("padding type", self)
-        self.layout().addWidget(self._paddingTypeLabel, 4, 0, 1, 1)
+        self.layout().addWidget(self._paddingTypeLabel, 5, 0, 1, 1)
         self._paddingType = QComboBoxIgnoreWheel(self, scrollArea)
         for fbp_padding_type in _NabuPaddingType:
             self._paddingType.addItem(fbp_padding_type.value)
-        self.layout().addWidget(self._paddingType, 4, 1, 1, 1)
+        self.layout().addWidget(self._paddingType, 5, 1, 1, 1)
         self.registerWidget(self._paddingTypeLabel, "optional")
         self.registerWidget(self._paddingType, "optional")
 
         # sub region
         self._subRegionSelector = _NabuReconstructionSubRegion(parent=self)
-        self.layout().addWidget(self._subRegionSelector, 6, 0, 1, 2)
+        self.layout().addWidget(self._subRegionSelector, 7, 0, 1, 2)
 
         # iterations
         self._iterationsLabel = qt.QLabel("iterations", self)
-        self.layout().addWidget(self._iterationsLabel, 7, 0, 1, 1)
+        self.layout().addWidget(self._iterationsLabel, 8, 0, 1, 1)
         self._iterationSB = qt.QSpinBox(parent=self)
-        self.layout().addWidget(self._iterationSB, 7, 1, 1, 1)
+        self.layout().addWidget(self._iterationSB, 8, 1, 1, 1)
         self._iterationSB.setMinimum(1)
         self._iterationSB.setMaximum(9999)
         # not supported for now so hidden
@@ -320,14 +379,14 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         self._binSubSamplingGB = _BinSubSampling(
             "binning and subsampling", parent=self, scrollArea=scrollArea
         )
-        self.layout().addWidget(self._binSubSamplingGB, 8, 0, 1, 2)
+        self.layout().addWidget(self._binSubSamplingGB, 9, 0, 1, 2)
 
         # optimization algorithm:
         # set has default value for now, because has only one at the moment
 
         # weight total variation
         self._tvLabel = qt.QLabel("total variation weight", self)
-        self.layout().addWidget(self._tvLabel, 9, 0, 1, 1)
+        self.layout().addWidget(self._tvLabel, 10, 0, 1, 1)
         self._totalVariationWeight = qt.QDoubleSpinBox(self)
         self._totalVariationWeight.setMinimum(0.0)
         self._totalVariationWeight.setMaximum(1.0)
@@ -343,7 +402,7 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         self._preconditioningFilter.setToolTip(
             'Whether to enable "filter ' 'preconditioning" for iterative' " methods"
         )
-        self.layout().addWidget(self._preconditioningFilter, 9, 0, 1, 2)
+        self.layout().addWidget(self._preconditioningFilter, 10, 0, 1, 2)
         # not supported for now so hidden
         self._preconditioningFilter.hide()
 
@@ -352,28 +411,47 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         self._positivityConstraintCB.setToolTip(
             "Whether to enforce a " "positivity constraint in the " "reconstruction."
         )
-        self.layout().addWidget(self._positivityConstraintCB, 10, 0, 1, 2)
+        self.layout().addWidget(self._positivityConstraintCB, 11, 0, 1, 2)
         # not supported for now so hidden
         self._positivityConstraintCB.hide()
 
+        # clip_outer_circle option
+        self._clipOuterCircleCB = qt.QCheckBox("clip outer circle", self)
+        self._clipOuterCircleCB.setToolTip(
+            "Whether to set to zero voxels falling outside of the reconstruction region"
+        )
+        self.layout().addWidget(self._clipOuterCircleCB, 12, 0, 1, 2)
+        self.registerWidget(self._clipOuterCircleCB, "optional")
+
+        # centered axis option
+        self._centeredAxisCB = qt.QCheckBox("centered axis", self)
+        self._centeredAxisCB.setToolTip("")
+        self.layout().addWidget(self._centeredAxisCB, 13, 0, 1, 2)
+        self.registerWidget(self._centeredAxisCB, "optional")
+
         # translation movement file
         self._transMvtFileLabel = qt.QLabel("translation movement file", self)
-        self.layout().addWidget(self._transMvtFileLabel, 11, 0, 1, 1)
+        self.layout().addWidget(self._transMvtFileLabel, 22, 0, 1, 1)
         self._transMvtFileWidget = TranslationMvtFileWidget(self)
-        self.layout().addWidget(self._transMvtFileWidget, 11, 1, 1, 1)
+        self.layout().addWidget(self._transMvtFileWidget, 22, 1, 1, 1)
         self.registerWidget(self._transMvtFileLabel, "advanced")
         self.registerWidget(self._transMvtFileWidget, "advanced")
 
+        # angle files (if the user want's to overwrite rotation angles)
+        self._angleFileLabel = qt.QLabel("angles file", self)
+        self.layout().addWidget(self._angleFileLabel, 23, 0, 1, 1)
+        self._anglesFileWidget = AnglesFileWidget(self)
+        self.layout().addWidget(self._anglesFileWidget, 23, 1, 1, 1)
+        self.registerWidget(self._angleFileLabel, "advanced")
+        self.registerWidget(self._anglesFileWidget, "advanced")
+
         # spacer for style
         spacer = qt.QWidget(self)
         spacer.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Expanding)
         self.layout().addWidget(spacer, 200, 1, 1, 1)
 
-        # expose API
-        self.getSlices = self._slicesWidget.getSlices
-        self.setSlices = self._slicesWidget.setSlices
-
         # set up
+        self.setNabuPlane("XY")
         self._fbpFilterCB.setChecked(True)
         fbp_item = self._methodQCB.findText(_NabuReconstructionMethods.FBP.value)
         self._methodQCB.setCurrentIndex(fbp_item)
@@ -386,8 +464,11 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         self._totalVariationWeight.setValue(1.0e-2)
         self._preconditioningFilter.setChecked(True)
         self._positivityConstraintCB.setChecked(True)
+        self._clipOuterCircleCB.setChecked(False)
+        self._centeredAxisCB.setChecked(False)
 
         # connect signal / slot
+        self._axisQCB.currentIndexChanged.connect(self._slicesChanged)
         self._methodQCB.currentIndexChanged.connect(self._methodChanged)
         self._angleOffsetQDSB.editingFinished.connect(self._angleOffsetChanged)
         self._fbpFilterCB.toggled.connect(self._FBPFilterTypeChanged)
@@ -401,6 +482,15 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         self._transMvtFileWidget.fileChanged.connect(self._mvtFileChanged)
         self._slicesWidget.sigSlicesChanged.connect(self._slicesChanged)
         self._binSubSamplingGB.binningChanged.connect(self._binningChanged)
+        self._anglesFileWidget._qle.textChanged.connect(self._anglesFilechanged)
+        self._clipOuterCircleCB.toggled.connect(self._clipOuterCircleChanged)
+        self._centeredAxisCB.toggled.connect(self._centeredAxisChanged)
+
+    def getSlices(self):
+        return self._slicesWidget.getSlices()
+
+    def setSlices(self, slices):
+        self._slicesWidget.setSlices(slices=slices)
 
     def hideSlicesInterface(self):
         self._slicesWidget.hide()
@@ -439,10 +529,19 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
     def _mvtFileChanged(self, *args, **kwargs):
         self._signalConfChanged("translation_movements_file")
 
+    def _clipOuterCircleChanged(self, *args, **kwargs):
+        self._signalConfChanged("clip_outer_circle")
+
+    def _centeredAxisChanged(self, *args, **kwargs):
+        self._signalConfChanged("centered_axis")
+
     def _binningChanged(self, *args, **kwargs):
         self._signalConfChanged("binning")
         self._signalConfChanged("binning_z")
 
+    def _anglesFilechanged(self, *args, **kwargs):
+        self._signalConfChanged("angles_file")
+
     def setScan(self, scan):
         raise NotImplementedError()
 
@@ -466,15 +565,15 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         else:
             return None
 
-    def setFBPFilterType(self, filter):
-        if type(filter) is str and filter.lower() == "none":
-            filter = None
-        if filter is None:
+    def setFBPFilterType(self, filter_type):
+        if type(filter_type) is str and filter_type.lower() == "none":
+            filter_type = None
+        if filter_type is None:
             self._fbpFilterCB.setChecked(False)
         else:
             self._fbpFilterCB.setChecked(True)
-            filter = _NabuFBPFilterType(filter)
-            filter_index = self._fbpFilterType.findText(filter.value)
+            filter_type = _NabuFBPFilterType(filter_type)
+            filter_index = self._fbpFilterType.findText(filter_type.value)
             self._fbpFilterType.setCurrentIndex(filter_index)
 
     def getFBPPaddingType(self) -> _NabuPaddingType:
@@ -525,7 +624,7 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         return self._binSubSamplingGB.getVerticalBinning()
 
     def setVerticalBinning(self, binning):
-        return self._binSubSamplingGB.getVerticalBinning(binning=binning)
+        return self._binSubSamplingGB.setVerticalBinning(binning=binning)
 
     def getProjSubsampling(self):
         return self._binSubSamplingGB.getProjSubsampling()
@@ -533,6 +632,32 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
     def setProjSubsampling(self, subsampling):
         return self._binSubSamplingGB.setProjSubsampling(subsampling=subsampling)
 
+    def getAnglesFile(self) -> str:
+        return self._anglesFileWidget.getFile()
+
+    def setAnglesFile(self, angles_file: str):
+        return self._anglesFileWidget.setFile(angles_file)
+
+    def getClipOuterCircle(self) -> bool:
+        return self._clipOuterCircleCB.isChecked()
+
+    def setClipOuterCircle(self, checked: bool) -> None:
+        self._clipOuterCircleCB.setChecked(checked)
+
+    def getCenteredAxis(self) -> bool:
+        return self._centeredAxisCB.isChecked()
+
+    def setCenteredAxis(self, checked: bool):
+        self._centeredAxisCB.setChecked(checked)
+
+    def getNabuPlane(self) -> NabuPlane:
+        """return over which axis we expect to do the reconstruction"""
+        return NabuPlane.from_value(self._axisQCB.currentText())
+
+    def setNabuPlane(self, axis: typing.Union[str, NabuPlane]):
+        axis = NabuPlane.from_value(axis)
+        self._axisQCB.setCurrentText(axis.value)
+
     @docstring(_NabuStageConfigBase)
     def getConfiguration(self) -> dict:
         fbp_filter_type = self.getFBPFilterType()
@@ -542,8 +667,8 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
             fbp_filter_type = fbp_filter_type.value
         config = {
             "method": self.getMethod().value,
-            "angles_file": self.__angle_files,
-            # 'rotation_axis_position'  # if setted should be done upstream because managed by 'tomwer' axis
+            "slice_plane": self.getNabuPlane().value,
+            "angles_file": self.getAnglesFile(),
             "axis_correction_file": "",  # not managed for now
             "angle_offset": self.getAngleOffset(),
             "fbp_filter_type": fbp_filter_type,
@@ -555,6 +680,8 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
             "positivity_constraint": int(self.isPositivityConstraintEnable()),
             "rotation_axis_position": "",
             "translation_movements_file": self.getTranslationMvtFile() or "",
+            "clip_outer_circle": int(self.getClipOuterCircle()),
+            "centered_axis": int(self.getCenteredAxis()),
         }
         config.update(self._subRegionSelector.getConfiguration())
         return config
@@ -570,7 +697,7 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
         if "method" in config:
             self.setMethod(config["method"])
         if "angles_file" in config:
-            self.__angle_files = config["angles_file"]
+            self.setAnglesFile(config["angles_file"])
         if "angle_offset" in config:
             self.setAngleOffset(value=config["angle_offset"])
         if "fbp_filter_type" in config:
@@ -589,6 +716,12 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
             self.setPositivityConstraintEnable(int(config["positivity_constraint"]))
         if "translation_movements_file" in config:
             self.setTranslationMvtFile(config["translation_movements_file"])
+        if "clip_outer_circle" in config:
+            self.setClipOuterCircle(bool(config["clip_outer_circle"]))
+        if "centered_axis" in config:
+            self.setCenteredAxis(bool(config["centered_axis"]))
+        if "slice_plane" in config:
+            self.setNabuPlane(config["slice_plane"])
         self._subRegionSelector.setConfiguration(config=config)
 
     def _signalConfChanged(self, param):
@@ -596,7 +729,6 @@ class _NabuReconstructionConfig(qt.QWidget, _NabuStageConfigBase):
 
 
 class _SubRegionEditor(qt.QObject):
-
     sigConfChanged = qt.Signal(str)
     """Signal emitted each type a parameter is edited"""
 
@@ -679,33 +811,33 @@ class _SubRegionEditor(qt.QObject):
         else:
             return None
 
-    def setSubRegionMin(self, min: typing.Union[None, int]) -> None:
+    def setSubRegionMin(self, sub_region_min: typing.Union[None, int]) -> None:
         """
 
         :param min: if min is None or -1 wr will expand it to - infinity
         """
-        if min in (-1, 0):
-            min = None
-        if min is None:
+        if sub_region_min in (-1, 0):
+            sub_region_min = None
+        if sub_region_min is None:
             self._minCB.setChecked(False)
         else:
             self._minCB.setChecked(True)
-            self._minQLE.setText(str(min))
+            self._minQLE.setText(str(sub_region_min))
 
-    def setSubRegionMax(self, max) -> None:
+    def setSubRegionMax(self, sub_region_max) -> None:
         """
 
         :param max: if max is None or -1 wr will expand it to infinity
         """
-        if type(max) is str:
-            max = int(max)
-        if max == -1:
-            max = None
-        if max is None:
+        if type(sub_region_max) is str:
+            sub_region_max = int(sub_region_max)
+        if sub_region_max == -1:
+            sub_region_max = None
+        if sub_region_max is None:
             self._maxCB.setChecked(False)
         else:
             self._maxCB.setChecked(True)
-            self._maxQLE.setText(str(max))
+            self._maxQLE.setText(str(sub_region_max))
 
     def getSubRegion(self) -> tuple:
         """
@@ -715,14 +847,18 @@ class _SubRegionEditor(qt.QObject):
         """
         return self.getSubRegionMin(), self.getSubRegionMax()
 
-    def setSubRegion(self, min: typing.Union[None, int], max: typing.Union[None, int]):
+    def setSubRegion(
+        self,
+        sub_region_min: typing.Union[None, int],
+        sub_region_max: typing.Union[None, int],
+    ):
         """
 
         :param tuple min:
         :param tuple max:
         """
-        self.setSubRegionMin(min)
-        self.setSubRegionMax(max)
+        self.setSubRegionMin(sub_region_min)
+        self.setSubRegionMax(sub_region_max)
 
     def _signalConfChanged(self, param):
         self.sigConfChanged.emit(param)
@@ -818,7 +954,6 @@ class _NabuReconstructionSubRegion(qt.QGroupBox, _NabuStageConfigBase):
 
 
 class _BinSubSampling(qt.QGroupBox):
-
     binningChanged = qt.Signal()
     """signal emitted when binning change"""
 
@@ -881,11 +1016,3 @@ class _BinSubSampling(qt.QGroupBox):
 
     def _valueUpdated(self, *args, **kwargs):
         self.binningChanged.emit()
-
-
-if __name__ == "__main__":
-    app = qt.QApplication([])
-    widget = _NabuReconstructionConfig(None, None)
-    widget.show()
-    print(widget.getConfiguration())
-    app.exec_()
