@@ -27,28 +27,32 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "04/03/2019"
 
-from silx.gui import qt
-from ...utils.flow import FlowDirection
-from .nabuconfig import NabuConfiguration
+import logging
+
 from nabu.pipeline.config import get_default_nabu_config
 from nabu.pipeline.fullfield.nabu_config import (
     nabu_config as nabu_fullfield_default_config,
 )
-from tomwer.core.process.reconstruction.nabu.utils import _NabuMode, ConfigurationLevel
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.gui.reconstruction.nabu.nabuflow import NabuFlowArea
-from tomoscan.scanbase import _FOV
+from silx.gui import icons as silx_icons
+from silx.gui import qt
 from silx.utils.enum import Enum as _Enum
+from nxtomo.nxobject.nxdetector import FOV
+
+from tomwer.core.process.reconstruction.nabu.utils import _NabuMode
+from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.gui import icons as tomwer_icons
-from .action import (
+from tomwer.gui.reconstruction.nabu.nabuflow import NabuFlowArea
+from tomwer.gui.utils.buttons import PadlockButton
+
+from ...utils.flow import FlowDirection
+from tomwer.gui.configuration.action import (
     BasicConfigurationAction,
     ExpertConfigurationAction,
     FilterAction,
     MinimalisticConfigurationAction,
 )
-from silx.gui import icons as silx_icons
-from tomwer.gui.utils.buttons import PadlockButton
-import logging
+from tomwer.gui.configuration.level import ConfigurationLevel
+from .nabuconfig import NabuConfiguration
 
 _logger = logging.getLogger(__name__)
 
@@ -202,7 +206,6 @@ class NabuDialog(qt.QDialog):
 
 
 class NabuWindow(qt.QMainWindow):
-
     sigConfigChanged = qt.Signal()
     """Signal emitted when the configuration change"""
 
@@ -248,7 +251,7 @@ class NabuWindow(qt.QMainWindow):
 
         # add load option
         self.__loadAction = qt.QAction(self)
-        self.__loadAction.setToolTip("load nabu configuration from a .cfg file")
+        self.__loadAction.setToolTip("load nabu configuration from a text file")
         # load_icon = style.standardIcon(qt.QStyle.SP_)
         load_icon = silx_icons.getQIcon("document-open")
         self.__loadAction.setIcon(load_icon)
@@ -257,7 +260,7 @@ class NabuWindow(qt.QMainWindow):
 
         # add save option
         self.__saveAction = qt.QAction(self)
-        self.__saveAction.setToolTip("save nabu configuration to a .cfg file")
+        self.__saveAction.setToolTip("save nabu configuration to a text file")
         save_icon = silx_icons.getQIcon("document-save")
         self.__saveAction.setIcon(save_icon)
         toolbar.addAction(self.__saveAction)
@@ -285,7 +288,7 @@ class NabuWindow(qt.QMainWindow):
         self.hidePaganinInterface = self._mainWidget.hidePaganinInterface
 
         # connect signal / slot
-        self._mainWidget.sigConfigChanged.connect(self._triggerConfigChanged)
+        self._mainWidget.sigConfigChanged.connect(self.sigConfigChanged)
 
         # set up
         self._resetParameters()
@@ -317,8 +320,8 @@ class NabuWindow(qt.QMainWindow):
             self.setConfigurationLevel(level=configuration_level)
 
     def _saveParameters(self):
-
         from nabu.pipeline.config import generate_nabu_configfile
+
         from tomwer.core.process.reconstruction.nabu.nabuslices import (
             interpret_tomwer_configuration,
         )
@@ -346,10 +349,11 @@ class NabuWindow(qt.QMainWindow):
 
     def _resetParameters(self):
         default_config = get_default_nabu_config(nabu_fullfield_default_config)
+        self._mainWidget._configuration._preProcessingWidget._sinoRingsOpts.resetConfiguration()
         default_config["tomwer_slices"] = "middle"
         default_config["preproc"]["ccd_filter_enabled"] = False
         default_config["preproc"]["double_flatfield_enabled"] = False
-        default_config["preproc"]["flatfield_enabled"] = True
+        default_config["preproc"]["flatfield"] = True
         default_config["preproc"]["take_logarithm"] = True
         self.setConfiguration(default_config)
 
@@ -363,9 +367,17 @@ class NabuWindow(qt.QMainWindow):
             config = parse_nabu_config_file(inputFile)
             self.setConfiguration(config)
 
-    def askForNabuconfigFile(self, acceptMode=qt.QFileDialog.AcceptSave):
+    def askForNabuconfigFile(  # pragma: no cover
+        self, acceptMode=qt.QFileDialog.AcceptSave
+    ):
         dialog = qt.QFileDialog(self)
-        dialog.setNameFilters(["*.cfg"])
+        dialog.setNameFilters(
+            [
+                "Configuration files (*.cfg *.conf *.config)",
+                "Any files (*)",
+            ]
+        )
+
         dialog.setAcceptMode(acceptMode)
         dialog.setFileMode(qt.QFileDialog.AnyFile)
 
@@ -376,8 +388,8 @@ class NabuWindow(qt.QMainWindow):
         filesSelected = dialog.selectedFiles()
         if filesSelected is not None and len(filesSelected) > 0:
             output = filesSelected[0]
-            if not output.endswith(".cfg"):
-                output = ".".join((output, "cfg"))
+            if not output.endswith((".cfg", ".conf", ".config")):
+                output = f"{output}.conf"
             return output
 
     def _userModeChanged(self, *args, **kwargs):
@@ -421,9 +433,6 @@ class NabuWindow(qt.QMainWindow):
     def isModeLocked(self):
         return self._mainWidget.isModeLocked()
 
-    def _triggerConfigChanged(self):
-        self.sigConfigChanged.emit()
-
 
 class NabuWidget(qt.QWidget):
     """
@@ -543,9 +552,9 @@ class NabuWidget(qt.QWidget):
         """Tune the configuration from scan information if possible"""
         assert isinstance(scan, TomwerScanBase)
         if scan.field_of_view is not None and not self.isModeLocked():
-            if scan.field_of_view == _FOV.FULL:
+            if scan.field_of_view == FOV.FULL:
                 self.setMode(_NabuMode.FULL_FIELD)
-            elif scan.field_of_view == _FOV.HALF:
+            elif scan.field_of_view == FOV.HALF:
                 self.setMode(_NabuMode.HALF_ACQ)
         if scan.sa_delta_beta_params is not None:
             db = scan.sa_delta_beta_params.selected_delta_beta_value
