@@ -28,21 +28,29 @@ __license__ = "MIT"
 __date__ = "04/08/2020"
 
 
+import typing
+from asyncio.log import logger
+
 from silx.gui import qt
+
 from tomwer.core.process.reconstruction.nabu.utils import (
-    ConfigurationLevel,
     _NabuStages,
 )
-from tomwer.gui.reconstruction.nabu.action import (
+from tomwer.gui import icons
+from tomwer.gui.configuration.action import (
     BasicConfigurationAction,
     ExpertConfigurationAction,
     MinimalisticConfigurationAction,
 )
+from tomwer.gui.configuration.level import ConfigurationLevel
 from tomwer.gui.reconstruction.nabu.nabuconfig.base import _NabuStageConfigBase
-from tomwer.gui import icons
+from tomwer.gui.reconstruction.nabu.nabuconfig.output import (
+    NabuOutputLocationWidget,
+    QNabuFileFormatComboBox,
+)
 
 
-class _SliceSelector(qt.QWidget):
+class SliceSelector(qt.QWidget):
     """
     Widget to select a value for defining a ROI to reconstruction (start or
     end)
@@ -144,7 +152,7 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
         self._groupBox.setLayout(qt.QGridLayout())
         self.layout().addWidget(self._groupBox, 2, 0, 1, 2)
 
-        self._fromSlice = _SliceSelector(
+        self._fromSlice = SliceSelector(
             label="from slice",
             auto_alias="start",
             is_end=False,
@@ -155,7 +163,7 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
         self._groupBox.layout().addWidget(self._fromSlice)
         self.registerWidget(self._groupBox, "required")
 
-        self._toSlice = _SliceSelector(
+        self._toSlice = SliceSelector(
             label="to slice",
             auto_alias="end",
             is_end=True,
@@ -166,39 +174,63 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
         self._groupBox.layout().addWidget(self._toSlice)
 
         # option to compute histogram
-        self._histogramCB = qt.QCheckBox("compute volume values histogram", self)
+        self._histogramCB = qt.QCheckBox("compute volume histogram of values", self)
         self.layout().addWidget(self._histogramCB, 3, 0, 1, 2)
         self.registerWidget(self._histogramCB, "required")
 
+        # output format can be redefined
+        self._redefineNabuFileFormat = qt.QGroupBox(
+            "change received file format from 'nabu slice'", self
+        )
+        self._redefineNabuFileFormat.setCheckable(True)
+        self._redefineNabuFileFormat.setLayout(qt.QVBoxLayout())
+        self._fileformatCB = QNabuFileFormatComboBox(self._redefineNabuFileFormat)
+        self._redefineNabuFileFormat.layout().addWidget(self._fileformatCB)
+        self.layout().addWidget(self._redefineNabuFileFormat, 4, 0, 1, 2)
+        self.registerWidget(self._fileformatCB, "advanced")
+        self.registerWidget(self._redefineNabuFileFormat, "advanced")
+
+        # redefine output location
+        self._redefineNabuOutputLocation = qt.QGroupBox(
+            "change received reconstruction output location from 'nabu slice'", self
+        )
+        self._redefineNabuOutputLocation.setCheckable(True)
+        self._redefineNabuOutputLocation.setLayout(qt.QVBoxLayout())
+        self._outputLocationWidget = NabuOutputLocationWidget(parent=self)
+        self._redefineNabuOutputLocation.layout().addWidget(self._outputLocationWidget)
+        self.layout().addWidget(self._redefineNabuOutputLocation, 5, 0, 1, 2)
+        self.registerWidget(self._redefineNabuOutputLocation, "advanced")
+        self.registerWidget(self._outputLocationWidget, "advanced")
+
         # gpu percentage
         self._labelGPUFract = qt.QLabel("GPU percentage usage", self)
-        self.layout().addWidget(self._labelGPUFract, 4, 0, 1, 1)
+        self.layout().addWidget(self._labelGPUFract, 50, 0, 1, 1)
         self.registerWidget(self._labelGPUFract, "advanced")
         self._spbGPUFract = qt.QSpinBox(self)
         self._spbGPUFract.setSingleStep(10)
         self._spbGPUFract.setSuffix("%")
         self._spbGPUFract.setMinimum(1)
         self._spbGPUFract.setMaximum(100)
-        self.layout().addWidget(self._spbGPUFract, 4, 1, 1, 1)
+        self.layout().addWidget(self._spbGPUFract, 50, 1, 1, 1)
         self._spbGPUFract.setToolTip("Which fraction of GPU memory to use.")
         self.registerWidget(self._spbGPUFract, "advanced")
 
         # cpu percentage
         self._labelCPUFract = qt.QLabel("CPU percentage usage", self)
-        self.layout().addWidget(self._labelCPUFract, 5, 0, 1, 1)
+        self.layout().addWidget(self._labelCPUFract, 51, 0, 1, 1)
         self.registerWidget(self._labelCPUFract, "advanced")
         self._spbCPUFract = qt.QSpinBox(self)
         self._spbCPUFract.setSingleStep(10)
         self._spbCPUFract.setSuffix("%")
         self._spbCPUFract.setMinimum(1)
         self._spbCPUFract.setMaximum(100)
-        self.layout().addWidget(self._spbCPUFract, 5, 1, 1, 1)
+        self.layout().addWidget(self._spbCPUFract, 52, 1, 1, 1)
         self._spbCPUFract.setToolTip("Which fraction of CPU memory to use.")
         self.registerWidget(self._spbCPUFract, "advanced")
 
         # opt use phase paganin
         self._usePhaseQCB = qt.QCheckBox("Use phase paganin")
-        self.layout().addWidget(self._usePhaseQCB, 6, 0, 1, 2)
+        self.layout().addWidget(self._usePhaseQCB, 53, 0, 1, 2)
         self.registerWidget(self._usePhaseQCB, "advanced")
         self._usePhaseQCB.setToolTip(
             "Whether to use a margin when " "performing phase retrieval."
@@ -210,9 +242,12 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
         self.layout().addWidget(spacer, 99, 0, 1, 1)
 
         # set up
+        self._histogramCB.setChecked(True)
         self._spbGPUFract.setValue(90)
         self._spbCPUFract.setValue(90)
         self._usePhaseQCB.setChecked(True)
+        self._redefineNabuFileFormat.setChecked(False)
+        self._redefineNabuOutputLocation.setChecked(False)
         self.setConfigurationLevel("required")
 
         # connect signal / slot
@@ -221,6 +256,52 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
         self._spbGPUFract.valueChanged.connect(self._triggerSigConfChanged)
         self._spbCPUFract.valueChanged.connect(self._triggerSigConfChanged)
         self._usePhaseQCB.toggled.connect(self._triggerSigConfChanged)
+        self._outputLocationWidget._outputDirQLE.editingFinished.connect(
+            self._triggerSigConfChanged
+        )
+        self._outputLocationWidget.sigOutputChanged.connect(self._triggerSigConfChanged)
+        self._redefineNabuOutputLocation.toggled.connect(self._triggerSigConfChanged)
+
+    def redefineOutputLocation(self):
+        return self._redefineNabuOutputLocation.isChecked()
+
+    def setRedefineOutputLocation(self, redefine: bool):
+        self._redefineNabuOutputLocation.setChecked(redefine)
+
+    def getNabuOutputLocation(self):
+        if self.redefineOutputLocation():
+            return self._outputLocationWidget.getOutputDir()
+        else:
+            return None
+
+    def setNabuOutputLocation(self, location: typing.Optional[str]):
+        if location is None:
+            self._redefineNabuOutputLocation.setChecked(False)
+        else:
+            self._redefineNabuOutputLocation.setChecked(True)
+            self._outputLocationWidget.setOutputDirMode("other")
+            self._outputLocationWidget.setOutputDir(location)
+
+    def redefineNabuFileFormat(self):
+        return self._redefineNabuFileFormat.isChecked()
+
+    def getNabuFileFormat(self):
+        if self.redefineNabuFileFormat():
+            return self._fileformatCB.currentText()
+        else:
+            return None
+
+    def setNabuFileFormat(self, file_format: typing.Optional[str]):
+        if file_format is None:
+            self._redefineNabuFileFormat.setChecked(False)
+        else:
+            assert isinstance(file_format, str)
+            self._redefineNabuFileFormat.setChecked(True)
+            idx = self._fileformatCB.findText(file_format)
+            if idx < 0:
+                logger.error(f"unable to find file format {file_format}")
+            else:
+                self._fileformatCB.setCurrentIndex(idx)
 
     def _triggerSigConfChanged(self, *args, **kwargs):
         self.sigConfigChanged.emit()
@@ -229,13 +310,13 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
         return self._spbCPUFract.value() / 100.0
 
     def setCPUFract(self, value):
-        return self._spbCPUFract.setValue(value * 100.0)
+        return self._spbCPUFract.setValue(int(value * 100.0))
 
     def getGPUFract(self):
         return self._spbGPUFract.value() / 100.0
 
     def setGPUFract(self, value):
-        return self._spbGPUFract.setValue(value * 100.0)
+        return self._spbGPUFract.setValue(int(value * 100.0))
 
     def getStartZ(self):
         return self._fromSlice.value()
@@ -277,6 +358,10 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
             "postproc": {
                 "output_histogram": int(self.isHistogramRequested()),
             },
+            "new_output_file_format": self.getNabuFileFormat() or "",
+            "new_output_location": self.getNabuOutputLocation() or "",
+            "output_dir_mode": self._outputLocationWidget.getOutputDirMode().value,
+            "overwrite_output_location": self.redefineOutputLocation(),
         }
 
     def setConfiguration(self, config):
@@ -292,6 +377,22 @@ class NabuVolumeWidget(_NabuStageConfigBase, qt.QWidget):
             self.setPhasePaganinRequested(config["use_phase_margin"])
         if "postproc" in config:
             self.setPostProcConfiguration(config["postproc"])
+        if "new_output_file_format" in config:
+            new_output_file_format = config["new_output_file_format"]
+            if new_output_file_format == "":
+                new_output_file_format = None
+            self.setNabuFileFormat(new_output_file_format)
+        if "new_output_location" in config:
+            new_output_location = config["new_output_location"]
+            if new_output_location == "":
+                new_output_location = None
+            self.setNabuOutputLocation(new_output_location)
+        output_dir_mode = config.get("output_dir_mode", None)
+        if output_dir_mode is not None:
+            self._outputLocationWidget.setOutputDirMode(output_dir_mode)
+        overwrite_output_location = config.get("overwrite_output_location", None)
+        if overwrite_output_location is not None:
+            self.setRedefineOutputLocation(overwrite_output_location)
 
     def setPostProcConfiguration(self, config):
         if "output_histogram" in config:
@@ -340,6 +441,7 @@ class NabuVolumeWindow(qt.QMainWindow):
 
         self._minimalisticAction = MinimalisticConfigurationAction(toolbar)
         menu.addAction(self._minimalisticAction)
+        self.__configurationModesGroup.addAction(self._minimalisticAction)
         self._basicConfigAction = BasicConfigurationAction(toolbar)
         menu.addAction(self._basicConfigAction)
         self.__configurationModesGroup.addAction(self._basicConfigAction)
@@ -347,16 +449,9 @@ class NabuVolumeWindow(qt.QMainWindow):
         menu.addAction(self._expertConfiguration)
         self.__configurationModesGroup.addAction(self._expertConfiguration)
 
-        # expose API
-        self.getConfiguration = self._mainWidget.getConfiguration
-        self.setConfiguration = self._mainWidget.setConfiguration
-        self.setConfigurationLevel = self._mainWidget.setConfigurationLevel
-        self.getConfigurationLevel = self._mainWidget.getConfigurationLevel
-        self.setScan = self._mainWidget.setScan
-
         # set up
-        self._basicConfigAction.setChecked(True)
         self._userModeChanged(action=self._basicConfigAction)
+        self._basicConfigAction.setChecked(True)
 
         # connect signal / slot
         self._mainWidget.sigConfigChanged.connect(self._triggerSigConfigChanged)
@@ -372,8 +467,21 @@ class NabuVolumeWindow(qt.QMainWindow):
         elif action is self._expertConfiguration:
             level = ConfigurationLevel.ADVANCED
         else:
-            raise ValueError("unknow configuration level")
+            level = ConfigurationLevel.REQUIRED
         self._mainWidget.setConfigurationLevel(level)
 
+    # expose API
     def getConfigurationLevel(self):
         self._mainWidget.getConfigurationLevel()
+
+    def setConfigurationLevel(self, level):
+        self._mainWidget.setConfigurationLevel(level=level)
+
+    def getConfiguration(self):
+        return self._mainWidget.getConfiguration()
+
+    def setConfiguration(self, config):
+        return self._mainWidget.setConfiguration(config=config)
+
+    def setScan(self, scan):
+        return self._mainWidget.setScan(scan=scan)
