@@ -34,24 +34,26 @@ __license__ = "MIT"
 __date__ = "01/02/2021"
 
 
-from silx.gui import qt
-from tomwer.gui import icons
-from tomwer.gui.reconstruction.nabu.slices import NabuWindow
-from tomwer.gui.reconstruction.axis.radioaxis import _CalculationWidget
-from tomwer.synctools.axis import QAxisRP
-from tomwer.core.process.reconstruction.nabu.utils import _NabuMode
-from tomwer.core.process.reconstruction.saaxis.params import SAAxisParams
-from tomwer.gui.reconstruction.scores.scoreplot import ScorePlot as _ScorePlot
-from tomwer.gui.reconstruction.scores.scoreplot import CorSelection
-from tomwer.gui.reconstruction.saaxis.corrangeselector import SliceAndCorWidget
-from tomwer.gui.utils.buttons import TabBrowsersButtons
-from tomwer.gui.reconstruction.scores.control import ControlWidget
-from tomwer.core.process.reconstruction.saaxis.saaxis import SAAxisProcess
-from tomwer.core.process.reconstruction.axis import mode as axis_mode
-import numpy
-from typing import Union
 import logging
+from typing import Union
 
+import numpy
+from silx.gui import qt
+
+from tomwer.core.process.reconstruction.axis import mode as axis_mode
+from tomwer.core.process.reconstruction.nabu.utils import _NabuMode, slice_index_to_int
+from tomwer.core.process.reconstruction.saaxis.params import SAAxisParams
+from tomwer.core.process.reconstruction.saaxis.saaxis import SAAxisTask
+from tomwer.gui import icons
+from tomwer.gui.reconstruction.axis.radioaxis import _CalculationWidget
+from tomwer.gui.reconstruction.nabu.slices import NabuWindow
+from tomwer.gui.reconstruction.saaxis.corrangeselector import SliceAndCorWidget
+from tomwer.gui.reconstruction.scores.control import ControlWidget
+from tomwer.gui.reconstruction.scores.scoreplot import CorSelection
+from tomwer.gui.reconstruction.scores.scoreplot import ScorePlot as _ScorePlot
+from tomwer.gui.utils.buttons import TabBrowsersButtons
+from tomwer.gui.utils.scandescription import ScanNameLabelAndShape
+from tomwer.synctools.axis import QAxisRP
 
 _logger = logging.getLogger(__file__)
 
@@ -64,14 +66,14 @@ class ScorePlot(_ScorePlot, constructor=CorSelection):
 
     def _updateScores(self):
         scan = self.__scan() if self.__scan else None
+        img_width = None
         if scan is not None:
             if scan.saaxis_params:
                 scan.saaxis_params.score_method = self.getScoreMethod()
                 img_width = scan.dim_1
                 # update autofocus
-                SAAxisProcess.autofocus(scan)
-        else:
-            img_width = None
+                SAAxisTask.autofocus(scan)
+
         self.setVarScores(
             scores=self._scores,
             score_method=self.getScoreMethod(),
@@ -129,22 +131,18 @@ class NabuAutoCorDiag(qt.QDialog):
         self.layout().addWidget(self._buttons)
 
         self._buttons.button(qt.QDialogButtonBox.Ok).clicked.connect(
-            self._requestAutomaticCor
+            self.sigRequestAutoCor
         )
         self._buttons.button(qt.QDialogButtonBox.Ok).setText("compute")
 
-    def _requestAutomaticCor(self, *args, **kwargs):
-        self.sigRequestAutoCor.emit()
-
 
 class _SAAxisTabWidget(qt.QTabWidget):
-
     sigConfigurationChanged = qt.Signal()
     """signal emitted each time the 'input' configuration changed.
     like slice to reconstruct, number of reconstruction, research width,
     nabu reconstruction parameters..."""
 
-    def __init__(self, parent):
+    def __init__(self, parent, backend=None):
         qt.QTabWidget.__init__(self, parent)
         # select slice & cor range
 
@@ -158,7 +156,7 @@ class _SAAxisTabWidget(qt.QTabWidget):
         nabu_icon = icons.getQIcon("nabu")
         self.addTab(self._nabuSettings, nabu_icon, "reconstruction settings")
         # results
-        self._resultsViewer = ScorePlot(self, variable_name="cor")
+        self._resultsViewer = ScorePlot(self, variable_name="cor", backend=backend)
         results_icon = icons.getQIcon("results")
         self.addTab(self._resultsViewer, results_icon, "reconstructed slices")
 
@@ -183,7 +181,6 @@ class _SAAxisTabWidget(qt.QTabWidget):
         self.setVolumeSize = self._resultsViewer.setVolumeSize
         self.setCurrentCorValue = self._resultsViewer.setCurrentVarValue
         self.getCurrentCorValue = self._resultsViewer.getCurrentVarValue
-        self.getScoreMethod = self._resultsViewer.getScoreMethod
         self.getEstimatedCorPosition = self._sliceAndCorWidget.getEstimatedCorPosition
         self.setEstimatedCorPosition = self._sliceAndCorWidget.setEstimatedCorPosition
         self.getNReconstruction = self._sliceAndCorWidget.getNReconstruction
@@ -201,6 +198,7 @@ class _SAAxisTabWidget(qt.QTabWidget):
         self.getSlicesRange = self._sliceAndCorWidget.getSlicesRange
         self.setSlicesRange = self._sliceAndCorWidget.setSlicesRange
         self.loadSinogram = self._sliceAndCorWidget.loadSinogram
+        self.saveReconstructedSlicesTo = self._resultsViewer.saveReconstructedSlicesTo
         # expose signals
         self.sigStartSinogramLoad = self._sliceAndCorWidget.sigStartSinogramLoad
         self.sigEndSinogramLoad = self._sliceAndCorWidget.sigEndSinogramLoad
@@ -252,11 +250,15 @@ class _SAAxisTabWidget(qt.QTabWidget):
         return {
             "research_width": self.getResearchWidth(),
             "n_reconstruction": self.getNReconstruction(),
-            "slice_indexes": self.getReconstructionSlices(),
+            "slice_index": self.getReconstructionSlices(),
             "nabu_params": nabu_config,
             "mode": self.getReconstructionMode().value,
             "score_method": self.getScoreMethod().value,
             "estimated_cor": self.getEstimatedCorPosition(),
+            "output_dir": self.getNabuReconsParams()
+            .get("output", {})
+            .get("location", None)
+            or None,
         }
 
     def setConfiguration(self, config):
@@ -264,8 +266,7 @@ class _SAAxisTabWidget(qt.QTabWidget):
             config = config.to_dict()
         if not isinstance(config, dict):
             raise TypeError(
-                "config should be a dictionary or a SAAxisParams. "
-                "Not {}".format(type(config))
+                f"config should be a dictionary or a SAAxisParams. Not {type(config)}"
             )
 
         research_width = config.get("research_width", None)
@@ -277,7 +278,7 @@ class _SAAxisTabWidget(qt.QTabWidget):
         estimated_cor = config.get("estimated_cor", None)
         if estimated_cor is not None:
             self.setEstimatedCorPosition(estimated_cor)
-        slice_indexes = config.get("slice_indexes", None)
+        slice_indexes = config.get("slice_index", None)
         if slice_indexes is not None:
             self.setReconstructionSlices(slice_indexes)
         if "nabu_params" in config:
@@ -302,25 +303,27 @@ class _SAAxisTabWidget(qt.QTabWidget):
         self._nabuSettings = None
         super().close()
 
-    def _stopAnimationThread(self):
-        self._resultsViewer._stopAnimationThread()
-        self._sliceAndCorWidget._sinogramViewer._stopAnimationThread()
-
 
 class SAAxisWindow(qt.QMainWindow):
     """
     Widget used to determine half-automatically the center of rotation
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, backend=None):
         qt.QMainWindow.__init__(self, parent)
         self._scan = None
         self._automaticCorWidget = None
         self._qaxis_rp = QAxisRP()
         self.setWindowFlags(qt.Qt.Widget)
         # central widget
-        self._tabWidget = _SAAxisTabWidget(self)
-        self.setCentralWidget(self._tabWidget)
+        self._mainWidget = qt.QWidget(self)
+        self._mainWidget.setLayout(qt.QVBoxLayout())
+
+        self._scanInfo = ScanNameLabelAndShape(self)
+        self._mainWidget.layout().addWidget(self._scanInfo)
+        self._tabWidget = _SAAxisTabWidget(self, backend=backend)
+        self._mainWidget.layout().addWidget(self._tabWidget)
+        self.setCentralWidget(self._mainWidget)
         # next and previous buttons for browsing the tab widget
         self._browserButtons = TabBrowsersButtons(self)
         self._dockWidgetBrwButtons = qt.QDockWidget(self)
@@ -356,7 +359,7 @@ class SAAxisWindow(qt.QMainWindow):
         self.getCors = self._tabWidget.getCors
         self.getMode = self._tabWidget.getReconstructionMode
         self.loadSinogram = self._tabWidget.loadSinogram
-        self.isAutoFocusLock = self._tabWidget.isAutoFocusLock
+        self.saveReconstructedSlicesTo = self._tabWidget.saveReconstructedSlicesTo
         # expose signals
         self.sigValidated = self._saaxisControl.sigValidateRequest
         self.sigStartSinogramLoad = self._tabWidget.sigStartSinogramLoad
@@ -374,9 +377,6 @@ class SAAxisWindow(qt.QMainWindow):
         self._saaxisControl.sigComputationRequest.connect(self._launchReconstructions)
         self._saaxisControl.sigValidateRequest.connect(self._validate)
 
-    def stop(self):
-        self._stopAnimationThread()
-
     def showResults(self):
         self._tabWidget.showResults()
 
@@ -409,6 +409,7 @@ class SAAxisWindow(qt.QMainWindow):
     def setScan(self, scan):
         self._scan = scan
         self._tabWidget.setScan(scan)
+        self._scanInfo.setScan(scan)
         self._updateSinogramLine()
         self._loadEstimatedCorFromScan(scan)
 
@@ -443,17 +444,11 @@ class SAAxisWindow(qt.QMainWindow):
     def _updateSinogramLine(self):
         r_slice = self.getReconstructionSlices()
         if r_slice == "middle":
-            if self._scan is not None and self._scan.dim_2 is not None:
-                line = self._scan.dim_2 // 2
-            else:
-                line = 1024
+            line = slice_index_to_int(slice_index="middle", scan=self._scan, axis="XY")
         else:
             line = list(r_slice.values())[0]
         self._tabWidget.getSinogramViewer().setLine(line)
 
-    def setEstimatedCorPosition(self, value):
-        self.setEstimatedCorPosition(value=value)
-
     def _autoCorRequested(self):
         window = self.getAutomaticCorWindow()
         window.activateWindow()
@@ -487,8 +482,8 @@ class SAAxisWindow(qt.QMainWindow):
         else:
             other_cors = ()
         sino_viewer.setCorRange(
-            min=cors[0],
-            max=cors[-1],
+            cor_min=cors[0],
+            cor_max=cors[-1],
             estimated_cor=estimated_cor,
             other_cors=other_cors,
         )
@@ -505,9 +500,6 @@ class SAAxisWindow(qt.QMainWindow):
         if idx >= 0:
             self._tabWidget.setCurrentIndex(idx)
 
-    def _stopAnimationThread(self):
-        self._tabWidget._stopAnimationThread()
-
     def close(self):
         self._tabWidget.close()
         self._tabWidget = None
