@@ -35,32 +35,37 @@ __license__ = "MIT"
 __date__ = "25/02/2021"
 
 
+import logging
+import os
+import weakref
+from contextlib import AbstractContextManager
+from typing import Union
+
+import numpy
+from matplotlib import image as _matplotlib_image
 from silx.gui import qt
 from silx.gui.plot import PlotWidget
-from tomwer.gui.reconstruction.saaxis.dimensionwidget import DimensionWidget
-from tomwer.gui.visualization.dataviewer import ImageStack
-from tomwer.gui.utils.vignettes import VignettesQDialog
-from contextlib import AbstractContextManager
-from tomwer.gui.utils.buttons import PadlockButton
-from tomwer.gui import icons
-from tomwer.io.utils.h5pyutils import DatasetReader
+
 from tomwer.core.process.reconstruction.scores.params import ScoreMethod
-from typing import Union
-import weakref
-import numpy
-import logging
+from tomwer.gui import icons, settings
+from tomwer.gui.reconstruction.saaxis.dimensionwidget import DimensionWidget
+from tomwer.gui.utils.buttons import PadlockButton
+from tomwer.gui.utils.vignettes import VignettesQDialog
+from tomwer.gui.visualization.dataviewer import ImageStack
+from tomwer.io.utils import get_default_directory
+from tomwer.io.utils.h5pyutils import DatasetReader
+from tomwer.io.utils.utils import get_slice_data
 
 _logger = logging.getLogger(__name__)
 
 
 class VariableScorePlot(PlotWidget):
-
     sigMouseWheelActive = qt.Signal(object)
 
     MARKER_COLOR = "#ff292199"
 
-    def __init__(self, parent):
-        PlotWidget.__init__(self, parent)
+    def __init__(self, parent, backend=None):
+        PlotWidget.__init__(self, parent, backend=backend)
         self._scores = {}
         self.setAxesDisplayed(False)
         self.setMaximumHeight(150)
@@ -132,6 +137,11 @@ class VariableSelection(qt.QWidget):
         self._scoreMethodCB = qt.QComboBox(self)
         self.layout().addWidget(self._scoreMethodCB, 0, 1, 1, 1)
         for method in ScoreMethod:
+            if method is ScoreMethod.TOMO_CONSISTENCY:
+                # for now we avoid this method. Does not provide great results and is very time consuming
+                # I guess when we will add it we will need to first display slices
+                # and then only compute the score. It might be an "on demand" score but not computed by default.
+                continue
             self._scoreMethodCB.addItem(method.value)
         # default score method is 1 / tv
         idx = self._scoreMethodCB.findText(ScoreMethod.TV_INVERSE.value)
@@ -143,7 +153,7 @@ class VariableSelection(qt.QWidget):
         # TODO: this should be inherite by a cor plot (to display variable and absloute value)
         # cor value
         self._currentVarValue = qt.QGroupBox(self)
-        self._currentVarValue.setTitle("current {} value".format(self._variable_name))
+        self._currentVarValue.setTitle(f"current {self._variable_name} value")
         self._currentVarValue.setLayout(qt.QFormLayout())
         self.layout().addWidget(self._currentVarValue, 0, 3, 3, 2)
 
@@ -152,7 +162,7 @@ class VariableSelection(qt.QWidget):
         self.layout().addWidget(self._currentcorRB, 0, 5, 1, 1)
         self._autofocusRB = qt.QRadioButton("autofocus", self)
         self._autofocusRB.setToolTip(
-            "Take the {} with the best score".format(self._variable_name)
+            f"Take the {self._variable_name} with the best score"
         )
         self.layout().addWidget(self._autofocusRB, 1, 5, 1, 1)
         # lock autofocus button
@@ -205,15 +215,15 @@ class VariableSelection(qt.QWidget):
         colormap = None
         if self.parent() and hasattr(self.parent(), "getPlotWidget"):
             master_plot = self.parent().getPlotWidget()
-            colormap = master_plot.getColorBarWidget().getColormap()
+            colormap = master_plot.getPlotWidget().getColorBarWidget().getColormap()
         dialog = VignettesQDialog(
-            value_name="{} position".format(self._variable_name),
+            value_name=f"{self._variable_name} position",
             score_name="score",
             value_format="{:.3f}",
             score_format="{:.3f}",
             colormap=colormap,
         )
-        dialog.setWindowTitle("{} - vignettes".format(self.getWindowTitle()))
+        dialog.setWindowTitle(f"{self.getWindowTitle()} - vignettes")
 
         # set scores
         dialog.setScores(self._scores, score_method=self.getScoreMethod())
@@ -281,13 +291,14 @@ class SingleValueSelection(VariableSelection):
         self._currentVarValue.layout().addRow("value:", self._valueLE)
 
     def setValue(self, value):
-        self._valueLE.setText("{:.2f}".format(value))
+        self._valueLE.setText(f"{value:.2f}")
 
     def getWindowTitle(self):
         raise NotImplementedError("Base class")
 
 
 class DelaBetaSelection(SingleValueSelection):
+    # FIXME: fix typo Dela vs Delta
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         idx = self._scoreMethodCB.findText(ScoreMethod.TOMO_CONSISTENCY.value)
@@ -311,7 +322,7 @@ class CorSelection(VariableSelection):
             self._absoluteVarValueLE,
         )
         try:
-            import pyopencl  # noqa F401
+            import pyopencl  # noqa F401 pylint: disable=E0401
         except Exception:
             idx = self._scoreMethodCB.findText(ScoreMethod.TOMO_CONSISTENCY.value)
             if idx >= 0:
@@ -321,12 +332,12 @@ class CorSelection(VariableSelection):
         if relative_value is None:
             self._relativeVarValueLE.clear()
         else:
-            self._relativeVarValueLE.setText("{:.3f}".format(relative_value))
+            self._relativeVarValueLE.setText(f"{relative_value:.3f}")
         if relative_value is None or self._img_width is None:
             self._absoluteVarValueLE.clear()
         else:
             absolute_value = relative_value + self._img_width / 2.0
-            self._absoluteVarValueLE.setText("{:.3f}".format(absolute_value))
+            self._absoluteVarValueLE.setText(f"{absolute_value:.3f}")
 
     def getWindowTitle(self):
         return "sa axis - vignettes"
@@ -435,7 +446,7 @@ class _VariableValueLabels(qt.QWidget):
         for i_var in var_indexes:
             var_value = self._var_values[i_var]
             var_px_pos = var_px_positions[i_var]
-            txt_value = "{:.3f}".format(var_value)
+            txt_value = f"{var_value:.3f}"
             # apply rotation to "invert" the painter rotation requested to
             # paint oblique text
             with PainterRotationCM(
@@ -483,6 +494,7 @@ class ScorePlot(qt.QWidget):
         parent=None,
         variable_name=None,
         dims_colors=("#ffff5a", "#62efff", "#ff5bff"),
+        backend=None,
     ):
         if not isinstance(variable_name, str):
             raise TypeError("a variable name should be provided as a string")
@@ -495,11 +507,21 @@ class ScorePlot(qt.QWidget):
         # define GUI
         self.setLayout(qt.QGridLayout())
         # main plot
-        self._plot = ImageStack(self, show_overview=False)
+        self._plot = ImageStack(self, show_overview=False, backend=backend)
+        self._plot.getPlotWidget().setYAxisInverted(settings.Y_AXIS_DOWNWARD)
+
         self._plot.getPlotWidget().setKeepDataAspectRatio(True)
         self._plot._sliderDockWidget.hide()
         self._plot.getPlotWidget().getColorBarWidget().hide()
         self._plot.getPlotWidget().getPositionInfoWidget().hide()
+        # add save all action
+        save_all_icon = icons.getQIcon("multi-document-save")
+        self._saveAllSnapshot = qt.QAction(
+            save_all_icon, "save all screenshot as png", self
+        )
+        toolbar = self._plot.getPlotWidget().toolBar()
+        toolbar.addSeparator()
+        toolbar.addAction(self._saveAllSnapshot)
         # hide dock widget
         self._plot._reconsInfoDockWidget.hide()
         self._plot._tableDockWidget.hide()
@@ -554,7 +576,7 @@ class ScorePlot(qt.QWidget):
         self._varSlider.valueChanged.connect(self._plot.setCurrentUrlIndex)
         self._varSlider.valueChanged.connect(self._sliderReleased)
         self._varSlider.valueChanged.connect(self._updateVarValue)
-        self._varScore.sigMouseWheelActive.connect(self._propagateSPWheelEvent)
+        self._varScore.sigMouseWheelActive.connect(self._varSlider.wheelEvent)
         self._varValueWidget.sigAutoFocusLocked.connect(self._autoFocusLockChanged)
         self._varValueWidget.sigAutoFocusUnLocked.connect(self._autoFocusLockChanged)
         self._varValueWidget.sigSelectionModeChanged.connect(
@@ -564,15 +586,21 @@ class ScorePlot(qt.QWidget):
             self._varSelectedFromVignettes
         )
         self._varValueWidget.sigScoreMethodChanged.connect(self._updateScores)
+        self._saveAllSnapshot.triggered.connect(self._saveAllReconstructedSlices)
 
         # set up
         # for now we don't want to use the volume and voxel size
         self._volumeSizeW.hide()
         self._voxelSizeW.hide()
         self._axisLabel.hide()
+        # use mean mean+/-3std for the cor
+        plotWidget = self._plot.getPlotWidget()
+        colormap = plotWidget.getDefaultColormap()
+        colormap.setAutoscaleMode(colormap.STDDEV3)
+        plotWidget.setDefaultColormap(colormap=colormap)
 
-    def __init_subclass__(cls, constructor, **kwargs):
-        super().__init_subclass__(**kwargs)
+    def __init_subclass__(cls, constructor, **kwargs):  # pylint: disable=E0302
+        super().__init_subclass__()
         cls._CorSlectionBuilder = constructor
 
     def getScoreMethod(self):
@@ -585,12 +613,7 @@ class ScorePlot(qt.QWidget):
         self.sigConfigurationChanged.emit()
 
     def getPlotWidget(self):
-        return self._plot.getPlotWidget()
-
-    def _stopAnimationThread(self):
-        self._plot._freeLoadingThreads()
-        if self._plot._plot.updateThread.is_alive():
-            self._plot._plot.updateThread.stop()
+        return self._plot
 
     def close(self):
         self._plot.close()
@@ -639,9 +662,6 @@ class ScorePlot(qt.QWidget):
     def _varSelectedFromVignettes(self, value):
         self._varSlider.setVarValue(value)
 
-    def _propagateSPWheelEvent(self, event):
-        self._varSlider.wheelEvent(event)
-
     def _get_closest_var(self, value):
         """return the closest cor value to value or None if no cor value
         defined"""
@@ -690,6 +710,8 @@ class ScorePlot(qt.QWidget):
             return
         for cor_value, cor_info in scores.items():
             url, score = cor_info
+            if score is None:
+                continue  # in case score calculation failed
             self._var_values.append(cor_value)
             score_list.append(score.get(score_method))
             self._urls.append(url)
@@ -710,7 +732,8 @@ class ScorePlot(qt.QWidget):
             self._plot.setUrls(urls=self._urls)
             self._varSlider.setVarValues(self._var_values)
             old = self._varSlider.blockSignals(True)
-            self._varSlider.setVarValue(self._var_values[0])
+            if len(self._var_values) > 0:
+                self._varSlider.setVarValue(self._var_values[0])
             self._varSlider.blockSignals(old)
 
         old = self._varValueWidget.blockSignals(True)
@@ -789,6 +812,47 @@ class ScorePlot(qt.QWidget):
             )
         )
 
+    def _saveAllReconstructedSlices(self):  # pragma: no cover
+        """
+        save all reconstructed slices to a folder provided by the user
+        """
+        dialog = qt.QFileDialog(self, directory=get_default_directory())
+        dialog.setFileMode(qt.QFileDialog.DirectoryOnly)
+        if not dialog.exec_():
+            dialog.close()
+            return
+
+        output_folder = dialog.selectedFiles()[0]
+        try:
+            self.saveReconstructedSlicesTo(output_folder=output_folder)
+        except Exception as e:
+            _logger.error(f"Fail to save snapshots. Error is {e}")
+        else:
+            _logger.info(f"snapshots have been saved to {output_folder}")
+
+    def saveReconstructedSlicesTo(self, output_folder):
+        """dump all slices with score as name"""
+        # make sure the output dir exists
+        os.makedirs(output_folder, exist_ok=True)
+
+        i_missing_score = 0
+        for score, slice_url in zip(self._var_values, self._urls):
+            try:
+                data = get_slice_data(slice_url)
+                if score is None:
+                    file_name = f"no_score_{i_missing_score}.png"
+                    i_missing_score += 1
+                else:
+                    file_name = f"{score}.png"
+                _matplotlib_image.imsave(
+                    os.path.join(output_folder, file_name),
+                    data,
+                    format="png",
+                )
+
+            except Exception as e:
+                _logger.error(f"Failed to dump {slice_url.path()}. Raison is {e}")
+
 
 class _VarSlider(qt.QWidget):
     def __init__(self, parent, orientation):
@@ -818,59 +882,7 @@ class _VarSlider(qt.QWidget):
                 value = where[0]
                 if isinstance(value, numpy.ndarray):
                     value = value[0]
-                self._slider.setValue(value)
+                self._slider.setValue(int(value))
 
     def value(self):
         return self._slider.value()
-
-
-if __name__ == "__main__":
-    from silx.image.phantomgenerator import PhantomGenerator
-    from silx.io.url import DataUrl
-    import tempfile
-    import os
-    import h5py
-    import shutil
-    from scipy import stats
-
-    app = qt.QApplication([])
-    img_width = 216
-    data = PhantomGenerator.get2DPhantomSheppLogan(n=img_width)
-    nb_cor = 50
-    urls = []
-    slices_folder = tempfile.mkdtemp()
-    for i in range(nb_cor):
-        noise = (
-            numpy.random.random(img_width * img_width).reshape(img_width, img_width)
-            * 100.0
-        )
-        file_path = os.path.join(slices_folder, "slice_{}.hdf5".format(i))
-        with h5py.File(file_path, mode="a") as h5f:
-            h5f["data"] = noise + data
-        urls.append(DataUrl(file_path=file_path, data_path="data", scheme="silx"))
-    variance = 1
-    mu = 0.5
-    sigma = numpy.sqrt(variance)
-    x = numpy.linspace(mu - 3 * sigma, mu + 3 * sigma, nb_cor)
-    scores = stats.norm.pdf(x, mu, sigma)
-
-    cor_values = numpy.linspace(80, 120, nb_cor)
-    scores_dict = {}
-    for cor, url, score in zip(cor_values, urls, scores):
-        scores_dict[cor] = (url, score)
-
-    window1 = ScorePlot(variable_name="var x")
-    window1.setVarScores(scores=scores_dict, score_method="std")
-    window1.show()
-    from tomwer.core.scan.scanbase import TomwerScanBase
-    from tomwer.core.process.reconstruction.saaxis.params import SAAxisParams
-    from tomwer.core.process.reconstruction.saaxis.saaxis import SAAxisProcess
-
-    scan = TomwerScanBase()
-    scan.saaxis_params = SAAxisParams()
-    scan.saaxis_params.scores = scores_dict
-    SAAxisProcess.autofocus(scan)
-    window1.setScan(scan)
-    app.exec_()
-
-    shutil.rmtree(slices_folder)
