@@ -27,21 +27,24 @@ __license__ = "MIT"
 __date__ = "16/06/2021"
 
 
-from tomwer.gui.reconstruction.axis import AxisWindow
-from tomwer.test.utils import skip_gui_test
-from tomwer.test.utils import UtilsTest
-from tomwer.core.process.reconstruction.axis.mode import AxisMode
-from tomwer.core.scan.edfscan import EDFTomoScan
-from tomwer.synctools.axis import QAxisRP
-from silx.gui.utils.testutils import TestCaseQt
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from silx.gui import qt
-import numpy.random
-import pytest
-import tempfile
 import os
 import shutil
+import tempfile
+
 import h5py
+import numpy.random
+import pytest
+from silx.gui import qt
+from silx.gui.utils.testutils import TestCaseQt
+
+from tomwer.core.process.reconstruction.axis.mode import AxisMode
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.gui.reconstruction.axis import AxisWindow
+from tomwer.gui.reconstruction.axis.radioaxis import _ManualFramesSelection
+from tomwer.synctools.axis import QAxisRP
+from tomwer.tests.datasets import TomwerCIDatasets
+from tomwer.tests.utils import skip_gui_test
+from tomwer.core.utils.scanutils import MockNXtomo
 
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
@@ -49,33 +52,55 @@ class TestWindowAxis(TestCaseQt):
     """Test that the axis widget work correctly"""
 
     def setUp(self):
+        self.tempdir = tempfile.mkdtemp()
         self.recons_params = QAxisRP()
         self._window = AxisWindow(axis_params=self.recons_params)
-        self.scan_path = UtilsTest.getEDFDataset("D2_H2_T2_h_")
-        self.scan = EDFTomoScan(self.scan_path)
+        self.scan = MockNXtomo(
+            scan_path=os.path.join(self.tempdir, "nx_tomo.nx"),
+            n_proj=10,
+            n_ini_proj=10,
+            scan_range=180,
+            dim=20,
+            energy=12.3,
+        ).scan
         self._window.show()
         self.qWaitForWindowExposed(self._window)
 
     def tearDown(self):
+        shutil.rmtree(self.tempdir)
         self._window.setAttribute(qt.Qt.WA_DeleteOnClose)
         self._window.close()
 
     def testSetImagesNumpyArray(self):
         """Test the setImages function"""
         radio_axis = self._window._axisWidget._radioAxis
-        self.assertEqual(radio_axis.getPlot().getPlot().getActiveImage(), None)
+        radio_axis_plot = self._window._axisWidget._radioAxis.getPlot()
+        numpy.testing.assert_array_equal(
+            radio_axis_plot.getPlot().getActiveImage().getData(), numpy.empty((0, 0, 4))
+        )
         imgA = numpy.random.random((100, 100))
         imgB = numpy.random.random((100, 100))
         radio_axis.setImages(imgA=imgA, imgB=imgB, flipB=True)
         self.qapp.processEvents()
-        self.assertNotEqual(radio_axis.getPlot().getPlot().getActiveImage(), None)
+        # make sur the image has been set
+        assert radio_axis_plot.getPlot().getImage(legend="image1").getData().shape == (
+            100,
+            100,
+        )
 
     def testSetScan(self):
         """Test the setScan function"""
         radio_axis_plot = self._window._axisWidget._radioAxis.getPlot()
-        self.assertEqual(radio_axis_plot.getPlot().getActiveImage(), None)
+        numpy.testing.assert_array_equal(
+            radio_axis_plot.getPlot().getActiveImage().getData(), numpy.empty((0, 0, 4))
+        )
         self._window.setScan(self.scan)
-        self.assertNotEqual(radio_axis_plot.getPlot().getActiveImage(), None)
+        self.qapp.processEvents()
+        # make sur the image has been set
+        assert radio_axis_plot.getPlot().getImage(legend="image1").getData().shape == (
+            20,
+            20,
+        )
 
     def testShiftButtons(self):
         """Test that the 'left', 'right', ... buttons and the shift steps are
@@ -87,7 +112,10 @@ class TestWindowAxis(TestCaseQt):
         self.qapp.processEvents()
         self.assertTrue(radio_axis.getXShift() == 0)
         self.assertTrue(radio_axis.getYShift() == 0)
-        self.assertTrue(radio_axis.getShiftStep() == 1.0)
+        self.assertTrue(
+            radio_axis.getShiftStep() == 1.0,
+            f"shift is {radio_axis.getShiftStep()} when 1.0 expected",
+        )
         radio_axis._controlWidget._shiftControl._leftButton.click()
         self.qapp.processEvents()
         self.assertTrue(radio_axis.getXShift() == -1)
@@ -96,7 +124,7 @@ class TestWindowAxis(TestCaseQt):
         self.qapp.processEvents()
         self.assertTrue(radio_axis.getXShift() == 0)
         self.assertTrue(radio_axis.getYShift() == 0)
-        radio_axis._controlWidget._displacementSelector._fineButton.toggle()
+        radio_axis._controlWidget._displacementSelector._fineButton.click()
         self.qapp.processEvents()
         self.assertTrue(radio_axis.getShiftStep() == 0.1)
         radio_axis._controlWidget._shiftControl._topButton.click()
@@ -126,13 +154,9 @@ class TestWindowAxis(TestCaseQt):
         # note : for now only the xshift is managed !
         self._window.setReconsParams(axis_params)
         radio_axis = self._window._axisWidget._radioAxis
-        self.assertEqual(radio_axis.getMode(), AxisMode.manual)
         self.assertEqual(radio_axis.getXShift(), -6.0)
         self.assertEqual(radio_axis.getYShift(), 0.0)
 
-    def testAxisCalculationSaveLoad(self):
-        pass
-
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
 class TestWindowsAxisSetScan(TestCaseQt):
@@ -147,16 +171,17 @@ class TestWindowsAxisSetScan(TestCaseQt):
         # create dataset
         self.master_file = os.path.join(self.scan_dir, "frm_edftomomill_twoentries.nx")
         shutil.copyfile(
-            UtilsTest.getH5Dataset(folderID="frm_edftomomill_twoentries.nx"),
+            TomwerCIDatasets.get_dataset(
+                "h5_datasets/frm_edftomomill_twoentries.nx",
+            ),
             self.master_file,
         )
-        self.scan = HDF5TomoScan(scan=self.master_file, entry="entry0000")
+        self.scan = NXtomoScan(scan=self.master_file, entry="entry0000")
 
     def tearDown(self):
         self.widget.setAttribute(qt.Qt.WA_DeleteOnClose)
         self.widget.close()
         self.widget = None
-        TestCaseQt.tearDown(self)
 
     def patch_fov(self, value: str):
         with h5py.File(self.scan.master_file, mode="a") as h5s:
@@ -165,3 +190,40 @@ class TestWindowsAxisSetScan(TestCaseQt):
                 if "instrument/detector/field_of_view" in entry_node:
                     del entry_node["instrument/detector/field_of_view"]
                 entry_node["instrument/detector/field_of_view"] = value
+
+
+def test__getClosestAssociatedAngle():
+    """
+    test _getClosestAssociatedAngle function
+    """
+    assert (
+        _ManualFramesSelection._getClosestAssociatedAngle(angle=12.3, angles=(12.3,))
+        == 12.3
+    )
+    assert (
+        _ManualFramesSelection._getClosestAssociatedAngle(
+            angle=12.3, angles=(2.06, 10.3, 100.6)
+        )
+        == 10.3
+    )
+    assert (
+        _ManualFramesSelection._getClosestAssociatedAngle(
+            angle=100.2, angles=(2.06, 10.3, 100.6)
+        )
+        == 100.6
+    )
+    assert (
+        _ManualFramesSelection._getClosestAssociatedAngle(angle=12.3, angles=()) is None
+    )
+    assert (
+        _ManualFramesSelection._getClosestAssociatedAngle(
+            angle=300.2, angles=(2.06, 10.3, 100.6)
+        )
+        == 100.6
+    )
+    assert (
+        _ManualFramesSelection._getClosestAssociatedAngle(
+            angle=-12, angles=(2.06, 10.3, 100.6)
+        )
+        == 2.06
+    )
