@@ -28,21 +28,24 @@ __date__ = "05/02/2020"
 
 
 import os
+
+import pytest
 from silx.gui import qt
-from tomwer.test.utils import skip_gui_test
-from tomwer.gui.reconstruction.nabu.nabuflow import NabuFlowControl
+from silx.gui.utils.testutils import TestCaseQt
+
+from tomwer.gui.reconstruction.nabu.nabuconfig.output import _NabuOutputConfig
+from tomwer.gui.reconstruction.nabu.nabuconfig.phase import _NabuPhaseConfig
 from tomwer.gui.reconstruction.nabu.nabuconfig.preprocessing import (
     _NabuPreProcessingConfig,
-    _RingCorrectionMethod,
+    RingCorrectionMethod,
 )
-from tomwer.gui.reconstruction.nabu.nabuconfig.phase import _NabuPhaseConfig
 from tomwer.gui.reconstruction.nabu.nabuconfig.reconstruction import (
     _NabuReconstructionConfig,
 )
-from tomwer.gui.reconstruction.nabu.nabuconfig.output import _NabuOutputConfig
+from tomwer.gui.reconstruction.nabu.nabuflow import NabuFlowControl
 from tomwer.gui.reconstruction.nabu.volume import NabuVolumeWidget
-from silx.gui.utils.testutils import TestCaseQt
-import pytest
+from tomwer.tests.utils import skip_gui_test
+from tomwer.core.process.output import ProcessDataOutputDirMode
 
 
 class ProcessClass:
@@ -104,12 +107,11 @@ class TestNabuPreProcConfig(TestCaseQt):
         self.nabuWidget.close()
         del self.nabuWidget
         self.qapp.processEvents()
-        TestCaseQt.tearDown(self)
 
     def testGetInitialConfiguration(self):
         """Test that the get configuration is working"""
         ini_conf = {
-            "flatfield_enabled": 1,
+            "flatfield": 1,
             "double_flatfield_enabled": 0,
             "dff_sigma": 0.0,
             "ccd_filter_enabled": 0,
@@ -117,14 +119,18 @@ class TestNabuPreProcConfig(TestCaseQt):
             "log_min_clip": 1e-6,
             "log_max_clip": 10.0,
             "take_logarithm": True,
-            "sino_rings_correction": _RingCorrectionMethod.NONE.value,
+            "normalize_srcurrent": 0,
+            "sino_rings_correction": RingCorrectionMethod.NONE.value,
+            "sino_rings_options": "sigma=1.0 ; levels=10 ; padding=False",
+            "tilt_correction": "",
+            "autotilt_options": "",
         }
         self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)
 
     def testSetConfiguration(self):
         """Test that the set configuration is working"""
         conf = {
-            "flatfield_enabled": 0,
+            "flatfield": 0,
             "double_flatfield_enabled": 1,
             "dff_sigma": 2.0,
             "ccd_filter_enabled": 1,
@@ -132,7 +138,11 @@ class TestNabuPreProcConfig(TestCaseQt):
             "log_min_clip": 1e-3,
             "log_max_clip": 250.0,
             "take_logarithm": False,
-            "sino_rings_correction": _RingCorrectionMethod.MUNCH.value,
+            "normalize_srcurrent": 1,
+            "sino_rings_correction": RingCorrectionMethod.MUNCH.value,
+            "sino_rings_options": "sigma=1.4 ; levels=11 ; padding=True",
+            "tilt_correction": "1d-correlation",
+            "autotilt_options": "low_pass=1; high_pass=20",
         }
         self.nabuWidget.setConfiguration(conf=conf)
         self.assertEqual(self.nabuWidget.getConfiguration(), conf)
@@ -148,7 +158,6 @@ class TestNabuPhaseConfig(TestCaseQt):
         self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
         self.nabuWidget.close()
         del self.nabuWidget
-        TestCaseQt.tearDown(self)
 
     def testGetInitialConfiguration(self):
         """Test that the get configuration is working"""
@@ -158,6 +167,10 @@ class TestNabuPhaseConfig(TestCaseQt):
             "padding_type": "edge",
             "unsharp_coeff": 0,
             "unsharp_sigma": 0,
+            "beam_shape": "parallel",
+            "ctf_advanced_params": " length_scale=1e-05; lim1=1e-05; lim2=0.2; normalize_by_mean=True",
+            "ctf_geometry": " z1_v=None; z1_h=None; detec_pixel_size=None; magnification=True",
+            "ctf_translations_file": "",
         }
         self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)
 
@@ -169,6 +182,10 @@ class TestNabuPhaseConfig(TestCaseQt):
             "padding_type": "zeros",
             "unsharp_coeff": 3.6,
             "unsharp_sigma": 2.1,
+            "beam_shape": "cone",
+            "ctf_advanced_params": " length_scale=1e-05; lim1=1e-05; lim2=0.2; normalize_by_mean=True",
+            "ctf_geometry": " z1_v=0.0; z1_h=0.0; detec_pixel_size=None; magnification=True",
+            "ctf_translations_file": "",
         }
         self.nabuWidget.setConfiguration(conf)
         self.nabuWidget.show()
@@ -197,12 +214,12 @@ class TestNabuReconstructionConfig(TestCaseQt):
         self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
         self.nabuWidget.close()
         del self.nabuWidget
-        TestCaseQt.tearDown(self)
 
     def testGetInitialConfiguration(self):
         """Test that the get configuration is working"""
         ini_conf = {
             "method": "FBP",
+            "slice_plane": "XY",
             "angles_file": "",
             "axis_correction_file": "",
             "angle_offset": 0.0,
@@ -221,6 +238,8 @@ class TestNabuReconstructionConfig(TestCaseQt):
             "rotation_axis_position": "",
             "positivity_constraint": 1,
             "translation_movements_file": "",
+            "clip_outer_circle": 0,
+            "centered_axis": 0,
         }
         self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)
 
@@ -228,6 +247,7 @@ class TestNabuReconstructionConfig(TestCaseQt):
         """Test that the set configuration is working"""
         ini_conf = {
             "method": "FBP",
+            "slice_plane": "XZ",
             "angles_file": "",
             "axis_correction_file": "",
             "angle_offset": 12.5,
@@ -246,9 +266,11 @@ class TestNabuReconstructionConfig(TestCaseQt):
             "rotation_axis_position": "",
             "positivity_constraint": 0,
             "translation_movements_file": "my_file.csv",
+            "clip_outer_circle": 1,
+            "centered_axis": 1,
         }
         self.nabuWidget.setConfiguration(ini_conf)
-        qt.QApplication.processEvents()
+        self.qapp.processEvents()
         self.nabuWidget.show()
 
         # check visibility of some widgets
@@ -262,6 +284,9 @@ class TestNabuReconstructionConfig(TestCaseQt):
         self.assertTrue(subRegionWidget._zSubRegion._minQLE.isEnabled())
         self.assertFalse(subRegionWidget._zSubRegion._maxQLE.isEnabled())
         self.assertFalse(self.nabuWidget._preconditioningFilter.isChecked())
+        self.assertTrue(self.nabuWidget._clipOuterCircleCB.isChecked())
+        self.assertTrue(self.nabuWidget._centeredAxisCB.isChecked())
+        assert self.nabuWidget._axisQCB.currentText() == ini_conf["slice_plane"]
 
         # check the generated configuration
         self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)
@@ -280,20 +305,27 @@ class TestNabuOutputConfig(TestCaseQt):
         self.nabuWidget.close()
         self.nabuWidget = None
         self.qapp.processEvents()
-        TestCaseQt.tearDown(self)
 
     def testGetConfiguration(self):
-        ini_conf = {"file_format": "hdf5", "location": ""}
+        ini_conf = {
+            "file_format": "hdf5",
+            "location": "",
+            "output_dir_mode": "same folder as scan",
+        }
         self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)
 
     def testSetConfiguration(self):
-        conf = {"file_format": "tiff", "location": os.sep.join(("tmp", "my_output"))}
+        conf = {
+            "file_format": "tiff",
+            "location": os.sep.join(("tmp", "my_output")),
+            "output_dir_mode": "other",
+        }
         self.nabuWidget.setConfiguration(conf)
         self.nabuWidget.show()
         self.qapp.processEvents()
         # check some widget visibility
-        self.assertTrue(self.nabuWidget._outputDirQLE.isVisible())
-        self.assertFalse(self.nabuWidget._defaultOutput.isChecked())
+        self.assertTrue(self.nabuWidget._output_dir_widget._outputDirQLE.isVisible())
+        self.assertTrue(self.nabuWidget._output_dir_widget._otherDirRB.isChecked())
 
         self.assertEqual(self.nabuWidget.getConfiguration(), conf)
 
@@ -303,21 +335,25 @@ class TestNabuVolumeWidget(TestCaseQt):
     def setUp(self):
         TestCaseQt.setUp(self)
         self.nabuWidget = NabuVolumeWidget(parent=None)
+        self.nabuWidget.setConfigurationLevel("advanced")
 
     def tearDown(self):
         self.nabuWidget.setAttribute(qt.Qt.WA_DeleteOnClose)
         self.nabuWidget.close()
         self.nabuWidget = None
-        TestCaseQt.tearDown(self)
 
     def testGetConfiguration(self):
         ini_conf = {
             "start_z": 0,
             "end_z": -1,
             "gpu_mem_fraction": 0.9,
-            "postproc": {"output_histogram": 0},
+            "output_dir_mode": ProcessDataOutputDirMode.IN_SCAN_FOLDER.value,
+            "overwrite_output_location": False,
+            "postproc": {"output_histogram": 1},
             "cpu_mem_fraction": 0.9,
             "use_phase_margin": True,
+            "new_output_file_format": "",
+            "new_output_location": "",
         }
         self.assertEqual(self.nabuWidget.getConfiguration(), ini_conf)
 
@@ -329,8 +365,17 @@ class TestNabuVolumeWidget(TestCaseQt):
             "postproc": {"output_histogram": 0},
             "cpu_mem_fraction": 0.1,
             "use_phase_margin": False,
+            "new_output_file_format": "hdf5",
+            "new_output_location": "/new/location",
         }
         self.nabuWidget.setConfiguration(conf)
-        self.nabuWidget.show()
         self.qapp.processEvents()
+
+        # update the config dict has setting an other custom output directory update some items
+        conf.update(
+            {
+                "overwrite_output_location": True,
+                "output_dir_mode": ProcessDataOutputDirMode.OTHER.value,
+            }
+        )
         self.assertEqual(self.nabuWidget.getConfiguration(), conf)
