@@ -27,27 +27,28 @@ __license__ = "MIT"
 __date__ = "03/02/2021"
 
 
-from tomwer.test.utils import skip_gui_test
-from tomwer.gui.reconstruction.saaxis.dimensionwidget import DimensionWidget
-from tomoscan.unitsystem import metricsystem
+import os
+import shutil
+import tempfile
+
+import h5py
+import numpy
+import pytest
+from silx.gui import qt
 from silx.gui.utils.testutils import TestCaseQt
-from tomwer.core.utils.scanutils import MockHDF5
-from tomwer.gui.reconstruction.saaxis.saaxis import SAAxisWindow
+from silx.io.url import DataUrl
+from pyunitsystem import metricsystem
+
 from tomwer.core.process.reconstruction.saaxis.params import (
-    SAAxisParams,
     ReconstructionMode,
+    SAAxisParams,
 )
 from tomwer.core.process.reconstruction.scores import ComputedScore
 from tomwer.core.process.reconstruction.scores.params import ScoreMethod
-from silx.io.url import DataUrl
-from silx.gui import qt
-import unittest
-import pytest
-import numpy
-import shutil
-import tempfile
-import os
-import h5py
+from tomwer.core.utils.scanutils import MockNXtomo
+from tomwer.gui.reconstruction.saaxis.dimensionwidget import DimensionWidget
+from tomwer.gui.reconstruction.saaxis.saaxis import SAAxisWindow
+from tomwer.tests.utils import skip_gui_test
 
 
 @pytest.mark.skipif(skip_gui_test(), reason="skip gui test")
@@ -128,7 +129,7 @@ class TestSAAxisWindow(TestCaseQt):
         self._cor_scores = {}
 
         for i in range(self._N_OUTPUT_URLS):
-            output_file = os.path.join(self.folder, "recons_{}.h5".format(i))
+            output_file = os.path.join(self.folder, f"recons_{i}.h5")
             with h5py.File(output_file, mode="a") as h5f:
                 h5f["data"] = numpy.random.random(100 * 100).reshape(100, 100)
                 url = DataUrl(file_path=output_file, data_path="data", scheme="silx")
@@ -143,7 +144,7 @@ class TestSAAxisWindow(TestCaseQt):
         # create a scan
         self.folder = tempfile.mkdtemp()
         dim = 10
-        mock = MockHDF5(
+        mock = MockNXtomo(
             scan_path=self.folder, n_proj=10, n_ini_proj=10, scan_range=180, dim=dim
         )
         mock.add_alignment_radio(index=10, angle=90)
@@ -187,7 +188,7 @@ class TestSAAxisWindow(TestCaseQt):
         """Insure configuration setter works"""
         self._window.setSlicesRange(0, 20)
         configuration = {
-            "slice_indexes": {"Slice": 2},
+            "slice_index": {"Slice": 2},
             "estimated_cor": 0.24,
             "research_width": 0.8,
             "n_reconstruction": 5,
@@ -195,7 +196,7 @@ class TestSAAxisWindow(TestCaseQt):
         self._window.setConfiguration(configuration)
         res_conf = self._window.getConfiguration()
         for key in (
-            "slice_indexes",
+            "slice_index",
             "estimated_cor",
             "research_width",
             "n_reconstruction",
@@ -203,19 +204,11 @@ class TestSAAxisWindow(TestCaseQt):
             self.assertEqual(configuration[key], res_conf[key])
 
     def testSetResults(self):
-        """Test setting results"""
+        """Test setting results and saving result to a folder"""
         self._window.setCorScores(self._cor_scores, score_method="standard deviation")
-
-
-def suite():
-    test_suite = unittest.TestSuite()
-    for ui in (
-        TestDimensionWidget,
-        TestSAAxisWindow,
-    ):
-        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
-    return test_suite
-
-
-if __name__ == "__main__":
-    unittest.main(defaultTest="suite")
+        # test saving snapshots
+        with tempfile.TemporaryDirectory() as output_png_imgs:
+            final_dir = os.path.join(output_png_imgs, "test/create/it")
+            self._window.saveReconstructedSlicesTo(final_dir)
+            assert os.path.exists(final_dir)
+            assert len(os.listdir(final_dir)) == len(self._cor_scores)
