@@ -30,12 +30,18 @@ __license__ = "MIT"
 __date__ = "19/03/2018"
 
 
-from collections import OrderedDict
-from tomwer.gui.samplemoved.selectiontable import AngleSelectionTable
-from silx.gui import qt
-import silx.io.utils
-import silx.io.url
 import logging
+import weakref
+from collections import OrderedDict
+
+import silx.io.url
+import silx.io.utils
+from silx.gui import qt
+from tomoscan.esrf.scan.utils import get_data
+
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.samplemoved.selectiontable import AngleSelectionTable
+from tomwer.gui.settings import Y_AXIS_DOWNWARD
 
 _logger = logging.getLogger(__name__)
 try:
@@ -64,14 +70,16 @@ class SampleMovedWidget(qt.QMainWindow):
     two projections
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, backend=None):
         qt.QMainWindow.__init__(self, parent)
+        self._scan = None
         self.setWindowFlags(qt.Qt.Widget)
         self._isConnected = False
         self._images = {}
         self._symmetricalStates = {"first": False, "second": False}
-        self._plot = CompareImages(parent=self)
+        self._plot = CompareImages(parent=self, backend=backend)
         self._plot.setWindowFlags(qt.Qt.Widget)
+        self._plot.getPlot().setYAxisInverted(Y_AXIS_DOWNWARD)
         self._on_load_callback = []
 
         self._topWidget = self.getControlWidget()
@@ -137,6 +145,23 @@ class SampleMovedWidget(qt.QMainWindow):
         self._selectionTable.clear()
         self._images = {}
 
+    @property
+    def scan(self):
+        if self._scan is None or self._scan() is None:
+            return None
+        else:
+            return self._scan()
+
+    def setScan(self, scan):
+        if scan is None:
+            self._scan = None
+        elif not isinstance(scan, TomwerScanBase):
+            raise TypeError(
+                f"scan is expected to be None or an instance of {TomwerScanBase}. {type(scan)} provided"
+            )
+        else:
+            self._scan = weakref.ref(scan)
+
     def setImages(self, images):
         """
         Set the images in a key value system. Key should be in
@@ -181,31 +206,32 @@ class SampleMovedWidget(qt.QMainWindow):
 
     def _updatePlot(self):
         imgA, imgB = self._selectionTable.getSelection()
-        dataImgA = dataImgB = None
-        if imgA is not None:
-            dataImgA = silx.io.utils.get_data(silx.io.url.DataUrl(path=imgA))
-            for callback in self._on_load_callback:
-                callback(dataImgA)
-        if imgB is not None:
-            dataImgB = silx.io.utils.get_data(silx.io.url.DataUrl(path=imgB))
-            for callback in self._on_load_callback:
-                callback(dataImgB)
+        dataImgA = self._changeImageA(imgA)
+        dataImgB = self._changeImageA(imgB)
         if dataImgA is not None and dataImgB is not None:
             self._plot.setData(image1=dataImgA, image2=dataImgB)
 
     def _changeImageA(self, img):
         if img is not None:
-            dataImgA = silx.io.utils.get_data(silx.io.url.DataUrl(path=img))
-            for callback in self._on_load_callback:
-                callback(dataImgA)
+            imgAUrl = silx.io.url.DataUrl(path=img)
+            if self.scan is None:
+                dataImgA = get_data(imgAUrl)
+            else:
+                idxImgA = self.scan.get_url_proj_index(imgAUrl)
+                dataImgA = self.scan.flat_field_correction((imgAUrl,), (idxImgA,))[0]
             self._plot.setImage1(image1=dataImgA)
+            return dataImgA
 
     def _changeImageB(self, img):
         if img is not None:
-            dataImgB = silx.io.utils.get_data(silx.io.url.DataUrl(path=img))
-            for callback in self._on_load_callback:
-                callback(dataImgB)
+            imgBUrl = silx.io.url.DataUrl(path=img)
+            if self.scan is None:
+                dataImgB = get_data(imgBUrl)
+            else:
+                idxImgB = self.scan.get_url_proj_index(imgBUrl)
+                dataImgB = self.scan.flat_field_correction((imgBUrl,), (idxImgB,))[0]
             self._plot.setImage2(image2=dataImgB)
+            return dataImgB
 
     def setConfiguration(self, config):
         if config == "manual":
