@@ -30,13 +30,15 @@ __license__ = "MIT"
 __date__ = "17/08/2021"
 
 
+import functools
+import logging
 import os
 from collections import OrderedDict
-import functools
+
 from silx.gui import qt
 from silx.io.url import DataUrl
+
 from tomwer.gui.imagefromfile import FileWithImage
-import logging
 
 logger = logging.getLogger(__name__)
 
@@ -125,7 +127,7 @@ class SelectionTable(qt.QTableWidget):
                     self._checkBoxes[radioUrl][whichImg].setChecked(False)
                     self._checkBoxes[radioUrl][whichImg].blockSignals(False)
 
-    def getSelection(self):
+    def getSelection(self, with_indexes=False):
         """
 
         :return: url selected for img A and img B.
@@ -136,7 +138,10 @@ class SelectionTable(qt.QTableWidget):
                 imgA = radioUrl
             if self._checkBoxes[radioUrl]["img B"].isChecked():
                 imgB = radioUrl
-        return imgA, imgB
+        if with_indexes:
+            raise NotImplementedError
+        else:
+            return imgA, imgB
 
     def setSelection(self, url_img_a, url_img_b):
         """
