@@ -28,23 +28,25 @@ __author__ = ["P. Paleo", "H. Payno"]
 __license__ = "MIT"
 __date__ = "24/05/2016"
 
+import functools
+import logging
+import os
+import time
+from collections import OrderedDict, namedtuple
+from typing import Iterable, Optional
+
+import numpy
 from silx.gui import qt
 from silx.gui.plot import Plot2D
-from . import utils
-from tomwer.gui import settings
-import numpy
-import os
-from collections import OrderedDict
-import tomwer.resources
-from .imagefromfile import ImageFromFile, FileWithImage, _Image
 from silx.gui.plot.actions import PlotAction
 from silx.gui.plot.actions.control import YAxisInvertedAction
 from silx.io.url import DataUrl
-import logging
-from tomwer.gui import icons
-import functools
-import time
-from collections import namedtuple, Iterable
+
+import tomwer.resources
+from tomwer.gui import icons, settings
+
+from . import utils
+from .imagefromfile import FileWithImage, ImageFromFile, _Image
 
 logger = logging.getLogger(__name__)
 
@@ -65,7 +67,7 @@ class _QImageStackPlot(qt.QWidget):
         )
     )
 
-    def __init__(self, parent, sliderVertical=False):
+    def __init__(self, parent, sliderVertical=False, backend=None):
         qt.QWidget.__init__(self, parent)
         self._transpose = False
         """If we want to transpose the image like imagj by default"""
@@ -80,7 +82,7 @@ class _QImageStackPlot(qt.QWidget):
         mainwidget.setLayout(mainLayout)
         layout.addWidget(mainwidget)
 
-        self._plot = Plot2D(self)
+        self._plot = Plot2D(self, backend=backend)
 
         def find_y_inverted(actions):
             for act in actions:
@@ -204,7 +206,7 @@ class _QImageStackPlot(qt.QWidget):
         self._qspinbox.blockSignals(False)
 
     def createStackImageInstance(self):
-        return _StackImage()
+        return _StackImage(images=None)
 
     def showImage(self, index):
         """
@@ -232,25 +234,6 @@ class _QImageStackPlot(qt.QWidget):
         self._qspinbox.blockSignals(False)
         self._qslider.blockSignals(False)
 
-    def setImage(self, index, data):
-        """
-        Set the given data as the image for the given index
-
-        :param int index: the index of the image to set
-        :param numpy.ndarray data: the image
-        """
-        _data = data
-        if data is None:
-            _data = self.IMG_NOT_FOUND
-        self.images[index] = _data
-
-    def isEmpty(self):
-        """
-
-        :return bool: True if no reconstruction has been set yet
-        """
-        return self.images.empty()
-
     def clear(self):
         self.setImages(None)
 
@@ -276,13 +259,15 @@ class _QImageFileStackPlot(_QImageStackPlot):
     file
     """
 
-    def __init__(self, parent):
+    def __init__(self, parent, backend=None):
         """
         Constructor
 
         :param parent: the Qt parent widget
         """
-        _QImageStackPlot.__init__(self, parent)
+        _QImageStackPlot.__init__(self, parent, backend=backend)
+        self._imagejThreads = []
+        # threads used to open frames with image j
 
         self._loadingMode = _LoadingModeToolButton(parent=self)
         self.getControlWidget().layout().addWidget(self._loadingMode)
@@ -388,10 +373,6 @@ class _QImageFileStackPlot(_QImageStackPlot):
             self._qslider.valueChanged.connect(self.changeActiveImageFrmQSlider)
             self._qspinbox.valueChanged.connect(self.changeActiveImageFrmQSpinBox)
 
-    def setLoadingImage(self, sliceNb):
-        """Set the status of the image to not loaded"""
-        self.setImage(sliceNb, _QImageFileStackPlot.IMG_LOADING)
-
     def setImages(self, imagesFiles):
         """
         Set the stack of images files
@@ -414,7 +395,9 @@ class _QImageFileStackPlot(_QImageStackPlot):
             logger.warning("No active image defined")
         else:
             try:
-                utils.open_url_with_image_j(active_image.url)
+                self._imagejThreads.append(
+                    utils.open_url_with_image_j(active_image.url)
+                )
             except OSError as e:
                 msg = qt.QMessageBox(self)
                 msg.setIcon(qt.QMessageBox.Warning)
@@ -495,9 +478,13 @@ class _QImageFileStackPlot(_QImageStackPlot):
             self._qlFileName.setText("")
         super(_QImageFileStackPlot, self).clear()
 
+    def close(self):
+        [thread.quit() for thread in self._imagejThreads]
+        super.close()
+
 
 class _StackImage(object):
-    def __init__(self, images):
+    def __init__(self, images: Optional[Iterable]):
         _images = images
         if _images is None:
             _images = {}
@@ -516,6 +503,11 @@ class _StackImage(object):
         else:
             return self._images[index]
 
+    def setImages(self, images: Iterable):
+        self.clear()
+        for image in images:
+            self.addImage(image)
+
     def size(self):
         return len(self._images)
 
@@ -543,9 +535,9 @@ class _StackImageToLoad(_StackImage, qt.QObject):
     """Signal emitted when the loading mode is changed. Parameter are new mode,
     old mode"""
 
-    def __init__(self, loadingMode=LAZY_LOADING):
+    def __init__(self, loadingMode=LAZY_LOADING, images: Optional[Iterable] = None):
         assert loadingMode in (LAZY_LOADING, ASAP_LOADING, ON_SHOW_LOADING)
-        _StackImage.__init__(self, images=None)
+        _StackImage.__init__(self, images=images)
         qt.QObject.__init__(self)
         self._mode = loadingMode
         self._url_to_index = {}
@@ -563,7 +555,7 @@ class _StackImageToLoad(_StackImage, qt.QObject):
 
     def clear(self):
         for img in self._images.values():
-            img.sigLoaded.disconnect(self._loaded)
+            img.sigLoaded.disconnect(self.sigImageLoaded)
         _StackImage.clear(self)
         self._url_to_index.clear()
 
@@ -573,10 +565,10 @@ class _StackImageToLoad(_StackImage, qt.QObject):
         elif isinstance(image, DataUrl):
             image_ = ImageFromFile(url=image)
         else:
-            raise TypeError("Unmanaged type:" + type(image))
+            raise TypeError(f"Unmanaged type: {type(image)}")
         self._url_to_index[image_.url.path()] = self.nextIndex
         _StackImage.addImage(self, image_)
-        image_.sigLoaded.connect(self._loaded)
+        image_.sigLoaded.connect(self.sigImageLoaded)
         if self._mode == ASAP_LOADING:
             image_.load(self._force_sync)
 
@@ -617,11 +609,11 @@ class _StackImageToLoad(_StackImage, qt.QObject):
         return self._mode
 
     def _loadAllImages(self):
-        for imgIndex, img in self._images.items():
-            img.load_recons_params(self._force_sync)
-
-    def _loaded(self, url_path):
-        self.sigImageLoaded.emit(url_path)
+        for _, img in self._images.items():
+            try:
+                img.load(sync=self._force_sync)
+            except Exception as e:
+                logger.error(e)
 
     def setForceSync(self, value):
         self._force_sync = value
