@@ -30,16 +30,19 @@ __license__ = "MIT"
 __date__ = "19/03/2018"
 
 
-from silx.gui import qt
-from tomwer.gui.stackplot import _QImageFileStackPlot
-from tomwer.core.scan.edfscan import EDFTomoScan
+import logging
 from collections import OrderedDict
-from tomwer.gui.qfolderdialog import QScanDialog
+from typing import Union
+
+import h5py
+from silx.gui import qt
+from tomoscan.factory import Factory
+from tomoscan.identifier import VolumeIdentifier
+
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.core.scan.scanfactory import ScanFactory
-import logging
-import h5py
-import os
+from tomwer.gui.qfolderdialog import QDataDialog
+from tomwer.gui.stackplot import _QImageFileStackPlot
 
 _logger = logging.getLogger(__name__)
 
@@ -64,11 +67,11 @@ class _ImageStack(qt.QMainWindow):
         copy.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, backend=None):
         qt.QMainWindow.__init__(self, parent)
         self.setWindowFlags(qt.Qt.Widget)
         self._scans = set()
-        self._viewer = _QImageFileStackPlot(parent=self)
+        self._viewer = _QImageFileStackPlot(parent=self, backend=backend)
         self._viewer.addFolderName(True)
         self.setCentralWidget(self._viewer)
 
@@ -96,27 +99,22 @@ class _ImageStack(qt.QMainWindow):
 
         self.setLoadingMode("lazy loading")
 
-    def addLeafScan(self, scanID):
-        if isinstance(scanID, TomwerScanBase):
-            self._scans.add(scanID.path)
-        else:
-            assert type(scanID) is str
-            self._scans.add(scanID)
+    def addLeafScan(self, scanID: Union[str, TomwerScanBase]):
+        self._scans.add(scanID)
         self._update()
 
     def _addNewFolder(self):
-        dialog = QScanDialog(self, multiSelection=True)
+        dialog = QDataDialog(self, multiSelection=True)
 
         if not dialog.exec_():
             dialog.close()
             return
 
-        for folder in dialog.filesSelected():
-            assert os.path.isdir(folder)
+        for folder in dialog.files_selected():
             self.addLeafScan(folder)
 
     def removeLeafScan(self, scanID):
-        self._scans.remove(scanID)
+        self._scans.remove(str(scanID))
         self._update()
 
     def extractImages(self):
@@ -135,8 +133,8 @@ class _ImageStack(qt.QMainWindow):
         for scan in all_images:
             try:
                 oScanSlices = OrderedDict(sorted(all_images[scan].items()))
-                for slice in oScanSlices:
-                    ordered_imgs.append(all_images[scan][slice])
+                for scan_slice in oScanSlices:
+                    ordered_imgs.append(all_images[scan][scan_slice])
             except TypeError:
                 # in case algorithm could extract recons index
                 ordered_imgs = ordered_imgs + list(all_images[scan].values())
@@ -175,10 +173,28 @@ class SliceStack(_ImageStack):
         """
         slices = {}
         for scan in self._scans:
-            # TODO: should also work for hdf5 files. Call ScanFactory ?
-            imgs = EDFTomoScan.get_reconstructions_paths(scan, withIndex=True)
-            if len(imgs) > 0:
-                slices[scan] = imgs
+            if not isinstance(scan, TomwerScanBase):
+                scan_obj = ScanFactory.create_scan_object(scan_path=scan)
+            else:
+                scan_obj = scan
+            if (
+                scan_obj.latest_reconstructions is not None
+                and len(scan_obj.latest_reconstructions) > 0
+            ):
+
+                def get_urls(volume_id: VolumeIdentifier):
+                    volume = Factory.create_tomo_object_from_identifier(volume_id)
+                    return list(volume.browse_data_urls())
+
+                urls = list()
+                for volume_id in scan_obj.latest_reconstructions:
+                    urls.extend(get_urls(volume_id))
+
+                slices[str(scan)] = dict(enumerate(urls))
+            else:
+                imgs = dict(enumerate(scan_obj.get_reconstructions_urls()))
+                if len(imgs) > 0:
+                    slices[str(scan)] = imgs
         return slices
 
 
@@ -197,7 +213,11 @@ class RadioStack(_ImageStack):
         slices = {}
         for scan in self._scans:
             # manage hdf5
-            if h5py.is_hdf5(scan):
+            if isinstance(scan, TomwerScanBase):
+                imgs = scan.projections
+                if len(imgs) > 0:
+                    slices[str(scan)] = imgs
+            elif h5py.is_hdf5(scan):
                 try:
                     scans = ScanFactory.create_scan_objects(scan_path=scan)
                 except Exception as e:
