@@ -27,11 +27,14 @@ __license__ = "MIT"
 __date__ = "04/02/2020"
 
 
+from typing import Optional
+
 from silx.gui import qt
-import os
+
+from tomwer.core.scan.scanbase import TomwerScanBase
 
 
-class ScanNameLabel(qt.QWidget):
+class ScanNameLabelAndShape(qt.QWidget):
     """Scan to display the scan name"""
 
     def __init__(self, parent):
@@ -50,16 +53,31 @@ class ScanNameLabel(qt.QWidget):
         )
         self.layout().addWidget(self._scanNameLabel)
 
+        self._shapeLabel = qt.QLabel("", self)
+        self._shapeLabel.setAlignment(qt.Qt.AlignLeft)
+        self._shapeLabel.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Minimum)
+        self.layout().addWidget(self._shapeLabel)
+
         # set up
         self.clear()
 
-    def setScan(self, scan):
+    def setScan(self, scan: Optional[TomwerScanBase]):
         if scan is None or scan.path is None:
             self.clear()
+        elif not isinstance(scan, TomwerScanBase):
+            raise TypeError(
+                f"Scan is expected to be an  instance of {TomwerScanBase}. Get {type(scan)} instead"
+            )
         else:
-            self._scanNameLabel.setText(os.path.basename(scan.path))
-            self._scanNameLabel.setToolTip(scan.path)
+            assert isinstance(scan, TomwerScanBase)
+            self._scanNameLabel.setText(scan.get_identifier().short_description())
+            self._scanNameLabel.setToolTip(scan.get_identifier().to_str())
+
+            shape_x = scan.dim_1 if scan.dim_1 is not None else "?"
+            shape_y = scan.dim_2 if scan.dim_2 is not None else "?"
+            self._shapeLabel.setText(f"dims: x={shape_x}, y={shape_y}")
 
     def clear(self):
         self._scanNameLabel.setText("-")
         self._scanNameLabel.setToolTip("")
+        self._shapeLabel.setText("")
