@@ -27,35 +27,39 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "13/08/2021"
 
-from silx.gui import qt
-from silx.utils.enum import Enum as _Enum
-from silx.gui.utils.signal import SignalProxy
-import weakref
 import logging
-from silx.gui.plot.ImageStack import UrlLoader, ImageStack as _ImageStack
+import weakref
+
+from silx.gui import qt
+from silx.gui.dialog.ColormapDialog import DisplayMode
+from silx.gui.plot.ImageStack import ImageStack as _ImageStack
+from silx.gui.plot.ImageStack import UrlLoader
+from silx.gui.utils.signal import SignalProxy
+from silx.io.url import DataUrl
+from silx.utils.enum import Enum as _Enum
+
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.core.utils.ftseriesutils import get_vol_file_shape
-from silx.io.url import DataUrl
+from tomwer.core.volume.volumebase import TomwerVolumeBase
+from tomwer.core.volume.volumefactory import VolumeFactory
+from tomwer.gui import icons
+from tomwer.gui.settings import Y_AXIS_DOWNWARD
+from tomwer.gui.visualization.fullscreenplot import FullScreenPlot2D
 from tomwer.gui.visualization.reconstructionparameters import ReconstructionParameters
 from tomwer.gui.visualization.scanoverview import ScanOverviewWidget
 from tomwer.io.utils.utils import get_slice_data
 
 try:
     from PIL import Image
-except ImportError:
-    has_PIL = False
+except ImportError:  # pragma: no cover
+    has_PIL = False  # pragma: no cover
 else:
     has_PIL = True
-try:
-    import cv2
-except ImportError:
-    has_cv2 = False
-else:
-    has_cv2 = True
+import os
+import time
+
 import numpy
 import numpy.lib.npyio
-import time
-import os
 
 _logger = logging.getLogger(__name__)
 
@@ -68,13 +72,16 @@ class DataViewer(qt.QMainWindow):
     sigConfigChanged = qt.Signal()
     """Signal emitted when the settings (display mode, options...) changed. """
 
-    def __init__(self, parent):
-        qt.QMainWindow.__init__(self, parent)
+    def __init__(self, parent, show_overview=True, backend=None):
+        super().__init__(parent)
         self.setWindowFlags(qt.Qt.Widget)
         self._scan = None
-
         # viewer
-        self._viewer = ImageStack(parent=self)
+        self._viewer = ImageStack(
+            parent=self, show_overview=show_overview, backend=backend
+        )
+        self._viewer.getPlotWidget().getMaskAction().setVisible(False)
+        self._viewer.getPlotWidget().setYAxisInverted(Y_AXIS_DOWNWARD)
         # set an UrlLoader managing .npy and .vol
         self._viewer.getPlotWidget().setKeepDataAspectRatio(True)
         self.setCentralWidget(self._viewer)
@@ -94,12 +101,14 @@ class DataViewer(qt.QMainWindow):
         self._controls = DisplayControl(parent=self)
         self._controlsDW = qt.QDockWidget(self)
         self._controlsDW.setWidget(self._controls)
-        self._controlsDW.setFeatures(qt.QDockWidget.DockWidgetMovable)
+        self._controlsDW.setWindowTitle("infos")
         self.addDockWidget(qt.Qt.TopDockWidgetArea, self._controlsDW)
+        self._controlsDW.setTitleBarWidget(qt.QWidget(self))
+        self._controlsDW.setFloating(False)
 
         # connect signal / slot
         self._controls.sigDisplayModeChanged.connect(self._updateDisplay)
-        self._controls.sigDisplayModeChanged.connect(self._reportSettingsUpdate)
+        self._controls.sigDisplayModeChanged.connect(self.sigConfigChanged)
 
     def getPlotWidget(self):
         return self._viewer.getPlotWidget()
@@ -107,8 +116,17 @@ class DataViewer(qt.QMainWindow):
     def getCurrentUrl(self):
         return self._viewer.getCurrentUrl()
 
-    def cleanBeforeQuit(self):
-        self._viewer._plot.updateThread.stop()
+    def setScanInfoVisible(self, visible: bool):
+        self._controls.setScanInfoVisible(visible=visible)
+
+    def setDisplayModeVisible(self, visible: bool):
+        self._controls.setDisplayModeVisible(visible=visible)
+
+    def setScanOverviewVisible(self, visible: bool) -> None:
+        self._viewer.setScanOverviewVisible(visible=visible)
+
+    def getUrlListDockWidget(self):
+        return self._viewer.getUrlListDockWidget()
 
     def close(self):
         self.stop()
@@ -167,6 +185,7 @@ class DataViewer(qt.QMainWindow):
             self._viewer.reset()
         else:
             assert isinstance(self._scan(), TomwerScanBase)
+            slices_metadata = {}
             if self.getDisplayMode() is _DisplayMode.RADIOS:
                 # update the normalization function from the viewer if needed
                 if self.getRadioOption() is _RadioMode.NORMALIZED:
@@ -187,12 +206,37 @@ class DataViewer(qt.QMainWindow):
                     slices = self._scan().latest_reconstructions
                 else:
                     slices = self._scan().get_reconstructions_urls()
+                # convert volumes identifiers to DataUrl
+                slices_urls = []
+                for identifier in slices:
+                    if isinstance(identifier, TomwerVolumeBase):
+                        volume = identifier
+                    else:
+                        volume = VolumeFactory.create_tomo_object_from_identifier(
+                            identifier=identifier
+                        )
+                    urls = list(volume.browse_data_urls())
+                    try:
+                        metadata = volume.metadata or volume.load_metadata()
+                    except Exception:
+                        metadata = None
+                    slices_urls.extend(urls)
+                    slices_metadata.update({url.path(): metadata for url in urls})
+
+                slices = slices_urls
+
             elif self.getDisplayMode() is _DisplayMode.DARKS:
                 self._viewer.setNormalizationFct(None)
                 slices = self._scan().darks
             elif self.getDisplayMode() is _DisplayMode.FLATS:
                 self._viewer.setNormalizationFct(None)
                 slices = self._scan().flats
+            elif self.getDisplayMode() is _DisplayMode.REDUCED_DARKS:
+                self._viewer.setNormalizationFct(None)
+                slices = self._scan().load_reduced_darks(return_as_url=True)
+            elif self.getDisplayMode() is _DisplayMode.REDUCED_FLATS:
+                self._viewer.setNormalizationFct(None)
+                slices = self._scan().load_reduced_flats(return_as_url=True)
             else:
                 raise ValueError("DisplayMode should be RADIOS or SLICES")
 
@@ -202,13 +246,12 @@ class DataViewer(qt.QMainWindow):
                     for key, value in sorted(slices.items(), key=lambda item: item[0])
                 ]
             if slices is not None and len(slices) > 0:
+                # TODO: set metadata as well ??? For each url get the metadata as a dict ???
+                self._viewer.setMetadatas(slices_metadata)
                 self._viewer.setUrls(slices)
             else:
                 self._viewer.reset()
 
-    def _reportSettingsUpdate(self):
-        self.sigConfigChanged.emit()
-
     def clear(self):
         self._scan = None
         self._viewer.reset()
@@ -221,10 +264,12 @@ class DataViewer(qt.QMainWindow):
 
 
 class _DisplayMode(_Enum):
-    RADIOS = "radios"
+    RADIOS = "projections-radios"
     SLICES = "slices"
-    DARKS = "darks"
-    FLATS = "flats"
+    DARKS = "raw darks"
+    FLATS = "raw flats"
+    REDUCED_DARKS = "reduced darks"
+    REDUCED_FLATS = "reduced flats"
 
 
 class _RadioMode(_Enum):
@@ -253,18 +298,18 @@ class DisplayControl(qt.QWidget):
         self.layout().setSpacing(0)
 
         # scan information
-        scanLab = qt.QLabel("scan:", self)
-        scanLab.setFixedWidth(40)
-        scanLab.setAlignment(qt.Qt.AlignRight | qt.Qt.AlignVCenter)
-        self.layout().addWidget(scanLab, 0, 0, 1, 1)
+        self._scanLab = qt.QLabel("scan:", self)
+        self._scanLab.setFixedWidth(40)
+        self._scanLab.setAlignment(qt.Qt.AlignRight | qt.Qt.AlignVCenter)
+        self.layout().addWidget(self._scanLab, 0, 0, 1, 1)
         self._scanQLE = qt.QLineEdit("", self)
         self._scanQLE.setReadOnly(True)
         self.layout().addWidget(self._scanQLE, 0, 1, 1, 3)
 
         # display information
-        displayLab = qt.QLabel("display:", self)
-        displayLab.setAlignment(qt.Qt.AlignRight | qt.Qt.AlignVCenter)
-        self.layout().addWidget(displayLab, 1, 0, 1, 1)
+        self._displayLab = qt.QLabel("display:", self)
+        self._displayLab.setAlignment(qt.Qt.AlignRight | qt.Qt.AlignVCenter)
+        self.layout().addWidget(self._displayLab, 1, 0, 1, 1)
         self._displayMode = qt.QComboBox(self)
         for mode in _DisplayMode:
             self._displayMode.addItem(mode.value)
@@ -300,6 +345,14 @@ class DisplayControl(qt.QWidget):
         self._radioMode.currentTextChanged.connect(self._updateOptions)
         self._sliceMode.currentTextChanged.connect(self._updateOptions)
 
+    def setScanInfoVisible(self, visible):
+        self._scanLab.setVisible(visible)
+        self._scanQLE.setVisible(visible)
+
+    def setDisplayModeVisible(self, visible):
+        self._displayLab.setVisible(visible)
+        self._displayMode.setVisible(visible)
+
     def getDisplayMode(self) -> _DisplayMode:
         """
 
@@ -337,9 +390,12 @@ class DisplayControl(qt.QWidget):
             config = mode, self.getRadioOption()
         elif mode is _DisplayMode.SLICES:
             config = mode, self.getSliceOption()
-        elif mode is _DisplayMode.DARKS:
-            config = mode, None
-        elif mode is _DisplayMode.FLATS:
+        elif mode in (
+            _DisplayMode.DARKS,
+            _DisplayMode.FLATS,
+            _DisplayMode.REDUCED_DARKS,
+            _DisplayMode.REDUCED_FLATS,
+        ):
             config = mode, None
         else:
             raise ValueError("mode should be RADIOS or SLICES")
@@ -355,19 +411,34 @@ class DisplayControl(qt.QWidget):
 
 
 class ImageStack(_ImageStack):
-    def __init__(self, parent, show_overview=True):
+    """
+    Image stack dedicated to data display.
+
+    It deal for example with data normalization...
+    """
+
+    def __init__(self, parent, show_overview=True, backend=None):
         self._normFct = None
         self._url_indexes = None
-        super(ImageStack, self).__init__(parent)
+        super().__init__(parent)
+        self.getPlotWidget().setBackend(backend)
+
+        # tune colormap dialog to have histogram by default
+        colormapAction = self.getPlotWidget().getColormapAction()
+        colormapDialog = colormapAction.getColormapDialog()
+        colormapDialog.getHistogramWidget().setDisplayMode(DisplayMode.HISTOGRAM)
+        colormapAction.setColormapDialog(colormapDialog)
+
         self.setUrlLoaderClass(_TomwerUrlLoader)
         # hide axis to be display
-        self._plot.getPlotWidget().setAxesDisplayed(False)
+        self._plot.setAxesDisplayed(False)
         self._loadSliceParams = False
         self._resetZoom = True
 
         # add dock widget for reconstruction parameters
         self._reconsInfoDockWidget = qt.QDockWidget(parent=self)
         self._reconsWidgetScrollArea = qt.QScrollArea(self)
+        self._reconsWidgetScrollArea.setWidgetResizable(True)
         self._reconsWidget = ReconstructionParameters(self)
         self._reconsWidgetScrollArea.setWidget(self._reconsWidget)
         self._reconsInfoDockWidget.setWidget(self._reconsWidgetScrollArea)
@@ -375,36 +446,65 @@ class ImageStack(_ImageStack):
         self.addDockWidget(qt.Qt.RightDockWidgetArea, self._reconsInfoDockWidget)
 
         # add scan overview dock widget
-        self._scanOverviewDockerWidget = qt.QDockWidget(parent=self)
-        self._scanOverviewDockerWidget.setMaximumHeight(300)
-        self._scanOverviewWidget = ScanOverviewWidget(self)
-        self._scanOverviewDockerWidget.setWidget(self._scanOverviewWidget)
-        self._scanOverviewDockerWidget.setFeatures(qt.QDockWidget.DockWidgetMovable)
-        self.addDockWidget(qt.Qt.RightDockWidgetArea, self._scanOverviewDockerWidget)
+        if show_overview:
+            self._scanOverviewDockerWidget = qt.QDockWidget(parent=self)
+            self._scanOverviewDockerWidget.setMaximumHeight(300)
+            self._scanOverviewWidget = ScanOverviewWidget(self)
+            self._scanOverviewDockerWidget.setWidget(self._scanOverviewWidget)
+            self._scanOverviewDockerWidget.setFeatures(qt.QDockWidget.DockWidgetMovable)
+            self.addDockWidget(
+                qt.Qt.RightDockWidgetArea, self._scanOverviewDockerWidget
+            )
+        else:
+            self._scanOverviewWidget = None
+            self._scanOverviewDockerWidget = None
 
         # for now hide the "control"
         self._urlsTable._toggleButton.hide()
 
+        # add an action to plot the url-image 'full size'
+        fullScreenIcon = icons.getQIcon("full_screen")
+        self._fullScreenAction = qt.QAction(fullScreenIcon, "pop up full screen")
+        self.getPlotWidget().toolBar().addAction(self._fullScreenAction)
+        self._fullScreenAction.triggered.connect(self._popCurrentImageFullScreen)
+
         # set up
         self.setAutoResetZoom(False)
-        if not show_overview:
-            self._scanOverviewDockerWidget.hide()
+
+    def _popCurrentImageFullScreen(self, *args, **kwargs):
+        new_plot = FullScreenPlot2D()
+        active_image = self.getPlotWidget().getActiveImage()
+        if active_image is None or active_image.getData(copy=False) is None:
+            return
+        url = self.getCurrentUrl()
+        window_title = f"Plot of {url.path()}" if url is not None else "Image Plot"
+
+        new_plot.setWindowTitle(window_title)
+        # reuse the same colormap for conveniance (user modification on it will be applied everywhere)
+        new_plot.setDefaultColormap(self.getPlotWidget().getDefaultColormap())
+
+        # add the current image
+        new_plot.addImage(active_image.getData(copy=True))
+
+        new_plot.showFullScreen()
 
     def setScan(self, scan):
-        self._scanOverviewWidget.setScan(scan=scan)
+        if self._scanOverviewWidget is not None:
+            self._scanOverviewWidget.setScan(scan=scan)
+
+    def getUrlListDockWidget(self):
+        return self._tableDockWidget
 
     def resetZoom(self):
-        self._plot.getPlotWidget().resetZoom()
+        self._plot.resetZoom()
 
     def setLimits(self, x_min, x_max, y_min, y_max):
-        plot = self._plot.getPlotWidget()
-        plot.setLimits(xmin=x_min, ymin=y_min, xmax=x_max, ymax=y_max)
+        self._plot.setLimits(xmin=x_min, ymin=y_min, xmax=x_max, ymax=y_max)
 
     def getLimits(self):
-        plot = self._plot.getPlotWidget()
         limits = []
-        limits.extend(plot.getGraphXLimits())
-        limits.extend(plot.getGraphYLimits())
+        limits.extend(self._plot.getGraphXLimits())
+        limits.extend(self._plot.getGraphYLimits())
         return tuple(limits)
 
     def setSliceReconsParamsVisible(self, visible):
@@ -414,7 +514,8 @@ class ImageStack(_ImageStack):
 
     def setScanOverviewVisible(self, visible):
         """show or not overview of the scan"""
-        self._scanOverviewDockerWidget.setVisible(visible)
+        if self._scanOverviewDockerWidget is not None:
+            self._scanOverviewDockerWidget.setVisible(visible)
 
     def setNormalizationFct(self, fct, url_indexes=None):
         self._normFct = fct
@@ -435,12 +536,21 @@ class ImageStack(_ImageStack):
                 return
 
             if data.ndim != 2:
-                if data.ndim == 3 and data.shape[0] == 1:
-                    data = data.reshape((data.shape[1], data.shape[2]))
+                if data.ndim == 3:
+                    if data.shape[0] == 1:
+                        # if reconstruction along z
+                        data = data.reshape((data.shape[1], data.shape[2]))
+                    elif data.shape[1] == 1:
+                        # if reconstruction along y
+                        data = data.reshape((data.shape[0], data.shape[2]))
+                    elif data.shape[2] == 1:
+                        # if reconstruction along z
+                        data = data.reshape((data.shape[0], data.shape[1]))
+                    else:
+                        _logger.warning(f"Image Stack only manage 2D data. Url: {url}")
+                        return
                 else:
-                    _logger.warning(
-                        "Image Stack only manage 2D data. Url: {}" "".format(url)
-                    )
+                    _logger.warning(f"Image Stack only manage 2D data. Url: {url}")
                     return
             if self._normFct is None:
                 self._urlData[url] = data
@@ -449,10 +559,14 @@ class ImageStack(_ImageStack):
                 self._urlData[url] = norm_data
 
             if self.getCurrentUrl().path() == url:
-                self._plot.setData(self._urlData[url])
+                self._plot.addImage(self._urlData[url])
+                if hasattr(self, "getWaiterOverlay"):
+                    self.getWaiterOverlay().hide()
+                else:
+                    self._waitingOverlay.hide()
                 if self._resetZoom:
                     self._resetZoom = False
-                    self._plot.getPlotWidget().resetZoom()
+                    self._plot.resetZoom()
 
             if sender in self._loadingThreads:
                 self._loadingThreads.remove(sender)
@@ -461,21 +575,28 @@ class ImageStack(_ImageStack):
     def setResetZoomOnNextIteration(self, reset):
         self._resetZoom = reset
 
-    def setCurrentUrl(self, url):
-        if self._loadSliceParams:
+    def setCurrentUrl(self, url: DataUrl):
+        if url in ("", None):
+            url = None
+        elif isinstance(url, str):
+            url = DataUrl(path=url)
+        elif not isinstance(url, DataUrl):
+            raise TypeError
+        if url is None:
+            pass
+            # FIXME: add a function to clear the metadata widget
+        elif self._loadSliceParams:
             try:
-                self._reconsWidget.setUrl(url)
-            except Exception as e:
-                _logger.info(
-                    "Unable to set reconstruction parameters from"
-                    " {}. Not handled for pyhst reconstructions. "
-                    "Error is {}".format(url, str(e))
+                self._reconsWidget.setVolumeMetadata(
+                    self._metadatas.get(url.path(), None)
                 )
+            except Exception:
+                _logger.info(f"Failed to find any metadata for {url.path()}.")
         super().setCurrentUrl(url)
 
     def setUrls(self, urls: list):
         _ImageStack.setUrls(self, urls)
-        listWidget = self._urlsTable._urlsTable._listWidget
+        listWidget = self._urlsTable._urlsTable
         items = []
         for i in range(listWidget.count()):
             # TODO: do this on the fly
@@ -484,43 +605,29 @@ class ImageStack(_ImageStack):
                 url = DataUrl(path=item.text())
             except Exception:
                 _logger.info(
-                    "fail to deduce data of last modification for "
-                    "{}".format(item.text())
+                    f"fail to deduce data of last modification for {item.text()}"
                 )
             else:
                 if os.path.exists(url.file_path()):
                     lst_m = time.ctime(os.path.getmtime(url.file_path()))
-                    item.setToolTip("last modification : {}".format(lst_m))
+                    item.setToolTip(f"last modification : {lst_m}")
             items.append(listWidget.item(i))
 
+    def setMetadatas(self, metadatas: dict):
+        """
+        :param dict metadata: for each url define a dict of the metadata
+        """
+        self._metadatas = metadatas
+
 
 class _TomwerUrlLoader(UrlLoader):
     """
     Thread use to load DataUrl
     """
 
-    def __init__(self, parent, url):
-        super(UrlLoader, self).__init__(parent=parent)
-        assert isinstance(url, DataUrl)
-        self.url = url
-        self.data = None
-
     def run(self):
         if self.url.file_path().endswith(".vol"):
             self.data = self._load_vol()
-        elif self.url.scheme() == "cv2":
-            if has_cv2:
-                self.data = cv2.imread(self.url.file_path(), -1)
-            else:
-                self.data = None
-                _logger.warning(
-                    f"need to install cv2 to read cast file {self.url.file_path()} because fabio fails to read float16"
-                )
-        elif self.url.scheme() == "numpy":
-            data = numpy.load(self.url.file_path())
-            if isinstance(data, numpy.lib.npyio.NpzFile):
-                data = data["result"]
-            self.data = data
         elif self.url.scheme() == "tomwer":
             if has_PIL:
                 self.data = numpy.array(Image.open(self.url.file_path()))
@@ -539,9 +646,7 @@ class _TomwerUrlLoader(UrlLoader):
             except ValueError:
                 self.data = None
                 _logger.warning(
-                    "Fail to open {}. Maybe the reconstruction failed.".format(
-                        self.url.path()
-                    )
+                    f"Fail to open {self.url.path()}. Maybe the reconstruction failed."
                 )
 
     def _load_vol(self):
@@ -585,9 +690,7 @@ class _TomwerUrlLoader(UrlLoader):
                         data = data.reshape(shape)
                     except ValueError:
                         _logger.warning(
-                            "unable to fix shape for raw file {}. "
-                            "Look for information in {}"
-                            "".format(raw_file, info_file)
+                            f"unable to fix shape for raw file {raw_file}. Look for information in {info_file}"
                         )
                         try:
                             sqr = int(numpy.sqrt(len(data)))
@@ -595,15 +698,12 @@ class _TomwerUrlLoader(UrlLoader):
                             data = data.reshape(shape)
                         except ValueError:
                             _logger.info(
-                                "deduction of shape size for {} "
-                                "failed".format(raw_file)
+                                f"deduction of shape size for {raw_file} failed"
                             )
                             data = None
                         else:
                             _logger.warning(
-                                f"try deducing shape size for {raw_file} "
-                                "might be an incorrect "
-                                "interpretation"
+                                f"try deducing shape size for {raw_file} might be an incorrect interpretation"
                             )
         if self.url.data_slice() is None:
             return data
