@@ -28,43 +28,49 @@ contains gui relative frame difference display
 
 __authors__ = ["H. Payno"]
 __license__ = "MIT"
-__date__ = "07/12/2021"
+__date__ = "23/03/2022"
 
 
 import functools
 import logging
 import os
+
 import numpy
 from processview.core.dataset import DatasetIdentifier
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.utils.image import shift_img
-from tomwer.io.utils import get_slice_data
-from tomwer.gui.qfolderdialog import QScanDialog
-from tomwer.gui.reconstruction.axis.CompareImages import CompareImages as _CompareImages
-from tomwer.gui.visualization.diffviewer.shiftwidget import TwoFramesShiftTab
+from silx.gui import icons as silx_icons
 from silx.gui import qt
 from silx.io.url import DataUrl
 from silx.utils.enum import Enum as _Enum
 
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils.image import shift_img
+from tomwer.gui.qfolderdialog import QDataDialog
+from tomwer.gui.reconstruction.axis.CompareImages import CompareImages as _CompareImages
+from tomwer.gui.settings import Y_AXIS_DOWNWARD
+from tomwer.gui.utils.completer import UrlCompleterDialog
+from tomwer.gui.visualization.diffviewer.shiftwidget import TwoFramesShiftTab
+from tomwer.io.utils import get_slice_data
+
 _logger = logging.getLogger(__name__)
 
 
 class CompareImages(_CompareImages):
-    def __init__(self, parent=None):
-        super().__init__(parent=parent)
+    def __init__(self, parent=None, backend=None):
+        super().__init__(parent=parent, backend=backend)
         # ignore Pan with arrow keys. This should be used by the "arrowControlWidget"
         self.getPlot().setPanWithArrowKeys(False)
         self._firstVisible = True
 
-    def setVisible(self, bool):
+    def setVisible(self, visible):
         if self._firstVisible:
             # hack to enforce matplotlib to replot the widget.
             # because it looks like it is missing to find the correct viewport
             # when not visible
             self.getPlot().resetZoom()
             self._firstVisible = False
-        super().setVisible(bool)
+        super().setVisible(visible)
 
 
 class _FrameSelector(qt.QWidget):
@@ -86,21 +92,31 @@ class _FrameSelector(qt.QWidget):
     def __init__(self, parent=None):
         qt.QWidget.__init__(self, parent)
         self._scan = None
-
-        self.setLayout(qt.QFormLayout())
+        self._currentFrameUrlsText = []
+        self.setLayout(qt.QGridLayout())
+        self.layout().addWidget(qt.QLabel("frame type", self), 0, 0, 1, 1)
         self._frameTypeCB = qt.QComboBox(self)
         for frame_type in _FrameSelector.FrameType:
             self._frameTypeCB.addItem(frame_type.value)
-        self.layout().addRow("frame type", self._frameTypeCB)
+        self.layout().addWidget(self._frameTypeCB, 0, 1, 1, 3)
+
+        self.layout().addWidget(qt.QLabel("frame", self), 1, 0, 1, 1)
         self._frameUrlCB = qt.QComboBox(self)
-        self.layout().addRow("frame", self._frameUrlCB)
+        self.layout().addWidget(self._frameUrlCB, 1, 1, 1, 3)
+        search_icon = silx_icons.getQIcon("zoom")
+        self._researchUrlButton = qt.QPushButton(parent=self, icon=search_icon)
+        self._researchUrlButton.setToolTip("search for a specific url")
+        self._researchUrlButton.setFixedWidth(30)
+        self.layout().addWidget(self._researchUrlButton, 1, 4, 1, 1)
+
         self._proj_normalized = qt.QCheckBox("corrected", self)
-        self.layout().addRow("", self._proj_normalized)
+        self.layout().addWidget(self._proj_normalized, 2, 1, 1, 1)
 
         # connect signal / slot
         self._frameTypeCB.currentIndexChanged.connect(self._typeChanged)
         self._frameUrlCB.currentIndexChanged.connect(self._urlChanged)
         self._proj_normalized.toggled.connect(self._correctionChanged)
+        self._researchUrlButton.released.connect(self._searchUrl)
 
         # default settings
         self._proj_normalized.setChecked(True)
@@ -132,6 +148,8 @@ class _FrameSelector(qt.QWidget):
             return
 
         urls = []
+        angles = []
+        self._currentFrameUrlsText.clear()
         self._frameUrlCB.clear()
         if type_selected == self.FrameType.DARKS:
             if self._scan.darks is not None:
@@ -140,7 +158,12 @@ class _FrameSelector(qt.QWidget):
             if self._scan.flats is not None:
                 urls = self._scan.flats.values()
         elif type_selected == self.FrameType.PROJ:
-            if self._scan.projections is not None:
+            if isinstance(self._scan, NXtomoScan):
+                angles_and_urls = self._scan.get_proj_angle_url(with_alignment=False)
+                for angle, url in angles_and_urls.items():
+                    urls.append(url)
+                    angles.append(angle)
+            else:
                 urls = self._scan.projections.values()
         elif type_selected == self.FrameType.ALIGN_PROJ:
             if self._scan.alignment_projections is not None:
@@ -148,21 +171,46 @@ class _FrameSelector(qt.QWidget):
         elif type_selected == self.FrameType.RECON_SLICES:
             urls = self._scan.get_reconstructions_urls()
         else:
-            raise ValueError("Type {} not managed".format(type_selected))
+            raise ValueError(f"Type {type_selected} not managed")
         urls = sorted(urls, key=lambda url: url.path())
-        for url in urls:
+
+        # if thre is some angles missing, avoiding setting any angle because they are probably wrong
+        # this will probably fail with EDF but this is legacy
+        if len(angles) != len(urls):
+            angles = []
+
+        for i_url, url in enumerate(urls):
+            if len(angles) > 0:
+                text = f"angle={angles[i_url]}&"
+            else:
+                text = ""
             if url.data_slice() is not None:
-                text = "?slice=".join(
+                text = f"?{text}slice=".join(
                     (os.path.basename(url.file_path()), str(url.data_slice()))
                 )
             else:
                 text = os.path.basename(url.file_path())
             user_data = url.path()
             self._frameUrlCB.addItem(text, user_data)
+            self._currentFrameUrlsText.append(text)
 
     def _urlChanged(self, *args, **kwargs):
         self.sigSelectedUrlChanged.emit()
 
+    def _searchUrl(self):
+        current_url = self._frameUrlCB.currentText()
+        dialog = UrlCompleterDialog(
+            current_url=current_url,
+            urls=tuple(self._currentFrameUrlsText),
+        )
+        if dialog.exec():
+            new_url = dialog.selected_url()
+            idx = self._frameUrlCB.findText(new_url)
+            if idx >= 0:
+                self._frameUrlCB.setCurrentIndex(idx)
+            else:
+                _logger.warning(f"Unable to find {new_url} in the list of urls")
+
     def _correctionChanged(self, *args, **kwargs):
         self.sigCorrectionChanged.emit()
 
@@ -180,6 +228,7 @@ class _FrameSelector(qt.QWidget):
     def clear(self):
         self._scan = None
         self._frameUrlCB.clear()
+        self._currentFrameUrlsText = []
 
 
 class _FramesSelector(qt.QWidget):
@@ -199,21 +248,29 @@ class _FramesSelector(qt.QWidget):
     def __init__(self, parent=None):
         qt.QWidget.__init__(self, parent)
         self.setLayout(qt.QGridLayout())
+        self.setContentsMargins(0, 0, 0, 0)
+        self.layout().setContentsMargins(0, 0, 0, 0)
         self._leftScanCB = _ScanComboBox(self)
+        self._leftScanCB.layout().setContentsMargins(0, 0, 0, 0)
         self.layout().addWidget(self._leftScanCB, 0, 0, 1, 4)
         self._rightScanCB = _ScanComboBox(self)
+        self._rightScanCB.layout().setContentsMargins(0, 0, 0, 0)
         self.layout().addWidget(self._rightScanCB, 0, 4, 1, 4)
 
         self._leftSelector = _FrameSelector(parent=self)
+        self._leftSelector.layout().setSpacing(2)
         self.layout().addWidget(self._leftSelector, 1, 0, 3, 4)
+        self._leftSelector.layout().setContentsMargins(0, 0, 0, 0)
         self._rightSelector = _FrameSelector(parent=self)
+        self._rightSelector.layout().setSpacing(2)
         self.layout().addWidget(self._rightSelector, 1, 4, 3, 4)
+        self._rightSelector.layout().setContentsMargins(0, 0, 0, 0)
 
         # signal / slot connection
-        self._leftSelector.sigCorrectionChanged.connect(self._leftFrameUpdReq)
-        self._rightSelector.sigCorrectionChanged.connect(self._rightFrameUpdReq)
-        self._leftSelector.sigSelectedUrlChanged.connect(self._leftFrameUpdReq)
-        self._rightSelector.sigSelectedUrlChanged.connect(self._rightFrameUpdReq)
+        self._leftSelector.sigCorrectionChanged.connect(self.sigLeftFrameUpdateReq)
+        self._rightSelector.sigCorrectionChanged.connect(self.sigRightFrameUpdateReq)
+        self._leftSelector.sigSelectedUrlChanged.connect(self.sigLeftFrameUpdateReq)
+        self._rightSelector.sigSelectedUrlChanged.connect(self.sigRightFrameUpdateReq)
         self._leftScanCB.sigScanChanged.connect(self._leftScanChanged)
         self._rightScanCB.sigScanChanged.connect(self._rightScanChanged)
         self._leftScanCB.sigRequestScanAdd.connect(self._addScanFrmDialogLeft)
@@ -223,21 +280,21 @@ class _FramesSelector(qt.QWidget):
         return self._leftSelector.getScan()
 
     def setLeftScan(self, scan):
-        self._leftScanCB.setScan(scan.get_dataset_identifier())
+        self._leftScanCB.setScan(scan.get_identifier())
         self._leftSelector.setScan(scan)
 
     def getRightScan(self):
         return self._rightSelector.getScan()
 
     def setRightScan(self, scan):
-        self._rightScanCB.setScan(scan.get_dataset_identifier())
+        self._rightScanCB.setScan(scan.get_identifier())
         self._rightSelector.setScan(scan)
 
     def addScan(self, scan):
         if not isinstance(scan, TomwerScanBase):
-            raise TypeError("scan should be an instance of {}".format(TomwerScanBase))
-        self._leftScanCB.addScan(scan.get_dataset_identifier())
-        self._rightScanCB.addScan(scan.get_dataset_identifier())
+            raise TypeError(f"scan should be an instance of {TomwerScanBase}")
+        self._leftScanCB.addScan(scan.get_identifier())
+        self._rightScanCB.addScan(scan.get_identifier())
 
     def _leftScanChanged(self):
         self._leftSelector.setScan(self._leftScanCB.getCurrentScan())
@@ -257,12 +314,6 @@ class _FramesSelector(qt.QWidget):
     def getRightUrl(self):
         return self._rightSelector.getCurrentUrl()
 
-    def _rightFrameUpdReq(self):
-        self.sigRightFrameUpdateReq.emit()
-
-    def _leftFrameUpdReq(self):
-        self.sigLeftFrameUpdateReq.emit()
-
     def _addScanFrmDialogLeft(self):
         self._addScanFrmDialog(link_to="left")
 
@@ -272,7 +323,7 @@ class _FramesSelector(qt.QWidget):
     def _addScanFrmDialog(self, link_to: str):
         assert link_to in ("left", "right"), "add scan should be in " "'left', 'right'"
 
-        dialog = QScanDialog(self, multiSelection=True)
+        dialog = QDataDialog(self, multiSelection=True)
 
         if not dialog.exec_():
             dialog.close()
@@ -285,21 +336,19 @@ class _FramesSelector(qt.QWidget):
                 scans = ScanFactory.create_scan_objects(scan_path=folder)
             except Exception as e:
                 _logger.error(
-                    "cannot create scan instances from {}. Error is {}".format(
-                        folder, e
-                    )
+                    f"cannot create scan instances from {folder}. Error is {e}"
                 )
             else:
                 for scan in scans:
-                    self._leftScanCB.addScan(scan.get_dataset_identifier())
-                    self._rightScanCB.addScan(scan.get_dataset_identifier())
+                    self._leftScanCB.addScan(scan.get_identifier())
+                    self._rightScanCB.addScan(scan.get_identifier())
                     scan_to_set = scan
 
         if scan_to_set is not None:
             if link_to == "left":
-                self._leftScanCB.setScan(scan_id=scan_to_set.get_dataset_identifier())
+                self._leftScanCB.setScan(scan_id=scan_to_set.get_identifier())
             else:
-                self._rightScanCB.setScan(scan_id=scan_to_set.get_dataset_identifier())
+                self._rightScanCB.setScan(scan_id=scan_to_set.get_identifier())
 
     def clear(self):
         self._leftScanCB.clear()
@@ -316,7 +365,7 @@ class DiffFrameViewer(qt.QMainWindow):
     or dark.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, backend=None):
         qt.QMainWindow.__init__(self, parent)
         self.setWindowFlags(qt.Qt.Widget)
         # add frame selector
@@ -324,8 +373,8 @@ class DiffFrameViewer(qt.QMainWindow):
         self._framesSelector.setContentsMargins(0, 0, 0, 0)
         self._framesSelector.layout().setContentsMargins(0, 0, 0, 0)
         self._framesSelectorDW = qt.QDockWidget(parent=self)
+        self._framesSelectorDW.setWindowTitle("inputs")
         self._framesSelectorDW.setWidget(self._framesSelector)
-        self._framesSelectorDW.setFeatures(qt.QDockWidget.DockWidgetMovable)
         self.addDockWidget(qt.Qt.TopDockWidgetArea, self._framesSelectorDW)
 
         # add shift dock widget
@@ -334,17 +383,21 @@ class DiffFrameViewer(qt.QMainWindow):
 
         self._shiftDW = qt.QDockWidget(parent=self)
         self._shiftDW.setWidget(self._shiftsWidget)
-        self._shiftDW.setFeatures(qt.QDockWidget.DockWidgetMovable)
-        self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._shiftDW)
+        self._shiftDW.setWindowTitle("shifts")
+        self.addDockWidget(qt.Qt.TopDockWidgetArea, self._shiftDW)
 
         # define central widget
-        self._mainWidget = CompareImages(parent=self)
+        self._mainWidget = CompareImages(parent=self, backend=backend)
         self._mainWidget.setVisualizationMode(
             CompareImages.VisualizationMode.COMPOSITE_A_MINUS_B
         )
+        self._mainWidget.getPlot().setYAxisInverted(Y_AXIS_DOWNWARD)
 
         self.setCentralWidget(self._mainWidget)
 
+        # tabify the two dock widget to reduce space occupy
+        self.tabifyDockWidget(self._shiftDW, self._framesSelectorDW)
+
         # toolbar
         # add toolbar
         toolbar = qt.QToolBar(self)
@@ -352,13 +405,6 @@ class DiffFrameViewer(qt.QMainWindow):
         toolbar.setFloatable(False)
         self.addToolBar(qt.Qt.TopToolBarArea, toolbar)
 
-        # add filtering
-        style = qt.QApplication.instance().style()
-        icon = style.standardIcon(qt.QStyle.SP_DialogResetButton)
-        self._clearAction = qt.QAction(icon, "Clear", toolbar)
-        toolbar.addAction(self._clearAction)
-        self._clearAction.triggered.connect(self.clear)
-
         # set up
         self._mainWidget.setAutoResetZoom(False)
         self._shiftsWidget.setFocus(qt.Qt.OtherFocusReason)
@@ -415,7 +461,7 @@ class DiffFrameViewer(qt.QMainWindow):
             if data.ndim == 3 and data.shape[0] == 1:
                 data = data.reshape((data.shape[1], data.shape[2]))
             else:
-                _logger.error("Cannot display {}. Should be 2D".format(url.path()))
+                _logger.error(f"Cannot display {url.path()}. Should be 2D")
                 return
 
         if self.isFrameALRFlip():
@@ -448,7 +494,7 @@ class DiffFrameViewer(qt.QMainWindow):
             if data.ndim == 3 and data.shape[0] == 1:
                 data = data.reshape((data.shape[1], data.shape[2]))
             else:
-                _logger.error("Cannot display {}. Should be 2D".format(url.path()))
+                _logger.error(f"Cannot display {url.path()}. Should be 2D")
                 return
         if self.isFrameBLRFlip():
             data = numpy.fliplr(data)
@@ -457,10 +503,6 @@ class DiffFrameViewer(qt.QMainWindow):
         )
         self._mainWidget.setImage2(shifted_image)
 
-    def clear(self):
-        self._framesSelector.clear()
-        self._mainWidget.getPlot().clear()
-
     def _frameShiftsChanged(self):
         self._resetLeftFrame()
         self._resetRightFrame()
@@ -494,7 +536,7 @@ class _ScanComboBox(qt.QWidget):
 
         # connect signal / slot
         self._scansCB.currentIndexChanged.connect(self._scanChanged)
-        self._addButton.pressed.connect(self._addScanCallback)
+        self._addButton.released.connect(self.sigRequestScanAdd)
 
     def clear(self):
         self._scans = set()
@@ -522,13 +564,7 @@ class _ScanComboBox(qt.QWidget):
         try:
             scan = DatasetIdentifier.recreate_dataset(dataset_id)
         except Exception as e:
-            _logger.error(
-                "Fail to recreate dataset from {}. Reason is {}."
-                "".format(str(dataset_id), e)
-            )
+            _logger.error(f"Fail to recreate dataset from {dataset_id}. Reason is {e}.")
             return None
         else:
             return scan
-
-    def _addScanCallback(self):
-        self.sigRequestScanAdd.emit()
