@@ -31,11 +31,14 @@ __license__ = "MIT"
 __date__ = "26/03/2020"
 
 
-from silx.gui import qt
-from silx.gui.plot.ImageStack import _PlotWithWaitingLabel
 import logging
 import weakref
 
+from silx.gui import qt
+from silx.gui.plot import Plot2D
+from silx.gui.widgets.WaitingOverlay import WaitingOverlay
+from tomwer.gui.settings import Y_AXIS_DOWNWARD
+
 _logger = logging.getLogger(__name__)
 
 
@@ -50,19 +53,29 @@ class SinogramViewer(qt.QMainWindow):
     sigSinoLoadEnded = qt.Signal()
     """Signal emitted when a computation is ended"""
 
-    def __init__(self, parent=None, scan=None, opts_orientation=qt.Qt.Vertical):
+    def __init__(
+        self,
+        parent=None,
+        scan=None,
+        opts_orientation=qt.Qt.Vertical,
+        backend=None,
+    ):
         qt.QMainWindow.__init__(self, parent)
         self._scan = None
         self._sinoInfoCache = None
         # used to memorize sinogram properties when load it.
         # Contains (str(scan), line, oversampling)
 
-        self._plot = _PlotWithWaitingLabel(parent=self)
-        plot = self._plot.getPlotWidget()
-        plot.getDefaultColormap().setVRange(None, None)
-        plot.setAxesDisplayed(False)
-        plot.setKeepDataAspectRatio(True)
-        self._plot.setWaiting(False)
+        self._plot = Plot2D(parent=self)
+        self._plotWaiter = WaitingOverlay(self._plot)
+        self._plotWaiter.hide()
+        self._plotWaiter.setIconSize(qt.QSize(30, 30))
+
+        self._plot.getMaskAction().setVisible(False)
+        self._plot.setYAxisInverted(Y_AXIS_DOWNWARD)
+        self._plot.getDefaultColormap().setVRange(None, None)
+        self._plot.setAxesDisplayed(False)
+        self._plot.setKeepDataAspectRatio(True)
         self._dockOpt = qt.QDockWidget(self)
         self._options = SinogramOpts(parent=self, orientation=opts_orientation)
         self._dockOpt.setWidget(self._options)
@@ -79,7 +92,7 @@ class SinogramViewer(qt.QMainWindow):
         self._loadingThread.finished.connect(self._sinogram_loaded)
 
         # expose API
-        self.getActiveImage = plot.getActiveImage
+        self.getActiveImage = self._plot.getActiveImage
 
         # set up
         if scan is not None:
@@ -108,22 +121,19 @@ class SinogramViewer(qt.QMainWindow):
         """
         self._options.setRadioLine(line)
 
+    def getLine(self):
+        return self._options.getRadioLine()
+
     def setSubsampling(self, value):
         self._options.setSubsampling(value)
 
     def _updatePlot(self, sinogram):
-        self._plot.getPlotWidget().addImage(data=sinogram)
-        self._plot.getPlotWidget().replot()
-
-    def _editionLocked(self, locked):
-        self.sigAxisEditionLocked.emit(locked)
-
-    def _applyRequested(self):
-        self.sigApply.emit()
+        self._plot.addImage(data=sinogram)
+        self._plot.replot()
 
     def _sinogram_loaded(self):
         """callback when the sinogram is loaded"""
-        self._plot.setWaiting(False)
+        self._plotWaiter.hide()
         if self._scan is None or self._scan() is None:
             return
         assert self._sinoInfoCache is not None
@@ -134,7 +144,7 @@ class SinogramViewer(qt.QMainWindow):
             return
 
         # note: cache avoid reading data twice here.
-        sinogram = self._scan().get_sinogram(line=line, subsampling=subsampling)
+        sinogram = self._scan().get_normed_sinogram(line=line, subsampling=subsampling)
         self._updatePlot(sinogram=sinogram)
         self.sigSinoLoadEnded.emit()
         self._options.setEnabled(True)
@@ -148,7 +158,7 @@ class SinogramViewer(qt.QMainWindow):
             )
             return
         # update scan
-        self._plot.setWaiting(True)
+        self._plotWaiter.show()
         self.sigSinoLoadStarted.emit()
         self._sinoInfoCache = (
             str(self._scan()),
@@ -156,34 +166,33 @@ class SinogramViewer(qt.QMainWindow):
             self._options.getSubsampling(),
         )
         self._loadingThread.init(
-            scan=self._scan(),
+            data=self._scan(),
             line=self._options.getRadioLine(),
             subsampling=int(self._options.getSubsampling()),
         )
         self._loadingThread.start()
 
     def clear(self):
-        self._plot.getPlotWidget().clear()
+        self._plot.clear()
 
     def close(self):
-        self._stopAnimationThread()
+        self._plotWaiter.hide()
         self._plot.close()
         self._plot = None
         super().close()
 
-    def _stopAnimationThread(self):
-        self._plot.stopUpdateThread()
-
 
 class _LoadSinoThread(qt.QThread):
-    def init(self, scan, line, subsampling):
-        self._scan = scan
+    def init(self, data, line, subsampling):
+        self._scan = data
         self._line = line
         self._subsampling = subsampling
 
     def run(self):
         try:
-            self._scan.get_sinogram(line=self._line, subsampling=self._subsampling)
+            self._scan.get_normed_sinogram(
+                line=self._line, subsampling=self._subsampling
+            )
         except ValueError as e:
             _logger.error(e)
 
@@ -250,15 +259,12 @@ class SinogramOpts(qt.QDialog):
 
         # connect signal / slot
         self._buttons.button(qt.QDialogButtonBox.Apply).clicked.connect(
-            self._updateSinogram
+            self.sigUpdateRequested
         )
 
     def setLineSelectionVisible(self, visible):
         self._lineSelWidget.setVisible(visible)
 
-    def _updateSinogram(self):
-        self.sigUpdateRequested.emit()
-
     def setScan(self, scan):
         old = self.blockSignals(True)
         # update line max and value
