@@ -1,63 +1,24 @@
-# coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
-
-__authors__ = ["H. Payno"]
-__license__ = "MIT"
-__date__ = "05/08/2020"
-
-
-from silx.gui import qt
-from tomoscan.io import HDF5File
-from silx.gui.data.DataViewerFrame import DataViewerFrame
-from silx.gui.dialog.ColormapDialog import ColormapDialog
-from silx.gui.data.DataViews import DataViewHooks
-from silx.io.utils import get_data
-from silx.gui.data.DataViews import IMAGE_MODE
-import os
-from tomwer.core.scan.scanbase import TomwerScanBase
 import logging
-from tomwer.core.utils.ftseriesutils import get_vol_file_shape
 import weakref
-from tomwer.gui.visualization.reconstructionparameters import ReconstructionParameters
-import silx
-from silx.gui.colors import Colormap
+import h5py
+from typing import Union
+
 import numpy
 import numpy.lib.npyio
+import silx
+from silx.gui import qt
+from silx.gui.colors import Colormap
+from silx.gui.data.DataViewerFrame import DataViewerFrame
+from silx.gui.data.DataViews import IMAGE_MODE, DataViewHooks
+from silx.gui.dialog.ColormapDialog import ColormapDialog
+from tomoscan.esrf.volume.hdf5volume import HDF5Volume
+from tomoscan.factory import Factory
+from tomoscan.identifier import VolumeIdentifier
+from tomoscan.volumebase import VolumeBase
 
-try:
-    from PIL import Image
-except ImportError:
-    has_PIL = False
-else:
-    has_PIL = True
-try:
-    import cv2
-except ImportError:
-    has_cv2 = False
-else:
-    has_cv2 = True
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.gui.visualization.reconstructionparameters import ReconstructionParameters
+from silx.gui.widgets.WaitingOverlay import WaitingOverlay
 
 _logger = logging.getLogger(__name__)
 
@@ -254,6 +215,9 @@ class _TomoApplicationContext(DataViewHooks):
 class VolumeViewer(qt.QMainWindow):
     def __init__(self, parent):
         qt.QMainWindow.__init__(self, parent)
+
+        self._volume_loaded_in_background = (None, None)
+        # store the volume loaded in the background and the thread used for it as (volume_identifier, thread)
         self._centralWidget = DataViewerFrame(parent=self)
         self.__context = _TomoApplicationContext(self)
         self._centralWidget.setGlobalHooks(self.__context)
@@ -261,6 +225,12 @@ class VolumeViewer(qt.QMainWindow):
             qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding
         )
         self.setCentralWidget(self._centralWidget)
+        # waiter overlay to notify user loading is on-going
+        self._waitingOverlay = WaitingOverlay(self._centralWidget)
+        self._waitingOverlay.setIconSize(qt.QSize(30, 30))
+        self._waitingOverlay.hide()
+
+        # display scan information when possible
         self._infoWidget = _ScanInfo(parent=self)
 
         # top level dock widget to display information regarding the scan
@@ -274,6 +244,7 @@ class VolumeViewer(qt.QMainWindow):
         # add dock widget for reconstruction parameters
         self._reconsInfoDockWidget = qt.QDockWidget(parent=self)
         self._reconsWidgetScrollArea = qt.QScrollArea(self)
+        self._reconsWidgetScrollArea.setWidgetResizable(True)
         self._reconsWidget = ReconstructionParameters(self)
         self._reconsWidgetScrollArea.setWidget(self._reconsWidget)
         self._reconsInfoDockWidget.setWidget(self._reconsWidgetScrollArea)
@@ -284,7 +255,6 @@ class VolumeViewer(qt.QMainWindow):
         """pointer to the hdf5 file since we want to set the HDF5Dataset for
         loading data on the fly and avoid loading everything into memory for
         hdf5."""
-        self.__first_load = True
         self.__last_mode = None
 
     def _close_h5_file(self):
@@ -297,150 +267,182 @@ class VolumeViewer(qt.QMainWindow):
         super().close()
 
     def setScan(self, scan):
-        if scan is None:
-            self.clear()
-        elif len(scan.latest_vol_reconstructions) == 0:
-            _logger.warning("No reconstructed volume for {}".format(scan))
-            self.clear()
-            self._infoWidget.setScan(scan)
-        else:
-            self._set_data_urls(urls=scan.latest_vol_reconstructions)
-            self._infoWidget.setScan(scan)
-
-    def _get_data(self, url):
-        if url.file_path().endswith(".vol"):
-            data = self._load_vol(url)
-        elif url.scheme() == "numpy":
-            data = numpy.load(url.file_path())
-            if isinstance(data, numpy.lib.npyio.NpzFile):
-                data = data["result"]
-        elif url.scheme() == "cv2":
-            if has_cv2:
-                data = cv2.imread(url.file_path(), -1)
-            else:
-                data = None
-                _logger.warning(
-                    f"need to install cv2 to read cast file {url.file_path()} because fabio fails to read float16"
-                )
-        elif url.scheme() == "tomwer":
-            if has_PIL:
-                data = numpy.array(Image.open(url.file_path()))
-                if url.data_slice() is not None:
-                    data = data[url.data_slice()]
-            else:
-                _logger.warning(
-                    "need to install Pillow to read file " + url.file_path()
-                )
-                data = None
-        elif url.scheme() == "silx":
-            self._h5_file = HDF5File(url.file_path(), mode="r", swmr=True)
-            if url.data_path() in self._h5_file:
-                data = self._h5_file[url.data_path()]
-            else:
-                data = None
-        else:
-            data = get_data(url)
-        return data
-
-    def _set_data_urls(self, urls):
         self.clear()
-        if len(urls) == 1:
-            # for hdf5 we have to be carreful to use the dataset to
-            # insure loading on the fly
-            url = urls[0]
-            data = self._get_data(url)
-            # set volume dataset
-            if data is not None:
-                self._set_volume(data)
-            # set reconstruction parameters
+        if scan is None:
+            return
+
+        elif len(scan.latest_vol_reconstructions) == 0:
+            _logger.warning(f"No reconstructed volume for {scan}")
+            self.clear()
+            self._infoWidget.setScan(scan)
+        else:
+            self._set_volumes(volumes=scan.latest_vol_reconstructions)
+            self._infoWidget.setScan(scan)
+
+    def setVolume(self, volume):
+        self.clear()
+        if volume is None:
+            return
+        self._set_volumes(volumes=(volume,))
+
+    def _get_data_volume(self, volume: VolumeBase):
+        """
+        load the data of the requested volume.
+        :return: (data: Optional[str], state: str) state can be "loaded", "loading" or "failed"
+        """
+        if not isinstance(volume, VolumeBase):
+            raise TypeError(
+                f"volume is expected to be an instance of {VolumeBase}. Not {type(volume)}"
+            )
+
+        self._close_h5_file()
+        if isinstance(volume, HDF5Volume):
             try:
-                self._reconsWidget.setUrl(url)
+                self._h5_file = h5py.File(volume.data_url.file_path(), mode="r")
+            except OSError:
+                self._h5_file = h5py.File(
+                    volume.data_url.file_path(),
+                    mode="r",
+                    libver="latest",
+                    swmr=True,
+                )
+
+            if volume.data_url.data_path() in self._h5_file:
+                data = self._h5_file[volume.data_url.data_path()]
+                state = "loaded"
+            else:
+                data = None
+                state = "failed"
+        elif volume.data is not None:
+            data = volume.data
+            state = "loaded"
+        else:
+            volume_id, _ = self._volume_loaded_in_background
+            if volume_id == volume.get_identifier().to_str():
+                # special case if the user send several time the volume which is currently loading
+                # in this case we just want to ignore the request to avoid reloading the volume
+                # can happen in the case of a large volume that take some time to be loaded.
+                pass
+            else:
+                _logger.warning(
+                    "Attempt to set a non HDF5 volume to the viewer. This requires to load all the data in memory. This can take a while"
+                )
+                self._stopExistingLoaderThread()
+                self._loadAndDisplay(volume)
+            state = "loading"
+            data = None
+
+        return data, state
+
+    def _loaderThreadFinished(self):
+        """Callback activated when a VolumeLoader thread is finished"""
+        sender = self.sender()
+        if not isinstance(sender, VolumeLoader):
+            raise TypeError("sender is expected to be a VolumeLoader")
+
+        self._stopExistingLoaderThread()
+        if sender.volume.data is None:
+            _logger.error(f"Failed to load volume {sender.volume.get_identifier()}")
+        else:
+            self.setVolume(sender.volume)
+
+    def _loadAndDisplay(self, volume):
+        """Load a thread and add a callback when loading is done"""
+        loader_thread = VolumeLoader(volume=volume)
+        self._volume_loaded_in_background = (
+            volume.get_identifier().to_str(),
+            loader_thread,
+        )
+        loader_thread.finished.connect(self._loaderThreadFinished)
+        loader_thread.start()
+
+    def _stopExistingLoaderThread(self):
+        """Will stop any existing loader thread. Make sure we load one volume at most at the time"""
+        _, loader_thread = self._volume_loaded_in_background
+        if loader_thread is not None:
+            loader_thread.finished.disconnect(self._loaderThreadFinished)
+            if loader_thread.isRunning():
+                loader_thread.quit()
+            self._volume_loaded_in_background = (None, None)
+
+    def _set_volumes(self, volumes: tuple):
+        self.clear()
+        # for now handle a single volume
+        if len(volumes) == 0:
+            pass
+        else:
+            if len(volumes) > 1:
+                _logger.warning(
+                    "Only one volume can be displayed. Will display the first one"
+                )
+            volume = volumes[0]
+            if isinstance(volume, (str, VolumeIdentifier)):
+                volume = Factory.create_tomo_object_from_identifier(volume)
+            elif isinstance(volume, VolumeBase):
+                pass
+            else:
+                raise TypeError(
+                    f"Volume should be an instance of a Volume, a VolumeIdentifier or a string refering to a VolumeIdentifier. {type(volume)} provided"
+                )
+
+            # warning: load metadata before data because can get some conflict with the HDF5 reader flag if done after
+            try:
+                # warning: limitation expected for .vol as it gets two configuration file. The default one is vol.info and does not contains
+                # any of the metadata 'distance', 'pixel size'... but it is here for backward compatiblity
+                self._reconsWidget.setVolumeMetadata(
+                    volume.metadata or volume.load_metadata()
+                )
             except Exception as e:
                 _logger.info(
-                    "Unable to set reconstruction parameters from"
-                    " {}. Not handled for pyhst reconstructions. "
-                    "Error is {}".format(url, str(e))
+                    f"Unable to set reconstruction parameters from {volume.data_url}. Not handled for pyhst reconstructions. Error is {e}"
                 )
-        else:
-            data = [self._get_data(url) for url in urls]
-            data = numpy.array(data)
-            self._set_volume(data)
 
-    def _set_volume(self, volume):
+            data, state = self._get_data_volume(volume)
+            # set volume dataset
+            if state == "loading":
+                self._waitingOverlay.show()
+            elif state == "loaded":
+                self._waitingOverlay.hide()
+                if data is not None:
+                    self._set_volume(data)
+
+            elif state == "failed":
+                _logger.warning(
+                    f"Failed to load data from {volume.get_identifier().to_str()}"
+                )
+                return
+
+    def _set_volume(self, volume: Union[numpy.ndarray, h5py.Dataset]):
         self._centralWidget.setData(volume)
-        if self.__first_load:
-            self._centralWidget.setDisplayMode(IMAGE_MODE)
-            self.__first_load = False
-        elif self.__last_mode is not None:
-            self._centralWidget.setDisplayMode(self.__last_mode)
+        self._centralWidget.setDisplayMode(self.__last_mode or IMAGE_MODE)
 
     def clear(self):
-        self.__last_mode = self._centralWidget.displayMode()
+        if self._centralWidget.displayMode():
+            self.__last_mode = self._centralWidget.displayMode()
         self._close_h5_file()
-        self._centralWidget.setData(None)
         self._infoWidget.clear()
+        self._centralWidget.setData(None)
+        # if clear stop loading any volume
+        self._stopExistingLoaderThread()
 
     def sizeHint(self):
         return qt.QSize(600, 600)
 
-    # TODO: should be merged with dataviewer._load_vol function ?
-    def _load_vol(self, url):
-        """
-        load a .vol file
-        """
-        if url.file_path().lower().endswith(".vol.info"):
-            info_file = url.file_path()
-            raw_file = url.file_path().replace(".vol.info", ".vol")
-        else:
-            assert url.file_path().lower().endswith(".vol")
-            raw_file = url.file_path()
-            info_file = url.file_path().replace(".vol", ".vol.info")
 
-        if not os.path.exists(raw_file):
-            data = None
-            mess = f"Can't find raw data file {raw_file} associated with {info_file}"
-            _logger.warning(mess)
-        elif not os.path.exists(info_file):
-            mess = f"Can't find info file {info_file} associated with {raw_file}"
-            _logger.warning(mess)
-            data = None
-        else:
-            shape = get_vol_file_shape(info_file)
-            if None in shape:
-                _logger.warning(f"Fail to retrieve data shape for {info_file}.")
-                data = None
-            else:
-                try:
-                    numpy.zeros(shape)
-                except MemoryError:
-                    data = None
-                    _logger.warning(f"Raw file {raw_file} is too large for being read")
-                else:
-                    data = numpy.fromfile(
-                        raw_file, dtype=numpy.float32, count=-1, sep=""
-                    )
-                    try:
-                        data = data.reshape(shape)
-                    except ValueError:
-                        _logger.warning(
-                            f"unable to fix shape for raw file {raw_file}. "
-                            "Look for information in {info_file}"
-                        )
-                        try:
-                            sqr = int(numpy.sqrt(len(data)))
-                            shape = (1, sqr, sqr)
-                            data = data.reshape(shape)
-                        except ValueError:
-                            _logger.info(
-                                f"deduction of shape size for {raw_file} " "failed"
-                            )
-                            data = None
-                        else:
-                            _logger.warning(
-                                f"try deducing shape size for {raw_file} "
-                                "might be an incorrect interpretation"
-                            )
-        if url.data_slice() is None:
-            return data
-        else:
-            return data[url.data_slice()]
+class VolumeLoader(qt.QThread):
+    """
+    simple thread that load a volume in memory
+    """
+
+    def __init__(self, volume: VolumeBase) -> None:
+        super().__init__()
+        if not isinstance(volume, VolumeBase):
+            raise TypeError()
+        self.__volume = volume
+
+    def run(self):
+        self.__volume.load_data(store=True)
+
+    @property
+    def volume(self):
+        return self.__volume
