@@ -31,14 +31,26 @@ __license__ = "MIT"
 __date__ = "07/09/2020"
 
 
-import numpy.lib.npyio
-import os
-from silx.io.utils import get_data as silx_get_data
-from tomoscan.io import HDF5File
-from tomwer.core.utils import ftseriesutils
-from PIL import Image
 import logging
+import os
+
 import h5py
+import numpy.lib.npyio
+from PIL import Image
+from silx.io.utils import open as open_hdf5
+from tomoscan.esrf import has_glymur
+from tomoscan.esrf.scan.utils import get_data as tomoscan_get_data
+from typing import Union
+
+from tomwer.core.utils import ftseriesutils
+
+try:
+    import tifffile  # noqa #F401 needed for later possible lazy loading
+except ImportError:
+    has_tifffile = False
+else:
+    has_tifffile = True
+
 
 _logger = logging.getLogger(__name__)
 
@@ -50,21 +62,34 @@ def get_slice_data(url):
             ".vol.info"
         ) or url.file_path().lower().endswith(".vol"):
             data = _loadVol(url)
-        elif url.scheme() == "numpy":
-            data = numpy.load(url.file_path())
-            if isinstance(data, numpy.lib.npyio.NpzFile):
-                data = data["result"]
-            data = data
+
         elif url.scheme() == "tomwer":
             data = numpy.array(Image.open(url.file_path()))
             if url.data_slice() is not None:
                 data = data[url.data_slice()]
+        elif url.scheme() == ("tifffile"):
+            if not has_tifffile:
+                _logger.warning("tifffile must be installed to read tiff")
+                data = None
+            else:
+                data = tifffile.imread(url.file_path())
+                if url.data_slice() is not None:
+                    data = data[url.data_slice()]
+        elif url.scheme() in ("jp2", "jp2k", "glymur"):
+            if not has_glymur:
+                _logger.warning("glymur must be installed to read jpeg2000")
+                data = None
+            else:
+                import glymur
+
+                data = glymur.Jp2k(url.file_path())[:]
         else:
             try:
-                data = silx_get_data(url)
+                data = tomoscan_get_data(url)
             except Exception as e:
-                _logger.error(e)
-                _logger.warning("file %s not longer exists or is empty" % url)
+                _logger.warning(
+                    f"file {url} not longer exists or is empty. Error is {e}"
+                )
                 data = None
     else:
         _logger.warning("file %s not longer exists or is empty" % url)
@@ -84,47 +109,41 @@ def _loadVol(url):
 
     if not os.path.exists(rawFile):
         data = None
-        mess = "Can't find raw data file %s associated with %s" % (rawFile, infoFile)
+        mess = f"Can't find raw data file {rawFile} associated with {infoFile}"
         _logger.warning(mess)
     elif not os.path.exists(infoFile):
-        mess = "Can't find info file %s associated with %s" % (infoFile, rawFile)
+        mess = f"Can't find info file {infoFile} associated with {rawFile}"
         _logger.warning(mess)
         data = None
     else:
         shape = ftseriesutils.get_vol_file_shape(infoFile)
         if None in shape:
-            _logger.warning("Fail to retrieve data shape for %s." % infoFile)
+            _logger.warning(f"Fail to retrieve data shape for {infoFile}.")
             data = None
         else:
             try:
                 numpy.zeros(shape)
             except MemoryError:
                 data = None
-                _logger.warning(
-                    "Raw file %s is to large for being " "readed %s" % rawFile
-                )
+                _logger.warning(f"Raw file {rawFile} is to large for being readed")
             else:
                 data = numpy.fromfile(rawFile, dtype=numpy.float32, count=-1, sep="")
                 try:
                     data = data.reshape(shape)
                 except ValueError:
                     _logger.warning(
-                        "unable to fix shape for raw file %s. "
-                        "Look for information in %s"
-                        "" % (rawFile, infoFile)
+                        f"unable to fix shape for raw file {rawFile}. Look for information in {infoFile}"
                     )
                     try:
                         sqr = int(numpy.sqrt(len(data)))
                         shape = (1, sqr, sqr)
                         data = data.reshape(shape)
                     except ValueError:
-                        _logger.info("deduction of shape size for %s failed" % rawFile)
+                        _logger.info(f"deduction of shape size for {rawFile} failed")
                         data = None
                     else:
                         _logger.warning(
-                            "try deducing shape size for %s "
-                            "might be an incorrect "
-                            "interpretation" % rawFile
+                            f"try deducing shape size for {rawFile} might be an incorrect interpretation"
                         )
     if url.data_slice() is None:
         return data
@@ -143,7 +162,10 @@ def get_default_directory() -> str:
     ):
         return os.environ["TOMWER_DEFAULT_INPUT_DIR"]
     else:
-        return os.getcwd()
+        try:
+            return os.getcwd()
+        except FileNotFoundError:
+            return os.sep
 
 
 def format_stderr_stdout(stdout, stderr, config=None):
@@ -154,102 +176,109 @@ def format_stderr_stdout(stdout, stderr, config=None):
     else:
         assert isinstance(config, dict)
     return (
-        "############# nabu ############## \nconfig: {}\n"
-        "------------- stderr -------------\n{}\n"
-        "------------- stdout -------------\n{}\n".format(config, s_err, s_out)
+        f"############# nabu ############## \nconfig: {config}\n"
+        f"------------- stderr -------------\n{s_err}\n"
+        f"------------- stdout -------------\n{s_out}\n"
     )
 
 
-def get_linked_files_with_entry(hdf5_file: str, entry: str) -> tuple:
+def get_linked_files_with_entry(hdf5_file: str, entry: str) -> set:
     """
-    parse all dataset under entry and look for connection with external files from vds or ExternalLink
+    parse all datasets under the entry and look for connections with external files (vds or ExternalLink)
     """
-    datasets_to_treat = set()
-    final_datasets = set()
-    already_checked_dataset = set()
+    items_to_treat = set()  # abs_file_path, file_path, data_path
+    final_datasets = set()  # file_path, dataset_path
+    treated_items = set()  # abs_file_path, data_path
 
-    # first datasets to be tested
-    datasets_to_treat.add(
-        (
-            hdf5_file,
-            entry,
-        ),
-    )
-    while len(datasets_to_treat) > 0:
-        to_treat = list(datasets_to_treat)
-        datasets_to_treat.clear()
-        # browse this dataset
-        for (file_path, dataset_path) in to_treat:
-            if (file_path, dataset_path) in already_checked_dataset:
+    abs_hdf5_file = os.path.abspath(hdf5_file)
+    items_to_treat.add((abs_hdf5_file, hdf5_file, entry))
+
+    while len(items_to_treat) > 0:
+        to_treat = list(items_to_treat)
+        items_to_treat.clear()
+        for abs_file_path, file_path, data_path in to_treat:
+            item = abs_file_path, data_path
+            if item in treated_items:
                 continue
-            with HDF5File(file_path, mode="r") as h5f:
-                node = h5f.get(dataset_path, getlink=True)
+            dirname = os.path.dirname(abs_file_path)
+            with open_hdf5(abs_file_path) as h5f:
+                node = h5f.get(data_path, getlink=True)
                 if isinstance(node, h5py.ExternalLink):
-                    datasets_to_treat.add((node.filename, node.path))
-                node = h5f.get(dataset_path, getlink=False)
-                if isinstance(node, h5py.Dataset) and node.is_virtual:
-                    sub_file_and_dataset = get_linked_files_with_vds(
-                        file_path, dataset_path
+                    ext_file_path = node.filename
+                    if not os.path.isabs(ext_file_path):
+                        ext_file_path = os.path.join(dirname, ext_file_path)
+                    items_to_treat.add(
+                        (os.path.abspath(ext_file_path), node.filename, node.path)
                     )
-                    final_datasets.update(sub_file_and_dataset)
-                elif file_path != hdf5_file:
-                    final_datasets.add((file_path, dataset_path))
+                node = h5f.get(data_path, getlink=False)
+                if isinstance(node, h5py.Dataset) and node.is_virtual:
+                    final_datasets.update(
+                        get_linked_files_with_vds(abs_file_path, data_path)
+                    )
+                elif abs_file_path != abs_hdf5_file:
+                    final_datasets.add((file_path, data_path))
+
+                treated_items.add(item)
 
-                already_checked_dataset.add((file_path, dataset_path))
-                # browse contained dataset / group
                 if isinstance(node, h5py.Group):
                     for key in node.keys():
-                        full_path = "/".join((dataset_path, key))
-                        if (file_path, full_path) not in already_checked_dataset:
-                            datasets_to_treat.add((file_path, full_path))
+                        data_sub_path = "/".join((data_path, key))
+                        if (
+                            abs_file_path,
+                            data_sub_path,
+                        ) not in treated_items:
+                            items_to_treat.add(
+                                (abs_file_path, file_path, data_sub_path)
+                            )
     return final_datasets
 
 
-def get_linked_files_with_vds(hdf5_file: str, entry: str) -> tuple:
+def get_linked_files_with_vds(hdf5_file: str, dataset_path: str) -> set:
     """
     parse all virtual sources of a virtual dataset and return a set of files / dataset connected to it
-
-    warning: should keep relative path when found some
     """
-    datasets_to_treat = set()
-    final_datasets = set()
-    already_checked_dataset = set()
+    items_to_treat = set()  # abs_file_path, file_path, dataset_path
+    final_datasets = set()  # file_path, dataset_path
+    treated_items = set()  # abs_file_path, dataset_path
 
-    # first datasets to be tested
-    datasets_to_treat.add(
-        (
-            hdf5_file,
-            entry,
-        ),
-    )
+    abs_hdf5_file = os.path.abspath(hdf5_file)
+    items_to_treat.add((abs_hdf5_file, hdf5_file, dataset_path))
 
-    while len(datasets_to_treat) > 0:
-        to_treat = list(datasets_to_treat)
-        datasets_to_treat.clear()
-        for (file_path, dataset_path) in to_treat:
-            if (file_path, dataset_path) in already_checked_dataset:
+    while len(items_to_treat) > 0:
+        to_treat = list(items_to_treat)
+        items_to_treat.clear()
+        for abs_file_path, file_path, dataset_path in to_treat:
+            item = abs_file_path, dataset_path
+            if item in treated_items:
                 continue
-            with HDF5File(file_path, mode="r") as h5f:
+            dirname = os.path.dirname(abs_file_path)
+            with open_hdf5(abs_file_path) as h5f:
                 dataset = h5f[dataset_path]
-
                 if dataset.is_virtual:
                     for vs_info in dataset.virtual_sources():
-                        dirname = os.path.dirname(file_path)
-                        if dirname not in (None, ""):
-                            os.chdir(dirname)
-                        # handle relative link contained in the vds
-                        if os.path.isabs(vs_info.file_name):
-                            vs_file_path = os.path.realpath(vs_info.file_name)
-                        else:
-                            vs_file_path = vs_info.file_name
-                        datasets_to_treat.add(
+                        vs_file_path = vs_info.file_name
+                        if not os.path.isabs(vs_file_path):
+                            vs_file_path = os.path.join(dirname, vs_file_path)
+                        items_to_treat.add(
                             (
-                                vs_file_path,
+                                os.path.abspath(vs_file_path),
+                                vs_info.file_name,
                                 vs_info.dset_name,
                             )
                         )
                 else:
                     final_datasets.add((file_path, dataset_path))
-                already_checked_dataset.add((file_path, dataset_path))
+            treated_items.add(item)
 
     return final_datasets
+
+
+def str_to_dict(my_str: Union[str, dict]):
+    """convert a string as key_1=value_2;key_2=value_2 to a dict"""
+    if isinstance(my_str, dict):
+        return my_str
+    res = {}
+    for key_value in filter(None, my_str.split(";")):
+        key, value = key_value.split("=")
+        res[key] = value
+    return res
