@@ -28,15 +28,18 @@ __date__ = "16/11/2020"
 
 
 from silx.gui import qt
+
 from tomwer.core.process.control.datalistener import DataListener
-from ..processingstack import FIFO, ProcessingThread
 from tomwer.core.scan.blissscan import BlissScan
 
+from ..processingstack import FIFO, ProcessingThread
+
 try:
     # new HDF5Config class
     from nxtomomill.io.config import TomoHDF5Config as HDF5Config
 except ImportError:
     from nxtomomill.io.config import HDF5Config
+
 import logging
 
 _logger = logging.getLogger(__name__)
@@ -51,14 +54,14 @@ class DataListenerProcessStack(FIFO, qt.QObject):
         FIFO.__init__(self, process_id=-1)
         self._results = {}
 
-    def _process(self, scan, configuration, callback=None):
-        assert isinstance(scan, BlissScan)
+    def _process(self, data, configuration, callback=None):
+        assert isinstance(data, BlissScan)
         assert isinstance(configuration, HDF5Config)
-        _logger.info("DataListenerProcessStack is processing {}".format(str(scan)))
-        self._scan_currently_computed = scan
+        _logger.info(f"DataListenerProcessStack is processing {data}")
+        self._data_currently_computed = data
         self._computationThread.finished.connect(self._end_threaded_computation)
 
-        self._computationThread.init(scan=scan, configuration=configuration)
+        self._computationThread.init(data=data, configuration=configuration)
         # need to manage connect before starting it because
         self._computationThread.start()
 
@@ -71,7 +74,7 @@ class DataListenerProcessStack(FIFO, qt.QObject):
         thread.setParent(self)
         return thread
 
-    def _end_computation(self, scan, future_scan, callback):
+    def _end_computation(self, data, future_tomo_obj, callback):
         """
         callback when the computation thread is finished
 
@@ -79,9 +82,9 @@ class DataListenerProcessStack(FIFO, qt.QObject):
         """
         if callback is not None:
             callback()
-        if scan in self._results:
-            nx_scan = self._results[scan]
-            del self._results[scan]
+        if data in self._results:
+            nx_scan = self._results[data]
+            del self._results[data]
         else:
             nx_scan = None
         self.sigComputationEnded.emit(nx_scan, None)
@@ -103,20 +106,16 @@ class _DataListenerConverterThread(ProcessingThread):
         self._scan = None
         self._configuration = None
 
-    def init(self, scan, configuration):
-        if not isinstance(scan, BlissScan):
-            raise TypeError(
-                "Only manage BlissScan. {} is not managed" "".format(type(scan))
-            )
+    def init(self, data, configuration):
+        if not isinstance(data, BlissScan):
+            raise TypeError(f"Only manage BlissScan. {type(data)} is not managed")
         assert isinstance(configuration, HDF5Config)
-        self._scan = scan
+        self._scan = data
         self._configuration = configuration
 
     def run(self):
         self.sigComputationStarted.emit()
-        _logger.processStarted(
-            "Start conversion of bliss scan {}".format(str(self._scan))
-        )
+        _logger.processStarted(f"Start conversion of bliss scan {self._scan}")
 
         data_listener = DataListener()
         data_listener.set_configuration(self._configuration or {})
@@ -129,11 +128,9 @@ class _DataListenerConverterThread(ProcessingThread):
             )
         except Exception as e:
             _logger.processFailed(
-                "Conversion of bliss scan {}." "Reason is {}".format(str(self._scan), e)
+                f"Conversion of bliss scan {self._scan}. Reason is {e}"
             )
             scans = None
         else:
-            _logger.processSucceed(
-                "Conversion of bliss scan {}." "".format(str(self._scan))
-            )
+            _logger.processSucceed(f"Conversion of bliss scan {self._scan}.")
         self.parent().register_result(self._scan, scans)
