@@ -27,18 +27,21 @@ __license__ = "MIT"
 __date__ = "28/08/2020"
 
 
-from silx.gui import qt
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.edfscan import EDFTomoScan
-from tomwer.core.settings import get_lbsram_path, isOnLbsram
-from tomwer.core.utils import isLowOnMemory
-from ..processingstack import FIFO, ProcessingThread
-from tomwer.core.process.edit import darkflatpatch
-from processview.core.superviseprocess import SuperviseProcess
-from processview.core.manager import ProcessManager, DatasetState
-from silx.io.url import DataUrl
 import logging
 
+from processview.core.manager import DatasetState, ProcessManager
+from processview.core.superviseprocess import SuperviseProcess
+from silx.gui import qt
+from silx.io.url import DataUrl
+
+from tomwer.core.process.edit import darkflatpatch
+from tomwer.core.scan.edfscan import EDFTomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.settings import get_lbsram_path, isOnLbsram
+from tomwer.core.utils.lbsram import is_low_on_memory
+
+from ..processingstack import FIFO, ProcessingThread
+
 _logger = logging.getLogger(__name__)
 
 
@@ -51,28 +54,28 @@ class DarkFlatPatchProcessStack(FIFO, qt.QObject):
         FIFO.__init__(self, process_id=process_id)
         self._dry_run = False
 
-    def _process(self, scan, configuration, callback=None):
-        _logger.info("dark-flat patch stack is processing {}".format(str(scan)))
-        self._scan_currently_computed = scan
-        assert isinstance(scan, TomwerScanBase)
+    def _process(self, data, configuration, callback=None):
+        _logger.info(f"dark-flat patch stack is processing {data}")
+        self._data_currently_computed = data
+        assert isinstance(data, TomwerScanBase)
         self._computationThread.finished.connect(self._end_threaded_computation)
 
-        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
+        if isOnLbsram(data) and is_low_on_memory(get_lbsram_path()) is True:
             # if computer is running into low memory on lbsram skip it
-            mess = "low memory, skip dark-flat-patch reconstruction for", scan.path
+            mess = "low memory, skip dark-flat-patch reconstruction for", data.path
             _logger.processSkipped(mess)
             ProcessManager().notify_dataset_state(
-                dataset=scan,
+                dataset=data,
                 process=self,
                 state=DatasetState.SKIPPED,
                 details=mess,
             )
             self._end_threaded_computation()
         else:
-            self._computationThread.init(scan=scan, configuration=configuration)
+            self._computationThread.init(data=data, configuration=configuration)
             # need to manage connect before starting it because
             ProcessManager().notify_dataset_state(
-                dataset=scan,
+                dataset=data,
                 process=self,
                 state=DatasetState.ON_GOING,
                 details=None,
@@ -101,21 +104,25 @@ class _DarkFlatPatchProcessingThread(ProcessingThread, SuperviseProcess):
         self._scan = None
         self._configuration = None
 
-    def init(self, scan, configuration):
-        self._scan = scan
+    def init(self, data, configuration):
+        self._scan = data
         self._configuration = configuration
 
     def run(self):
+        # TODO: must be replace by calling the 'DarkFlatPatch' task directly
         self.sigComputationStarted.emit()
-        _logger.processStarted("{} Start dark-flat patch".format(str(self._scan)))
+        _logger.processStarted(f"{self._scan} Start dark-flat patch")
+        process = darkflatpatch.DarkFlatPatchTask(
+            inputs={
+                "data": self._scan,
+                "configuration": self._configuration,
+                "serialize_output_data": False,
+            }
+        )
         try:
-            darkflatpatch.apply_dark_flat_patch(
-                config=self._configuration, scan=self._scan
-            )
+            process.run()
         except Exception as e:
-            info = "Fail to patch dark-flat for {}. Reason is " "{}".format(
-                str(self._scan), e
-            )
+            info = f"Fail to patch dark-flat for {self._scan}. Reason is {e}"
             _logger.processFailed(info)
             ProcessManager().notify_dataset_state(
                 dataset=self._scan,
@@ -123,24 +130,28 @@ class _DarkFlatPatchProcessingThread(ProcessingThread, SuperviseProcess):
                 state=DatasetState.FAILED,
                 details=info,
             )
+            return
+
+        index = self._scan.pop_process_index()
+        if isinstance(self._scan, EDFTomoScan):
+            entry = None
         else:
-            index = self._scan.pop_process_index()
-            if isinstance(self._scan, EDFTomoScan):
-                entry = None
-            else:
-                entry = self._scan.entry
-            configuration_to_dump = self._configuration
-            keys = list(configuration_to_dump.keys())
-            for key in keys:
-                if isinstance(configuration_to_dump[key], DataUrl):
-                    configuration_to_dump[key] = configuration_to_dump[key].path()
+            entry = self._scan.entry
+        configuration_to_dump = self._configuration
+        keys = list(configuration_to_dump.keys())
+        for key in keys:
+            if isinstance(configuration_to_dump[key], DataUrl):
+                configuration_to_dump[key] = configuration_to_dump[key].path()
+
+        with self._scan.acquire_process_file_lock():
+
             try:
-                darkflatpatch.DarkFlatPatch._register_process(
+                darkflatpatch.DarkFlatPatchTask._register_process(
                     process_file=self._scan.process_file,
                     entry=entry,
                     configuration=configuration_to_dump,
                     results={},
-                    process=darkflatpatch.DarkFlatPatch,
+                    process=darkflatpatch.DarkFlatPatchTask,
                     process_index=index,
                     overwrite=True,
                 )
@@ -148,11 +159,11 @@ class _DarkFlatPatchProcessingThread(ProcessingThread, SuperviseProcess):
                 _logger.warning(
                     f"Fail to register DarkFlatPatch process. Reason is {e}"
                 )
-            info = "Dark-flat patched for {}." "".format(str(self._scan))
-            _logger.processSucceed(info)
-            ProcessManager().notify_dataset_state(
-                dataset=self._scan,
-                process=self,
-                state=DatasetState.SUCCEED,
-                details=info,
-            )
+        info = f"Dark-flat patched for {self._scan}."
+        _logger.processSucceed(info)
+        ProcessManager().notify_dataset_state(
+            dataset=self._scan,
+            process=self,
+            state=DatasetState.SUCCEED,
+            details=info,
+        )
