@@ -27,16 +27,18 @@ __license__ = "MIT"
 __date__ = "28/08/2020"
 
 
+import logging
+
+from processview.core.manager import DatasetState, ProcessManager
+from processview.core.superviseprocess import SuperviseProcess
 from silx.gui import qt
+
+from tomwer.core.process.edit import imagekeyeditor
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.core.settings import get_lbsram_path, isOnLbsram
-from tomwer.core.utils import isLowOnMemory
+from tomwer.core.utils.lbsram import is_low_on_memory
+
 from ..processingstack import FIFO, ProcessingThread
-from tomwer.core.process.edit import imagekeyeditor
-from tomoscan.esrf.hdf5scan import ImageKey
-from processview.core.superviseprocess import SuperviseProcess
-from processview.core.manager import ProcessManager, DatasetState
-import logging
 
 _logger = logging.getLogger(__name__)
 
@@ -49,27 +51,27 @@ class ImageKeyEditorProcessStack(FIFO, qt.QObject):
         FIFO.__init__(self, process_id=process_id)
         self._dry_run = False
 
-    def _process(self, scan, configuration, callback=None):
-        _logger.info("image-key edition is processing {}".format(str(scan)))
-        self._scan_currently_computed = scan
-        assert isinstance(scan, TomwerScanBase)
+    def _process(self, data, configuration, callback=None):
+        _logger.info(f"image-key edition is processing {data}")
+        self._data_currently_computed = data
+        assert isinstance(data, TomwerScanBase)
         self._computationThread.finished.connect(self._end_threaded_computation)
 
-        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
+        if isOnLbsram(data) and is_low_on_memory(get_lbsram_path()) is True:
             # if computer is running into low memory on lbsram skip it
-            mess = "low memory, skip image-key-edition for", scan.path
+            mess = f"low memory, skip image-key-edition for {data.path}"
             _logger.processSkipped(mess)
             ProcessManager().notify_dataset_state(
-                dataset=scan,
+                dataset=data,
                 process=self,
                 state=DatasetState.SKIPPED,
                 details=mess,
             )
             self._end_threaded_computation()
         else:
-            self._computationThread.init(scan=scan, configuration=configuration)
+            self._computationThread.init(data=data, configuration=configuration)
             ProcessManager().notify_dataset_state(
-                dataset=scan,
+                dataset=data,
                 process=self,
                 state=DatasetState.ON_GOING,
             )
@@ -97,21 +99,24 @@ class _ImageKeyEditorProcessingThread(ProcessingThread, SuperviseProcess):
         self._scan = None
         self._configuration = None
 
-    def init(self, scan, configuration):
-        self._scan = scan
+    def init(self, data, configuration):
+        self._scan = data
         self._configuration = configuration
 
     def run(self):
         self.sigComputationStarted.emit()
-        _logger.processStarted("Start image key edition of {}".format(str(self._scan)))
+        _logger.processStarted(f"Start image key edition of {self._scan}")
+        task = imagekeyeditor.ImageKeyEditorTask(
+            inputs={
+                "data": self._scan,
+                "serialize_output_data": False,
+                "configuration": self._configuration,
+            },
+        )
         try:
-            imagekeyeditor.change_image_key_control(
-                config=self._configuration, scan=self._scan
-            )
+            task.run()
         except Exception as e:
-            info = "Fail to edit image keys for {}. Reason is {}".format(
-                str(self._scan), e
-            )
+            info = f"Fail to edit image keys for {self._scan}. Reason is {e}"
             _logger.processFailed(info)
             ProcessManager().notify_dataset_state(
                 dataset=self._scan,
@@ -120,31 +125,7 @@ class _ImageKeyEditorProcessingThread(ProcessingThread, SuperviseProcess):
                 details=info,
             )
         else:
-            conf_to_dump = {}
-            if "modifications" in self._configuration:
-                for frame_index, new_frame_type in self._configuration[
-                    "modifications"
-                ].items():
-                    conf_to_dump[str(frame_index)] = ImageKey.from_value(
-                        new_frame_type
-                    ).value
-
-            index = self._scan.pop_process_index()
-            try:
-                imagekeyeditor.ImageKeyEditor._register_process(
-                    process_file=self._scan.process_file,
-                    entry=self._scan.entry,
-                    configuration=conf_to_dump,
-                    results={},
-                    process=imagekeyeditor.ImageKeyEditor,
-                    process_index=index,
-                    overwrite=True,
-                )
-            except Exception as e:
-                _logger.warning(
-                    f"Fail to register ImageKeyEditor process. Reason is {e}"
-                )
-            info = "Image key edited for {}.".format(str(self._scan))
+            info = f"Image key edited for {self._scan.get_identifier()}."
             _logger.processSucceed(info)
             ProcessManager().notify_dataset_state(
                 dataset=self._scan,
