@@ -28,10 +28,14 @@ __license__ = "MIT"
 __date__ = "08/01/2020"
 
 
-from queue import Queue
-from silx.gui import qt
-from processview.core.manager import ProcessManager, DatasetState
+from collections import deque
+from typing import Optional
+from tomwer.core.tomwer_object import TomwerObject
+from silx.gui.utils import blockSignals
+
+from processview.core.manager import DatasetState, ProcessManager
 from processview.core.superviseprocess import SuperviseProcess
+from silx.gui import qt
 
 
 class ProcessingThread(qt.QThread):
@@ -41,7 +45,7 @@ class ProcessingThread(qt.QThread):
     """Signal emitted when a computation is started"""
 
 
-class FIFO(Queue, SuperviseProcess):
+class FIFO(SuperviseProcess):
     """Processing Queue with a First In, First Out behavior"""
 
     sigComputationStarted = qt.Signal(object)
@@ -52,18 +56,22 @@ class FIFO(Queue, SuperviseProcess):
 
     def __init__(self, process_id=None):
         SuperviseProcess.__init__(self, process_id=process_id)
-        Queue.__init__(self)
+        self._deque = deque()
         self._computationThread = self._create_processing_thread(process_id=process_id)
         assert isinstance(self._computationThread, ProcessingThread)
         self._computationThread.sigComputationStarted.connect(
             self._start_threaded_computation
         )
         """scan process by the thread"""
-        self._scan_currently_computed = None
+        self._data_currently_computed = None
         """Scan computed currently"""
         self._processing = False
 
-    def add(self, scan, configuration=None, callback=None):
+    @property
+    def data_currently_computed(self) -> Optional[TomwerObject]:
+        return self._data_currently_computed
+
+    def add(self, data, configuration=None, callback=None):
         """
         add a scan to process
 
@@ -74,26 +82,27 @@ class FIFO(Queue, SuperviseProcess):
         try:
             if self.process_id not in (None, -1):
                 ProcessManager().notify_dataset_state(
-                    dataset=scan,
+                    dataset=data,
                     process=self,
                     state=DatasetState.PENDING,
                 )
         except Exception:
             pass
-        Queue.put(self, (scan, configuration, callback))
+
+        self.append((data, configuration, callback))
         if self.can_process_next():
             self._process_next()
 
-    def _process(self, scan, configuration, callback):
+    def _process(self, data, configuration, callback):
         raise NotImplementedError("Virtual class")
 
     def _process_next(self):
-        if Queue.empty(self):
+        if len(self) == 0:
             return
 
         self._processing = True
-        scan, configuration, callback = Queue.get(self)
-        self._process(scan=scan, configuration=configuration or {}, callback=callback)
+        data, configuration, callback = self.pop()
+        self._process(data=data, configuration=configuration or {}, callback=callback)
 
     def can_process_next(self):
         """
@@ -103,7 +112,7 @@ class FIFO(Queue, SuperviseProcess):
         """
         return not self._processing
 
-    def _end_computation(self, scan, future_scan, callback):
+    def _end_computation(self, data, future_tomo_obj, callback):
         """
         callback when the computation thread is finished
 
@@ -111,13 +120,13 @@ class FIFO(Queue, SuperviseProcess):
         """
         if callback is not None:
             callback()
-        sender = self.sender()
-        if hasattr(sender, "future_scan"):
-            future_scan = sender.future_scan
+        sender = self.sender()  # pylint: disable=E1101
+        if hasattr(sender, "future_tomo_obj"):
+            future_tomo_obj = sender.future_tomo_obj
         else:
-            future_scan = None
+            future_tomo_obj = None
 
-        self.sigComputationEnded.emit(scan, future_scan)
+        self.sigComputationEnded.emit(data, future_tomo_obj)
         self._processing = False
         if self.can_process_next():
             self._process_next()
@@ -138,22 +147,51 @@ class FIFO(Queue, SuperviseProcess):
             self._computationThread.wait()
 
     def _start_threaded_computation(self, *args, **kwargs):
-        self.sigComputationStarted.emit(self._scan_currently_computed)
+        self.sigComputationStarted.emit(self._data_currently_computed)
 
     def _end_threaded_computation(self, callback=None):
-        sender = self.sender()
-        if hasattr(sender, "future_scan"):
-            future_scan = sender.future_scan
+        sender = self.sender()  # pylint: disable=E1101
+        if hasattr(sender, "future_tomo_obj"):
+            future_tomo_obj = sender.future_tomo_obj
         else:
-            future_scan = None
+            future_tomo_obj = None
         self._end_computation(
-            scan=self._scan_currently_computed,
+            data=self._data_currently_computed,
             callback=callback,
-            future_scan=future_scan,
+            future_tomo_obj=future_tomo_obj,
         )
 
+    def cancel(self):
+        if self._computationThread.isRunning():
+            with blockSignals(self._computationThread):
+                self._computationThread.cancel()
+                # stop stack
+                self.stop()
+                # emit next
+                self.sigComputationEnded.emit(None, None)
+
     def stop(self):
-        while not self.empty():
-            self.get()
-        self._computationThread.blockSignals(True)
         self._computationThread.wait()
+        self._processing = False
+        self._data_currently_computed = None
+
+    # expose deque API
+    def append(self, value):
+        self._deque.append(value)
+
+    def clear(self):
+        self._deque.clear()
+
+    def pop(self):
+        return self._deque.pop()
+
+    def remove(self, value):
+        return self._deque.remove(value)
+
+    def __len__(self):
+        return len(self._deque)
+
+    def __contains__(self, value):
+        return value.get_identifier().to_str() in [
+            scan.get_identifier().to_str() for scan in self._deque
+        ]
