@@ -27,21 +27,23 @@ __license__ = "MIT"
 __date__ = "03/05/2019"
 
 
-from tomwer.core.process.reconstruction.axis import AxisProcess
-from tomwer.core.process.reconstruction.axis.axis import NoAxisUrl
-from tomwer.core.process.reconstruction.axis.mode import AxisMode
-from processview.core.manager import ProcessManager
-from tomwer.synctools.axis import QAxisRP
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from tomwer.core.settings import get_lbsram_path, isOnLbsram
-from tomwer.core.utils import isLowOnMemory
-from processview.core.manager import DatasetState
-from ..processingstack import FIFO, ProcessingThread
+import functools
+import logging
+
+from processview.core.manager import DatasetState, ProcessManager
 from processview.core.superviseprocess import SuperviseProcess
 from silx.gui import qt
-import logging
-import functools
+
+from tomwer.core.process.reconstruction.axis import AxisTask
+from tomwer.core.process.reconstruction.axis.axis import NoAxisUrl
+from tomwer.core.process.reconstruction.axis.mode import AxisMode
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.settings import get_lbsram_path, isOnLbsram
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.synctools.axis import QAxisRP
+
+from ..processingstack import FIFO, ProcessingThread
 
 _logger = logging.getLogger(__name__)
 
@@ -56,29 +58,29 @@ class AxisProcessStack(FIFO, qt.QObject):
         assert axis_params is not None
         self._axis_params = axis_params
 
-    def _process(self, scan, configuration, callback=None):
+    def _process(self, data, configuration, callback=None):
         ProcessManager().notify_dataset_state(
-            dataset=scan,
+            dataset=data,
             process=self,
             state=DatasetState.ON_GOING,
         )
-        assert isinstance(scan, TomwerScanBase)
-        if scan.axis_params is None:
-            scan.axis_params = QAxisRP()
-        self._scan_currently_computed = scan
-        self._axis_params.frame_width = scan.dim_1
+        assert isinstance(data, TomwerScanBase)
+        if data.axis_params is None:
+            data.axis_params = QAxisRP()
+        self._data_currently_computed = data
+        self._axis_params.frame_width = data.dim_1
         mode = self._axis_params.mode
-        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
+        if isOnLbsram(data) and is_low_on_memory(get_lbsram_path()) is True:
             # if computer is running into low memory on lbsram skip it
-            mess = "low memory, skip axis calculation", scan.path
+            mess = "low memory, skip axis calculation", data.path
             ProcessManager().notify_dataset_state(
-                dataset=scan, process=self._process_id, state=DatasetState.SKIPPED
+                dataset=data, process=self._process_id, state=DatasetState.SKIPPED
             )
             _logger.processSkipped(mess)
-            scan.axis_params.set_relative_value(None)
+            data.axis_params.set_relative_value(None)
             if callback is not None:
                 callback()
-            self.scan_ready(scan=scan)
+            self.scan_ready(scan=data)
         elif not self._axis_params.use_sinogram and mode in (
             AxisMode.manual,
             AxisMode.read,
@@ -88,71 +90,69 @@ class AxisProcessStack(FIFO, qt.QObject):
             cor = self._axis_params.relative_cor_value
             if cor is None:
                 cor = 0
-            scan._axis_params.set_relative_value(cor)
-            cor = scan._axis_params.relative_cor_value
-            if isinstance(scan, HDF5TomoScan):
-                entry = scan.entry
+            data._axis_params.set_relative_value(cor)
+            cor = data._axis_params.relative_cor_value
+            if isinstance(data, NXtomoScan):
+                entry = data.entry
             else:
                 entry = "entry"
             try:
-                with scan.acquire_process_file_lock():
-                    AxisProcess._register_process(
-                        process_file=scan.process_file,
+                with data.acquire_process_file_lock():
+                    AxisTask._register_process(
+                        process_file=data.process_file,
                         entry=entry,
-                        process=AxisProcess,
+                        process=AxisTask,
                         results={"center_of_rotation": cor if cor is not None else "-"},
                         configuration=self._axis_params.to_dict(),
-                        process_index=scan.pop_process_index(),
+                        process_index=data.pop_process_index(),
                         overwrite=True,
                     )
             except Exception as e:
                 _logger.warning(f"Fail to register Axis process. Reason is {e}")
             # If mode is read or manual the position_value is not computed and
             # we will keep the actual one (should have been defined previously)
-            self._end_computation(scan=scan, future_scan=None, callback=callback)
+            self._end_computation(data=data, future_tomo_obj=None, callback=callback)
 
         elif (
             not self._axis_params.use_sinogram
-            and mode not in AxisProcess._CALCULATIONS_METHODS
+            and mode not in AxisTask._CALCULATIONS_METHODS
         ):
-            _logger.warning("no method defined to compute {}".format(mode))
+            _logger.warning(f"no method defined to compute {mode}")
             if callback is not None:
                 callback()
             self._process_next()
         else:
             _logger.processStarted(
-                "Start cor calculation on {} ({})".format(
-                    str(scan), self._axis_params.get_simple_str()
-                )
+                f"Start cor calculation on {data} ({self._axis_params.get_simple_str()})"
             )
-            scan.axis_params.set_relative_value("...")
+            data.axis_params.set_relative_value("...")
             self._axis_params.set_relative_value("...")
             assert self._axis_params.relative_cor_value == "..."
-            self._scan_currently_computed = scan
-            self._computationThread.init(scan=scan, axis_params=configuration)
+            self._data_currently_computed = data
+            self._computationThread.init(data=data, axis_params=configuration)
             fct_callback = functools.partial(self._end_threaded_computation, callback)
             self._computationThread.finished.connect(fct_callback)
             self._computationThread.start()
 
-    def _end_computation(self, scan, future_scan, callback):
+    def _end_computation(self, data, future_tomo_obj, callback):
         """
         callback when the computation thread is finished
 
         :param scan: pass if no call to '_computationThread is made'
         """
-        assert isinstance(scan, TomwerScanBase)
+        assert isinstance(data, TomwerScanBase)
         assert self._axis_params is not None
         # copy result computed on scan on the AxisProcess reconsparams
         self._axis_params.set_relative_value(
-            scan.axis_params.relative_cor_value
+            data.axis_params.relative_cor_value
         )  # noqa
-        self._axis_params.frame_width = scan.dim_1
+        self._axis_params.frame_width = data.dim_1
         FIFO._end_computation(
-            self, scan=scan, future_scan=future_scan, callback=callback
+            self, data=data, future_tomo_obj=future_tomo_obj, callback=callback
         )
 
     def _end_threaded_computation(self, callback=None):
-        assert self._scan_currently_computed is not None
+        assert self._data_currently_computed is not None
         self._axis_params.set_relative_value(self._computationThread.center_of_rotation)
         self._computationThread.finished.disconnect()
         if callback:
@@ -182,16 +182,17 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
         self.__patch = {}
         """Used to patch some calculation method (for test purpose)"""
 
-    def init(self, scan, axis_params):
-        self._scan = scan
+    def init(self, data, axis_params):
+        self._scan = data
         self._axis_params = axis_params
 
     def run(self):
         self.sigComputationStarted.emit()
-        axis = AxisProcess(
+        axis = AxisTask(
             inputs={
                 "data": self._scan,
                 "axis_params": self._axis_params,
+                "serialize_output_data": False,
             },
             process_id=self.process_id,
         )
