@@ -27,20 +27,21 @@ __license__ = "MIT"
 __date__ = "03/05/2019"
 
 
-from tomwer.core.process.reconstruction.saaxis import SAAxisProcess
-from processview.core.manager import ProcessManager
-from tomwer.synctools.saaxis import QSAAxisParams
-from tomwer.synctools.axis import QAxisRP
+import functools
+import logging
+
+from processview.core.manager import DatasetState, ProcessManager
+from processview.core.superviseprocess import SuperviseProcess
+from silx.gui import qt
+
+from tomwer.core.process.reconstruction.saaxis import SAAxisTask
 from tomwer.core.scan.scanbase import TomwerScanBase
 from tomwer.core.settings import get_lbsram_path, isOnLbsram
-from tomwer.core.utils import isLowOnMemory
-from processview.core.manager import DatasetState
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.synctools.axis import QAxisRP
+from tomwer.synctools.saaxis import QSAAxisParams
+
 from ..processingstack import FIFO, ProcessingThread
-from processview.core.superviseprocess import SuperviseProcess
-from tomwer.io.utils import format_stderr_stdout
-from silx.gui import qt
-import logging
-import functools
 
 _logger = logging.getLogger(__name__)
 
@@ -62,53 +63,52 @@ class SAAxisProcessStack(FIFO, qt.QObject):
     def set_dry_run(self, dry_run):
         self._dry_run = dry_run
 
-    def _process(self, scan, configuration, callback=None):
+    def _process(self, data, configuration, callback=None):
         ProcessManager().notify_dataset_state(
-            dataset=scan,
+            dataset=data,
             process=self,
             state=DatasetState.ON_GOING,
         )
-        _logger.processStarted("start saaxis {}" "".format(str(scan)))
-        assert isinstance(scan, TomwerScanBase)
-        if scan.axis_params is None:
-            scan.axis_params = QAxisRP()
-        if scan.saaxis_params is None:
-            scan.saaxis_params = QSAAxisParams()
-        self._scan_currently_computed = scan
+        _logger.processStarted(f"start saaxis {data}")
+        assert isinstance(data, TomwerScanBase)
+        if data.axis_params is None:
+            data.axis_params = QAxisRP()
+        if data.saaxis_params is None:
+            data.saaxis_params = QSAAxisParams()
+        self._data_currently_computed = data
         saaxis_params = QSAAxisParams.from_dict(configuration)
-        saaxis_params.frame_width = scan.dim_1
-        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
+        saaxis_params.frame_width = data.dim_1
+        if isOnLbsram(data) and is_low_on_memory(get_lbsram_path()) is True:
             # if computer is running into low memory on lbsram skip it
-            mess = "low memory, skip saaxis calculation", scan.path
+            mess = f"low memory, skip saaxis calculation {data.path}"
             ProcessManager().notify_dataset_state(
-                dataset=scan, process=self._process_id, state=DatasetState.SKIPPED
+                dataset=data, process=self._process_id, state=DatasetState.SKIPPED
             )
             _logger.processSkipped(mess)
-            scan.axis_params.set_relative_value(None)
+            data.axis_params.set_relative_value(None)
             if callback is not None:
                 callback()
-            self.scan_ready(scan=scan)
+            self.scan_ready(scan=data)
         else:
-            self._scan_currently_computed = scan
-            self._computationThread.init(scan=scan, saaxis_params=saaxis_params)
+            self._data_currently_computed = data
+            self._computationThread.init(data=data, saaxis_params=saaxis_params)
             # need to manage connect before starting it because
             fct_callback = functools.partial(self._end_threaded_computation, callback)
             self._computationThread.finished.connect(fct_callback)
             self._computationThread.start()
 
-    def _end_computation(self, scan, future_scan, callback):
+    def _end_computation(self, data, future_tomo_obj, callback):
         """
         callback when the computation thread is finished
 
         :param scan: pass if no call to '_computationThread is made'
         """
-        assert isinstance(scan, TomwerScanBase)
+        assert isinstance(data, TomwerScanBase)
         FIFO._end_computation(
-            self, scan=scan, future_scan=future_scan, callback=callback
+            self, data=data, future_tomo_obj=future_tomo_obj, callback=callback
         )
 
     def _end_threaded_computation(self, callback=None):
-        assert self._scan_currently_computed is not None
         self._computationThread.finished.disconnect()
         if callback:
             callback()
@@ -136,8 +136,11 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
         self._patch_process_fct = None
         """function pointer to know which function to call for the axis
         calculation"""
-        self.__patch = {}
-        """Used to patch some calculation method (for test purpose)"""
+        self._current_processing = None
+        # processing currently runned. As we want to be able to cancel processing we need to keep a pointer on the
+        # 'activate' nabu processing to stop process if requested
+        self._task = None
+        # ewoks task for reconstruction
 
     def set_dry_run(self, dry_run):
         self._dry_run = dry_run
@@ -145,8 +148,8 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
     def patch_processing(self, process_fct):
         self._patch_process_fct = process_fct
 
-    def init(self, scan, saaxis_params):
-        self._scan = scan
+    def init(self, data, saaxis_params: dict):
+        self._scan = data
         self._saaxis_params = saaxis_params
 
     def run(self):
@@ -156,38 +159,24 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
             for cor in self._saaxis_params.cors:
                 scores[cor] = self._patch_process_fct(cor)
             self._scan.saaxis_params.scores = scores
-            SAAxisProcess.autofocus(scan=self._scan)
+            SAAxisTask.autofocus(scan=self._scan)
             self.center_of_rotation = self._scan.saaxis_params.autofocus
         else:
-            saaxis = SAAxisProcess(
+            self._task = SAAxisTask(
                 process_id=self.process_id,
                 inputs={
                     "dump_process": False,
                     "data": self._scan,
                     "dry_run": self._dry_run,
+                    "sa_axis_params": self._saaxis_params.to_dict(),
+                    "serialize_output_data": False,
                 },
             )
-            saaxis.set_configuration(self._saaxis_params)
-            try:
-                saaxis.run()
-            except Exception as e:
-                _logger.error(str(e))
-                mess = f"SAaxis computation for {str(self._scan)} failed."
-                state = DatasetState.FAILED
-            else:
-                mess = f"SAaxis computation for {str(self._scan)} succeed."
-                state = DatasetState.WAIT_USER_VALIDATION
-                self.center_of_rotation = self._scan.saaxis_params.autofocus
 
-            nabu_logs = []
-            for std_err, std_out in zip(saaxis.std_errs, saaxis.std_outs):
-                nabu_logs.append(format_stderr_stdout(stdout=std_out, stderr=std_err))
-            self._nabu_log = nabu_logs
-            nabu_logs.insert(0, mess)
-            details = "\n".join(nabu_logs)
-            ProcessManager().notify_dataset_state(
-                dataset=self._scan,
-                process=self,
-                state=state,
-                details=details,
-            )
+            self._task.run()
+            self.center_of_rotation = self._task.outputs.best_cor
+
+    def cancel(self):
+        if self._task is not None:
+            self._task.cancel()
+        self.quit()
