@@ -27,22 +27,23 @@ __license__ = "MIT"
 __date__ = "08/06/2021"
 
 
-from processview.core.manager import ProcessManager
-from tomwer.synctools.sadeltabeta import QSADeltaBetaParams
-from tomwer.synctools.axis import QAxisRP
-from tomwer.core.scan.scanbase import TomwerScanBase
-from tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta import (
-    SADeltaBetaProcess,
-)
-from tomwer.core.settings import get_lbsram_path, isOnLbsram
-from tomwer.core.utils import isLowOnMemory
-from processview.core.manager import DatasetState
-from ..processingstack import FIFO, ProcessingThread
-from processview.core.superviseprocess import SuperviseProcess
-from tomwer.io.utils import format_stderr_stdout
-from silx.gui import qt
-import logging
 import functools
+import logging
+
+from processview.core.manager import DatasetState, ProcessManager
+from processview.core.superviseprocess import SuperviseProcess
+from silx.gui import qt
+
+from tomwer.core.process.reconstruction.sadeltabeta.sadeltabeta import (
+    SADeltaBetaTask,
+)
+from tomwer.core.scan.scanbase import TomwerScanBase
+from tomwer.core.settings import get_lbsram_path, isOnLbsram
+from tomwer.core.utils.lbsram import is_low_on_memory
+from tomwer.synctools.axis import QAxisRP
+from tomwer.synctools.sadeltabeta import QSADeltaBetaParams
+
+from ..processingstack import FIFO, ProcessingThread
 
 _logger = logging.getLogger(__name__)
 
@@ -64,54 +65,54 @@ class SADeltaBetaProcessStack(FIFO, qt.QObject):
     def set_dry_run(self, dry_run):
         self._dry_run = dry_run
 
-    def _process(self, scan, configuration, callback=None):
+    def _process(self, data, configuration, callback=None):
         ProcessManager().notify_dataset_state(
-            dataset=scan,
+            dataset=data,
             process=self,
             state=DatasetState.ON_GOING,
         )
-        _logger.processStarted("start sa-delta-beta {}" "".format(str(scan)))
-        assert isinstance(scan, TomwerScanBase)
-        if scan.axis_params is None:
-            scan.axis_params = QAxisRP()
-        if scan.sa_delta_beta_params is None:
-            scan.sa_delta_beta_params = QSADeltaBetaParams()
-        self._scan_currently_computed = scan
+        _logger.processStarted(f"start sa-delta-beta {data}")
+        assert isinstance(data, TomwerScanBase)
+        if data.axis_params is None:
+            data.axis_params = QAxisRP()
+        if data.sa_delta_beta_params is None:
+            data.sa_delta_beta_params = QSADeltaBetaParams()
+        self._data_currently_computed = data
         sa_delta_beta_params = QSADeltaBetaParams.from_dict(configuration)
-        if isOnLbsram(scan) and isLowOnMemory(get_lbsram_path()) is True:
+        if isOnLbsram(data) and is_low_on_memory(get_lbsram_path()) is True:
             # if computer is running into low memory on lbsram skip it
-            mess = "low memory, skip sa-delta-beta-axis calculation", scan.path
+            mess = f"low memory, skip sa-delta-beta-axis calculation {data.path}"
             ProcessManager().notify_dataset_state(
-                dataset=scan, process=self._process_id, state=DatasetState.SKIPPED
+                dataset=data, process=self._process_id, state=DatasetState.SKIPPED
             )
             _logger.processSkipped(mess)
-            scan.sa_delta_beta_params.set_value(None)
+            data.sa_delta_beta_params.set_value(None)
             if callback is not None:
                 callback()
-            self.scan_ready(scan=scan)
+            self.scan_ready(scan=data)
         else:
-            self._scan_currently_computed = scan
+            self._data_currently_computed = data
             self._computationThread.init(
-                scan=scan, sa_delta_beta_params=sa_delta_beta_params
+                data=data, sa_delta_beta_params=sa_delta_beta_params
             )
             # need to manage connect before starting it because
             fct_callback = functools.partial(self._end_threaded_computation, callback)
             self._computationThread.finished.connect(fct_callback)
             self._computationThread.start()
 
-    def _end_computation(self, scan, future_scan, callback):
+    def _end_computation(self, data, future_tomo_obj, callback):
         """
         callback when the computation thread is finished
 
         :param scan: pass if no call to '_computationThread is made'
         """
-        assert isinstance(scan, TomwerScanBase)
+        assert isinstance(data, TomwerScanBase)
         FIFO._end_computation(
-            self, scan=scan, future_scan=future_scan, callback=callback
+            self, data=data, future_tomo_obj=future_tomo_obj, callback=callback
         )
 
     def _end_threaded_computation(self, callback=None):
-        assert self._scan_currently_computed is not None
+        assert self._data_currently_computed is not None
         self._computationThread.finished.disconnect()
         if callback:
             callback()
@@ -132,15 +133,15 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
             ProcessingThread.__init__(self, process_id=process_id)
         except TypeError:
             ProcessingThread.__init__(self)
-        self.center_of_rotation = None
+        self.db_value = None
         self._dry_run = False
         self._scan = None
         self._sa_delta_beta_params = None
         self._patch_process_fct = None
         """function pointer to know which function to call for the axis
         calculation"""
-        self.__patch = {}
-        """Used to patch some calculation method (for test purpose)"""
+        self._task = None
+        # ewoks task for reconstruction
 
     def set_dry_run(self, dry_run):
         self._dry_run = dry_run
@@ -148,8 +149,8 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
     def patch_processing(self, process_fct):
         self._patch_process_fct = process_fct
 
-    def init(self, scan, sa_delta_beta_params):
-        self._scan = scan
+    def init(self, data, sa_delta_beta_params):
+        self._scan = data
         self._sa_delta_beta_params = sa_delta_beta_params
 
     def run(self):
@@ -159,38 +160,24 @@ class _ProcessingThread(ProcessingThread, SuperviseProcess):
             for db in self._sa_delta_beta_params.delta_beta_values:
                 scores[db] = self._patch_process_fct(db)
             self._scan.sa_delta_beta_params.scores = scores
-            SADeltaBetaProcess.autofocus(scan=self._scan)
+            SADeltaBetaTask.autofocus(scan=self._scan)
             self.db_value = self._scan.sa_delta_beta_params.autofocus
         else:
-            sa_delta_beta = SADeltaBetaProcess(
+            self._task = SADeltaBetaTask(
                 inputs={
                     "dump_process": False,
                     "data": self._scan,
                     "dry_run": self._dry_run,
-                    "sa_delta_beta_params": self._sa_delta_beta_params,
+                    "sa_delta_beta_params": self._sa_delta_beta_params.to_dict(),
+                    "serialize_output_data": False,
                 },
                 process_id=self.process_id,
             )
-            try:
-                sa_delta_beta.run()
-            except Exception as e:
-                _logger.error(str(e))
-                mess = "SADeltaBeta computation for {} failed.".format(str(self._scan))
-                state = DatasetState.FAILED
-            else:
-                mess = "SADeltaBeta computation for {} succeed.".format(str(self._scan))
-                state = DatasetState.WAIT_USER_VALIDATION
-                self.db_value = self._scan.sa_delta_beta_params.autofocus
 
-            nabu_logs = []
-            for std_err, std_out in zip(sa_delta_beta.std_errs, sa_delta_beta.std_outs):
-                nabu_logs.append(format_stderr_stdout(stdout=std_out, stderr=std_err))
-            self._nabu_log = nabu_logs
-            nabu_logs.insert(0, mess)
-            details = "\n".join(nabu_logs)
-            ProcessManager().notify_dataset_state(
-                dataset=self._scan,
-                process=self,
-                state=state,
-                details=details,
-            )
+            self._task.run()
+            self.db_value = self._task.outputs.best_db
+
+    def cancel(self):
+        if self._task is not None:
+            self._task.cancel()
+        self.quit()
