@@ -31,15 +31,15 @@ __date__ = "11/12/2017"
 import os
 import shutil
 import tempfile
+
 import fabio
 import numpy
 from silx.gui.utils.testutils import TestCaseQt
+
+from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefsTask
+from tomwer.core.process.reconstruction.darkref.params import DKRFRP, ReduceMethod
 from tomwer.core.utils.scanutils import MockEDF
-from tomwer.core.process.reconstruction.darkref.darkrefs import DarkRefs
-from tomwer.core.process.reconstruction.darkref.params import DKRFRP
-from tomwer.core.process.reconstruction.darkref.params import Method
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.test.utils import UtilsTest
+from tomwer.tests.datasets import TomwerCIDatasets
 
 
 class TestDarkRefsBehavior(TestCaseQt):
@@ -57,7 +57,9 @@ class TestDarkRefsBehavior(TestCaseQt):
         """number of theoretical dark file the algorithm should create"""
         self.dataset_folder = os.path.join(self.tmpDir, self.datasetsID)
 
-        dataDir = UtilsTest.getEDFDataset(self.datasetsID)
+        dataDir = TomwerCIDatasets.get_dataset(
+            f"edf_datasets/{self.datasetsID}",
+        )
         shutil.copytree(dataDir, self.dataset_folder)
         files = os.listdir(self.dataset_folder)
         for _f in files:
@@ -71,19 +73,19 @@ class TestDarkRefsBehavior(TestCaseQt):
         self.qapp.processEvents()
         self.darkRef = None
         shutil.rmtree(self.tmpDir)
-        TestCaseQt.tearDown(self)
 
     def testDarkCreation(self):
         """Test that the dark is correctly computed"""
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.median
+        self.recons_params.flat_calc_method = ReduceMethod.NONE
+        self.recons_params.dark_calc_method = ReduceMethod.MEDIAN
 
-        dar_ref_process = DarkRefs(
+        dar_ref_process = DarkRefsTask(
             inputs={
                 "dark_ref_params": self.recons_params,
                 "force_sync": True,
                 "darkhst_prefix": "darkHST",
                 "data": self.dataset_folder,
+                "serialize_output_data": False,
             }
         )
         dar_ref_process.run()
@@ -102,7 +104,7 @@ class TestDarkRefsBehavior(TestCaseQt):
             self.assertEqual(
                 len(
                     dar_ref_process.getRefHSTFiles(
-                        self.dataset_folder, prefix=self.recons_params.ref_prefix
+                        self.dataset_folder, prefix=self.recons_params.flat_prefix
                     )
                 ),
                 0,
@@ -110,15 +112,16 @@ class TestDarkRefsBehavior(TestCaseQt):
 
     def testRefCreation(self):
         """Test that the dark is correctly computed"""
-        self.recons_params.ref_calc_method = Method.median
-        self.recons_params.dark_calc_method = Method.none
+        self.recons_params.flat_calc_method = ReduceMethod.MEDIAN
+        self.recons_params.dark_calc_method = ReduceMethod.NONE
 
-        dar_ref_process = DarkRefs(
+        dar_ref_process = DarkRefsTask(
             inputs={
                 "dark_ref_params": self.recons_params,
                 "force_sync": True,
                 "darkhst_prefix": "darkHST",
                 "data": self.dataset_folder,
+                "serialize_output_data": False,
             }
         )
 
@@ -133,66 +136,6 @@ class TestDarkRefsBehavior(TestCaseQt):
         self.assertTrue("ref0001_0000.edf" in os.listdir(self.dataset_folder))
         self.assertTrue("ref0001_0020.edf" in os.listdir(self.dataset_folder))
 
-    def testRemoveOption(self):
-        """Test that the remove option is working"""
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.none
-        self.recons_params._set_remove_opt(True)
-        dar_ref_process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "force_sync": True,
-                "darkhst_prefix": "darkHST",
-                "data": self.dataset_folder,
-            }
-        )
-        dar_ref_process.run()
-        self.qapp.processEvents()
-        self.assertFalse("darkend0000.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("dark0000.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("refHST0000.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("refHST0020.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("ref0000_0000.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("ref0000_0020.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("ref0001_0000.edf" in os.listdir(self.dataset_folder))
-        self.assertFalse("ref0001_0020.edf" in os.listdir(self.dataset_folder))
-
-    def testSkipOption(self):
-        """Test that the overwrite option is working"""
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.none
-        self.recons_params._set_skip_if_exist(True)
-
-        iniRefNFile = len(
-            DarkRefs.getRefHSTFiles(
-                self.dataset_folder, prefix=self.recons_params.ref_prefix
-            )
-        )
-        iniDarkNFile = len(
-            DarkRefs.getDarkHSTFiles(
-                self.dataset_folder, prefix=self.recons_params.dark_prefix
-            )
-        )
-
-        dar_ref_process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "force_sync": True,
-                "darkhst_prefix": "darkHST",
-                "data": self.dataset_folder,
-            }
-        )
-        dar_ref_process.run()
-        self.qapp.processEvents()
-        refs = DarkRefs.getRefHSTFiles(
-            self.dataset_folder, prefix=self.recons_params.ref_prefix
-        )
-        darks = DarkRefs.getDarkHSTFiles(
-            self.dataset_folder, prefix=self.recons_params.dark_prefix
-        )
-        self.assertTrue(len(refs) == iniRefNFile)
-        self.assertTrue(len(darks) == iniDarkNFile)
-
 
 class TestRefCalculationOneSerie(TestCaseQt):
     """
@@ -222,22 +165,23 @@ class TestRefCalculationOneSerie(TestCaseQt):
         )
 
         self.recons_params = DKRFRP()
-        self.darkRef = DarkRefs(
+        self.darkRef = DarkRefsTask(
             inputs={
                 "dark_ref_params": self.recons_params,
                 "data": self.tmp_dir,
+                "serialize_output_data": False,
             }
         )
         self.darkRef.setForceSync(True)
-        self.recons_params.ref_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"
+        self.recons_params.flat_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"
 
     def tearDown(self):
         shutil.rmtree(self.tmp_dir)
         super().tearDown()
 
     def testRefMedianCalculation(self):
-        self.recons_params.ref_calc_method = Method.median
-        self.recons_params.dark_calc_method = Method.none
+        self.recons_params.flat_calc_method = ReduceMethod.MEDIAN
+        self.recons_params.dark_calc_method = ReduceMethod.NONE
         self.darkRef.run()
         refHST = os.path.join(self.tmp_dir, "refHST0000.edf")
         self.assertTrue(os.path.isfile(refHST))
@@ -246,8 +190,8 @@ class TestRefCalculationOneSerie(TestCaseQt):
         )
 
     def testRefMeanCalculation(self):
-        self.recons_params.ref_calc_method = Method.average
-        self.recons_params.dark_calc_method = Method.none
+        self.recons_params.flat_calc_method = ReduceMethod.MEAN
+        self.recons_params.dark_calc_method = ReduceMethod.NONE
         self.darkRef.run()
         refHST = os.path.join(self.tmp_dir, "refHST0000.edf")
         self.assertTrue(os.path.isfile(refHST))
@@ -280,22 +224,23 @@ class TestRefCalculationThreeSerie(TestCaseQt):
                 file_desc.write(os.path.join(self.tmp_dir, refFile))
 
         self.recons_params = DKRFRP()
-        self.darkRef = DarkRefs(
+        self.darkRef = DarkRefsTask(
             inputs={
                 "dark_ref_params": self.recons_params,
                 "data": self.tmp_dir,
                 "force_sync": True,
+                "serialize_output_data": False,
             }
         )
-        self.recons_params.ref_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"
+        self.recons_params.flat_pattern = "ref*.*[0-9]{3,4}_[0-9]{3,4}"
 
     def tearDown(self):
         shutil.rmtree(self.tmp_dir)
         super().tearDown()
 
     def testRefMedianCalculation(self):
-        self.recons_params.ref_calc_method = Method.median
-        self.recons_params.dark_calc_method = Method.none
+        self.recons_params.flat_calc_method = ReduceMethod.MEDIAN
+        self.recons_params.dark_calc_method = ReduceMethod.NONE
         self.darkRef.run()
         for serie in self.series:
             refHST = os.path.join(self.tmp_dir, "refHST" + str(serie).zfill(4) + ".edf")
@@ -307,8 +252,8 @@ class TestRefCalculationThreeSerie(TestCaseQt):
             )
 
     def testRefMeanCalculation(self):
-        self.recons_params.ref_calc_method = Method.average
-        self.recons_params.dark_calc_method = Method.none
+        self.recons_params.flat_calc_method = ReduceMethod.MEAN
+        self.recons_params.dark_calc_method = ReduceMethod.NONE
         self.darkRef.run()
         for serie in self.series:
             refHST = os.path.join(self.tmp_dir, "refHST" + str(serie).zfill(4) + ".edf")
@@ -335,11 +280,12 @@ class TestDarkCalculationOneFrame(TestCaseQt):
         file_desc.write(os.path.join(self.tmp_dir, "darkend0000.edf"))
         assert len(os.listdir(self.tmp_dir)) is (1 + n_scan + n_info + n_xml)
         self.recons_params = DKRFRP()
-        self.darkRef = DarkRefs(
+        self.darkRef = DarkRefsTask(
             inputs={
                 "dark_ref_params": self.recons_params,
                 "force_sync": True,
                 "data": self.tmp_dir,
+                "serialize_output_data": False,
             }
         )
 
@@ -348,8 +294,8 @@ class TestDarkCalculationOneFrame(TestCaseQt):
         super().tearDown()
 
     def testDarkMeanCalculation(self):
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.average
+        self.recons_params.flat_calc_method = ReduceMethod.NONE
+        self.recons_params.dark_calc_method = ReduceMethod.MEAN
 
         self.darkRef.run()
         refHST = os.path.join(self.tmp_dir, "dark.edf")
@@ -359,8 +305,8 @@ class TestDarkCalculationOneFrame(TestCaseQt):
         )
 
     def testDarkMedianCalculation(self):
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.median
+        self.recons_params.flat_calc_method = ReduceMethod.NONE
+        self.recons_params.dark_calc_method = ReduceMethod.MEDIAN
 
         self.darkRef.run()
         refHST = os.path.join(self.tmp_dir, "dark.edf")
@@ -390,11 +336,12 @@ class TestDarkCalculation(TestCaseQt):
         file_desc.write(os.path.join(self.tmp_dir, "darkend0000.edf"))
         assert len(os.listdir(self.tmp_dir)) is (1 + n_scan + n_xml + n_info)
         self.recons_params = DKRFRP()
-        self.darkRef = DarkRefs(
+        self.darkRef = DarkRefsTask(
             inputs={
                 "data": self.tmp_dir,
                 "dark_ref_params": self.recons_params,
                 "force_sync": True,
+                "serialize_output_data": False,
             }
         )
 
@@ -403,8 +350,8 @@ class TestDarkCalculation(TestCaseQt):
         super().tearDown()
 
     def testDarkMeanCalculation(self):
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.average
+        self.recons_params.flat_calc_method = ReduceMethod.NONE
+        self.recons_params.dark_calc_method = ReduceMethod.MEAN
 
         self.darkRef.run()
         refHST = os.path.join(self.tmp_dir, "dark.edf")
@@ -414,8 +361,8 @@ class TestDarkCalculation(TestCaseQt):
         )
 
     def testDarkMedianCalculation(self):
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.median
+        self.recons_params.flat_calc_method = ReduceMethod.NONE
+        self.recons_params.dark_calc_method = ReduceMethod.MEDIAN
 
         self.darkRef.run()
         refHST = os.path.join(self.tmp_dir, "dark.edf")
@@ -434,7 +381,9 @@ class TestDarkAccumulation(TestCaseQt):
     def setUp(self):
         super().setUp()
         self.dataset = "bone8_1_"
-        dataDir = UtilsTest.getEDFDataset(self.dataset)
+        dataDir = TomwerCIDatasets.get_dataset(
+            f"edf_datasets/{self.dataset}",
+        )
         self.outputdir = tempfile.mkdtemp()
         shutil.copytree(src=dataDir, dst=os.path.join(self.outputdir, self.dataset))
         self.darkFile = os.path.join(self.outputdir, self.dataset, "dark.edf")
@@ -447,8 +396,8 @@ class TestDarkAccumulation(TestCaseQt):
         os.remove(self.darkFile)
 
         self.recons_params = DKRFRP()
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.median
+        self.recons_params.flat_calc_method = ReduceMethod.NONE
+        self.recons_params.dark_calc_method = ReduceMethod.MEDIAN
         self.recons_params.dark_pattern = "darkend*"
         self.recons_params.dark_prefix = "dark.edf"
 
@@ -459,10 +408,11 @@ class TestDarkAccumulation(TestCaseQt):
     def testComputation(self):
         """Test data bone8_1_ from id16b containing dark.edf of reference
         and darkend"""
-        dark_ref_process = DarkRefs(
+        dark_ref_process = DarkRefsTask(
             inputs={
                 "data": os.path.join(self.outputdir, self.dataset),
                 "force_sync": True,
+                "serialize_output_data": False,
             }
         )
 
@@ -471,113 +421,3 @@ class TestDarkAccumulation(TestCaseQt):
         with fabio.open(self.darkFile) as dsc:
             self.computed_dark = dsc.data
         self.assertTrue(numpy.array_equal(self.computed_dark, self.dark_reference))
-
-
-class TestPCOTomo(TestCaseQt):
-    """Test processing of DKRF are correct"""
-
-    def setUp(self):
-        TestCaseQt.setUp(self)
-        self.tmp_dir = tempfile.mkdtemp()
-        MockEDF.fastMockAcquisition(self.tmp_dir)
-
-        self.recons_params = DKRFRP()
-        self.recons_params.ref_calc_method = Method.none
-        self.recons_params.dark_calc_method = Method.none
-        self.recons_params.dark_pattern = ".*_dark_.*"
-        self.recons_params.ref_pattern = ".*_ref_.*"
-        self.recons_params._set_remove_opt(True)
-
-    def copyDataset(self, dataset):
-        folder = os.path.join(self.tmp_dir, dataset)
-        shutil.copytree(os.path.join(UtilsTest.getEDFDataset(dataset)), folder)
-        self.scan = ScanFactory.create_scan_object(scan_path=folder)
-
-    def tearDown(self):
-        shutil.rmtree(self.tmp_dir)
-        TestCaseQt.tearDown(self)
-
-    def testDark3Scan(self):
-        """
-        Make sure the processing dark field for
-        pcotomo_3scan_refdarkbeg_end_download are correct
-        """
-        self.dataset = "pcotomo_3scan_refdarkbeg_end_download"
-        self.copyDataset(self.dataset)
-        _file = os.path.join(
-            self.tmp_dir, "pcotomo_3scan_refdarkbeg_end_download", "dark.edf"
-        )
-        if os.path.isfile(_file):
-            os.remove(_file)
-        self.recons_params.dark_calc_method = Method.median
-        dar_ref_process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "data": self.scan,
-                "darkhst_prefix": "darkHST",
-                "force_sync": True,
-            }
-        )
-        dar_ref_process.run()
-        darkHSTFiles = DarkRefs.getDarkHSTFiles(
-            self.scan.path, prefix=self.recons_params.dark_prefix
-        )
-        self.assertEqual(len(darkHSTFiles), 2)
-        dark0000 = os.path.join(self.scan.path, "dark0000.edf")
-        dark1000 = os.path.join(self.scan.path, "dark1000.edf")
-        self.assertTrue(dark0000 in darkHSTFiles)
-        self.assertTrue(dark1000 in darkHSTFiles)
-
-    def testRef3Scan(self):
-        """
-        Make sure the processing flat field for
-        pcotomo_3scan_refdarkbeg_end_download are correct
-        """
-        self.dataset = "pcotomo_3scan_refdarkbeg_end_download"
-        self.copyDataset(self.dataset)
-        self.recons_params.ref_calc_method = Method.median
-        dar_ref_process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "data": self.scan,
-                "darkhst_prefix": "darkHST",
-                "force_sync": True,
-            }
-        )
-        dar_ref_process.run()
-        refHSTFiles = DarkRefs.getRefHSTFiles(
-            self.scan.path, prefix=self.recons_params.ref_prefix
-        )
-        self.assertEqual(len(refHSTFiles), 2)
-        f0000 = os.path.join(self.scan.path, "refHST0000.edf")
-        self.assertTrue(f0000 in refHSTFiles)
-        f1000 = os.path.join(self.scan.path, "refHST1000.edf")
-        self.assertTrue(f1000 in refHSTFiles)
-
-    def testDark2x2Scan(self):
-        self.dataset = "pcotomo_2x2scan_refdarkbeg_end_conti"
-        self.copyDataset(self.dataset)
-        self.recons_params.dark_calc_method = Method.median
-        dar_ref_process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "data": self.scan,
-                "darkhst_prefix": "darkHST",
-                "force_sync": True,
-            }
-        )
-        dar_ref_process.run()
-
-    def testRef2x2Scan(self):
-        self.dataset = "pcotomo_2x2scan_refdarkbeg_end_conti"
-        self.copyDataset(self.dataset)
-        self.recons_params.ref_calc_method = Method.median
-        dar_ref_process = DarkRefs(
-            inputs={
-                "dark_ref_params": self.recons_params,
-                "data": self.scan,
-                "darkhst_prefix": "darkHST",
-                "force_sync": True,
-            }
-        )
-        dar_ref_process.run()
