@@ -28,21 +28,24 @@ __license__ = "MIT"
 __date__ = "24/01/2017"
 
 
-from tomwer.synctools.datatransfert import ScanTransfer
-from tomwer.core.scan.scanfactory import ScanFactory
-from tomwer.core.utils import rebaseParFile
-from tomwer.core.utils.scanutils import MockEDF
-from tomwer.test.utils import UtilsTest
-from tomwer.core.scan.hdf5scan import HDF5TomoScan
-from nxtomomill import converter as nxtomomill_converter
-from silx.gui.utils.testutils import TestCaseQt
-from glob import glob
-import unittest
-import os
-import tempfile
-import shutil
 import logging
+import os
+import shutil
+import tempfile
+import unittest
+from glob import glob
+
 import pytest
+from nxtomomill.converter import from_h5_to_nx
+from nxtomomill.io.config.hdf5config import TomoHDF5Config
+from silx.gui.utils.testutils import TestCaseQt
+
+from tomwer.core.scan.nxtomoscan import NXtomoScan
+from tomwer.core.scan.scanfactory import ScanFactory
+from tomwer.core.utils.spec import rebaseParFile
+from tomwer.core.utils.scanutils import MockEDF
+from tomwer.synctools.datatransfert import ScanTransfer
+from tomwer.tests.datasets import TomwerCIDatasets
 
 try:
     from tomwer.synctools.rsyncmanager import RSyncManager
@@ -66,8 +69,10 @@ class TestEDFDataTransfert(TestCaseQt):
         MockEDF.fastMockAcquisition(self.sourcedir, n_radio=self.n_file)
         self.scan = ScanFactory.create_scan_object(self.sourcedir)
         assert os.path.isdir(self.sourcedir)
+        assert os.path.exists(self.sourcedir)
         self.targettedir = tempfile.mkdtemp()
         assert os.path.isdir(self.targettedir)
+        assert os.path.exists(self.targettedir)
         self.outputdir = os.path.join(
             self.targettedir, os.path.basename(self.sourcedir)
         )
@@ -77,7 +82,6 @@ class TestEDFDataTransfert(TestCaseQt):
             shutil.rmtree(self.sourcedir)
         if os.path.isdir(self.targettedir):
             shutil.rmtree(self.targettedir)
-        TestCaseQt.tearDown(self)
 
     def testMoveFiles(self):
         """
@@ -127,7 +131,7 @@ class TestEDFDataTransfert(TestCaseQt):
                 "dest_dir": self.targettedir,
                 "noRsync": True,
                 "move": True,
-                "force": False,
+                "overwrite": False,
             }
         )
         folder_trans_process.run()
@@ -141,7 +145,7 @@ class TestEDFDataTransfert(TestCaseQt):
                 "dest_dir": self.targettedir,
                 "noRsync": True,
                 "move": True,
-                "force": False,
+                "overwrite": False,
             }
         )
         with self.assertRaises(shutil.Error):
@@ -155,7 +159,7 @@ class TestEDFDataTransfert(TestCaseQt):
                 "dest_dir": self.targettedir,
                 "noRsync": True,
                 "move": True,
-                "force": True,
+                "overwrite": True,
             }
         )
         folder_trans_process.run()
@@ -177,7 +181,7 @@ class TestEDFDataTransfert(TestCaseQt):
                 "dest_dir": self.targettedir,
                 "noRsync": True,
                 "move": False,
-                "force": False,
+                "overwrite": False,
             }
         )
         folder_trans_process.run()
@@ -194,7 +198,7 @@ class TestEDFDataTransfert(TestCaseQt):
                 "dest_dir": self.targettedir,
                 "noRsync": True,
                 "move": False,
-                "force": False,
+                "overwrite": False,
             }
         )
 
@@ -209,7 +213,7 @@ class TestEDFDataTransfert(TestCaseQt):
                 "dest_dir": self.targettedir,
                 "noRsync": True,
                 "move": False,
-                "force": True,
+                "overwrite": True,
             }
         )
         folder_trans_process.run()
@@ -227,14 +231,15 @@ class TestEDFDataTransfert(TestCaseQt):
 
 
 @pytest.mark.skipif(RSyncManager().has_rsync() is False, reason="Rsync is missing")
-class TestHDFDataTransfert(unittest.TestCase):
+class TestHDFDataTransfert(TestCaseQt):
     """Make sure we can transfer data from bliss acquisition"""
 
     def setUp(self):
+        super().setUp()
         self.input_dir = tempfile.mkdtemp()
         self.output_dir = tempfile.mkdtemp()
         shutil.copytree(
-            UtilsTest.getBlissDataset(folderID="sample"),
+            TomwerCIDatasets.get_dataset("bliss/sample"),
             os.path.join(self.input_dir, "sample"),
         )
 
@@ -252,20 +257,22 @@ class TestHDFDataTransfert(unittest.TestCase):
             self.input_dir, "sample", "sample_29042021", "nx_file.nx"
         )
         # convert it to nx
-        files_entries = nxtomomill_converter.h5_to_nx(
-            input_file_path=self._sample_file,
-            output_file=output_file_path,
-            entries=(self._sample_file_entry,),
-            single_file=False,
-            ask_before_overwrite=False,
-            request_input=False,
-            file_extension=".nx",
-        )
-        assert len(files_entries) == 1
-        output_file_path, entry = files_entries[0]
+        configuration = TomoHDF5Config()
+        configuration.input_file = self._sample_file
+        configuration.output_file = output_file_path
+        files_entries = (self._sample_file_entry,)
+        configuration.entries = files_entries
+        configuration.single_file = False
+        configuration.overwrite = False
+        configuration.request_input = False
+        configuration.file_extension = ".nx"
+        results = from_h5_to_nx(configuration)
+
+        assert len(results) == 1
+        output_file_path, entry = results[0]
         assert os.path.exists(output_file_path)
         assert os.path.isfile(output_file_path)
-        self.scan = HDF5TomoScan(scan=output_file_path, entry=entry)
+        self.scan = NXtomoScan(scan=output_file_path, entry=entry)
 
         # register information regarding origin of the file
         from tomwer.core.process.control.datalistener import DataListener
@@ -297,13 +304,16 @@ class TestHDFDataTransfert(unittest.TestCase):
             inputs={
                 "data": self.scan,
                 "dest_dir": os.path.join(self.output_dir, "sample"),
-                "force_sync": True,
+                "block": True,
             }
         )
         process.run()
         output_scan = os.path.join(
             self.output_dir, "sample", "sample_29042021", "scan0002"
         )
+        input_scan = os.path.join(
+            self.input_dir, "sample", "sample_29042021", "scan0002"
+        )
         output_proposal_file = os.path.join(
             self.output_dir, "sample", "ihpayno_sample.h5"
         )
@@ -311,7 +321,7 @@ class TestHDFDataTransfert(unittest.TestCase):
             self.output_dir, "sample", "sample_29042021", "sample_29042021.h5"
         )
         self.assertTrue(os.path.exists(output_scan))
-        self.assertEqual(len(os.listdir(output_scan)), 3)
+        self.assertEqual(len(os.listdir(output_scan)), len(os.listdir(input_scan)))
         self.assertTrue(os.path.exists(output_proposal_file))
         self.assertTrue(os.path.exists(output_sample_file))
         other_scan_dir = os.path.join(self.output_dir, "sample", "scan0004")
@@ -323,7 +333,7 @@ class TestPreTransfert(unittest.TestCase):
 
     def setUp(self):
         unittest.TestCase.setUp(self)
-        folderDataset = UtilsTest.getEDFDataset("scan_3_")
+        folderDataset = TomwerCIDatasets.get_dataset("edf_datasets/scan_3_")
         self.tmpdir = tempfile.mkdtemp()
         self.outputfolder = tempfile.mkdtemp()
         scan_path = os.path.join(self.tmpdir, "scan_3_")
@@ -364,7 +374,7 @@ class TestPreTransfert(unittest.TestCase):
                 "dest_dir": self.outputfolder,
                 "data": self.scan,
                 "move": False,
-                "force": True,
+                "overwrite": True,
                 "noRsync": True,
             }
         )
