@@ -22,6 +22,9 @@
 # THE SOFTWARE.
 #
 # ###########################################################################*/
+"""
+some project general utils functions
+"""
 
 __authors__ = ["H. Payno", "T. Vincent"]
 __license__ = "MIT"
@@ -30,19 +33,16 @@ __date__ = "04/01/2018"
 
 import argparse
 import functools
+import importlib
+import logging
+import sys
+
 from silx.utils.launcher import Launcher as _Launcher
 from silx.utils.launcher import LauncherCommand as _LauncherCommand
-import sys
-import logging
-import importlib
 
 _logger = logging.getLogger(__file__)
 
 
-def addHTMLLine(txt):
-    return "<li>" + txt + "</li>"
-
-
 def _docstring(dest, origin):
     """Implementation of docstring decorator.
 
@@ -202,12 +202,12 @@ class Launcher(_Launcher):
             return 0
 
         if command_name not in self._commands:
-            print("Unknown command: %s", command_name)
+            print(f"Unknown command: {command_name}")
             self.print_help()
             return -1
 
         command = self._commands[command_name]
-        prog = "%s %s" % (self.prog, command.name)
+        prog = f"{self.prog} {command.name}"
         return command.execute([prog, "--help"])
 
     def execute(self, argv=None):
@@ -228,7 +228,7 @@ class Launcher(_Launcher):
         command_name = argv[1]
 
         if command_name == "--version":
-            print("%s version %s" % (self.prog, str(self.version)))
+            print(f"{self.prog} version {str(self.version)}")
             return 0
 
         if command_name in ["--help", "-h"]:
@@ -239,10 +239,10 @@ class Launcher(_Launcher):
             else:
                 command_name = argv[2]
                 command_argv = argv[2:] + ["--help"]
-                command_argv[0] = "%s %s" % (self.prog, command_argv[0])
+                command_argv[0] = f"{self.prog} {command_argv[0]}"
         else:
             command_argv = argv[1:]
-            command_argv[0] = "%s %s" % (self.prog, command_argv[0])
+            command_argv[0] = f"{self.prog} {command_argv[0]}"
 
         if command_name not in self._commands:
             print("Unknown command: %s" % command_name)
